/*
	BtAnalogStream.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "Debug.hpp"

#include "BtDigitizer.hpp"
#include "BtAnalogStream.hpp"

#if __ORIG_WAY__ // Does not work properly... It does not allow the Video Pin to connect properly!

// Helper macro
#define DECL_ANALOGVIDEO(maxx,maxy,fps,v0,v1,v2,v3,v4,v5,v6,v7,v8,v9,va) \
{ \
	/* KS_DATARANGE_ANALOGVIDEO */ \
	{ \
		{ \
			sizeof( KS_DATARANGE_ANALOGVIDEO ), \
			0, \
			sizeof (KS_TVTUNER_CHANGE_INFO),        /* SampleSize */ \
			0, \
			STATIC_KSDATAFORMAT_TYPE_ANALOGVIDEO,		/* MEDIATYPE_AnalogVideo*/ \
			{ v0, v1, v2, { v3, v4, v5, v6, v7, v8, v9, va } },  /* Norm */ \
			STATIC_KSDATAFORMAT_SPECIFIER_ANALOGVIDEO	/* FORMAT_AnalogVideo */ \
		} \
	}, \
  /* KS_ANALOGVIDEOINFO */ \
  { \
      { 0, 0, maxx, maxy },         /* rcSource*/ \
      { 0, 0, maxx, maxy },         /* rcTarget*/ \
      maxx,                    /* dwActiveWidth*/ \
      maxy,                    /* dwActiveHeight*/ \
      (10000000i64 / fps),                      /* REFERENCE_TIME  AvgTimePerFrame */ \
   } \
}

/* 
	PAL-BDGHI	= 920x576x50
	SECAM-L		= 920x576x50
	PAL-N       = 920x576x50
	PAL_60		= 920x480x60
	PAL-NC		= 744x576x50
	NTSC-50		= 756x576x50
	PAL-M		= 756x480x60
	NTSC-J		= 756x480x60
	NTSC		= 756x480x60
*/

KS_DATARANGE_ANALOGVIDEO BtAnalogStream::Format[] = {
	DECL_ANALOGVIDEO(768,576,25,0x482dde5, 0x7817, 0x11cf, 0x8a, 0x3, 0x0, 0xaa, 0x0, 0x6e, 0xcb, 0x65), //MEDIASUBTYPE_AnalogVideo_PAL_B
	DECL_ANALOGVIDEO(768,576,25,0x482dde6, 0x7817, 0x11cf, 0x8a, 0x3, 0x0, 0xaa, 0x0, 0x6e, 0xcb, 0x65), //MEDIASUBTYPE_AnalogVideo_PAL_D
	DECL_ANALOGVIDEO(768,576,25,0x482dde7, 0x7817, 0x11cf, 0x8a, 0x3, 0x0, 0xaa, 0x0, 0x6e, 0xcb, 0x65), //MEDIASUBTYPE_AnalogVideo_PAL_G
	DECL_ANALOGVIDEO(768,576,25,0x482dde8, 0x7817, 0x11cf, 0x8a, 0x3, 0x0, 0xaa, 0x0, 0x6e, 0xcb, 0x65), //MEDIASUBTYPE_AnalogVideo_PAL_H
	DECL_ANALOGVIDEO(768,576,25,0x482dde9, 0x7817, 0x11cf, 0x8a, 0x3, 0x0, 0xaa, 0x0, 0x6e, 0xcb, 0x65), //MEDIASUBTYPE_AnalogVideo_PAL_I
	DECL_ANALOGVIDEO(768,576,25,0x482ddf6, 0x7817, 0x11cf, 0x8a, 0x3, 0x0, 0xaa, 0x0, 0x6e, 0xcb, 0x65), //MEDIASUBTYPE_AnalogVideo_SECAM_L
	DECL_ANALOGVIDEO(768,576,25,0x482ddeb, 0x7817, 0x11cf, 0x8a, 0x3, 0x0, 0xaa, 0x0, 0x6e, 0xcb, 0x65), //MEDIASUBTYPE_AnalogVideo_PAL_N
	DECL_ANALOGVIDEO(768,480,30,0x482ddec, 0x7817, 0x11cf, 0x8a, 0x3, 0x0, 0xaa, 0x0, 0x6e, 0xcb, 0x65), //MEDIASUBTYPE_AnalogVideo_PAL_60
	DECL_ANALOGVIDEO(744,576,25,0x482dded, 0x7817, 0x11cf, 0x8a, 0x3, 0x0, 0xaa, 0x0, 0x6e, 0xcb, 0x65), //MEDIASUBTYPE_AnalogVideo_PAL_NC
	DECL_ANALOGVIDEO(756,480,30,0x482dde2, 0x7817, 0x11cf, 0x8a, 0x3, 0x0, 0xaa, 0x0, 0x6e, 0xcb, 0x65), //MEDIASUBTYPE_AnalogVideo_NTSC_M
	DECL_ANALOGVIDEO(756,480,30,0x482ddea, 0x7817, 0x11cf, 0x8a, 0x3, 0x0, 0xaa, 0x0, 0x6e, 0xcb, 0x65), //MEDIASUBTYPE_AnalogVideo_PAL_M
	DECL_ANALOGVIDEO(756,480,30,0x482dde3, 0x7817, 0x11cf, 0x8a, 0x3, 0x0, 0xaa, 0x0, 0x6e, 0xcb, 0x65), //MEDIASUBTYPE_AnalogVideo_NTSC_M_J ??
	DECL_ANALOGVIDEO(756,576,25,0x482dde4, 0x7817, 0x11cf, 0x8a, 0x3, 0x0, 0xaa, 0x0, 0x6e, 0xcb, 0x65)  //MEDIASUBTYPE_AnalogVideo_NTSC_50 ??
};

//---------------------------------------------------------------------------
//  Analog Video Stream Formats
//---------------------------------------------------------------------------
PKSDATAFORMAT BtAnalogStream::Formats[] =
{
	(PKSDATAFORMAT) &BtAnalogStream::Format[0],
	(PKSDATAFORMAT) &BtAnalogStream::Format[1],
	(PKSDATAFORMAT) &BtAnalogStream::Format[2],
	(PKSDATAFORMAT) &BtAnalogStream::Format[3],
	(PKSDATAFORMAT) &BtAnalogStream::Format[4],
	(PKSDATAFORMAT) &BtAnalogStream::Format[5],
	(PKSDATAFORMAT) &BtAnalogStream::Format[6],
	(PKSDATAFORMAT) &BtAnalogStream::Format[7],
	(PKSDATAFORMAT) &BtAnalogStream::Format[8],
	(PKSDATAFORMAT) &BtAnalogStream::Format[9],
	(PKSDATAFORMAT) &BtAnalogStream::Format[10]
};

#else

KS_DATARANGE_ANALOGVIDEO BtAnalogStream::Format[] = 
{ 
	/* KS_DATARANGE_ANALOGVIDEO */ 
	{ 
		{ 
			sizeof( KS_DATARANGE_ANALOGVIDEO ), 
			0, 
			sizeof (KS_TVTUNER_CHANGE_INFO),			/* SampleSize */ 
			0, 
			{STATIC_KSDATAFORMAT_TYPE_ANALOGVIDEO},		/* MEDIATYPE_AnalogVideo*/ 
			{STATIC_KSDATAFORMAT_SUBTYPE_NONE},			/* Norm */ 
			{STATIC_KSDATAFORMAT_SPECIFIER_ANALOGVIDEO}	/* FORMAT_AnalogVideo */ 
		} 
	}, 
	/* KS_ANALOGVIDEOINFO */ 
	{ 
		{0, 0, 720, 480},         /* rcSource*/ 
		 0, 0, 720, 480,         /* rcTarget*/ 
		720,                      /* dwActiveWidth*/ 
		480,                      /* dwActiveHeight*/ 
		0                         /* REFERENCE_TIME  AvgTimePerFrame */ 
	} 
};

PKSDATAFORMAT BtAnalogStream::Formats[] =
{
	(PKSDATAFORMAT) &BtAnalogStream::Format[0],
};

#endif

// Number of the analog Formats!
#define NUM_ANALOG_FORMATS (sizeof (BtAnalogStream::Formats) / sizeof (PKSDATAFORMAT))

// -----------------------------------------------------------------
// Analog Video Input Stream
// -----------------------------------------------------------------

// HW_STREAM_INFORMATION -------------------------------------------
HW_STREAM_INFORMATION BtAnalogStream::StreamInfo_Template = 
{
	1,												// NumberOfPossibleInstances
	KSPIN_DATAFLOW_IN,								// DataFlow
	TRUE,											// DataAccessible
	NUM_ANALOG_FORMATS,								// NumberOfFormatArrayEntries
	BtAnalogStream::Formats,		                // StreamFormatsArray
	{
		0,											// ClassReserved[0]
		0,											// ClassReserved[1]
		0,											// ClassReserved[2]
		0											// ClassReserved[3]
	},
	0,												// NumStreamPropArrayEntries
	0,												// StreamPropertiesArray
	0,												// NumStreamEventArrayEntries;
	0,												// StreamEventsArray;
	(GUID *) &PINNAME_VIDEO_ANALOGVIDEOIN,			// Category
	(GUID *) &PINNAME_VIDEO_ANALOGVIDEOIN,			// Name
	1,												// MediumsCount
	NULL,											// Mediums : Patched when per instance data is created.
	FALSE
};


// HW_STREAM_OBJECT ------------------------------------------------
HW_STREAM_OBJECT BtAnalogStream::StreamObject =
{
	sizeof (HW_STREAM_OBJECT),              // SizeOfThisPacket
	STREAM_IDX_ANALOG,                      // StreamNumber
	0,                                      // HwStreamExtension
	BtDigitizer::cbReceiveDataPacket,		// HwReceiveDataPacket
	BtDigitizer::cbReceiveCtrlPacket,		// HwReceiveControlPacket
	{ NULL, 0 },                            // HW_CLOCK_OBJECT
	FALSE,                                  // Dma
	FALSE,                                  // Pio
	0,										// HwDeviceExtension
	0,                                      // StreamHeaderMediaSpecific
	0,                                      // StreamHeaderWorkspace 
	FALSE,                                  // Allocator 
	NULL,                                   // HwEventRoutine
	{ 0, 0 },                               // Reserved[2]
};


// Constructor
BtAnalogStream::BtAnalogStream(BtDigitizer* dev) : 
	digitizer(dev) , bIsOpen(FALSE) , KSState(KSSTATE_STOP)
{
	// Create a copy of the Stream information...
	memcpy(&StreamInfo,&StreamInfo_Template,sizeof(StreamInfo_Template));
	// Fix the template to point to our medium
	StreamInfo.Mediums = (const KSPIN_MEDIUM* ) &(dev->Mediums[3]);
}

NTSTATUS BtAnalogStream::Open( IN PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtAnalogStream::Open"));

	if (bIsOpen) { // Already open?
		return STATUS_INVALID_PARAMETER;
	}
	bIsOpen = TRUE;

	// Pass our callbacks !
	// the structure of the driver is such that a single callback could be used
	// for all stream requests. But the code below could be used to supply
	// different entry points for different streams

	pSrb->StreamObject->ReceiveDataPacket			= StreamObject.ReceiveDataPacket;
	pSrb->StreamObject->ReceiveControlPacket		= StreamObject.ReceiveControlPacket;
	pSrb->StreamObject->Dma							= StreamObject.Dma;
	pSrb->StreamObject->Allocator					= StreamObject.Allocator;

	// The PIO flag must be set when the mini driver will be accessing the data
	// buffers passed in using logical addressing

	pSrb->StreamObject->Pio							= StreamObject.Pio;
	pSrb->StreamObject->StreamHeaderMediaSpecific	= StreamObject.StreamHeaderMediaSpecific;
	pSrb->StreamObject->HwClockObject				= StreamObject.HwClockObject;

	KSState = KSSTATE_STOP;

	DUMP(("Created!\n"));
	return STATUS_SUCCESS;
}

NTSTATUS BtAnalogStream::Close( IN PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtAnalogStream::Close"));
	if (!bIsOpen) { // Already closed?
		return STATUS_INVALID_PARAMETER;
	}
	bIsOpen = FALSE;
	KSState = KSSTATE_STOP;

	return STATUS_SUCCESS;
}

/*
** BtAnalogStream::FormatFromRange()
**
**   Returns a DATAFORMAT from a DATARANGE
**
** Arguments:
**
**         IN PHW_STREAM_REQUEST_BLOCK pSrb
**
** Returns:
**
**   TRUE if the format is supported
**   FALSE if the format cannot be suppored
**
** Side Effects:  none
*/

NTSTATUS BtAnalogStream::FormatFromRange( IN PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtAnalogStream::FormatFromRange"));

	PSTREAM_DATA_INTERSECT_INFO IntersectInfo;
	PKSDATARANGE                DataRange;
	ULONG                       FormatSize;
	ULONG                       j;
	PKSDATAFORMAT*				pAvailableFormats = Formats;

	IntersectInfo = pSrb->CommandData.IntersectInfo;
	DataRange = IntersectInfo->DataRange;


	//
	// Is the caller trying to get the format, or the size of the format?
	//

	const bool OnlyWantsSize = (IntersectInfo->SizeOfDataFormatBuffer == sizeof( ULONG ) ) ||
							   (IntersectInfo->SizeOfDataFormatBuffer == 0 )
								;

	//
	// Walk the formats supported by the stream searching for a match
	// of the three GUIDs which together define a DATARANGE
	//

	for ( j = 0; j < NUM_ANALOG_FORMATS; j++, pAvailableFormats++ ) {

		if ( !CompareGUIDsAndFormatSize( DataRange, *pAvailableFormats, TRUE )) {
			continue;
		}

		//
		// Now that the three GUIDs match, switch on the Specifier
		// to do a further type specific check
		//

		// -------------------------------------------------------------------
		// Specifier FORMAT_AnalogVideo for KS_ANALOGVIDEOINFO
		// -------------------------------------------------------------------

		if ( DataRange->Specifier == KSDATAFORMAT_SPECIFIER_ANALOGVIDEO  ) {

			//
			// For analog video, the DataRange and DataFormat
			// are identical, so just copy the whole structure
			//

			PKS_DATARANGE_ANALOGVIDEO pDataRangeVideo = (PKS_DATARANGE_ANALOGVIDEO) *pAvailableFormats;

			// MATCH FOUND!
			FormatSize = sizeof( KS_DATARANGE_ANALOGVIDEO )	;

			if ( OnlyWantsSize ) {	
				break;
			}

			// Caller wants the full data format
			if ( IntersectInfo->SizeOfDataFormatBuffer < FormatSize ) {
               
				DUMP(("Buffer too small\n"));
				return STATUS_BUFFER_TOO_SMALL;
			}

			RtlCopyMemory( IntersectInfo->DataFormatBuffer,
				pDataRangeVideo, sizeof( KS_DATARANGE_ANALOGVIDEO ) );

			((PKSDATAFORMAT)IntersectInfo->DataFormatBuffer)->FormatSize = FormatSize;

			break;
		} 

		// -------------------------------------------------------------------
		// No MATCH!
		// -------------------------------------------------------------------

		else 
		{
			ERROR(( "STATUS_NO_MATCH\n" ));
			return STATUS_NO_MATCH;
		}

	} // End of loop on all formats for this stream

	if ( OnlyWantsSize ) {

        // Is the return buffer size = 0 ?
        if(IntersectInfo->SizeOfDataFormatBuffer == 0) {
			DUMP(("Proxy wants to know the required buffer size\n"));
            // the proxy wants to know the actuall buffer size to allocate.
            pSrb->ActualBytesTransferred = FormatSize;
            return STATUS_BUFFER_OVERFLOW;
        }

		DUMP(("only wants size\n" ));
        *(PULONG) IntersectInfo->DataFormatBuffer = FormatSize;
        pSrb->ActualBytesTransferred = sizeof(ULONG);
		return STATUS_SUCCESS;
	}

	pSrb->ActualBytesTransferred = FormatSize;
	DUMP(("done\n" ));

	return STATUS_SUCCESS;
}

// Notification of Channel Change:
/* Note: This is NOT synchronized to the processing of the other streams
 *	and can be called concurrently  */
NTSTATUS BtAnalogStream::ChangeNotifyChannels( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtAnalogStream::ChangeNotifyChannels"));

	digitizer->SendChangeChannelNotification(pSrb);

	return STATUS_SUCCESS;
}

// Called to get description of the Stream. Must be a Static Struct.
HW_STREAM_INFORMATION& BtAnalogStream::Info( void )
{
	TRACE(("BtAnalogStream::Info"));

	// Return a pointer to the stream descriptor
	return StreamInfo;
}

BOOL BtAnalogStream::IsOpen( void )
{
	TRACE(("BtAnalogStream::IsOpen"));

	return bIsOpen;
}

NTSTATUS BtAnalogStream::GetState( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtAnalogStream::GetState"));
		
	pSrb->CommandData.StreamState = KSState;
	pSrb->ActualBytesTransferred = sizeof (KSSTATE);

	// A very odd rule:
	// When transitioning from stop to pause, DShow tries to preroll
	// the graph.  Capture sources can't preroll, and indicate this
	// by returning VFW_S_CANT_CUE in user mode.  To indicate this
	// condition from drivers, they must return ERROR_NO_DATA_DETECTED
	//
	// JayBo says KSSTATE_ACQUIRE should return success 

	if (pSrb->CommandData.StreamState == KSSTATE_PAUSE) {
		return STATUS_NO_DATA_DETECTED;
	}

	return STATUS_SUCCESS;
}

NTSTATUS BtAnalogStream::SetState( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtAnalogStream::SetState"));

	// determine which new state is requested and set it
	KSState = pSrb->CommandData.StreamState;

	return STATUS_SUCCESS;
}

NTSTATUS BtAnalogStream::ProposeFormat( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtAnalogStream::ProposeFormat"));
	return STATUS_SUCCESS;
}

NTSTATUS BtAnalogStream::SetFormat( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtAnalogStream::SetFormat"));
	return STATUS_SUCCESS;
}

NTSTATUS BtAnalogStream::GetFormat( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtAnalogStream::GetFormat"));

	ERROR(("Not implemented yet")); // BUGBUG

	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS BtAnalogStream::GetProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtAnalogStream::GetProperty"));

	ERROR(("Property not implemented\n"));
	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS BtAnalogStream::SetProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtAnalogStream::SetProperty"));

	ERROR(("Property not implemented\n"));
	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS BtAnalogStream::IndicateMasterClock( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtAnalogStream::IndicateMasterClock"));
	return STATUS_SUCCESS;
}
