/*
	BtAnalogStream.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef BTANALOGSTREAM_HPP_
#define BTANALOGSTREAM_HPP_

#include "Stream.hpp"

// Forward declarations
class BtDigitizer;

// This class represents the analog video Stream 
class BtAnalogStream : public AnalogStream {

	public:
		static KS_DATARANGE_ANALOGVIDEO Format[];
		static PKSDATAFORMAT			Formats[];
		static HW_STREAM_INFORMATION	StreamInfo_Template;
		static HW_STREAM_OBJECT			StreamObject;

	protected:
		// Our per instance stream information and Medium
		HW_STREAM_INFORMATION	StreamInfo;

	private:

		// Pointer to the digitizer object
		BtDigitizer*	digitizer;
		BOOL			bIsOpen;
		KSSTATE			KSState;			// Stream Status

	public:

		BtAnalogStream(BtDigitizer* dev);
		
		virtual NTSTATUS Open( IN PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS Close( IN PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS FormatFromRange( IN PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS GetState( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS SetState( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS ProposeFormat( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS SetFormat( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS GetFormat( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS GetProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS SetProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS IndicateMasterClock( PHW_STREAM_REQUEST_BLOCK pSrb );


		virtual HW_STREAM_INFORMATION& Info( void );
		virtual BOOL IsOpen( void );

	protected:

		virtual NTSTATUS ChangeNotifyChannels( PHW_STREAM_REQUEST_BLOCK pSrb );

};

#endif

