/*
	BtCaptureStream.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* Bits		biCompression */
/* RGB32	BI_RGB */


 
 
/*
 RGB565	
 h.biClrUsed = 3;
 h.biBitCount = 16;
 h.biCompression = BI_BITFIELDS;
 *((LPDWORD)pbmi->bmiColors) = 0xF800;
 *((LPDWORD)pbmi->bmiColors+1) = 0x07E0;
 *((LPDWORD)pbmi->bmiColors+2) = 0x001F;
 RGB555
 h.biClrUsed = 3;
 h.biBitCount = 16;
 h.biCompression = BI_BITFIELDS;
 *((LPDWORD)pbmi->bmiColors) = 0x7C00;
 *((LPDWORD)pbmi->bmiColors+1) = 0x03E0;
 *((LPDWORD)pbmi->bmiColors+2) = 0x001F;
 RGB8
 h.biClrUsed = 0;
 h.biBitCount = 8;
 h.biCompression = BI_RGB;
 RGB24
 h.biClrUsed = 0;
 h.biBitCount = 24;
 h.biCompression = BI_RGB; // Can be also BI_BITFIELDS .. NO!!!
 RGB32
 h.biClrUsed = 0;
 h.biBitCount = 24;
 h.biCompression = BI_BITFIELDS or BI_RGB;
*/


#ifndef BTCAPTURESTREAM_HPP_
#define BTCAPTURESTREAM_HPP_

#include "BtVideoStream.hpp"
#include "BtPreviewStream.hpp"


// This class represents the capture pin.
class BtVCaptureStream : public BtVideoStream {

	public:

		static KS_DATARANGE_VIDEO			FormatRGB24[];
		static KS_DATARANGE_VIDEO			FormatRGB32[];
		static KS_DATARANGE_VIDEO			FormatYUY2[];
		static KS_DATARANGE_VIDEO			FormatYVU9[];
		static KS_DATARANGE_VIDEO			FormatUYVY[];
		static KS_DATARANGE_VIDEO			FormatY41P[];
		static KS_DATARANGE_VIDEO			FormatYV12[];
		static KS_DATARANGE_VIDEO			FormatI420[];
		static KS_DATARANGE_VIDEO			FormatRGB555[]; // Per ActiveMovie impelemtation
		static KS_DATARANGE_VIDEO_WITHMASK	FormatRGB565[]; // No other way to describe it
		static KS_DATARANGE_VIDEO2			FormatRGB24_2[];
		static KS_DATARANGE_VIDEO2			FormatRGB32_2[];
		static KS_DATARANGE_VIDEO2			FormatYUY2_2[];
		static KS_DATARANGE_VIDEO2			FormatYVU9_2[];
		static KS_DATARANGE_VIDEO2			FormatUYVY_2[];
		static KS_DATARANGE_VIDEO2			FormatY41P_2[];
		static KS_DATARANGE_VIDEO2			FormatYV12_2[];
		static KS_DATARANGE_VIDEO2			FormatI420_2[];
		static KS_DATARANGE_VIDEO2			FormatRGB555_2[];
		static KS_DATARANGE_VIDEO2_WITHMASK	FormatRGB565_2[]; // No other way to describe it
		static PKSDATAFORMAT				Formats[];
		static HW_STREAM_INFORMATION		StreamInfo;
		static HW_STREAM_OBJECT				StreamObject;
		static const KSPROPERTY_ITEM		Connection[];
		static const KSPROPERTY_ITEM		DroppedFrames[];
		static const KSPROPERTY_ITEM		Allocator[];
		static KSPROPERTY_SET				Properties[];

	protected:

		// Pointer to the preview Stream
		BtVPreviewStream* Preview;

	public:

		BtVCaptureStream(BtDigitizer *dev,BtVPreviewStream* Prev);
		virtual ~BtVCaptureStream();

	protected:

		// Function called to Timestamp Buffers
		virtual void TimeStamp( PKSSTREAM_HEADER pHeader , PHW_STREAM_REQUEST_BLOCK pSrb );

		// Events on channels
		virtual NTSTATUS Create();
		virtual NTSTATUS Destroy();

	public:

		virtual HW_STREAM_INFORMATION& Info( void ); 

	protected:

		// Functions
		virtual NTSTATUS GetConnectionProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) ;
		virtual NTSTATUS GetAllocatorProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) ;

	protected:

		// Used to try to let preview run while we are capturing at a lower framerate than available!
		virtual const VideoDataBuf OnSampling(bool GetSample);
};

#endif

