/*
	BtDigitizer.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


  Beware:
	If using VIDEOINFOHEADER2 to get fields, check KSPROPERTY_VIDEOCONTROL_FRAME_RATES_S -> Just don't care for fields!
*/


#include "Debug.hpp"
#include "btguids.hpp"
#include "BtDigitizer.hpp"
#include "CppRt.hpp"		// To get new and delete
#include "Registry.hpp"

// To resolve those sizeofs...
#include "BtTVAudio.cpp"
#include "BtXBar.cpp"
#include "BtTuner.cpp"
#include "BtVideoProcAmp.cpp"
#include "BtVideoDec.cpp"
#include "BtVideoCtl.cpp"
#include "BtBt848.cpp"
#include "BtAudio.cpp"
#include "BtIR.cpp"
#include "BtVideoWindow.cpp"

//---------------------------------------------------------------------------
// Topology
//---------------------------------------------------------------------------

//
// All of the property sets supported by the adapter
//
KSPROPERTY_SET BtDigitizer::PropertySets[] =
{
	DEFINE_KSPROPERTY_SET
	(
		&PROPSETID_VIDCAP_CROSSBAR,					
		SIZEOF_ARRAY(BtXBar::Properties),
		BtXBar::Properties,
		0,								
		NULL							
	),
	DEFINE_KSPROPERTY_SET
	(
		&PROPSETID_TUNER,					
		SIZEOF_ARRAY(BtTuner::Properties),
		BtTuner::Properties,			
		0,								
		NULL							
	),
	DEFINE_KSPROPERTY_SET
    ( 
        &PROPSETID_VIDCAP_TVAUDIO,
        SIZEOF_ARRAY(BtTVAudio::Properties),
        BtTVAudio::Properties,
        0, 
        NULL
    ),
    DEFINE_KSPROPERTY_SET
    (
        &PROPSETID_VIDCAP_VIDEOPROCAMP,
        SIZEOF_ARRAY(BtVideoProcAmp::Properties),
        BtVideoProcAmp::Properties,
        0,
        NULL
    ),
    DEFINE_KSPROPERTY_SET
    (
        &PROPSETID_VIDCAP_VIDEODECODER,
        SIZEOF_ARRAY(BtVideoDec::Properties),
        BtVideoDec::Properties,
        0,
        NULL
    ),
	DEFINE_KSPROPERTY_SET
    ( 
        &PROPSETID_VIDCAP_VIDEOCONTROL,
        SIZEOF_ARRAY(BtVideoCtl::Properties),
        BtVideoCtl::Properties,
        0, 
        NULL
    ),
    DEFINE_KSPROPERTY_SET
    ( 
        &PROPSETID_CUSTOMBT848,
        SIZEOF_ARRAY(BtBt848::Properties),
        BtBt848::Properties,
        0, 
        NULL
    ),
    DEFINE_KSPROPERTY_SET
    ( 
        &PROPSETID_AUDIO,
        SIZEOF_ARRAY(BtAudio::Properties),
        BtAudio::Properties,
        0, 
        NULL
    ),
    DEFINE_KSPROPERTY_SET
    ( 
        &PROPSETID_IR,
        SIZEOF_ARRAY(BtIR::Properties),
        BtIR::Properties,
        0, 
        NULL
    ),
    DEFINE_KSPROPERTY_SET
    ( 
        &PROPSETID_VIDEOWINDOW,
        SIZEOF_ARRAY(BtVideoWindow::Properties),
        BtVideoWindow::Properties,
        0, 
        NULL
    )
};

// Categories define what the device does.
GUID BtDigitizer::AdapterCategories[] = {
    { STATIC_KSCATEGORY_VIDEO },
    { STATIC_KSCATEGORY_CAPTURE },
    { STATIC_KSCATEGORY_TVAUDIO },
	{ STATIC_KSCATEGORY_CROSSBAR },
	{ STATIC_KSCATEGORY_TVTUNER }
};

KSTOPOLOGY BtDigitizer::Topology = {
   SIZEOF_ARRAY (BtDigitizer::AdapterCategories),
   (GUID*) &BtDigitizer::AdapterCategories,
   0,
   NULL,
   0,
   NULL
};

//---------------------------------------------------------------------------
// The Main stream header
//---------------------------------------------------------------------------

HW_STREAM_HEADER BtDigitizer::StreamHeader = 
{
	DRIVER_STREAM_COUNT,							// NumberOfStreams
	sizeof( HW_STREAM_INFORMATION ),				// Future proofing
	SIZEOF_ARRAY (BtDigitizer::PropertySets),		// NumDevPropArrayEntries
	(PKSPROPERTY_SET) BtDigitizer::PropertySets,	// DevicePropertiesArray
	0,												// NumDevEventArrayEntries;
	NULL,											// DeviceEventsArray;
	&BtDigitizer::Topology,							// Pointer to Device Topology
	NULL,											// DeviceEventRoutine
	0,
	0
};

// Template of mediums
KSPIN_MEDIUM BtDigitizer::Mediums_Template[4] = {
        {STATIC_GUID_NULL,  0, 0},					// Pin 0  Capture
        {STATIC_GUID_NULL,  0, 0},					// Pin 1  Preview
        {STATIC_GUID_NULL,  0, 0},					// Pin 2  VBI
        {M_GUID3,			0, 0}					// Pin 3  Analog Video In
};

BOOL BtDigitizer::PinDirection [4] = {
        TRUE,										// Output Pin 0
        TRUE,										// Output Pin 1
        TRUE,										// Output Pin 2
        FALSE										// Input  Pin 3
};

GUID BtDigitizer::Categories [4] = {
    STATIC_PINNAME_VIDEO_CAPTURE,					// Pin 0
    STATIC_PINNAME_VIDEO_PREVIEW,					// Pin 1
    STATIC_PINNAME_VIDEO_VBI,						// Pin 2
    STATIC_PINNAME_VIDEO_ANALOGVIDEOIN				// Pin 3
};

KSPIN_MEDIUM BtDigitizer::StandardMedium = {
    STATIC_KSMEDIUMSETID_Standard,  0, 0
};

#define FPSx100_TO_NS(x) (1000000000i64 / LONGLONG(x))

// List of available framerates!
LONGLONG BtDigitizer::FramerateList[] =
{
	FPSx100_TO_NS(  10),
	FPSx100_TO_NS(  25),
	FPSx100_TO_NS(  50),
	FPSx100_TO_NS( 100),
	FPSx100_TO_NS( 200),
	FPSx100_TO_NS( 300),
	FPSx100_TO_NS( 400),
	FPSx100_TO_NS( 500),
	FPSx100_TO_NS( 600),
	FPSx100_TO_NS( 700),
	FPSx100_TO_NS( 800),
	FPSx100_TO_NS( 900),
	FPSx100_TO_NS(1000),
	FPSx100_TO_NS(1100),
	FPSx100_TO_NS(1200),
	FPSx100_TO_NS(1300),
	FPSx100_TO_NS(1400),
	FPSx100_TO_NS(1500),
	FPSx100_TO_NS(1600),
	FPSx100_TO_NS(1700),
	FPSx100_TO_NS(1800),
	FPSx100_TO_NS(1900),
	FPSx100_TO_NS(2000),
	FPSx100_TO_NS(2100),
	FPSx100_TO_NS(2200),
	FPSx100_TO_NS(2300),
	FPSx100_TO_NS(2400),
	FPSx100_TO_NS(2500),
	FPSx100_TO_NS(2600),
	FPSx100_TO_NS(2700),
	FPSx100_TO_NS(2800),
	FPSx100_TO_NS(2900),
	FPSx100_TO_NS(2997),
	FPSx100_TO_NS(3000),
	FPSx100_TO_NS(3100),
	FPSx100_TO_NS(3200),
	FPSx100_TO_NS(3300),
	FPSx100_TO_NS(3400),
	FPSx100_TO_NS(3500),
	FPSx100_TO_NS(3600),
	FPSx100_TO_NS(3700),
	FPSx100_TO_NS(3800),
	FPSx100_TO_NS(3900),
	FPSx100_TO_NS(4000),
	FPSx100_TO_NS(4100),
	FPSx100_TO_NS(4200),
	FPSx100_TO_NS(4300),
	FPSx100_TO_NS(4400),
	FPSx100_TO_NS(4500),
	FPSx100_TO_NS(4600),
	FPSx100_TO_NS(4700),
	FPSx100_TO_NS(4800),
	FPSx100_TO_NS(4900),
	FPSx100_TO_NS(5000),
	FPSx100_TO_NS(5100),
	FPSx100_TO_NS(5200),
	FPSx100_TO_NS(5300),
	FPSx100_TO_NS(5400),
	FPSx100_TO_NS(5500),
	FPSx100_TO_NS(5600),
	FPSx100_TO_NS(5700),
	FPSx100_TO_NS(5800),
	FPSx100_TO_NS(5900),
	FPSx100_TO_NS(5994),
	FPSx100_TO_NS(6000)
};

// Some global operators!
inline LONGLONG operator-( const LARGE_INTEGER &first, const LARGE_INTEGER &second )
{
   return first.QuadPart - second.QuadPart;
}

inline bool operator>( const LARGE_INTEGER &first, const LARGE_INTEGER &second )
{
   return bool( first.QuadPart > second.QuadPart );
}

inline bool operator>=( const LARGE_INTEGER &first, const LARGE_INTEGER &second )
{
   return bool( first.QuadPart >= second.QuadPart );
}


/*
** DriverEntry()
**
**   This routine is called when the driver is first loaded by PnP.
**   It in turn, calls upon the stream class to perform registration services.
**
** Arguments:
**
**   DriverObject - 
**          Driver object for this driver 
**
**   RegistryPath - 
**          Registry path string for this driver's key
**
** Returns:
**
**   Results of StreamClassRegisterAdapter()
**
** Side Effects:  none
*/

extern "C" 
	ULONG STREAMAPI DriverEntry ( 
	IN PDRIVER_OBJECT DriverObject, 
	IN PUNICODE_STRING RegistryPath  
	)
{

	// Init Runtime first (TRACE can't be used 'till that) !
	InitCppRT();

	TRACE_EP(("DriverEntry"));

	HW_INITIALIZATION_DATA     HwInitData;
	ULONG                      ReturnValue;

	// Zero init the HW_INITIALIZATION_DATA
	RtlZeroMemory( &HwInitData, sizeof( HwInitData ));
	HwInitData.HwInitializationDataSize = sizeof( HwInitData );

	// Set the Adapter entry points for the driver

	HwInitData.HwInterrupt					= (PHW_INTERRUPT)& BtDigitizer::cbInterrupt;
	HwInitData.HwReceivePacket				= BtDigitizer::cbReceiveCtrlPacket;
	HwInitData.HwCancelPacket				= BtDigitizer::cbCancelPacket; 
	HwInitData.HwRequestTimeoutHandler		= BtDigitizer::cbTimeoutPacket;// Default is OK!
	HwInitData.DeviceExtensionSize			= sizeof(BtDigitizer::DATA_EXTENSION);
	HwInitData.PerRequestExtensionSize		= 0; 
	HwInitData.FilterInstanceExtensionSize	= 0;
	HwInitData.PerStreamExtensionSize		= 0;
	HwInitData.BusMasterDMA					= TRUE;
	HwInitData.Dma24BitAddresses			= FALSE;
	HwInitData.BufferAlignment				= FILE_LONG_ALIGNMENT;
	HwInitData.DmaBufferSize				= RISCProgramsSize;

    // Don't rely on the stream class using raised IRQL to synchronize
    // execution.  This single paramater most affects the overall structure
    // of the driver.

	HwInitData.TurnOffSynchronization		= TRUE;

	ReturnValue = StreamClassRegisterAdapter(DriverObject, RegistryPath, &HwInitData);

	DUMP(("StreamClassRegisterAdapter returned %x\n", ReturnValue));

	return ReturnValue;
}

extern "C"
VOID STREAMAPI Unload( 
    IN PDRIVER_OBJECT DriverObject 
    )
{
	TRACE_EP(("Unload"));

	// Terminate usage of Runtime
	TermCppRT();
}
 
// I know what i am doing!
#pragma warning( disable: 4355 )

// Constructor: 
BtDigitizer::BtDigitizer(const PDEVICE_OBJECT pPDO_) :

	// Init Base Class
	Adapter (pPDO_) ,

	// No card by default
	Card(NULL),

	// Initialization of Streams
	streamPreview(this),	// Note: this is valid here because this is NOT a base class
	streamCapture(this,&streamPreview),
	streamVBI(this),
	streamAnalog(this),

	// Use Even video field for preview
	bUseOddFieldForPreview(false),

	// Not inited by now!
	InitedOK(false),	

	// Init instance to 0...
	Instance((ULONG)-1),

	m_TunerVideoSubChannel(0),
	m_TunerAudioSubChannel(0),
	m_TunerChannel(0),
	m_TunerCountry(0)
{

	// Build the Mediums descriptor of the Digitizer
	memcpy(BtDigitizer::Mediums,BtDigitizer::Mediums_Template,sizeof(BtDigitizer::Mediums_Template));
}

// The initialization fn!
bool BtDigitizer::Init(const PVOID HwDeviceExtension,const DWORD_PTR dwBase)
{
	// Init all the parts that compose a Bt board... From here, Hw ints will be passed to BtCard class (shared ints first)
	Card = new BtCard(pPDO, HwDeviceExtension, dwBase);
	if (!Card) {
		return false;
	}

	// Check if inited ok!
	if (!Card->Init()) {
		return false;
	}

	// Use the HwExtension pointer as Instance data...
	Instance = (ULONG)(ULONG_PTR)HwDeviceExtension;

	// Init the Streams Array!
	SupportedStreams[0] = &streamCapture;
	SupportedStreams[1] = &streamPreview;
	SupportedStreams[2] = &streamVBI;
	SupportedStreams[3] = &streamAnalog;

	// -Set the Medium instance of the Digitizer
	BtDigitizer::Mediums[3].Id = Instance; // This is the only one that has a NON NULL GUID

	// -Set the Medium instance of the Tuner
	BtTuner::Mediums[0].Id = Instance; 
	BtTuner::Mediums[1].Id = Instance;

	// -Set the Medium instance of the TVAudio
	BtTVAudio::Mediums[0].Id = Instance;
	BtTVAudio::Mediums[1].Id = Instance;

	// We will create the Crossbar pins descriptions based on 
	// the capabilities of the card
	DWORD Pin = 0;

	// First, place the tuner(s) info if we have them.
	if (GetBtCard().HasTunerInput()) {
		BtXBar::InputPinInfo[0].Direction		= KSPIN_DATAFLOW_IN;
		BtXBar::InputPinInfo[0].Conn			= VideoConTuner;
		BtXBar::InputPinInfo[0].PinType			= KS_PhysConn_Video_Tuner;
		BtXBar::InputPinInfo[0].RelatedPinIndex	= 0;/*the audio pin*/
		BtXBar::InputPinInfo[0].Medium			= BtTuner::Mediums[0];
		BtXBar::Mediums[0]						= BtTuner::Mediums[0];
		BtXBar::PinDirection[0]					= FALSE;
		Pin++;
	}

	// Second, the SVideo inputs if we have them.
	DWORD VideoInPin = Pin;
	DWORD Idx = GUID_INDEX;
	DWORD PinToDo;
	for (PinToDo = 0; Idx < GUID_COUNT &&  PinToDo < (DWORD)GetBtCard().SVideoInputCount(); PinToDo++) {
		BtXBar::InputPinInfo[Pin].Direction			= KSPIN_DATAFLOW_IN;
		BtXBar::InputPinInfo[Pin].Conn				= VideoConSVideo0 + PinToDo;
		BtXBar::InputPinInfo[Pin].PinType			= KS_PhysConn_Video_SVideo;
		BtXBar::InputPinInfo[Pin].RelatedPinIndex	= 0;/*the audio pin*/
		BtXBar::InputPinInfo[Pin].Medium.Set		= STATIC_GUID[Idx];
		BtXBar::InputPinInfo[Pin].Medium.Id			= Instance;
		BtXBar::InputPinInfo[Pin].Medium.Flags		= 0;
		BtXBar::Mediums[Pin] = BtXBar::InputPinInfo[Pin].Medium;
		BtXBar::PinDirection[Pin]					= FALSE;
		Idx++;
		Pin++;
	}

	// 3rd, the Composite inputs if we have them.
	for (PinToDo = 0; Idx < GUID_COUNT &&  PinToDo < (DWORD)GetBtCard().CompositeInputCount(); PinToDo++) {
		BtXBar::InputPinInfo[Pin].Direction			= KSPIN_DATAFLOW_IN;
		BtXBar::InputPinInfo[Pin].Conn				= VideoConComposite0 + PinToDo;
		BtXBar::InputPinInfo[Pin].PinType			= KS_PhysConn_Video_Composite;
		BtXBar::InputPinInfo[Pin].RelatedPinIndex	= 0;/*the audio pin*/
		BtXBar::InputPinInfo[Pin].Medium.Set		= STATIC_GUID[Idx];
		BtXBar::InputPinInfo[Pin].Medium.Id			= Instance;
		BtXBar::InputPinInfo[Pin].Medium.Flags		= 0;
		BtXBar::Mediums[Pin]= BtXBar::InputPinInfo[Pin].Medium;
		BtXBar::PinDirection[Pin]					= FALSE;
		Idx++;
		Pin++;
	}

	// Third, the serial digital inputs if we have one or more of them...
	for (PinToDo = 0; Idx < GUID_COUNT &&  PinToDo < (DWORD)GetBtCard().SerialDigitalInputCount(); PinToDo++) {
		BtXBar::InputPinInfo[Pin].Direction			= KSPIN_DATAFLOW_IN;
		BtXBar::InputPinInfo[Pin].Conn				= VideoConSDI0 + PinToDo;
		BtXBar::InputPinInfo[Pin].PinType			= KS_PhysConn_Video_SerialDigital;
		BtXBar::InputPinInfo[Pin].RelatedPinIndex	= 0;/*the audio pin*/
		BtXBar::InputPinInfo[Pin].Medium.Set		= STATIC_GUID[Idx];
		BtXBar::InputPinInfo[Pin].Medium.Id			= Instance;
		BtXBar::InputPinInfo[Pin].Medium.Flags		= 0;
		BtXBar::Mediums[Pin] = BtXBar::InputPinInfo[Pin].Medium;
		BtXBar::PinDirection[Pin]					= FALSE;
		Idx++;
		Pin++;
	}

	// 4th, the parallel digital inputs if we have one or more of them...
	for (PinToDo = 0; Idx < GUID_COUNT && PinToDo < (DWORD)GetBtCard().ParallelDigitalInputCount(); PinToDo++) {
		BtXBar::InputPinInfo[Pin].Direction			= KSPIN_DATAFLOW_IN;
		BtXBar::InputPinInfo[Pin].Conn				= VideoConPDI0 + PinToDo;
		BtXBar::InputPinInfo[Pin].PinType			= KS_PhysConn_Video_ParallelDigital;
		BtXBar::InputPinInfo[Pin].RelatedPinIndex	= 0;/*the audio pin*/
		BtXBar::InputPinInfo[Pin].Medium.Set		= STATIC_GUID[Idx];
		BtXBar::InputPinInfo[Pin].Medium.Id			= Instance;
		BtXBar::InputPinInfo[Pin].Medium.Flags		= 0;
		BtXBar::Mediums[Pin] = BtXBar::InputPinInfo[Pin].Medium;
		BtXBar::PinDirection[Pin]					= FALSE;
		Idx++;
		Pin++;
	}

	// 5th, the RGB inputs if we have one or more of them...
	for (PinToDo = 0; Idx < GUID_COUNT &&  PinToDo < (DWORD)GetBtCard().RGBInputCount(); PinToDo++) {
		BtXBar::InputPinInfo[Pin].Direction			= KSPIN_DATAFLOW_IN;
		BtXBar::InputPinInfo[Pin].Conn				= VideoConRGB0 + PinToDo;
		BtXBar::InputPinInfo[Pin].PinType			= KS_PhysConn_Video_RGB;
		BtXBar::InputPinInfo[Pin].RelatedPinIndex	= 0;/*the audio pin*/
		BtXBar::InputPinInfo[Pin].Medium.Set		= STATIC_GUID[Idx];
		BtXBar::InputPinInfo[Pin].Medium.Id			= Instance;
		BtXBar::InputPinInfo[Pin].Medium.Flags		= 0;
		BtXBar::Mediums[Pin] = BtXBar::InputPinInfo[Pin].Medium;
		BtXBar::PinDirection[Pin]					= FALSE;
		Idx++;
		Pin++;
	}

	// 6th, the Component inputs if we have one or more of them...
	for (PinToDo = 0; Idx < GUID_COUNT && PinToDo < (DWORD)GetBtCard().ComponentVideoInputCount(); PinToDo++) {
		BtXBar::InputPinInfo[Pin].Direction			= KSPIN_DATAFLOW_IN;
		BtXBar::InputPinInfo[Pin].Conn				= VideoConComponent0 + PinToDo;
		BtXBar::InputPinInfo[Pin].PinType			= KS_PhysConn_Video_YRYBY;
		BtXBar::InputPinInfo[Pin].RelatedPinIndex	= 0;/*the audio pin*/
		BtXBar::InputPinInfo[Pin].Medium.Set		= STATIC_GUID[Idx];
		BtXBar::InputPinInfo[Pin].Medium.Id			= Instance;
		BtXBar::InputPinInfo[Pin].Medium.Flags		= 0;
		BtXBar::Mediums[Pin].Set					= STATIC_GUID[Idx];
		BtXBar::Mediums[Pin].Flags					= 0;
		BtXBar::Mediums[Pin].Id						= Instance;
		BtXBar::PinDirection[Pin]					= FALSE;
		Idx++;
		Pin++;
	}

	// After all the Video pins, there follows all the audio pins, if available.
	DWORD AudioPinTuner = 0xFF;
	DWORD AudioPinLine  = 0xFF;

	// Remember the first audio pin input
	ulFirstAudioInPinInput = Pin;

	// If we have a tuner, then we have an audio port for it!
	if (GetBtCard().HasTunerInput() && GetBtCard().AudioInputs() != 0) {
		AudioPinTuner = Pin;
		BtXBar::InputPinInfo[Pin].Direction			= KSPIN_DATAFLOW_IN;
		BtXBar::InputPinInfo[Pin].Conn				= AudioConTuner;
		BtXBar::InputPinInfo[Pin].PinType			= KS_PhysConn_Audio_Tuner;
		BtXBar::InputPinInfo[Pin].RelatedPinIndex	= 0;/*the video pin is 0 for the tuner!*/
		BtXBar::InputPinInfo[Pin].Medium			= BtTVAudio::Mediums[1];
		BtXBar::Mediums[Pin]						= BtTVAudio::Mediums[1];
		BtXBar::PinDirection[Pin]					= FALSE;
		Pin++;
	}

	// If we have any kind of input port, AND we have audio inputs, then we assume we have
	// an Audio Line in.
	BOOL VideoInCount = GetBtCard().SVideoInputCount()+
						GetBtCard().CompositeInputCount()+
						GetBtCard().SerialDigitalInputCount()+
						GetBtCard().ParallelDigitalInputCount()+
						GetBtCard().RGBInputCount()+
						GetBtCard().ComponentVideoInputCount();
	if (VideoInCount > GUID_COUNT) {
		VideoInCount = GUID_COUNT;
	}

	if (VideoInCount > 0 && Idx < GUID_COUNT && GetBtCard().AudioInputs() != 0) {
		AudioPinLine = Pin;
		BtXBar::InputPinInfo[Pin].Direction			= KSPIN_DATAFLOW_IN;
		BtXBar::InputPinInfo[Pin].Conn				= AudioConLineIn;
		BtXBar::InputPinInfo[Pin].PinType			= KS_PhysConn_Audio_Line;
		BtXBar::InputPinInfo[Pin].RelatedPinIndex	= VideoInPin; // Asociate with first Video LineIn
		BtXBar::InputPinInfo[Pin].Medium.Set		= STATIC_GUID[Idx];
		BtXBar::InputPinInfo[Pin].Medium.Id			= Instance;
		BtXBar::InputPinInfo[Pin].Medium.Flags		= 0;
		BtXBar::Mediums[Pin]= BtXBar::InputPinInfo[Pin].Medium;
		BtXBar::PinDirection[Pin]					= FALSE;
		Pin++;
		Idx++;
	}

	// Well, all inputs are done, we must fixup the related pin index field of
	// the video pins.
	if (GetBtCard().HasTunerInput()) {
		// Tuner 
		BtXBar::InputPinInfo[0].RelatedPinIndex = (AudioPinTuner!= 0xFF) ? AudioPinTuner : 0;
	}

	for (PinToDo = 0; PinToDo < (DWORD)VideoInCount; PinToDo++) {
		BtXBar::InputPinInfo[PinToDo+VideoInPin].RelatedPinIndex = (AudioPinLine != 0xFF) ? AudioPinLine : 0;
	}

	// Remember the number of input pins
	ulInputPinEntries = Pin;

	// We have all the input pins properly setup, now create the output pins
	DWORD OutPin = 0;
	ulOutputPinVideo = 0xFF; // Assume no video output pin
	if (GetBtCard().HasTunerInput() || VideoInCount) {
		BtXBar::OutputPinInfo[OutPin].Direction			= KSPIN_DATAFLOW_OUT;
		BtXBar::OutputPinInfo[OutPin].Conn				= VideoConnector(0);
		BtXBar::OutputPinInfo[OutPin].PinType			= KS_PhysConn_Video_VideoDecoder;
		BtXBar::OutputPinInfo[OutPin].RelatedPinIndex	= (GetBtCard().AudioInputs() != 0) ? 1 : 0;
		BtXBar::OutputPinInfo[OutPin].Medium			= BtDigitizer::Mediums[3];
		BtXBar::Mediums[Pin]							= BtDigitizer::Mediums[3]; // - VideoDecoderOut
		BtXBar::PinDirection[Pin]						= TRUE;
		// Remember the index of the output video pin
		ulOutputPinVideo = OutPin;
		Pin++; OutPin++;
	}

	// Assume no audio out pin
	ulOutputPinAudio = 0xFF;
	if (GetBtCard().AudioInputs() != 0) {
		BtXBar::OutputPinInfo[OutPin].Direction			= KSPIN_DATAFLOW_OUT;
		BtXBar::OutputPinInfo[OutPin].Conn				= AudioConnector(0);
		BtXBar::OutputPinInfo[OutPin].PinType			= KS_PhysConn_Audio_AudioDecoder;
		BtXBar::OutputPinInfo[OutPin].RelatedPinIndex	= 0; // associated to the KS_PhysConn_Video_VideoDecoder pin
		BtXBar::OutputPinInfo[OutPin].Medium.Set		= GUID_NULL;
		BtXBar::OutputPinInfo[OutPin].Medium.Id			= 0; // As it is a NULL GUID
		BtXBar::OutputPinInfo[OutPin].Medium.Flags		= 0;
		BtXBar::Mediums[Pin].Set						= GUID_NULL; // KS_PhysConn_Audio_AudioDecoder
		BtXBar::Mediums[Pin].Flags						= 0;
		BtXBar::Mediums[Pin].Id							= 0; // As it is a NULL GUID
		BtXBar::PinDirection[Pin]						= TRUE;
		// Remember the index of the output audio pin.
		ulOutputPinAudio = OutPin;
		Pin++; OutPin++;
	}

	// Init routing (by default)
	ulOutputPinIsRoutedTo[ 0 ] = 0; // The Video output
	ulOutputPinIsRoutedTo[ 1 ] =  // The audio output
		((AudioPinTuner != 0xFF) ? AudioPinTuner : 
		((AudioPinLine  != 0xFF) ? AudioPinLine : 0));

	
	// Remember the number of output entries
	ulOutputPinEntries = OutPin;

	// And remember the number of entries
	ulPinEntries = Pin;

	// Get default values and set them!
	ReadRegistryValues();

	// All Done!!!
	InitedOK = true;
	return true;
}

void BtDigitizer::Uninit()
{
	TRACE(("BtDigitizer::Uninit"));

	if (InitedOK) {

		// Now close all the videoStreams
		streamCapture.Close(NULL);
		streamPreview.Close(NULL);
		streamAnalog.Close(NULL);

	}

	// Now kill the Card
	if (Card) {
		// Uninit Card...
		Card->Uninit();

		// Make sure no more ints will be routed there...
		AcquireISR();
		BtCard* c = Card; Card = NULL;
		ReleaseISR();

		// And finally destroy the adapter
		delete c;
	}

	// Don't forget to call the base class uninit
	Adapter::Uninit();
}

BtDigitizer::~BtDigitizer()
{
	TRACE(("BtDigitizer::~BtDigitizer"));
}

/////////////////////////////////////////////////////////////////////////////
// Method:  BYTE BtDigitizer::ReadRegistryValues( void )
// Purpose: Read registry to get all parameters
// Input:   None
// Output:  None
// Return:  
/////////////////////////////////////////////////////////////////////////////
void BtDigitizer::ReadRegistryValues(void)
{
	LONG lVal,lVal2;
	Registry reg(pPDO);

	// Read the default field to use for preview
	reg.ReadKey("UseOddFieldForPreview", bUseOddFieldForPreview, 0);

	// Crossbar
	reg.ReadKey("LastVideoSource",ulOutputPinIsRoutedTo[ 0 ],1);
	reg.ReadKey("LastAudioSource",ulOutputPinIsRoutedTo[ 1 ],3);

	// Video ProcAmp
	reg.ReadKey("AutoGamma",lVal,1L);
	GetBtCard().SetAutoGamma(BOOL(lVal));

	reg.ReadKey("Gamma",lVal,10L);
	GetBtCard().SetGamma( lVal );

	reg.ReadKey("ColorEnable",lVal,LONG(true));
	GetBtCard().SetColorEnable( lVal );

	reg.ReadKey("BacklightCompensation", lVal,0L);
	GetBtCard().SetBacklightCompensation( lVal );

	reg.ReadKey("WhiteBalance", lVal ,0L);
	GetBtCard().SetWhiteBalance( lVal );

	reg.ReadKey("AutoSharpness", lVal , LONG(true) );
	GetBtCard().SetAutoSharpness( lVal );

	reg.ReadKey("Sharpness", lVal , 10000L );
	GetBtCard().SetSharpness( lVal );

	reg.ReadKey("Brightness", lVal ,750L);
	GetBtCard().SetBrightness( lVal );

	reg.ReadKey("Contrast", lVal ,100L);
	GetBtCard().SetContrast( lVal );

	reg.ReadKey("Hue", lVal ,0L);
	GetBtCard().SetHue( lVal );

	reg.ReadKey("AutoSaturation", lVal , LONG(true) );
	GetBtCard().SetAutoSaturation( lVal );

	reg.ReadKey("Saturation", lVal ,100L);
	GetBtCard().SetSaturation( lVal );

	// Tuner
	// Read Video Standard
	reg.ReadKey( "TunerVideoStd", lVal, 0L);
	GetBtCard().TunerSetCurrentStandard(KS_AnalogVideoStandard(lVal));

	// Read CurrentMode
	reg.ReadKey( "TunerMode", lVal, LONG(KSPROPERTY_TUNER_MODE_TV) );
	GetBtCard().TunerSetMode(lVal);

	// Read CurrentFrecuency
	reg.ReadKey( "CurrFreq", lVal, 0L );
	GetBtCard().TunerSetFrequency( lVal ,KS_TUNER_TUNING_FINE );

	GetBtCard().VideoSetInput( VideoConnector(InputPinInfo[ ulOutputPinIsRoutedTo[ 0 ] ].Conn) );

	// Video Decoder
	reg.ReadKey("VideoStandard",lVal,0L);
	GetBtCard().VideoSetCurrentStandard( KS_AnalogVideoStandard(lVal) );

	// Set the audio input
	GetBtCard().AudioSetInput( AudioConnector(InputPinInfo[ ulOutputPinIsRoutedTo[ 1 ] ].Conn) );

	// Set the Audio mode
	reg.ReadKey("AudioMode",lVal,0L);
	GetBtCard().AudioSetMode(lVal);

	// Read the audio Settings and restore them!
	reg.ReadKey("VolumeLeft",lVal, 32768L);
	reg.ReadKey("VolumeRight",lVal2, 32768L);
	lVal  -= 32768L;
	lVal2 -= 32768L;
	GetBtCard().AudioSetVolume(lVal,lVal2);
	
	reg.ReadKey("Loudness",lVal, 32768L);
	lVal  -= 32768L;
	GetBtCard().AudioSetLoudness(lVal);

	reg.ReadKey("Bass",lVal, 32768L);
	reg.ReadKey("Treble",lVal2, 32768L);
	lVal  -= 32768L;
	lVal2 -= 32768L;
	GetBtCard().AudioSetBassTreble(lVal,lVal2);

	LONG Bands[MAX_EQUALIZER_BANDS];
	for (int i=0;i<MAX_EQUALIZER_BANDS;i++) {
		char buffer[16];
		strcpy(buffer,"EqualizerBand-?");
		buffer[14] = char('A' + i);
		reg.ReadKey(buffer,Bands[i],32768L );
		Bands[i] -= 32768L;
	}
	GetBtCard().AudioSetEqualizer(Bands);

	DOLBY_PARAMS dp;
	reg.ReadKey("DolbyDelay",lVal, 0L);
	dp.delay = BYTE(lVal);
	reg.ReadKey("DolbyMode",lVal, LONG(DPL_Through) );
	dp.mode = BYTE(lVal);
	reg.ReadKey("DolbyNoise",lVal, 0L );
	dp.noise = BYTE(lVal);
	reg.ReadKey("DolbyPanorama",lVal, 0L );
	dp.pan = BYTE(lVal);
	reg.ReadKey("DolbyPrologicMode",lVal, LONG(DPLMPl_Normal ));
	dp.plmode = BYTE(lVal);
	reg.ReadKey("DolbyReverb",lVal, 0L);
	dp.rev = BYTE(lVal);
	reg.ReadKey("DolbySpatial",lVal, 0L);
	dp.spatial = BYTE(lVal);
	GetBtCard().AudioSetDolby(dp);
	
}


/////////////////////////////////////////////////////////////////////////////
// Method:  BYTE BtDigitizer::WriteRegistryValues( void )
// Purpose: Write all parameters to registry
// Input:   None
// Output:  None
// Return:  
/////////////////////////////////////////////////////////////////////////////
void BtDigitizer::WriteRegistryValues(void)
{
	Registry reg(pPDO);

	// Video Decoder
	reg.WriteKey("VideoStandard",GetBtCard().VideoGetCurrentStandard());

	// Video ProcAmp
	reg.WriteKey("AutoGamma",GetBtCard().GetAutoGamma());
	reg.WriteKey("Gamma",GetBtCard().GetGamma());
	reg.WriteKey("ColorEnable",GetBtCard().GetColorEnable());
	reg.WriteKey("BacklightCompensation", GetBtCard().GetBacklightCompensation());
	reg.WriteKey("WhiteBalance", GetBtCard().GetWhiteBalance());
	reg.WriteKey("AutoSharpness", GetBtCard().GetAutoSharpness());
	reg.WriteKey("Sharpness", GetBtCard().GetSharpness());
	reg.WriteKey("Brightness", GetBtCard().GetBrightness());
	reg.WriteKey("Contrast", GetBtCard().GetContrast());
	reg.WriteKey("Hue", GetBtCard().GetHue());
	reg.WriteKey("AutoSaturation", GetBtCard().GetAutoSaturation());
	reg.WriteKey("Saturation", GetBtCard().GetSaturation());

	// Crossbar
	reg.WriteKey("LastVideoSource",ulOutputPinIsRoutedTo[ 0 ]);
	reg.WriteKey("LastAudioSource",ulOutputPinIsRoutedTo[ 1 ]);

	// Write the default field to use for preview
	reg.WriteKey("UseOddFieldForPreview", bUseOddFieldForPreview );

	// Tuner

	// Write Video Standard
	reg.WriteKey("TunerVideoStd", GetBtCard().TunerGetCurrentStandard() );
	// Write CurrentFrecuency
	reg.WriteKey("CurrFreq", GetBtCard().TunerGetFrequency() );
	// Write CurrentMode
	reg.WriteKey("TunerMode", GetBtCard().TunerGetMode() );

	// Write the Audio Mode and Settings!
	reg.WriteKey("AudioMode", GetBtCard().AudioGetCurrentMode() );
	reg.WriteKey("VolumeLeft",GetBtCard().AudioGetLeftVolume() + 32768L );
	reg.WriteKey("VolumeRight",GetBtCard().AudioGetRightVolume() + 32768L );
	reg.WriteKey("Loudness",GetBtCard().AudioGetLoudness() + 32768L );
	reg.WriteKey("Bass",GetBtCard().AudioGetBass() + 32768L );
	reg.WriteKey("Treble",GetBtCard().AudioGetTreble() + 32768L );

	LONG Bands[MAX_EQUALIZER_BANDS];
	GetBtCard().AudioGetEqualizer(Bands);
	for (int i=0;i<MAX_EQUALIZER_BANDS;i++) {
		char buffer[16];
		strcpy(buffer,"EqualizerBand-?");
		buffer[14] = char('A' + i);
		reg.WriteKey(buffer,Bands[i] + 32768L );
	}

	DOLBY_PARAMS dp;
	GetBtCard().AudioGetDolby(dp);
	reg.WriteKey("DolbyDelay",LONG(dp.delay));
	reg.WriteKey("DolbyMode",LONG(dp.mode));
	reg.WriteKey("DolbyNoise",LONG(dp.noise));
	reg.WriteKey("DolbyPanorama",LONG(dp.pan));
	reg.WriteKey("DolbyPrologicMode",LONG(dp.plmode));
	reg.WriteKey("DolbyReverb",LONG(dp.rev));
	reg.WriteKey("DolbySpatial",LONG(dp.spatial));
}
	
// Called by analog Channel to Send notifications of Channel Change

/* Note: This is NOT synchronized to the processing of the other streams
 *	and can be called concurrently  */
void BtDigitizer::SendChangeChannelNotification(PHW_STREAM_REQUEST_BLOCK pSrb)
{
	TRACE(("BtDigitizer::SendChangeChannelNotification"));

	// Notify all our output channels!
	streamCapture.ChangeNotification(pSrb);
	streamPreview.ChangeNotification(pSrb);
	streamVBI.ChangeNotification(pSrb);

}

// Callbacks
void STREAMAPI BtDigitizer::cbReceiveCtrlPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE_EP(( "BtDigitizer::cbReceiveCtrlPacket" ));

	PDATA_EXTENSION pDataExt = (PDATA_EXTENSION)pSrb->HwDeviceExtension;

	if( pDataExt == NULL ) {

		ERROR(( "DataExtension = NULL\n" ));
		pSrb->Status = STATUS_INVALID_PARAMETER;
		CompleteCtrlPacket( pSrb );
		return;
	}

	if( pSrb->Command == SRB_INITIALIZE_DEVICE ) { // Special case: Adapter does not exist yet!

		DUMP(( "( SRB_INITIALIZE_DEVICE )\n" ));
		pSrb->Status = Initialize( pSrb );
		CompleteCtrlPacket( pSrb );
		return;
	}

	if (pDataExt->adapter == NULL) {
		ERROR(("adapter == NULL\n"));
		pSrb->Status = STATUS_SUCCESS; // All requests are ackd without performing them!
		CompleteCtrlPacket( pSrb );
		return;
	}

#if DBG
	if (pDataExt->adapter->Signature != ADAPTER_SIGNATURE) {
		ERROR(( "Invalid signature for adapter!" ));
		pSrb->Status = STATUS_INVALID_PARAMETER;
		CompleteCtrlPacket( pSrb );
	} else
#endif

	{

		// Get pointer to Device and call our function (assumes InitOK was called!)
		pDataExt->adapter->ReceiveCtrlPacket(pSrb);

		// If we must exit... Dispatch the last message by hand!
		if (pDataExt->adapter->MustQuit()) {
			PHW_STREAM_REQUEST_BLOCK S = pDataExt->adapter->MustQuit();
			S->Status = UnInitialize( S );
			CompleteCtrlPacket( S );
		}
	}
}


/*
 * BtDigitizer::Interrupt()
 *
 *   Routine is called when an interrupt at the IRQ level specified by the
 *   ConfigInfo structure passed to the HwInitialize routine is received.
 *
 *   Note: IRQs may be shared, so the device should ensure the IRQ received
 *          was expected 
 *
 * Arguments:
 *
 *  HwDeviceExtension - the device extension for the hardware interrupt ... 
 *
 * Returns:
 *
 * Side Effects:  none
 */


BOOLEAN BtDigitizer::HwInterrupt( void )
{
	TRACE(("BtDigitizer::HwInterrupt"));

	// If card is not inited yet, do not pass ints to it (specially shared ints)
	if (Card == NULL) {
		return FALSE;
	}

	// Pass the interrupt to BtCard !
	return GetBtCard().HwInterrupt();
}


/*
 * BtDigitizer::Initialize()
 *
 *   This routine is called when an SRB_INITIALIZE_DEVICE request is received
 *
 * Arguments:
 *
 *   pSrb - pointer to stream request block for the Initialize command
 *
 * Returns:
 *
 * Side Effects:  none
 */

NTSTATUS BtDigitizer::Initialize( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(( "BtDigitizer::Initialize" ));

	// initialize ourselves
	PPORT_CONFIGURATION_INFORMATION ConfigInfo = pSrb->CommandData.ConfigInfo;
	PDATA_EXTENSION pDataExt = (PDATA_EXTENSION)ConfigInfo->HwDeviceExtension;

	// Assume no adapter by now
	pDataExt->adapter = 0;
	
	// Check IO Ranges
	if ( ConfigInfo->NumberOfAccessRanges != 1 ) {
		ERROR (("illegal config info\n" ));
		return STATUS_NO_SUCH_DEVICE;
	}


	// Return number of streams supported
	ConfigInfo->StreamDescriptorSize = 	sizeof( HW_STREAM_HEADER ) + DRIVER_STREAM_COUNT * sizeof( HW_STREAM_INFORMATION );

	// Create an instance of the adapter
	BtDigitizer * adapter = new BtDigitizer( ConfigInfo->PhysicalDeviceObject );
	// Make sure initialization is successfull
	if ( !adapter ) {
		ERROR(("Error initializing Adapter\n"));
		return STATUS_INSUFFICIENT_RESOURCES;
	}
	
	// get ready for Rock & Roll! (allow calling of our adapter)
	pDataExt->adapter = adapter; // Be careful: If using Shared ints (very likely), Adapter::HwInterrupt() could be called, but we are not ready to handle them...

	// Initialize adapter
	if (!adapter->Init( pDataExt , 
						(DWORD_PTR)ConfigInfo->AccessRanges[0].RangeStart.QuadPart  // Base address of the adapter
						) ) {

		// Free all resources
		pDataExt->adapter = NULL;
		delete adapter;

		ERROR(("Error initializing Adapter\n"));
		return STATUS_INSUFFICIENT_RESOURCES;
	}

	return STATUS_SUCCESS;
	
}

/*  BtDigitizer::CompleteInitialization()
 *
 *   This routine is called when an SRB_COMPLETE_INITIALIZATION request is received
 *
 * Arguments:
 *
 *   pSrb - pointer to stream request block
 *
 * Returns:
 *
 * Side Effects:  none
 */

NTSTATUS BtDigitizer::CompleteInitialization( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::CompleteInitialization"));

	NTSTATUS Status;
   
	// Create the Registry blobs that DShow uses to create
	// graphs via Mediums

	// Register the TVAudio decoder

    Status = StreamClassRegisterFilterWithNoKSPins (
                    pPDO,								// IN PDEVICE_OBJECT   DeviceObject,
                    &KSCATEGORY_TVAUDIO,				// IN GUID           * InterfaceClassGUID,
                    SIZEOF_ARRAY (BtTVAudio::Mediums),  // IN ULONG            PinCount,
                    BtTVAudio::PinDirection,            // IN ULONG          * Flags,
					BtTVAudio::Mediums,                 // IN KSPIN_MEDIUM   * MediumList,
                    NULL								// IN GUID           * CategoryList
            );

	// Register the Crossbar
	Status = StreamClassRegisterFilterWithNoKSPins(
					pPDO,		
					&KSCATEGORY_CROSSBAR,			
					ulPinEntries,				
					BtXBar::PinDirection,		
					BtXBar::Mediums,		
					NULL
			);						

	// Register the tuner
	Status = StreamClassRegisterFilterWithNoKSPins(
					pPDO,		
					&KSCATEGORY_TVTUNER,
					SIZEOF_ARRAY(BtTuner::Mediums),				
					BtTuner::PinDirection,		
					BtTuner::Mediums,
					NULL
			);				
	
    // Register the Capture filter
    // Note:  This should be done automatically be MSKsSrv.sys,
    // when that component comes on line (if ever) ...

    Status = StreamClassRegisterFilterWithNoKSPins (
                    pPDO,								// IN PDEVICE_OBJECT   DeviceObject,
                    &KSCATEGORY_CAPTURE,				// IN GUID           * InterfaceClassGUID,
                    SIZEOF_ARRAY (BtDigitizer::Mediums),// IN ULONG            PinCount,
                    BtDigitizer::PinDirection,          // IN ULONG          * Flags,
                    BtDigitizer::Mediums,               // IN KSPIN_MEDIUM   * MediumList,
                    BtDigitizer::Categories             // IN GUID           * CategoryList
            );

	return Status;
}

/*
** BtDigitizer::GetStreamInfo()
**
**   Returns the information of all streams that are supported by the
**   mini-driver
**
** Arguments:
**
**   pSrb - Pointer to the STREAM_REQUEST_BLOCK
**        pSrb->HwDeviceExtension - will be the hardware device extension for
**                                  as initialised in HwInitialise
**
** Returns:
**
** Side Effects:  none
*/

NTSTATUS BtDigitizer::GetStreamInfo( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::GetStreamInfo"));

	// pick up the pointer to header which preceeds the stream info structs
	PHW_STREAM_HEADER pstrhdr = (PHW_STREAM_HEADER)&(pSrb->CommandData.StreamBuffer->StreamHeader);

	// pick up the pointer to the stream information data structure
	PHW_STREAM_INFORMATION pstrinfo = (PHW_STREAM_INFORMATION)&(pSrb->CommandData.StreamBuffer->StreamInfo);
	
	// verify that the buffer is large enough to hold our return data
	if (pSrb->NumberOfBytesToTransfer == 0) {

		pSrb->ActualBytesTransferred = 
			sizeof( HW_STREAM_HEADER ) + sizeof( HW_STREAM_INFORMATION ) * DRIVER_STREAM_COUNT ;
		return STATUS_BUFFER_OVERFLOW;

	} else if (pSrb->NumberOfBytesToTransfer >= 
		sizeof( HW_STREAM_HEADER ) + sizeof( HW_STREAM_INFORMATION ) * DRIVER_STREAM_COUNT ) {

		ENSURE( pSrb->NumberOfBytesToTransfer >=
			sizeof( HW_STREAM_HEADER ) +
			sizeof( HW_STREAM_INFORMATION ) * DRIVER_STREAM_COUNT );

		// Set the header
		*pstrhdr = StreamHeader;

		// stuff the contents of each HW_STREAM_INFORMATION struct
		*pstrinfo++ = streamCapture.Info();
		*pstrinfo++ = streamPreview.Info();
		*pstrinfo++ = streamVBI.Info();
		*pstrinfo++ = streamAnalog.Info();

		return STATUS_SUCCESS;
	} 

	return STATUS_BUFFER_TOO_SMALL;
}


// -------------------------------------------------------------------
// A few notes about property set handling
//  
// Property sets used in here are of two varieties, those that have
// default values, ranges, and stepping, such as VideoProcAmp and CameraControl,
// and those which don't have defaults and ranges, such as TVTuner and
// Crossbar.
// 
// Default values and stepping are established by tables in capprop.h,
// no code is required to implement this other than initally creating the tables.
// 
// Many of the property sets require the ability to modify a number
// of input parameters.  Since KS doesn't allow this inherently, you'll
// note that some property sets require copying the provided input parameters
// to the ouput parameter list, effectively creating a "read, modify, write"
// capability.  For this reason, the input and output parameter lists
// use identical structures.
//
// On an SRB_GET_DEVICE_PROPERTY, read-only input data to the driver is provided as:
//      pSrb->CommandData.PropertyInfo
//
// ... while the output data pointer is:
//      pSrb->CommandData.PropertyInfo.PropertyInfo
// 
// -------------------------------------------------------------------




/* Method: BtDigitizer::GetProperty()
 * Purpose: Selects which adapter property to get
 */
NTSTATUS BtDigitizer::GetProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::GetProperty"));
	PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;

	if ( PROPSETID_VIDCAP_VIDEOPROCAMP == pSPD->Property->Set )  {
		return GetVideoProcAmpProperty( pSrb );
	} else if ( PROPSETID_VIDCAP_VIDEODECODER == pSPD->Property->Set ) {
		return GetVideoDecProperty( pSrb );
	} else if ( PROPSETID_VIDCAP_TVAUDIO == pSPD->Property->Set )  {
		return GetTVAudioProperty( pSrb );
	} else if ( PROPSETID_VIDCAP_CROSSBAR == pSPD->Property->Set )  {
		return GetXBarProperty( pSrb );
	} else if ( PROPSETID_TUNER == pSPD->Property->Set )  {
		return GetTunerProperty( pSrb );
	} else if ( PROPSETID_CUSTOMBT848 == pSPD->Property->Set ) {
		return GetBt848CustomProperty( pSrb );
	} else if ( PROPSETID_VIDCAP_VIDEOCONTROL == pSPD->Property->Set ) {
		return GetVideoControlProperty (pSrb);
	} else if ( PROPSETID_AUDIO == pSPD->Property->Set ) {
		return GetAudioProperty (pSrb);
	} else if ( PROPSETID_IR == pSPD->Property->Set ) {
		return GetIRProperty (pSrb);
	} else if ( PROPSETID_VIDEOWINDOW == pSPD->Property->Set ) {
		return GetVideoWindowProperty (pSrb);
	}

	ERROR(("unrecognized GUID: pSrb(%x), pSPD->Property->Set(%x)\n", pSrb, pSPD->Property->Set));
	return STATUS_NOT_IMPLEMENTED;
}

/* Method: BtDigitizer::SetProperty
 * Purpose: Selects which adapter property to set
 */
NTSTATUS BtDigitizer::SetProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::SetProperty"));
	PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;

	if ( PROPSETID_VIDCAP_VIDEOPROCAMP == pSPD->Property->Set )  {
		NTSTATUS stat = SetVideoProcAmpProperty( pSrb );
		if (stat == STATUS_SUCCESS) {
			WriteRegistryValues();
		}
		return stat;
	} else if ( PROPSETID_VIDCAP_VIDEODECODER == pSPD->Property->Set )  {
		NTSTATUS stat = SetVideoDecProperty( pSrb );
		if (stat == STATUS_SUCCESS) {
			WriteRegistryValues();
		}
		return stat;
	} else if ( PROPSETID_VIDCAP_TVAUDIO == pSPD->Property->Set )  {
		NTSTATUS stat = SetTVAudioProperty( pSrb );
		if (stat == STATUS_SUCCESS) {
			WriteRegistryValues();
		}
		return stat;
	} else if ( PROPSETID_VIDCAP_CROSSBAR == pSPD->Property->Set )  {
		NTSTATUS stat = SetXBarProperty( pSrb );
		if (stat == STATUS_SUCCESS) {
			WriteRegistryValues();
		}
		return stat;
	} else if ( PROPSETID_TUNER == pSPD->Property->Set )  {
		NTSTATUS stat = SetTunerProperty( pSrb );
		if (stat == STATUS_SUCCESS) {
			WriteRegistryValues();
		}
		return stat;
	} else if ( PROPSETID_VIDCAP_VIDEOCONTROL == pSPD->Property->Set ) {
		NTSTATUS stat = SetVideoControlProperty ( pSrb );
		if (stat == STATUS_SUCCESS) {
			WriteRegistryValues();
		}
		return stat;
	} else if ( PROPSETID_AUDIO == pSPD->Property->Set ) {
		NTSTATUS stat = SetAudioProperty( pSrb );
		if (stat == STATUS_SUCCESS) {
			WriteRegistryValues();
		}
		return stat;
	} else if ( PROPSETID_VIDEOWINDOW == pSPD->Property->Set ) {
		NTSTATUS stat = SetVideoWindowProperty( pSrb );
		if (stat == STATUS_SUCCESS) {
			WriteRegistryValues();
		}
		return stat;
	}

	ERROR(("unrecognized GUID: pSrb(%x), pSPD->Property->Set(%x)\n", pSrb, pSPD->Property->Set));
	return STATUS_NOT_IMPLEMENTED;
}

// -------------------------------------------------------------------
// Tuner functions
// -------------------------------------------------------------------

NTSTATUS BtDigitizer::GetTunerProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(( "BtDigitizer::GetTunerProperty" ));
	PSTREAM_PROPERTY_DESCRIPTOR	pSPD = pSrb->CommandData.PropertyInfo;
	pSrb->ActualBytesTransferred = 0;

	switch( pSPD->Property->Id ) 
	{
		case KSPROPERTY_TUNER_CAPS:
		{
			// Work with the output buffer ...
			PKSPROPERTY_TUNER_CAPS_S pCaps = 
				(PKSPROPERTY_TUNER_CAPS_S)pSPD->PropertyInfo;

			pCaps->ModesSupported = GetBtCard().TunerGetSupportedModes();
			pCaps->VideoMedium    = BtTuner::Mediums[ 0 ];
			pCaps->TVAudioMedium  = BtTuner::Mediums[ 1 ];
			pCaps->RadioAudioMedium.Set   = GUID_NULL; 
			pCaps->RadioAudioMedium.Id    = 0; // (because it is a NULL guid)
			pCaps->RadioAudioMedium.Flags = 0;

			pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_TUNER_CAPS_S);
			return STATUS_SUCCESS;
		}

		case KSPROPERTY_TUNER_FREQUENCY:
		{
			PKSPROPERTY_TUNER_FREQUENCY_S pFreq = 
				(PKSPROPERTY_TUNER_FREQUENCY_S) pSPD->PropertyInfo;

			pFreq->LastFrequency = pFreq->Frequency = GetBtCard().TunerGetFrequency();
			pFreq->TuningFlags = GetBtCard().TunerGetMethod();
			pFreq->VideoSubChannel = m_TunerVideoSubChannel;
			pFreq->AudioSubChannel = m_TunerAudioSubChannel;
			pFreq->Channel = m_TunerChannel;
			pFreq->Country = m_TunerCountry;

			pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_TUNER_FREQUENCY_S);
			return STATUS_SUCCESS;
		}

		case KSPROPERTY_TUNER_MODE:
		{
			// Work with the output buffer ...
			PKSPROPERTY_TUNER_MODE_S pMode = 
				(PKSPROPERTY_TUNER_MODE_S)pSPD->PropertyInfo;

			pMode->Mode = GetBtCard().TunerGetMode();

			pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_TUNER_MODE_S);
			return STATUS_SUCCESS;
		}
		case KSPROPERTY_TUNER_MODE_CAPS:
		{
			// Work with the output buffer ...
			PKSPROPERTY_TUNER_MODE_CAPS_S pCaps = 
				(PKSPROPERTY_TUNER_MODE_CAPS_S)pSPD->PropertyInfo;

			// Copy the requested mode to the output buffer (needed by TuneGetModeCaps() )
			pCaps->Mode = ((PKSPROPERTY_TUNER_MODE_CAPS_S) pSPD->Property)->Mode;

			// Get tuner Caps
			NTSTATUS ret = GetBtCard().TunerGetModeCaps(pCaps);

			if (ret != STATUS_SUCCESS) 
				return ret;

			pCaps->NumberOfInputs = 1;
			pCaps->Strategy = KS_TUNER_STRATEGY_DRIVER_TUNES;

			pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_TUNER_MODE_CAPS_S);
			return STATUS_SUCCESS;

		}
		case KSPROPERTY_TUNER_STATUS:
		{
			// Work with the output buffer ...
			PKSPROPERTY_TUNER_STATUS_S pStat = 
				(PKSPROPERTY_TUNER_STATUS_S)pSPD->PropertyInfo;

			// Return the status of the tuner

			// PLLOffset is in units of TuningGranularity 
			// SignalStrength is 0 to 100
			// Set Busy to 1 if tuning is still in process

			pStat->Busy = FALSE;
			pStat->CurrentFrequency = GetBtCard().TunerGetFrequency();
			pStat->PLLOffset = GetBtCard().TunerWasSignalDetected() ? 0 : 2;													// It shouldnt be needed, but some buggy programs try to use it (especially kstvtune.ax) , so fake it! (Driver tunes, not user!)
            pStat->SignalStrength = GetBtCard().TunerWasSignalDetected() ? 100 : 0;	// This shouldnt be used, but some buggy programs try to use it (especially kstvtune.ax) , so fake it so programs will know if signal is present! (Driver tunes, not user!)

			pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_TUNER_STATUS_S);
			return STATUS_SUCCESS;

		}
		case KSPROPERTY_TUNER_STANDARD:
		{
			// Work with the output buffer ...
			PKSPROPERTY_TUNER_STANDARD_S pStd = 
				(PKSPROPERTY_TUNER_STANDARD_S)pSPD->PropertyInfo;

			pStd->Standard = GetBtCard().TunerGetCurrentStandard();

			pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_TUNER_STANDARD_S);
			return STATUS_SUCCESS;

		}
		case KSPROPERTY_TUNER_INPUT:
		{
			// Work with the output buffer ...
			PKSPROPERTY_TUNER_INPUT_S pIn = 
				(PKSPROPERTY_TUNER_INPUT_S)pSPD->PropertyInfo;

			pIn->InputIndex = 0; // Use input 0 always (just one input)

			pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_TUNER_INPUT_S);
			return STATUS_SUCCESS;

		}
#if 0
		case KSPROPERTY_TUNER_IF_MEDIUM:
		{
			// Only Digital TV tuners should support this property

			// Work with the output buffer ...
			PKSPROPERTY_TUNER_IF_MEDIUM_S pMedium =
				(PKSPROPERTY_TUNER_IF_MEDIUM_S)pSPD->PropertyInfo;

			pMedium->IFMedium.Set = GUID_NULL;
			pMedium->IFMedium.Id  = 0; // (because it is a NULL GUID)
			pMedium->IFMedium.Flags = 0;
			
			pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_TUNER_IF_MEDIUM_S);
			return STATUS_SUCCESS;
		}
#endif
	}
	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS BtDigitizer::SetTunerProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(( "BtDigitizer::SetTunerProperty" ));
	PSTREAM_PROPERTY_DESCRIPTOR	pSPD = pSrb->CommandData.PropertyInfo;

	switch( pSPD->Property->Id ) 
	{
		case KSPROPERTY_TUNER_FREQUENCY:
		{
			PKSPROPERTY_TUNER_FREQUENCY_S pFreq = 
				(PKSPROPERTY_TUNER_FREQUENCY_S) pSPD->Property;

			NTSTATUS st = GetBtCard().TunerSetFrequency( 
				pFreq->Frequency , KS_TUNER_TUNING_FLAGS(pFreq->TuningFlags) );

			if (st == STATUS_SUCCESS) {
				m_TunerVideoSubChannel = pFreq->VideoSubChannel;
				m_TunerAudioSubChannel = pFreq->AudioSubChannel;
				m_TunerChannel = pFreq->Channel;
				m_TunerCountry = pFreq->Country;
			}
			return st;
		}
		case KSPROPERTY_TUNER_MODE:
		{

			PKSPROPERTY_TUNER_MODE_S pMode = 
				(PKSPROPERTY_TUNER_MODE_S) pSPD->Property;
				
			return GetBtCard().TunerSetMode( pMode->Mode );

		}
        case KSPROPERTY_TUNER_STANDARD:
        {

			PKSPROPERTY_TUNER_STANDARD_S pStandard = 
				(PKSPROPERTY_TUNER_STANDARD_S) pSPD->Property;

			return GetBtCard().TunerSetCurrentStandard( KS_AnalogVideoStandard(pStandard->Standard) );
			
        }
		case KSPROPERTY_TUNER_INPUT:
		{
			PKSPROPERTY_TUNER_INPUT_S pInput = 
				(PKSPROPERTY_TUNER_INPUT_S) pSPD->Property;
			//TunerInput = pInput_S->InputIndex;
			return STATUS_SUCCESS;
		}
	}
	return STATUS_NOT_IMPLEMENTED;
}

// -------------------------------------------------------------------
// CrossBar functions
// -------------------------------------------------------------------

NTSTATUS BtDigitizer::GetXBarProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(( "BtDigitizer::GetXBarProperty" ));
	PSTREAM_PROPERTY_DESCRIPTOR	pSPD = pSrb->CommandData.PropertyInfo;

	pSrb->ActualBytesTransferred = 0;
	
	switch( pSPD->Property->Id ) {

		case KSPROPERTY_CROSSBAR_CAPS:
			{
				PKSPROPERTY_CROSSBAR_CAPS_S pCaps = 
					(PKSPROPERTY_CROSSBAR_CAPS_S)pSPD->PropertyInfo;

				if (pSPD->PropertyOutputSize < sizeof (KSPROPERTY_CROSSBAR_CAPS_S))
					break;
					
				// Copy the input property info to the output property info
				::RtlCopyMemory( pCaps, pSPD->Property, sizeof(KSPROPERTY_CROSSBAR_CAPS_S) );

				pCaps->NumberOfInputs = ulInputPinEntries;
				pCaps->NumberOfOutputs = ulOutputPinEntries;

				pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_CROSSBAR_CAPS_S);
				return STATUS_SUCCESS;

			}
			break;
		case KSPROPERTY_CROSSBAR_PININFO:
			{
				PKSPROPERTY_CROSSBAR_PININFO_S  pPinInfo = 
					(PKSPROPERTY_CROSSBAR_PININFO_S)pSPD->PropertyInfo;

				if (pSPD->PropertyOutputSize < sizeof (KSPROPERTY_CROSSBAR_PININFO_S))
					break;

				// Copy the input property info to the output property info
				::RtlCopyMemory( pPinInfo, pSPD->Property, sizeof(KSPROPERTY_CROSSBAR_PININFO_S) );

				switch ( pPinInfo->Direction ) {
					case KSPIN_DATAFLOW_IN:

						if( pPinInfo->Index < ulInputPinEntries )	{
							pPinInfo->PinType			= InputPinInfo[ pPinInfo->Index ].PinType;
							pPinInfo->RelatedPinIndex	= InputPinInfo[ pPinInfo->Index ].RelatedPinIndex;
							pPinInfo->Medium			= InputPinInfo[ pPinInfo->Index ].Medium;

						} else	{
							ERROR(( "KSPROPERTY_CROSSBAR_PININFO: invalid index" ));
							return STATUS_INVALID_PARAMETER;
						}
						break;

					case KSPIN_DATAFLOW_OUT:
						if( pPinInfo->Index < ulOutputPinEntries )	{
							pPinInfo->PinType			= OutputPinInfo[ pPinInfo->Index ].PinType;
							pPinInfo->RelatedPinIndex	= OutputPinInfo[ pPinInfo->Index ].RelatedPinIndex;
							pPinInfo->Medium			= OutputPinInfo[ pPinInfo->Index ].Medium;

						} else {
							ERROR(( "KSPROPERTY_CROSSBAR_PININFO: invalid index" ));
							return STATUS_INVALID_PARAMETER;
						}
						break;
					default:
						ERROR(( "KSPROPERTY_CROSSBAR_PININFO: invalid direction" ));
						return STATUS_INVALID_PARAMETER;
				}

				pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_CROSSBAR_PININFO_S);
				return STATUS_SUCCESS;

			}
			break;
		
		case KSPROPERTY_CROSSBAR_CAN_ROUTE:
			{
				PKSPROPERTY_CROSSBAR_ROUTE_S  pRoute = 
					(PKSPROPERTY_CROSSBAR_ROUTE_S)pSPD->PropertyInfo;

				if (pSPD->PropertyOutputSize < sizeof (KSPROPERTY_CROSSBAR_ROUTE_S))
					break;

				// Copy the input property info to the output property info
				::RtlCopyMemory( pRoute, pSPD->Property, sizeof(KSPROPERTY_CROSSBAR_ROUTE_S) );

				pRoute->CanRoute = CanRoute( pRoute );
				pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_CROSSBAR_ROUTE_S);
				return STATUS_SUCCESS;

			}
			break;
		
		case KSPROPERTY_CROSSBAR_ROUTE:
			{
				PKSPROPERTY_CROSSBAR_ROUTE_S  pRoute = 
					(PKSPROPERTY_CROSSBAR_ROUTE_S)pSPD->PropertyInfo;

				if( pSPD->PropertyOutputSize < sizeof(KSPROPERTY_CROSSBAR_ROUTE_S) ) 
					break;

				// Copy the input property info to the output property info
				::RtlCopyMemory( pRoute, pSPD->Property, sizeof(KSPROPERTY_CROSSBAR_ROUTE_S) );

				if( pRoute->IndexOutputPin < ulOutputPinEntries ) {

					pRoute->IndexInputPin = ulOutputPinIsRoutedTo[ pRoute->IndexOutputPin ];
					pRoute->CanRoute = TRUE;

				} else {

					//pRoute->IndexInputPin = (DWORD)-1;
					pRoute->CanRoute = FALSE;

				}

				pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_CROSSBAR_ROUTE_S);
				return STATUS_SUCCESS;
			}
			break;

		default:

			ERROR(("bogus request" ));
			break;
	}
	return STATUS_INVALID_PARAMETER;
}


NTSTATUS BtDigitizer::SetXBarProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(( "BtDigitizer::SetXBarProperty" ));
	PSTREAM_PROPERTY_DESCRIPTOR	pSPD = pSrb->CommandData.PropertyInfo;

	switch( pSPD->Property->Id ) 
	{
		case KSPROPERTY_CROSSBAR_ROUTE:
			{
				PKSPROPERTY_CROSSBAR_ROUTE_S  pRoute = 
					(PKSPROPERTY_CROSSBAR_ROUTE_S)pSPD->Property;

				// Find out if we can route or not
				if (CanRoute( pRoute )) {

					if( pRoute->IndexOutputPin == ulOutputPinVideo ) { // Routing to Video Out

						// Video out
						GetBtCard().VideoSetInput( VideoConnector(InputPinInfo[ pRoute->IndexInputPin ].Conn) );

						// Save route
						ulOutputPinIsRoutedTo[ pRoute->IndexOutputPin ] = pRoute->IndexInputPin;

					} else if ( pRoute->IndexOutputPin == ulOutputPinAudio)	{ // Routing to Audio input

						if (pRoute->IndexInputPin == (~0) )  { // Mute

							// Special case!  Audio Routing of (-1) means mute!!!
							GetBtCard().AudioMute();
							
						} else {

							GetBtCard().AudioSetInput( AudioConnector(InputPinInfo[ pRoute->IndexInputPin ].Conn) );
							GetBtCard().AudioUnmute();

						}

						// Save route, except when muting
						if (pRoute->IndexInputPin != (~0) ) {
							ulOutputPinIsRoutedTo[ pRoute->IndexOutputPin ] = pRoute->IndexInputPin;
						}
					}
					return STATUS_SUCCESS;

				} else {

					return STATUS_INVALID_PARAMETER;
				}
			}
			break;
		
		default:

			ERROR(("bogus request" ));
			break;
	}

	return STATUS_INVALID_PARAMETER;
}

BOOL BtDigitizer::CanRoute( PKSPROPERTY_CROSSBAR_ROUTE_S pRoute )
{
	TRACE(( "BtDigitizer::CanRoute" ));
	DUMP(( "In=%d , Out =%d\n", pRoute->IndexInputPin, pRoute->IndexOutputPin ));

	if (pRoute->IndexOutputPin == ulOutputPinVideo) { // Video 
		if ( pRoute->IndexInputPin < ulFirstAudioInPinInput ) {
			DUMP(( "YES\n" ));	
			return TRUE;
		}
	} else if ( pRoute->IndexOutputPin == ulOutputPinAudio ) { // audio
		if ( pRoute->IndexInputPin == (~0) || // Muting ...
			 (pRoute->IndexInputPin >= ulFirstAudioInPinInput && 
			  pRoute->IndexInputPin < ulInputPinEntries )
			 ) {
			DUMP(( "YES\n" ));
			return TRUE;
		}
	}
	DUMP(( "NO\n" ));
	return FALSE;
}


// -------------------------------------------------------------------
// VideoProcAmp functions
// -------------------------------------------------------------------

NTSTATUS BtDigitizer::SetVideoProcAmpProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::SetVideoProcAmpProperty"));

	PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;
	PKSPROPERTY_VIDEOPROCAMP_S pS = (PKSPROPERTY_VIDEOPROCAMP_S) pSPD->Property;

	switch ( pSPD->Property->Id ) {
		case KSPROPERTY_VIDEOPROCAMP_GAMMA:
			{
				NTSTATUS st;
				if ((st = GetBtCard().SetAutoGamma( (pS->Flags == KSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO) )) != STATUS_SUCCESS) {
					return st;
				}
				st = GetBtCard().SetGamma( pS->Value );
				return st;
			}
		case KSPROPERTY_VIDEOPROCAMP_COLORENABLE:
			{
				NTSTATUS st = GetBtCard().SetColorEnable( pS->Value );
				return st;
			}
		case KSPROPERTY_VIDEOPROCAMP_BACKLIGHT_COMPENSATION:
			{
				NTSTATUS st = GetBtCard().SetBacklightCompensation( pS->Value );
				return st;
			}
		case KSPROPERTY_VIDEOPROCAMP_WHITEBALANCE:
			{
				NTSTATUS st = GetBtCard().SetWhiteBalance( pS->Value );
				return st;
			}
		case KSPROPERTY_VIDEOPROCAMP_SHARPNESS:
			{
				NTSTATUS st;
				if ((st = GetBtCard().SetAutoSharpness( (pS->Flags == KSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO) )) != STATUS_SUCCESS) {
					return st;
				}
				st = GetBtCard().SetSharpness( pS->Value );
				return st;
			}
		case KSPROPERTY_VIDEOPROCAMP_BRIGHTNESS:
			{
				NTSTATUS st = GetBtCard().SetBrightness( pS->Value );
				return st;
			}
		case KSPROPERTY_VIDEOPROCAMP_CONTRAST:
			{
				NTSTATUS st = GetBtCard().SetContrast( pS->Value );
				return st;
			}
		case KSPROPERTY_VIDEOPROCAMP_HUE:
			{
				NTSTATUS st = GetBtCard().SetHue( pS->Value );
				return st;
			}
		case KSPROPERTY_VIDEOPROCAMP_SATURATION:
			{
				NTSTATUS st;
				if ((st = GetBtCard().SetAutoSaturation( (pS->Flags == KSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO) )) != STATUS_SUCCESS) {
					return st;
				}
				st = GetBtCard().SetSaturation( pS->Value );
				return st;
			}
	}
	return STATUS_INVALID_PARAMETER;
}

/* Method: BtDigitizer::GetVideoProcAmpProperty()
 * Purpose: Gets various video procamp properties
 */
NTSTATUS BtDigitizer::GetVideoProcAmpProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::GetVideoProcAmpProperty"));

	PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;
	PKSPROPERTY_VIDEOPROCAMP_S pS = (PKSPROPERTY_VIDEOPROCAMP_S) pSPD->PropertyInfo;    // pointer to the output buffer
	
	ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_VIDEOPROCAMP_S) );
	pSrb->ActualBytesTransferred = 0;

	switch ( pSPD->Property->Id ) {
		case KSPROPERTY_VIDEOPROCAMP_GAMMA:
			pS->Value = GetBtCard().GetGamma();
			pS->Flags = GetBtCard().GetAutoGamma() ? KSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO : KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;
			pS->Capabilities = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL | KSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;

			pSrb->ActualBytesTransferred = sizeof( KSPROPERTY_VIDEOPROCAMP_S );
			return STATUS_SUCCESS;

		case KSPROPERTY_VIDEOPROCAMP_BACKLIGHT_COMPENSATION:
			pS->Value = GetBtCard().GetBacklightCompensation();
			pS->Flags = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;
			pS->Capabilities = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;

			pSrb->ActualBytesTransferred = sizeof( KSPROPERTY_VIDEOPROCAMP_S );
			return STATUS_SUCCESS;

		case KSPROPERTY_VIDEOPROCAMP_COLORENABLE:
			pS->Value = GetBtCard().GetColorEnable();
			pS->Flags = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;
			pS->Capabilities = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;

			pSrb->ActualBytesTransferred = sizeof( KSPROPERTY_VIDEOPROCAMP_S );
			return STATUS_SUCCESS;

		case KSPROPERTY_VIDEOPROCAMP_WHITEBALANCE:
			pS->Value = GetBtCard().GetWhiteBalance();
			pS->Flags = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;
			pS->Capabilities = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;

			pSrb->ActualBytesTransferred = sizeof( KSPROPERTY_VIDEOPROCAMP_S );
			return STATUS_SUCCESS;

		case KSPROPERTY_VIDEOPROCAMP_SHARPNESS:
			pS->Value = GetBtCard().GetSharpness();
			pS->Flags = GetBtCard().GetAutoSharpness() ? KSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO : KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;
			pS->Capabilities = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL | KSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;

			pSrb->ActualBytesTransferred = sizeof( KSPROPERTY_VIDEOPROCAMP_S );
			return STATUS_SUCCESS;

		case KSPROPERTY_VIDEOPROCAMP_BRIGHTNESS:
			pS->Value = GetBtCard().GetBrightness();
			pS->Flags = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;
			pS->Capabilities = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;

			pSrb->ActualBytesTransferred = sizeof( KSPROPERTY_VIDEOPROCAMP_S );
			return STATUS_SUCCESS;

		case KSPROPERTY_VIDEOPROCAMP_CONTRAST:
			pS->Value = GetBtCard().GetContrast();
			pS->Flags = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;
			pS->Capabilities = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;

			pSrb->ActualBytesTransferred = sizeof( KSPROPERTY_VIDEOPROCAMP_S );
			return STATUS_SUCCESS;

		case KSPROPERTY_VIDEOPROCAMP_HUE:
			pS->Value = GetBtCard().GetHue();
			pS->Flags = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;
			pS->Capabilities = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;

			pSrb->ActualBytesTransferred = sizeof( KSPROPERTY_VIDEOPROCAMP_S );
			return STATUS_SUCCESS;

		case KSPROPERTY_VIDEOPROCAMP_SATURATION:
			pS->Value = GetBtCard().GetSaturation();
			pS->Flags = GetBtCard().GetAutoSaturation() ? KSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO : KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;
			pS->Capabilities = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL | KSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;

			pSrb->ActualBytesTransferred = sizeof( KSPROPERTY_VIDEOPROCAMP_S );
			return STATUS_SUCCESS;
	}

	ERROR(("KSPROPERTY_??? (%d) ***\n", pSPD->Property->Id));
	return STATUS_INVALID_PARAMETER;
}

/* Method: BtDigitizer::SetVideoDecProperty
 * Purpose: Manipulates various video decoder properties
 * Input: SRB
 * Output: None
 */
NTSTATUS BtDigitizer::SetVideoDecProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::SetVideoDecProperty"));

	PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;

	switch ( pSPD->Property->Id ) {
		case KSPROPERTY_VIDEODECODER_STANDARD: {
			
			DUMP(("KSPROPERTY_VIDEODECODER_STANDARD\n"));

			PKSPROPERTY_VIDEODECODER_S pVDecStd = 
				(PKSPROPERTY_VIDEODECODER_S)pSPD->Property;

			DUMP(("Format = %x\n", pVDecStd->Value));

			NTSTATUS st = GetBtCard().VideoSetCurrentStandard( KS_AnalogVideoStandard( pVDecStd->Value) );

			// And tell the streams format has changed!
			streamCapture.RescaleCropping();
			streamPreview.RescaleCropping();

			return st;
		}
		case KSPROPERTY_VIDEODECODER_OUTPUT_ENABLE: {
			//pHwDevExt->VideoDecoderOutputEnable = pS->Value;
			return STATUS_SUCCESS;
	    }

		case KSPROPERTY_VIDEODECODER_VCR_TIMING: {
			//pHwDevExt->VideoDecoderVCRTiming = pS->Value;
			return STATUS_SUCCESS;
		}

	}
	return STATUS_INVALID_PARAMETER;
}

/* Method: BtDigitizer::GetVideoDecProperty
 * Purpose: Obtains various video decoder properties
 * Input: SRB
 * Output: None
 */
NTSTATUS BtDigitizer::GetVideoDecProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::GetVideoDecProperty"));
   
	PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;
	pSrb->ActualBytesTransferred = 0;

	switch ( pSPD->Property->Id ) {
		case KSPROPERTY_VIDEODECODER_CAPS: {

			DUMP(("KSPROPERTY_VIDEODECODER_CAPS\n"));

			PKSPROPERTY_VIDEODECODER_CAPS_S pVDecCaps = 
				(PKSPROPERTY_VIDEODECODER_CAPS_S)pSPD->PropertyInfo;

			pVDecCaps->StandardsSupported = GetBtCard().VideoGetSupportedStandards();
			pVDecCaps->Capabilities =	KS_VIDEODECODER_FLAGS_CAN_INDICATE_LOCKED;

			pVDecCaps->SettlingTime = 1;    // How long to delay after tuning
											// before locked indicator is valid ( i do not like 0)
			pVDecCaps->HSyncPerVSync = 6;   // HSync per VSync

			DUMP(("StandardsSupported = %x, Capabilities = %x, SettlingTime = %d, HSyncPerVSync = %d\n"
				,pVDecCaps->StandardsSupported
				,pVDecCaps->Capabilities
				,pVDecCaps->SettlingTime
				,pVDecCaps->HSyncPerVSync ));

			pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_VIDEODECODER_CAPS_S);
			return STATUS_SUCCESS;
		}
	case KSPROPERTY_VIDEODECODER_STANDARD: {

			DUMP(("KSPROPERTY_VIDEODECODER_STANDARD\n"));

			// Return the currently active analog video mode
			PKSPROPERTY_VIDEODECODER_S pVDecStd = 
				(PKSPROPERTY_VIDEODECODER_S)pSPD->PropertyInfo;

			pVDecStd->Value = (ULONG)GetCurrentVideoStandard();
	
			DUMP(("Standard = %x\n"	,pVDecStd->Value ));

			pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_VIDEODECODER_S);
			return STATUS_SUCCESS;
		}
	case KSPROPERTY_VIDEODECODER_STATUS: {

			DUMP(("KSPROPERTY_VIDEODECODER_STATUS\n"));

			PKSPROPERTY_VIDEODECODER_STATUS_S pVDecStat = 
				(PKSPROPERTY_VIDEODECODER_STATUS_S)pSPD->PropertyInfo;
			
			pVDecStat->NumberOfLines = (GetBtCard().Is525LinesVideo()) ? 525 : 625;
			pVDecStat->SignalLocked = GetBtCard().IsHLocked();

			DUMP(("NumberOfLines = %d , SignalLocked = %d\n",pVDecStat->NumberOfLines,pVDecStat->SignalLocked));

			pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_VIDEODECODER_STATUS_S);
			return STATUS_SUCCESS;
		}
	case KSPROPERTY_VIDEODECODER_VCR_TIMING: {

			PKSPROPERTY_VIDEODECODER_S pS = 
				(PKSPROPERTY_VIDEODECODER_S) pSPD->PropertyInfo;    // pointer to the data

			pS->Value = TRUE; //pHwDevExt->VideoDecoderVCRTiming;

		    pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_VIDEODECODER_S);
			return STATUS_SUCCESS;
		}
	}
	ERROR(("KSPROPERTY_??? (%d) ***\n", pSPD->Property->Id));
	return STATUS_INVALID_PARAMETER;
}

// -------------------------------------------------------------------
// TVAudio functions
// -------------------------------------------------------------------

/*
** BtDigitizer::SetTVAudioProperty ()
**
**    Handles Set operations on the TVAudio property set.
**
** Arguments:
**
**      pSRB -
**          Pointer to the HW_STREAM_REQUEST_BLOCK 
**
** Returns:
**
** Side Effects:  none
*/


NTSTATUS BtDigitizer::SetTVAudioProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
    TRACE(("BtDigitizer::SetTVAudioProperty"));

    PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;

    switch (pSPD->Property->Id) {

		case KSPROPERTY_TVAUDIO_MODE: {
	        PKSPROPERTY_TVAUDIO_S pS = 
				(PKSPROPERTY_TVAUDIO_S) pSPD->Property;    

			// Set audio mode.
			return GetBtCard().AudioSetMode( pS->Mode );

		}
	}

    ERROR(("TVAUDIO_PROPERTY(%x) not supported\n", pSPD->Property->Id));
    return STATUS_INVALID_PARAMETER;
}

/*
** BtDigitizer::GetTVAudioProperty ()
**
**    Handles Get operations on the TVAudio property set.
**
** Arguments:
**
**      pSRB -
**          Pointer to the HW_STREAM_REQUEST_BLOCK 
**
** Returns:
**
** Side Effects:  none
*/

NTSTATUS BtDigitizer::GetTVAudioProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::GetTVAudioProperty"));

    PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;
    pSrb->ActualBytesTransferred = 0;

    switch (pSPD->Property->Id) {

    case KSPROPERTY_TVAUDIO_CAPS:
		{
			DUMP(("KSPROPERTY_TVAUDIO_CAPS\n"));

			PKSPROPERTY_TVAUDIO_CAPS_S pS = 
				(PKSPROPERTY_TVAUDIO_CAPS_S) pSPD->PropertyInfo;    // pointer to the data

	        ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_TVAUDIO_CAPS_S));
        
			pS->InputMedium  = BtTVAudio::Mediums[0];
			pS->OutputMedium = BtTVAudio::Mediums[1];

			// Report all of the possible audio decoding modes the hardware is capabable of
			pS->Capabilities = GetBtCard().AudioGetCaps();
			
			pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_TVAUDIO_CAPS_S);
			return STATUS_SUCCESS;
		}

        
    case KSPROPERTY_TVAUDIO_MODE:
	    {
		    DUMP(("KSPROPERTY_TVAUDIO_MODE\n"));

	        PKSPROPERTY_TVAUDIO_S pS = 
				(PKSPROPERTY_TVAUDIO_S) pSPD->PropertyInfo;    // pointer to the data

	        ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_TVAUDIO_S));

			// Report the currently selected mode
			pS->Mode = GetBtCard().AudioGetCurrentMode();

	        pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_TVAUDIO_S);
			return STATUS_SUCCESS;
		}


    case KSPROPERTY_TVAUDIO_CURRENTLY_AVAILABLE_MODES:
	    {
		    DUMP(("KSPROPERTY_TVAUDIO_CURRENTLY_AVAILABLE_MODES\n"));
	        PKSPROPERTY_TVAUDIO_S pS = 
				(PKSPROPERTY_TVAUDIO_S) pSPD->PropertyInfo;    // pointer to the data

			ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_TVAUDIO_S));

			// Report which audio modes could potentially be selected right now
			pS->Mode = GetBtCard().AudioGetSupportedModes();

	        pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_TVAUDIO_S);
			return STATUS_SUCCESS;
		}
    
    }

    ERROR(("default - unrecognized (%x)\n", pSPD->Property->Id));
    return STATUS_INVALID_PARAMETER;
}

// -------------------------------------------------------------------
// Audio functions
// -------------------------------------------------------------------

/*
** BtDigitizer::SetAudioProperty ()
**
**    Handles Set operations on the Audio property set.
**
** Arguments:
**
**      pSRB -
**          Pointer to the HW_STREAM_REQUEST_BLOCK 
**
** Returns:
**
** Side Effects:  none
*/


NTSTATUS BtDigitizer::SetAudioProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
    TRACE(("BtDigitizer::SetAudioProperty"));

    PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;

    switch (pSPD->Property->Id) {

		case KSPROPERTY_AUDIO_VOLUME: {
			PKSPROPERTY_AUDIO_VOLUME_S pS =
				(PKSPROPERTY_AUDIO_VOLUME_S) pSPD->Property;

			// Set Volume
			GetBtCard().AudioSetVolume(pS->Left,pS->Right);
			return STATUS_SUCCESS;
		}	

		case KSPROPERTY_AUDIO_BASSTREBLE: {
			PKSPROPERTY_AUDIO_BASSTREBLE_S pS =
				(PKSPROPERTY_AUDIO_BASSTREBLE_S) pSPD->Property;

			// Set Bass and Treble
			GetBtCard().AudioSetBassTreble(pS->Bass,pS->Treble);
			return STATUS_SUCCESS;
		}	

		case KSPROPERTY_AUDIO_LOUDNESS: {
			PKSPROPERTY_AUDIO_LOUDNESS_S pS =
				(PKSPROPERTY_AUDIO_LOUDNESS_S) pSPD->Property;

			// Set Loudness
			GetBtCard().AudioSetLoudness(pS->Loudness);

			return STATUS_SUCCESS;
		}	

		case KSPROPERTY_AUDIO_EQUALIZER: {
			PKSPROPERTY_AUDIO_EQUALIZER_S pS =
				(PKSPROPERTY_AUDIO_EQUALIZER_S) pSPD->Property;

			// Set Equalizer
			GetBtCard().AudioSetEqualizer(pS->Band);

			return STATUS_SUCCESS;
		}	

		case KSPROPERTY_AUDIO_DOLBY: {
			PKSPROPERTY_AUDIO_DOLBY_S pS =
				(PKSPROPERTY_AUDIO_DOLBY_S) pSPD->Property;

			// Set Dolby
			GetBtCard().AudioSetDolby(pS->params);

			return STATUS_SUCCESS;
		}	

	}

    ERROR(("AUDIO_PROPERTY(%x) not supported\n", pSPD->Property->Id));
    return STATUS_INVALID_PARAMETER;
}

/*
** BtDigitizer::GetAudioProperty ()
**
**    Handles Get operations on the TVAudio property set.
**
** Arguments:
**
**      pSRB -
**          Pointer to the HW_STREAM_REQUEST_BLOCK 
**
** Returns:
**
** Side Effects:  none
*/

NTSTATUS BtDigitizer::GetAudioProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::GetAudioProperty"));

    PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;
    pSrb->ActualBytesTransferred = 0;

    switch (pSPD->Property->Id) {

		case KSPROPERTY_AUDIO_VOLUME: {
			PKSPROPERTY_AUDIO_VOLUME_S pS =
				(PKSPROPERTY_AUDIO_VOLUME_S) pSPD->PropertyInfo;    // pointer to the data

			ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_AUDIO_VOLUME_S));
        
			pS->Left = GetBtCard().AudioGetLeftVolume();
			pS->Right = GetBtCard().AudioGetRightVolume();

			pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_AUDIO_VOLUME_S);
			return STATUS_SUCCESS;
		}

        
		case KSPROPERTY_AUDIO_BASSTREBLE: {
			PKSPROPERTY_AUDIO_BASSTREBLE_S pS =
				(PKSPROPERTY_AUDIO_BASSTREBLE_S) pSPD->PropertyInfo;    // pointer to the data

	        ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_AUDIO_BASSTREBLE_S));

			pS->Bass = GetBtCard().AudioGetBass();
			pS->Treble = GetBtCard().AudioGetTreble();

	        pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_AUDIO_BASSTREBLE_S);
			return STATUS_SUCCESS;
		}


		case KSPROPERTY_AUDIO_LOUDNESS: {
			PKSPROPERTY_AUDIO_LOUDNESS_S pS =
				(PKSPROPERTY_AUDIO_LOUDNESS_S) pSPD->PropertyInfo;    // pointer to the data

			ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_AUDIO_LOUDNESS_S));

			pS->Loudness = GetBtCard().AudioGetLoudness();

	        pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_AUDIO_LOUDNESS_S);
			return STATUS_SUCCESS;
		}

		case KSPROPERTY_AUDIO_EQUALIZER: {
			PKSPROPERTY_AUDIO_EQUALIZER_S pS =
				(PKSPROPERTY_AUDIO_EQUALIZER_S) pSPD->PropertyInfo;    // pointer to the data

			ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_AUDIO_EQUALIZER_S));

			// Get equalizer settings
			GetBtCard().AudioGetEqualizer(pS->Band);

	        pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_AUDIO_EQUALIZER_S);
			return STATUS_SUCCESS;
		}

		case KSPROPERTY_AUDIO_DOLBY: {
			PKSPROPERTY_AUDIO_DOLBY_S pS =
				(PKSPROPERTY_AUDIO_DOLBY_S) pSPD->PropertyInfo;    // pointer to the data

			ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_AUDIO_DOLBY_S));

			// Get Dolby settings
			GetBtCard().AudioGetDolby(pS->params);

	        pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_AUDIO_DOLBY_S);
			return STATUS_SUCCESS;
		}

		case KSPROPERTY_AUDIO_CAPS: {
			PKSPROPERTY_AUDIO_CAPS_S pS =
				(PKSPROPERTY_AUDIO_CAPS_S) pSPD->PropertyInfo;    // pointer to the data

			ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_AUDIO_CAPS_S));

			// Get Audio control caps
			GetBtCard().AudioControlCaps(pS->Caps,pS->EqualizerBands,pS->BandFrequency);

	        pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_AUDIO_CAPS_S);
			return STATUS_SUCCESS;
		}
    }

    ERROR(("default - unrecognized (%x)\n", pSPD->Property->Id));
    return STATUS_INVALID_PARAMETER;
}


/*
** BtDigitizer::GetIRProperty ()
**
**    Handles Get operations on the IR property set.
**
** Arguments:
**
**      pSRB -
**          Pointer to the HW_STREAM_REQUEST_BLOCK 
**
** Returns:
**
** Side Effects:  none
*/

NTSTATUS BtDigitizer::GetIRProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::GetIRProperty"));

    PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;
    pSrb->ActualBytesTransferred = 0;

    switch (pSPD->Property->Id) {

		case KSPROPERTY_IR_GETCODE: {
			PKSPROPERTY_IR_GETCODE_S pS =
				(PKSPROPERTY_IR_GETCODE_S) pSPD->PropertyInfo;    // pointer to the data

			ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_IR_GETCODE_S));
        
			pS->Code = GetBtCard().IRGetCode();

			pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_IR_GETCODE_S);
			return STATUS_SUCCESS;
		}

        
		case KSPROPERTY_IR_CAPS: {
			PKSPROPERTY_IR_CAPS_S pS =
				(PKSPROPERTY_IR_CAPS_S) pSPD->PropertyInfo;    // pointer to the data

	        ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_IR_CAPS_S));

			pS->Caps = 0;
			if (GetBtCard().IRAvailable()) {
				pS->Caps = KSPROPERTY_IR_CAPS_AVAILABLE;
			}

	        pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_IR_CAPS_S);
			return STATUS_SUCCESS;
		}

    }

    ERROR(("default - unrecognized (%x)\n", pSPD->Property->Id));
    return STATUS_INVALID_PARAMETER;
}


// -------------------------------------------------------------------
// VideoControl functions
// -------------------------------------------------------------------

/*
** BtDigitizer::SetVideoControlProperty ()
**
**    Handles Set operations on the VideoControl property set.
**
** Arguments:
**
**      pSRB -
**          Pointer to the HW_STREAM_REQUEST_BLOCK 
**
** Returns:
**
** Side Effects:  none
*/

NTSTATUS BtDigitizer::SetVideoControlProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::SetVideoControlProperty"));

	PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;
    PKSPROPERTY_VIDEOCONTROL_MODE_S pS = (PKSPROPERTY_VIDEOCONTROL_MODE_S) pSPD->Property;
    ULONG StreamIndex;

    // For this property set, the StreamIndex is always in the same place
    // for each property
    StreamIndex = ((PKSPROPERTY_VIDEOCONTROL_CAPS_S) pSPD->Property)->StreamIndex;

    // Verify the stream index is valid
    if ( StreamIndex >= SIZEOF_ARRAY(SupportedStreams) ) {
        return STATUS_INVALID_PARAMETER;
    }

    ENSURE (pSPD->PropertyInputSize >= sizeof (KSPROPERTY_VIDEOCONTROL_MODE_S));
    
    switch (pSPD->Property->Id) {

		case KSPROPERTY_VIDEOCONTROL_MODE:
		{
			return SupportedStreams[StreamIndex]->SetVideoControlMode(pS->Mode);
		}
    }
	return STATUS_INVALID_PARAMETER;
}

/*
** BtDigitizer::GetVideoControlProperty ()
**
**    Handles Get operations on the VideoControl property set.
**
** Arguments:
**
**      pSRB -
**          Pointer to the HW_STREAM_REQUEST_BLOCK 
**
** Returns:
**
** Side Effects:  none
*/

NTSTATUS BtDigitizer::GetVideoControlProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::GetVideoControlProperty"));

	PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;
    ULONG Id = pSPD->Property->Id;              // index of the property
	ULONG nS  = pSPD->PropertyOutputSize;        // size of data supplied
	pSrb->ActualBytesTransferred = 0;

    // For this property set, the StreamIndex is always in the same place
    // for each property
    ULONG StreamIndex = ((PKSPROPERTY_VIDEOCONTROL_CAPS_S) pSPD->Property)->StreamIndex;

    // Verify the stream index is valid
    if ( StreamIndex >= SIZEOF_ARRAY(SupportedStreams) ) {
        return STATUS_INVALID_PARAMETER;
    }

    switch (Id) {

		case KSPROPERTY_VIDEOCONTROL_CAPS:
		{
			PKSPROPERTY_VIDEOCONTROL_CAPS_S pS = 
				(PKSPROPERTY_VIDEOCONTROL_CAPS_S) pSPD->PropertyInfo;    // pointer to the data

			ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_VIDEOCONTROL_CAPS_S));
        
			ULONG Caps = SupportedStreams[StreamIndex]->GetVideoControlCaps(); 

			if (Caps) { // If some caps ...

				pS->VideoControlCaps = Caps;
				pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_VIDEOCONTROL_CAPS_S);
				return STATUS_SUCCESS;

			} else {

				return STATUS_NOT_SUPPORTED;

			}
		}
		case KSPROPERTY_VIDEOCONTROL_ACTUAL_FRAME_RATE:
		{
			PKSPROPERTY_VIDEOCONTROL_ACTUAL_FRAME_RATE_S pS = 
				(PKSPROPERTY_VIDEOCONTROL_ACTUAL_FRAME_RATE_S) pSPD->PropertyInfo;    // pointer to the data

			LONGLONG CurrentActualFrameRate = SupportedStreams[StreamIndex]->GetActualFramerateInNs(); 

			if (!CurrentActualFrameRate) { // If no fps...
				return STATUS_NOT_SUPPORTED;
			}

			if (nS == 0) {

				pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_VIDEOCONTROL_ACTUAL_FRAME_RATE_S);
				return STATUS_BUFFER_OVERFLOW;

			} else 
			if (nS >= sizeof(KSPROPERTY_VIDEOCONTROL_ACTUAL_FRAME_RATE_S)) {

				// If the stream is not open, we must NOT support this property!
				if (!SupportedStreams[StreamIndex]->IsOpen()) {
					return STATUS_NOT_SUPPORTED;
				}

				// Get current and maximum framerate
				LONGLONG CurrentMaxAvailableFrameRate = SupportedStreams[StreamIndex]->GetMaxAvailableFramerateInNs();
								
				// If no framerate, don't support property!
				if (!CurrentMaxAvailableFrameRate) {
					return STATUS_NOT_SUPPORTED;
				}

				// Return all requested info!
				pS->CurrentMaxAvailableFrameRate = CurrentMaxAvailableFrameRate;
				pS->CurrentActualFrameRate = CurrentActualFrameRate;

				pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_VIDEOCONTROL_ACTUAL_FRAME_RATE_S);
				return STATUS_SUCCESS;

			} else {

				return STATUS_NOT_SUPPORTED;

			}
		}

		// This one gives the app the framerates supported. 
		case KSPROPERTY_VIDEOCONTROL_FRAME_RATES:
		{
			// We must determine the amount of data to copy first

			// Get maximum framerate available - This assumes that we will use the VIDEOINFOHEADER struct
			// So, no field timing by now. Check if VIDEOINFO2 used
			LONGLONG CurrentMaxAvailableFrameRate = 1i64;

			// If stream is NOT open, use maximum available by videonorm, else use maximum available with open method
			if (!SupportedStreams[StreamIndex]->IsOpen()) {
				CurrentMaxAvailableFrameRate = GetBtCard().TimeBetweenFields() << 1i64;
			} else {
				CurrentMaxAvailableFrameRate = SupportedStreams[StreamIndex]->GetMaxAvailableFramerateInNs();
			}

			// Now we must count all entries that apply to us.
			LONG Count = 0;
			UINT Idx;
			for (Idx = 0; Idx < (sizeof(FramerateList) / sizeof(FramerateList[0])) ; Idx ++) {
				if (FramerateList[Idx] <= CurrentMaxAvailableFrameRate) {
					Count++;
				}
			}
			
			PKSPROPERTY_VIDEOCONTROL_FRAME_RATES_S pData =
				(PKSPROPERTY_VIDEOCONTROL_FRAME_RATES_S) pSPD->PropertyInfo;
	
			if (nS == 0) {

				pSrb->ActualBytesTransferred = sizeof(KSMULTIPLE_ITEM) + (Count * sizeof(LONGLONG) );
				return STATUS_BUFFER_OVERFLOW;

			} else if (nS >= sizeof(KSMULTIPLE_ITEM) + (Count * sizeof(LONGLONG) ) ) {

				UINT i;
				PKSMULTIPLE_ITEM pOutputBuf = (PKSMULTIPLE_ITEM) pSPD->PropertyInfo;
				LONGLONG * pDataPtr = (LONGLONG*) (pOutputBuf + 1);

				// Copy the input property info to the output property info
				pOutputBuf->Size = sizeof(KSMULTIPLE_ITEM) + (Count * sizeof(LONGLONG));
				pOutputBuf->Count = Count;

				// And copy all the framerates that apply.
				for (i = 0,Idx = 0; Idx < (sizeof(FramerateList) / sizeof(FramerateList[0])) ; Idx ++) {
					if (FramerateList[Idx] <= CurrentMaxAvailableFrameRate) {
						pDataPtr[i++] = FramerateList[Idx];
					}
				}

				ENSURE( i == UINT(Count) );

				pSrb->ActualBytesTransferred = sizeof(KSMULTIPLE_ITEM) + (Count * sizeof(LONGLONG));
			    return STATUS_SUCCESS;

			} 
			return STATUS_BUFFER_TOO_SMALL;

	    }
		

		case KSPROPERTY_VIDEOCONTROL_MODE:
		{
			PKSPROPERTY_VIDEOCONTROL_MODE_S pS = 
				(PKSPROPERTY_VIDEOCONTROL_MODE_S) pSPD->PropertyInfo;    // pointer to the data

			ULONG Caps = SupportedStreams[StreamIndex]->GetVideoControlCaps(); 

			if (!Caps) { // If no caps supported ... Don't support property
				return STATUS_NOT_SUPPORTED;
			}

			if (nS == 0){
				pSrb->ActualBytesTransferred =	sizeof(KSPROPERTY_VIDEOCONTROL_MODE_S);
				return STATUS_BUFFER_OVERFLOW;
			}
			else if (nS >= 	sizeof(KSPROPERTY_VIDEOCONTROL_MODE_S))	{

				pS->Mode = SupportedStreams[StreamIndex]->GetVideoControlMode();
				pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_VIDEOCONTROL_MODE_S);
				return STATUS_SUCCESS;

			}
			else {

				return STATUS_NOT_SUPPORTED;

			}
		}
    
    }
	return STATUS_INVALID_PARAMETER;
}


// Here there are all the fns. They are here (in the Get() method), because all of them return a value!
NTSTATUS BtDigitizer::GetBt848CustomProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::GetBt848CustomProperty"));

    PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;
    pSrb->ActualBytesTransferred = 0;

	switch( pSPD->Property->Id ) 
	{
		case KSPROPERTY_CUSTOMBT848_GPIO:
		{
			DUMP(( "KSPROPERTY_CUSTOMBT848_GPIO\n" ));
			PKSPROPERTY_CUSTOMBT848_GPIO_S	pS = 
				(PKSPROPERTY_CUSTOMBT848_GPIO_S) pSPD->PropertyInfo;	// Pointer to the output buffer

			ENSURE( pSPD->PropertyOutputSize >= sizeof(KSPROPERTY_CUSTOMBT848_GPIO_S) );

			// Copy the input property info to the output property info
			::RtlCopyMemory( pS, pSPD->Property, sizeof(KSPROPERTY_CUSTOMBT848_GPIO_S) );

			switch( pS->dwOperation )
			{
				case BT848_CUSTPROP_GPIO_SETGPOEBIT:	
					pS->dwReturnValue = GetBtCard().SetGPOE( pS->dwBit, (State) pS->dwState );
					break;
				case BT848_CUSTPROP_GPIO_SETGPOE:	
					GetBtCard().SetGPOE( pS->dwValue );
					pS->dwReturnValue = true;
					break;
				case BT848_CUSTPROP_GPIO_SETGPDATA:	
					pS->dwReturnValue = GetBtCard().SetGPDATA( (ULONG*)pS->adwData, pS->dwSize, pS->dwOffset );
					break;
				case BT848_CUSTPROP_GPIO_SETGPDATABITS:
					pS->dwReturnValue = GetBtCard().SetGPDATA( pS->dwFromBit, pS->dwToBit, pS->dwValue, pS->dwOffset );
					break;
				case BT848_CUSTPROP_GPIO_GETGPOEBIT:		
					pS->dwReturnValue = GetBtCard().GetGPOE( (int)pS->dwBit );
					break;
				case BT848_CUSTPROP_GPIO_GETGPOE:		
					pS->dwReturnValue = GetBtCard().GetGPOE();
					break;
				case BT848_CUSTPROP_GPIO_GETGPDATA:		
					pS->dwReturnValue = GetBtCard().GetGPDATA( (ULONG*)pS->adwData, pS->dwSize, pS->dwOffset );
					break;
				case BT848_CUSTPROP_GPIO_GETGPDATABITS:	
					pS->dwReturnValue = GetBtCard().GetGPDATA( pS->dwFromBit, pS->dwToBit, &(pS->dwValue), pS->dwOffset );
					break;
				default:
					//	error
					return STATUS_INVALID_PARAMETER;
			}
			pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_CUSTOMBT848_GPIO_S);
			return STATUS_SUCCESS;
		}
		case KSPROPERTY_CUSTOMBT848_I2C:
		{
			DUMP(( "KSPROPERTY_CUSTOMBT848_I2C\n"));
			PKSPROPERTY_CUSTOMBT848_I2C_S pS = 
				(PKSPROPERTY_CUSTOMBT848_I2C_S)pSPD->PropertyInfo;

			ENSURE( pSPD->PropertyInputSize >= sizeof(KSPROPERTY_CUSTOMBT848_I2C_S) );

			// Copy the input property info to the output property info
			::RtlCopyMemory( pS, pSPD->Property, sizeof(KSPROPERTY_CUSTOMBT848_I2C_S) );
			
			switch( pS->dwOperation )
			{
				case BT848_CUSTPROP_I2C_SETFREQ:		
					pS->dwReturnValue = GetBtCard().I2CSetFreq( pS->dwFreq );
					break;
				case BT848_CUSTPROP_I2C_GETFREQ:		
					pS->dwReturnValue = GetBtCard().I2CGetFreq();
					break;
				case BT848_CUSTPROP_I2C_WR:		
					pS->dwReturnValue = GetBtCard().I2CWR( pS->ucAddress, pS->ucOutBuf, pS->dwOutLen, pS->ucInBuf, pS->dwInLen, pS->bDontWaitACK == TRUE );
					break;
				case BT848_CUSTPROP_I2C_GETLASTERROR:
					pS->dwReturnValue = GetBtCard().I2CGetLastError();
					break;
				default:
					//	error
					return STATUS_INVALID_PARAMETER;
			}
			pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_CUSTOMBT848_I2C_S);
			return STATUS_SUCCESS;
		}
#ifdef __FULLDECODERACCESS__ // NOT RECOMMENDED AT ALL
		case KSPROPERTY_CUSTOMBT848_DECODER:
		{
			DUMP(( "KSPROPERTY_CUSTOMBT848_DECODER\n"));
			PKSPROPERTY_CUSTOMBT848_DECODER_S pS = 
				(PKSPROPERTY_CUSTOMBT848_DECODER_S)pSPD->PropertyInfo;

			ENSURE( pSPD->PropertyInputSize >= sizeof(KSPROPERTY_CUSTOMBT848_DECODER_S) );

			// Copy the input property info to the output property info
			::RtlCopyMemory( pS, pSPD->Property, sizeof(KSPROPERTY_CUSTOMBT848_DECODER_S) );
			
			switch( pS->dwOperation )
			{
				case BT848_CUSTPROP_DECODER_READ:		
					pS->dwValue = GetBtCard().ReadDecoder( pS->dwAddress );
					break;
				case BT848_CUSTPROP_DECODER_WRITE:		
					GetBtCard().WriteDecoder( pS->dwAddress , pS->dwValue );
					break;
				default:
					//	error
					return STATUS_INVALID_PARAMETER;
			}
			pSrb->ActualBytesTransferred = sizeof(KSPROPERTY_CUSTOMBT848_DECODER_S);
			return STATUS_SUCCESS;
		}
#endif
	}
	return STATUS_INVALID_PARAMETER;
}

// -------------------------------------------------------------------
// VideoWindow functions
// -------------------------------------------------------------------

/*
** BtDigitizer::SetVideoWindowProperty ()
**
**    Handles Set operations on the VideoWindow property set.
**
** Arguments:
**
**      pSRB -
**          Pointer to the HW_STREAM_REQUEST_BLOCK 
**
** Returns:
**
** Side Effects:  none
*/


NTSTATUS BtDigitizer::SetVideoWindowProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
    TRACE(("BtDigitizer::SetVideoWindowProperty"));

    PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;

    switch (pSPD->Property->Id) {

		case KSPROPERTY_VIDEOWINDOW_ANALOG: {
			PKSPROPERTY_VIDEOWINDOW_S pS =
				(PKSPROPERTY_VIDEOWINDOW_S) pSPD->Property;

			// Validate parameters...
			if (pS->dwChannel != VIDEOWINDOW_CHANNEL_CAPTURE &&
				pS->dwChannel != VIDEOWINDOW_CHANNEL_PREVIEW ) {
				return STATUS_INVALID_PARAMETER;
			}

			// Set the Analog Video Window
			return SupportedStreams[pS->dwChannel]->SetAnalogWindow(pS->rcRect);
		}	

	}

    ERROR(("VIDEOWINDOW_PROPERTY(%x) not supported\n", pSPD->Property->Id));
    return STATUS_INVALID_PARAMETER;
}

/*
** BtDigitizer::GetVideoWindowProperty ()
**
**    Handles Get operations on the VideoWindow property set.
**
** Arguments:
**
**      pSRB -
**          Pointer to the HW_STREAM_REQUEST_BLOCK 
**
** Returns:
**
** Side Effects:  none
*/

NTSTATUS BtDigitizer::GetVideoWindowProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::GetVideoWindowProperty"));

    PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;
    pSrb->ActualBytesTransferred = 0;

    switch (pSPD->Property->Id) {

		case KSPROPERTY_VIDEOWINDOW_ANALOG: {
			PKSPROPERTY_VIDEOWINDOW_S pS =
				(PKSPROPERTY_VIDEOWINDOW_S) pSPD->PropertyInfo;    // pointer to the data

			ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_VIDEOWINDOW_S));
        
			PKSPROPERTY_VIDEOWINDOW_S pSIn =
				(PKSPROPERTY_VIDEOWINDOW_S) pSPD->Property;

			// Validate parameters...
			if (pSIn->dwChannel != VIDEOWINDOW_CHANNEL_CAPTURE &&
				pSIn->dwChannel != VIDEOWINDOW_CHANNEL_PREVIEW ) {
				return STATUS_INVALID_PARAMETER;
			}

			NTSTATUS ret = SupportedStreams[pSIn->dwChannel]->GetAnalogWindow(pS->rcRect);
			pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_VIDEOWINDOW_S);
			return ret;
		}

		case KSPROPERTY_VIDEOWINDOW_MAXANALOG: {
			PKSPROPERTY_VIDEOWINDOW_S pS =
				(PKSPROPERTY_VIDEOWINDOW_S) pSPD->PropertyInfo;    // pointer to the data

			ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_VIDEOWINDOW_S));
        
			PKSPROPERTY_VIDEOWINDOW_S pSIn =
				(PKSPROPERTY_VIDEOWINDOW_S) pSPD->Property;

			// Validate parameters...
			if ((pSIn->dwChannel != VIDEOWINDOW_CHANNEL_CAPTURE &&
				pSIn->dwChannel != VIDEOWINDOW_CHANNEL_PREVIEW) || Card == NULL ) {
				return STATUS_INVALID_PARAMETER;
			}

			Card->VideoGetMaxFrame(pS->rcRect);
			pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_VIDEOWINDOW_S);
			return STATUS_SUCCESS;
		}

		case KSPROPERTY_VIDEOWINDOW_DIGITAL: {
			PKSPROPERTY_VIDEOWINDOW_S pS =
				(PKSPROPERTY_VIDEOWINDOW_S) pSPD->PropertyInfo;    // pointer to the data

			ENSURE (pSPD->PropertyOutputSize >= sizeof (KSPROPERTY_VIDEOWINDOW_S));
        
			PKSPROPERTY_VIDEOWINDOW_S pSIn =
				(PKSPROPERTY_VIDEOWINDOW_S) pSPD->Property;

			// Validate parameters...
			if (pSIn->dwChannel != VIDEOWINDOW_CHANNEL_CAPTURE &&
				pSIn->dwChannel != VIDEOWINDOW_CHANNEL_PREVIEW ) {
				return STATUS_INVALID_PARAMETER;
			}

			NTSTATUS ret = SupportedStreams[pSIn->dwChannel]->GetDigitalWindow(pS->rcRect);
			pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_VIDEOWINDOW_S);
			return ret;
		}
    }

    ERROR(("default - unrecognized (%x)\n", pSPD->Property->Id));
    return STATUS_INVALID_PARAMETER;
}


NTSTATUS BtDigitizer::PowerUp( void )
{
	TRACE(("BtDigitizer::PowerUp"));
	NTSTATUS ret = GetBtCard().PowerUp();

	// Make sure to reinit Cropping now.
	if (ret == STATUS_SUCCESS) {
		streamCapture.RescaleCropping();
		streamPreview.RescaleCropping();
	}
	return ret;
}

NTSTATUS BtDigitizer::PowerDown( void )
{
	TRACE(("BtDigitizer::PowerDown"));
	return GetBtCard().PowerDown();
}

// Unknown command processing
NTSTATUS BtDigitizer::UnknownCommand( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::UnknownCommand"));
	return STATUS_NOT_SUPPORTED;
}

/*
** BtDigitizer::StreamFormatFromRange()
**
**   Returns a DATAFORMAT from a DATARANGE
**
** Arguments:
**
**         IN PHW_STREAM_REQUEST_BLOCK pSrb
**
** Returns:
**
**   TRUE if the format is supported
**   FALSE if the format cannot be suppored
**
** Side Effects:  none
*/

NTSTATUS BtDigitizer::StreamFormatFromRange( IN PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::StreamFormatFromRange"));

	PSTREAM_DATA_INTERSECT_INFO IntersectInfo;
	IntersectInfo = pSrb->CommandData.IntersectInfo;

	// Get the Stream Number 
	ULONG StreamNumber = IntersectInfo->StreamNumber;
	if ( StreamNumber >= SIZEOF_ARRAY(SupportedStreams) ) {
		ERROR(("Invalid Stream number (%d)\n", StreamNumber));
		return STATUS_INVALID_PARAMETER;
	}

	DUMP(("Stream(%d) - pSrb(%x)\n", StreamNumber, pSrb));

	// Let the Stream handle the request!
	return SupportedStreams[StreamNumber]->FormatFromRange( pSrb );

}


/*
** BtDigitizer::StreamOpen()
**
**   This routine is called when an OpenStream SRB request is received
**
** Arguments:
**
**   pSrb - pointer to stream request block for the Open command
**
** Returns:
**
** Side Effects:  none
*/

NTSTATUS BtDigitizer::StreamOpen( IN PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::StreamOpen"));

	// Get the Stream Number to open
	ULONG StreamNumber = pSrb->StreamObject->StreamNumber;
	if ( StreamNumber >= SIZEOF_ARRAY(SupportedStreams) ) {
		ERROR(("Invalid Stream number (%d)\n", StreamNumber));

		return STATUS_INVALID_PARAMETER;
	}

	DUMP(("Stream(%d) - pSrb(%x)\n", StreamNumber, pSrb));

	// Reinit the Adapter if it is idle (To overcame DScaler modifications)
	GetBtCard().ReinitIfIdle();

	// Let the Stream handle the request!
	return SupportedStreams[StreamNumber]->Open( pSrb );
}

/*
** BtDigitizer::StreamClose()
**
**   This routine is called when an CloseStream SRB request is received
**
** Arguments:
**
**   pSrb - pointer to stream request block for the Open command
**
** Returns:
**
** Side Effects:  none
*/

NTSTATUS BtDigitizer::StreamClose( IN PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::StreamClose"));

	// Get the Stream Number to close
	ULONG StreamNumber = pSrb->StreamObject->StreamNumber;
	if ( StreamNumber >= SIZEOF_ARRAY(SupportedStreams) ) {
		ERROR(("Invalid Stram number (%d)\n", StreamNumber));

		return STATUS_INVALID_PARAMETER;
	}

	DUMP(("Stream(%d) - pSrb(%x)\n", StreamNumber, pSrb));

	// Let the Stream handle the request!
	return SupportedStreams[StreamNumber]->Close( pSrb );
}


NTSTATUS BtDigitizer::StreamGetState( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("BtDigitizer::StreamGetState")); 

	// Get the Stream Number
	ULONG StreamNumber = pSrb->StreamObject->StreamNumber;
	if ( StreamNumber >= SIZEOF_ARRAY(SupportedStreams) ) {
		ERROR(("Invalid Stream number (%d)\n", StreamNumber));

		return STATUS_INVALID_PARAMETER;
	}

	DUMP(("Stream(%d) - pSrb(%x)\n", StreamNumber, pSrb));

	// Let the Stream handle the request!
	return SupportedStreams[StreamNumber]->GetState( pSrb );
}

NTSTATUS BtDigitizer::StreamSetState( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("BtDigitizer::StreamSetState"));
	// Get the Stream Number
	ULONG StreamNumber = pSrb->StreamObject->StreamNumber;
	if ( StreamNumber >= SIZEOF_ARRAY(SupportedStreams) ) {
		ERROR(("Invalid Stream number (%d)\n", StreamNumber));

		return STATUS_INVALID_PARAMETER;
	}

	DUMP(("Stream(%d) - pSrb(%x)\n", StreamNumber, pSrb));

	// Let the Stream handle the request!
	return SupportedStreams[StreamNumber]->SetState( pSrb );
}

NTSTATUS BtDigitizer::StreamProposeFormat( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("BtDigitizer::StreamProposeFormat"));
	// Get the Stream Number
	ULONG StreamNumber = pSrb->StreamObject->StreamNumber;
	if ( StreamNumber >= SIZEOF_ARRAY(SupportedStreams) ) {
		ERROR(("Invalid Stream number (%d)\n", StreamNumber));

		return STATUS_INVALID_PARAMETER;
	}

	DUMP(("Stream(%d) - pSrb(%x)\n", StreamNumber, pSrb));

	// Let the Stream handle the request!
	return SupportedStreams[StreamNumber]->ProposeFormat( pSrb );
}

NTSTATUS BtDigitizer::StreamSetFormat( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("BtDigitizer::StreamSetFormat")); 
	// Get the Stream Number
	ULONG StreamNumber = pSrb->StreamObject->StreamNumber;
	if ( StreamNumber >= SIZEOF_ARRAY(SupportedStreams) ) {
		ERROR(("Invalid Stream number (%d)\n", StreamNumber));

		return STATUS_INVALID_PARAMETER;
	}

	DUMP(("Stream(%d) - pSrb(%x)\n", StreamNumber, pSrb));

	// Let the Stream handle the request!
	return SupportedStreams[StreamNumber]->SetFormat( pSrb );

}

NTSTATUS BtDigitizer::StreamGetFormat( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("BtDigitizer::StreamGetFormat"));
	// Get the Stream Number
	ULONG StreamNumber = pSrb->StreamObject->StreamNumber;
	if ( StreamNumber >= SIZEOF_ARRAY(SupportedStreams) ) {
		ERROR(("Invalid Stream number (%d)\n", StreamNumber));

		return STATUS_INVALID_PARAMETER;
	}

	DUMP(("Stream(%d) - pSrb(%x)\n", StreamNumber, pSrb));

	// Let the Stream handle the request!
	return SupportedStreams[StreamNumber]->GetFormat( pSrb );

}

NTSTATUS BtDigitizer::StreamGetProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("BtDigitizer::StreamGetProperty"));
	// Get the Stream Number
	ULONG StreamNumber = pSrb->StreamObject->StreamNumber;
	if ( StreamNumber >= SIZEOF_ARRAY(SupportedStreams) ) {
		ERROR(("Invalid Stream number (%d)\n", StreamNumber));

		return STATUS_INVALID_PARAMETER;
	}

	DUMP(("Stream(%d) - pSrb(%x)\n", StreamNumber, pSrb));

	// Let the Stream handle the request!
	return SupportedStreams[StreamNumber]->GetProperty( pSrb );

}

NTSTATUS BtDigitizer::StreamSetProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("BtDigitizer::StreamSetProperty")); 
	// Get the Stream Number
	ULONG StreamNumber = pSrb->StreamObject->StreamNumber;
	if ( StreamNumber >= SIZEOF_ARRAY(SupportedStreams) ) {
		ERROR(("Invalid Stream number (%d)\n", StreamNumber));

		return STATUS_INVALID_PARAMETER;
	}

	DUMP(("Stream(%d) - pSrb(%x)\n", StreamNumber, pSrb));

	// Let the Stream handle the request!
	return SupportedStreams[StreamNumber]->SetProperty( pSrb );

}

NTSTATUS BtDigitizer::StreamIndicateMasterClock( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("BtDigitizer::StreamIndicateMasterClock"));
	// Get the Stream Number
	ULONG StreamNumber = pSrb->StreamObject->StreamNumber;
	if ( StreamNumber >= SIZEOF_ARRAY(SupportedStreams) ) {
		ERROR(("Invalid Stream number (%d)\n", StreamNumber));

		return STATUS_INVALID_PARAMETER;
	}

	DUMP(("Stream(%d) - pSrb(%x)\n", StreamNumber, pSrb));

	// Let the Stream handle the request!
	return SupportedStreams[StreamNumber]->IndicateMasterClock( pSrb );

}

void BtDigitizer::StreamReceiveDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("BtDigitizer::StreamReceiveDataPacket"));
	// Get the Stream Number
	ULONG StreamNumber = pSrb->StreamObject->StreamNumber;
	if ( StreamNumber >= SIZEOF_ARRAY(SupportedStreams) ) {
		ERROR(("Invalid Stream number (%d)\n", StreamNumber));
		pSrb->Status = STATUS_INVALID_PARAMETER;
		CompleteDataPacket(pSrb);
	}

	DUMP(("Stream(%d) - pSrb(%x)\n", StreamNumber, pSrb));

	// Let the Stream handle the request!
	SupportedStreams[StreamNumber]->ReceiveDataPacket( pSrb );
}

void BtDigitizer::StreamCancelDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::StreamCancelDataPacket"));
		// Get the Stream Number
	ULONG StreamNumber = pSrb->StreamObject->StreamNumber;
	if ( StreamNumber >= SIZEOF_ARRAY(SupportedStreams) ) {
		ERROR(("Invalid Stream number (%d)\n", StreamNumber));
		return;
	}

	DUMP(("Stream(%d) - pSrb(%x)\n", StreamNumber, pSrb));

	// Let the Stream handle the request!
	SupportedStreams[StreamNumber]->CancelDataPacket( pSrb );
}

void BtDigitizer::StreamTimeoutDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizer::StreamTimeoutDataPacket"));
		// Get the Stream Number
	ULONG StreamNumber = pSrb->StreamObject->StreamNumber;
	if ( StreamNumber >= SIZEOF_ARRAY(SupportedStreams) ) {
		ERROR(("Invalid Stream number (%d)\n", StreamNumber));
		return;
	}

	DUMP(("Stream(%d) - pSrb(%x)\n", StreamNumber, pSrb));

	// Let the Stream handle the request!
	SupportedStreams[StreamNumber]->TimeoutDataPacket( pSrb );
}
