/*
	BtDigitizer.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef BTDIGITIZER_HPP_
#define BTDIGITIZER_HPP_

#include "WDMHeaders.hpp"

#include "BtBt848.hpp"
#include "BtVideoDec.hpp"
#include "BtVideoProcAmp.hpp"
#include "BtTVAudio.hpp"
#include "BtXBar.hpp"
#include "BtTuner.hpp"
#include "BtVideoCtl.hpp"
#include "BtAudio.hpp"
#include "BtIR.hpp"
#include "BtVideoWindow.hpp"
#include "custprop.h"

#include "Adapter.hpp"
#include "Hardware\BtCard.hpp"

// Indices to Streams
#define STREAM_IDX_CAPTURE       0
#define STREAM_IDX_PREVIEW       1
#define STREAM_IDX_VBI           2
#define STREAM_IDX_ANALOG        3
#define DRIVER_STREAM_COUNT		 4

// All the Streams supported!
#include "BtAnalogStream.hpp"
#include "BtCaptureStream.hpp"
#include "BtPreviewStream.hpp"
#include "BtVBIStream.hpp"

extern "C" {
	ULONG STREAMAPI DriverEntry( PDRIVER_OBJECT, PUNICODE_STRING );
	VOID STREAMAPI  Unload( IN PDRIVER_OBJECT DriverObject );
}

// Class declaration
class BtDigitizer :
		public Adapter ,
		public BtBt848 ,
		public BtVideoDec ,
		public BtVideoProcAmp ,
		public BtTVAudio ,
		public BtXBar ,
		public BtTuner ,
		public BtAudio , 
		public BtIR ,
		public BtVideoWindow {
	
	friend class BtAnalogStream;

	private:

		// All the parts that a Bt848 based Capture Card supports!
		BtCard*				Card;

		// All the Streams supported!
		BtVCaptureStream	streamCapture;
		BtVPreviewStream	streamPreview;
		BtVBIStream			streamVBI;
		BtAnalogStream		streamAnalog;

		// And an array to reference them by index!
		BaseStream *		SupportedStreams[DRIVER_STREAM_COUNT];

		// Some misc configs ... User controllable behaviour
		BOOL				bUseOddFieldForPreview;		// Means the Odd field will be used for preview, and the other one for capture, instead of default opposite


		// Initialization vars
		bool				InitedOK;
		ULONG				Instance; // The instance identifier

		// Some TV tuner information
		ULONG				m_TunerVideoSubChannel;
		ULONG				m_TunerAudioSubChannel;
		ULONG				m_TunerChannel;
		ULONG				m_TunerCountry;

	public:

		static KSPROPERTY_SET PropertySets[];
		static GUID AdapterCategories[];
		static KSTOPOLOGY Topology;
		static HW_STREAM_HEADER StreamHeader;
		static KSPIN_MEDIUM Mediums_Template[4];
		static BOOL PinDirection [4];
		static GUID Categories[4];
		static KSPIN_MEDIUM StandardMedium;
		static LONGLONG FramerateList[];
	
	protected:
		// A local instance copy - Will be accessed by BtAnalogStream class to get it's medium
		KSPIN_MEDIUM Mediums[4];

	protected:

		// Creation of BtAdapter: this == 0 after or before this
		static NTSTATUS Initialize( PHW_STREAM_REQUEST_BLOCK pSrb );

	public:

		// Callbacks
		static void STREAMAPI cbReceiveCtrlPacket( PHW_STREAM_REQUEST_BLOCK pSrb );

	public:

		virtual BOOLEAN HwInterrupt( void );

	public:

		BtDigitizer (const PDEVICE_OBJECT pPDO);

		bool Init(const PVOID HwDeviceExtension,const DWORD_PTR dwBase);
		virtual void Uninit( void );

		~BtDigitizer();

	protected:

		// Internal functions 
		// functions to process each command

		// Adapter related commands
		virtual NTSTATUS CompleteInitialization( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS GetProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS SetProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS PowerDown( void );
		virtual NTSTATUS PowerUp( void );
		virtual NTSTATUS GetStreamInfo( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS UnknownCommand( PHW_STREAM_REQUEST_BLOCK pSrb );

		// Stream directed commands
		virtual NTSTATUS StreamFormatFromRange( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamOpen( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamClose( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamGetState( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamSetState( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamProposeFormat( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamSetFormat( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamGetFormat( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamGetProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamSetProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamIndicateMasterClock( PHW_STREAM_REQUEST_BLOCK pSrb );

		// Stream packet interfase
		virtual void	 StreamReceiveDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual void	 StreamCancelDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual void	 StreamTimeoutDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb );

	protected:

		// Internal Commands
		void ReadRegistryValues();
		void WriteRegistryValues();
	
	protected:

		// Properties for each component
		NTSTATUS SetVideoProcAmpProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS GetVideoProcAmpProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS SetVideoDecProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS GetVideoDecProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS SetTVAudioProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS GetTVAudioProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS SetAudioProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS GetAudioProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS SetXBarProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS GetXBarProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS SetTunerProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS GetTunerProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS SetVideoControlProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS GetVideoControlProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS GetBt848CustomProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS GetIRProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS SetVideoWindowProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		NTSTATUS GetVideoWindowProperty( PHW_STREAM_REQUEST_BLOCK pSrb );

	public:

		// To let Analog Channel Send notifications to the other channels!
		void SendChangeChannelNotification(PHW_STREAM_REQUEST_BLOCK pSrb);

	private:

		BOOL CanRoute( PKSPROPERTY_CROSSBAR_ROUTE_S pRoute );

	public:

		// Interfase functions
		BtCard& GetBtCard() { return *Card; }
		KS_AnalogVideoStandard GetCurrentVideoStandard() { return GetBtCard().VideoGetCurrentStandard(); }
		BOOL UseOddFieldForPreview() const { return bUseOddFieldForPreview; }


};

#endif




