/*
	BtDigitizerStream.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef BTDIGITIZERSTREAM_H_
#define BTDIGITIZERSTREAM_H_

#include "Stream.hpp"
#include "SrbQueue.hpp"
#include "SpinLock.hpp"
#include "Hardware\BtCard.hpp"	
#include "DDraw.hpp"

// Forward Decls
class BtDigitizer;

// This class represents a Capture Stream (Capture,Preview or VBI)
// This class represents the common functionality of all digitalization Streams
class BtDigitizerStream : 
	public CaptureStream			// We are a capture Stream
{
	protected:

		// Queue of requests
		KSSTATE	 KSState;			// Stream Status
		SpinLock BufferLock;		// Spinlock to protect the data buffer
		SrbQueue BufferQueue;		// Queue of SRBs with data buffers

		// Pointers
		BtDigitizer  *digitizer;	// Pointer to the main class
		VideoChannel *vc;			// Pointer to the videochannel associated to this Stream

		// Handle para obtener el Tiempo
		HANDLE hMasterClock;		// Handle al objeto que es el Clockmaster

		// Flag to know if we have had buffers missing
		BOOL bDataDiscontinuity;	// True if we weren't able to get new buffers

		// Stride of buffer in bytes
		DWORD dwStride;

		// Flag to signal that Stream is open!
		BOOL bStreamOpened;

#ifdef ENABLE_DDRAW_STUFF
		// The DirectDraw object we will use to blit directly to surfaces (Xface to Overlay Mixer)
		DirectDraw					 DDraw;						// Encapsulates all the functionality required!
#endif

	public:

		BtDigitizerStream(BtDigitizer *dev);
		virtual ~BtDigitizerStream();

	protected:

		// Reset Statistic counters
		virtual void	 ResetCounters() = 0;
		virtual void	 IncPictureNumber() = 0;
		virtual void	 IncDropCount() = 0;

		// Events on channels
		virtual NTSTATUS Create() = 0;
		virtual NTSTATUS Destroy() = 0;

	public:
		virtual NTSTATUS Open( IN PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS Close( IN PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS FormatFromRange( IN PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual HW_STREAM_INFORMATION& Info( void ) = 0; 
		virtual ULONG    GetVideoControlCaps( void ) = 0;
		virtual ULONG    GetVideoControlMode( void ) = 0;
		virtual NTSTATUS SetVideoControlMode( ULONG mode ) = 0;
		virtual LONGLONG GetActualFramerateInNs( void ) = 0;
		virtual LONGLONG GetMaxAvailableFramerateInNs( void ) = 0;
		virtual NTSTATUS GetAnalogWindow( RECT& rc ) = 0;
		virtual NTSTATUS SetAnalogWindow( const RECT& rc ) = 0;
		virtual NTSTATUS GetDigitalWindow( RECT& rc ) = 0;
		virtual void TimeoutDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb ) ;
		virtual void CancelDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb ) ;

	protected:
		// Called when user program supplies a buffer to fill in		
		virtual void OnReadData( PHW_STREAM_REQUEST_BLOCK pSrb) ;
		virtual NTSTATUS GetState( PHW_STREAM_REQUEST_BLOCK pSrb ) ;
		virtual NTSTATUS SetState( PHW_STREAM_REQUEST_BLOCK pSrb ) ;
		virtual NTSTATUS ProposeFormat( PHW_STREAM_REQUEST_BLOCK pSrb ) ;
		virtual NTSTATUS SetFormat( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS GetFormat( PHW_STREAM_REQUEST_BLOCK pSrb ) ;
		virtual NTSTATUS GetProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) ;
		virtual NTSTATUS SetProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) ;
		virtual NTSTATUS IndicateMasterClock( PHW_STREAM_REQUEST_BLOCK pSrb ) ;

		// Functions
		virtual NTSTATUS GetConnectionProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS GetAllocatorProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS GetDroppedFramesProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;

		virtual BOOL     VerifyFormat( PKSDATAFORMAT pKSDataFormatToVerify ) = 0;

	public:
		// Function called to Notify channels of a Tuner Notification
		virtual void     ChangeNotification( PHW_STREAM_REQUEST_BLOCK pSrb );

	private:
		// To cancel all pending data Srbs
		void EmptyIncomingDataBufferQueue();

	public:
		virtual BOOL IsOpen( void );
		
};

#endif

