/*
	BtPreviewStream.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef BtPreviewStream_HPP_
#define BtPreviewStream_HPP_

#include "BtVideoStream.hpp"

// This class represents the preview pin.
class BtVPreviewStream : public BtVideoStream {
	public:

		static KS_DATARANGE_VIDEO			FormatRGB24[];
		static KS_DATARANGE_VIDEO			FormatRGB32[];
		static KS_DATARANGE_VIDEO			FormatYUY2[];
		static KS_DATARANGE_VIDEO			FormatYVU9[];
		static KS_DATARANGE_VIDEO			FormatUYVY[];
		static KS_DATARANGE_VIDEO			FormatY41P[];
		static KS_DATARANGE_VIDEO			FormatYV12[];
		static KS_DATARANGE_VIDEO			FormatI420[];
		static KS_DATARANGE_VIDEO			FormatRGB555[];
		static KS_DATARANGE_VIDEO_WITHMASK	FormatRGB565[]; // No other way to describe it!
		static KS_DATARANGE_VIDEO2			FormatRGB24_2[];
		static KS_DATARANGE_VIDEO2			FormatRGB32_2[];
		static KS_DATARANGE_VIDEO2			FormatYUY2_2[];
		static KS_DATARANGE_VIDEO2			FormatYVU9_2[];
		static KS_DATARANGE_VIDEO2			FormatUYVY_2[];
		static KS_DATARANGE_VIDEO2			FormatY41P_2[];
		static KS_DATARANGE_VIDEO2			FormatYV12_2[];
		static KS_DATARANGE_VIDEO2			FormatI420_2[];
		static KS_DATARANGE_VIDEO2			FormatRGB555_2[];
		static KS_DATARANGE_VIDEO2_WITHMASK	FormatRGB565_2[]; // No other way to describe it!
		static PKSDATAFORMAT				Formats[];
		static HW_STREAM_INFORMATION		StreamInfo;
		static HW_STREAM_OBJECT				StreamObject;
		static const KSPROPERTY_ITEM		Connection[];
		static const KSPROPERTY_ITEM		DroppedFrames[];
		static const KSPROPERTY_ITEM		Allocator[];
		static KSPROPERTY_SET				Properties[];

	protected:
		
		// If channel was allocd when we virtualized it!
		bool	bWasAlloc;

	public:

		BtVPreviewStream(BtDigitizer *dev);
		virtual ~BtVPreviewStream();

	protected:

		// Function called to Timestamp Buffers
		virtual void TimeStamp( PKSSTREAM_HEADER pHeader , PHW_STREAM_REQUEST_BLOCK pSrb );
		
		// Events on channels
		virtual NTSTATUS Create();
		virtual NTSTATUS Destroy();

	public:

		virtual HW_STREAM_INFORMATION& Info( void ); 

	protected:

		// Functions
		virtual NTSTATUS GetConnectionProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) ;
		virtual NTSTATUS GetAllocatorProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) ;

		// Special functions to give preference to the Capture Channel (used
		// to pass control of the videoChannel to the Capture channel)
		void CreateVirtualized();

		// Xfaces to the Capture Channel!
	public:

		// To force deallocation of VideoChannel to pass it to the Capture Stream
		bool DeallocChannel();

		// To return VideoChannel to the Preview Stream
		bool ReallocChannel();

		// To get buffers to be completed for previewing purposes
		const VideoDataBuf OnCaptureSample(VideoChannel* Capture, bool GetBuffer = false);

};


#endif

