/*
	BtVBIStream.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "Debug.hpp"
#include "BtVBIStream.hpp"
#include "Hardware\HwCapture\defaults.hpp"
#include "BtDigitizer.hpp"

// Note: To Check ... I don't have a VBI stream to test it!

#define KS_AnalogVideo_25fps \
	( KS_AnalogVideo_PAL_B | KS_AnalogVideo_PAL_D | \
	KS_AnalogVideo_PAL_G | KS_AnalogVideo_PAL_H | \
	KS_AnalogVideo_PAL_I | KS_AnalogVideo_SECAM_L | \
	KS_AnalogVideo_SECAM_B | KS_AnalogVideo_SECAM_D | \
	KS_AnalogVideo_SECAM_G | KS_AnalogVideo_SECAM_H | \
	KS_AnalogVideo_SECAM_K | KS_AnalogVideo_SECAM_K1 | \
	KS_AnalogVideo_PAL_N | KS_AnalogVideo_PAL_NC | KS_AnalogVideo_NTSC_50 )


#define KS_AnalogVideo_30fps \
	( KS_AnalogVideo_NTSC_M | KS_AnalogVideo_PAL_M | \
	  KS_AnalogVideo_NTSC_M_J | KS_AnalogVideo_PAL_60 )

KS_DATARANGE_VIDEO_VBI BtVBIStream::FormatNTSCVBI =
{
   // KSDATARANGE
   {
      {
         sizeof( KS_DATARANGE_VIDEO_VBI ),
         0,
         dwNTSCVBISamplesPerLine * dwNTSCVBILines,      // SampleSize
         0,                          // Reserved
         { STATIC_KSDATAFORMAT_TYPE_VBI },
         { STATIC_KSDATAFORMAT_SUBTYPE_RAW8 },
         { STATIC_KSDATAFORMAT_SPECIFIER_VBI }
      }
   },
   true,    // BOOL,  bFixedSizeSamples (all samples same size?)
   true,    // BOOL,  bTemporalCompression (all I frames?)
   KS_VIDEOSTREAM_VBI, // StreamDescriptionFlags  (KS_VIDEO_DESC_*)
   0,       // MemoryAllocationFlags   (KS_VIDEO_ALLOC_*)

   // _KS_VIDEO_STREAM_CONFIG_CAPS
   {
      { STATIC_KSDATAFORMAT_SPECIFIER_VBI },

		KS_AnalogVideo_30fps,                      // AnalogVideoStandard
      {
         dwNTSCVBISamplesPerLine, dwNTSCVBILines   // SIZE InputSize
      },
      {
         dwNTSCVBISamplesPerLine, dwNTSCVBILines   // SIZE MinCroppingSize;       smallest rcSrc cropping rect allowed
      },
      {
         dwNTSCVBISamplesPerLine, dwNTSCVBILines   // SIZE MaxCroppingSize;       largest rcSrc cropping rect allowed
      },
      1,           // int CropGranularityX;       // granularity of cropping size
      1,           // int CropGranularityY;
      1,           // int CropAlignX;             // alignment of cropping rect
      1,           // int CropAlignY;
      {
         dwNTSCVBISamplesPerLine, dwNTSCVBILines  // SIZE MinOutputSize;         // smallest bitmap stream can produce
      },
      {
         dwNTSCVBISamplesPerLine, dwNTSCVBILines  // SIZE MaxOutputSize;         // largest  bitmap stream can produce
      },
      1,          // int OutputGranularityX;     // granularity of output bitmap size
      2,          // int OutputGranularityY;
      0,          // StretchTapsX  (0 no stretch, 1 pix dup, 2 interp...)
      0,          // StretchTapsY
      0,          // ShrinkTapsX
      0,          // ShrinkTapsY
      dwNTSCFrameDuration,     // LONGLONG MinFrameInterval;  // 100 nS units
      dwNTSCFrameDuration,     // LONGLONG MaxFrameInterval;  // 100 nS units
      dwNTSCVBISamplesPerLine * dwNTSCFramesPerSecond * dwNTSCVBILines * 2 * 8, // LONG MinBitsPerSecond;
      dwNTSCVBISamplesPerLine * dwNTSCFramesPerSecond * dwNTSCVBILines * 2 * 8  // LONG MaxBitsPerSecond;
   },

   // KS_VBIINFOHEADER (default format)
   {
      dwNTSCVBIStart,      // StartLine  -- inclusive
      dwNTSCVBIEnd,        // EndLine    -- inclusive
      dwNTSCSamplingFrequency,
      dwNTSCVBIMinLineStartTime,    // (uS past HR LE) * 100
      dwNTSCVBIMaxLineStartTime,    // (uS past HR LE) * 100
      dwNTSCVBIActualLineStartTime, // (uS past HR LE) * 100
      dwNTSCVBIActualLineEndTime,   // (uS past HR LE) * 100
      KS_AnalogVideo_30fps,
      dwNTSCVBISamplesPerLine,
      dwNTSCVBIStrideInBytes,
      dwNTSCVBIBufferSize
   }
};


KS_DATARANGE_VIDEO_VBI BtVBIStream::FormatPALVBI =
{
   // KSDATARANGE
   {
      {
         sizeof( KS_DATARANGE_VIDEO_VBI ),
         0,
         //BUGBUG is this (dwPALVBILines) ok for PAL?
         dwPALVBISamplesPerLine * dwPALVBILines,      // SampleSize
         0,                          // Reserved
         { STATIC_KSDATAFORMAT_TYPE_VBI },
         { STATIC_KSDATAFORMAT_SUBTYPE_RAW8 },
         { STATIC_KSDATAFORMAT_SPECIFIER_VBI }
      }
   },
   true,    // BOOL,  bFixedSizeSamples (all samples same size?)
   true,    // BOOL,  bTemporalCompression (all I frames?)
   KS_VIDEOSTREAM_VBI, // StreamDescriptionFlags  (KS_VIDEO_DESC_*)
   0,       // MemoryAllocationFlags   (KS_VIDEO_ALLOC_*)

   // _KS_VIDEO_STREAM_CONFIG_CAPS
   {
      { STATIC_KSDATAFORMAT_SPECIFIER_VBI },
		KS_AnalogVideo_25fps,                       // AnalogVideoStandard
      {
         dwPALVBISamplesPerLine, dwPALVBILines  // SIZE InputSize
      },
      {
         dwPALVBISamplesPerLine, dwPALVBILines   // SIZE MinCroppingSize;       smallest rcSrc cropping rect allowed
      },
      {
         dwPALVBISamplesPerLine, dwPALVBILines   // SIZE MaxCroppingSize;       largest rcSrc cropping rect allowed
      },
      1,           // int CropGranularityX;       // granularity of cropping size
      1,           // int CropGranularityY;
      1,           // int CropAlignX;             // alignment of cropping rect
      1,           // int CropAlignY;
      {
         dwPALVBISamplesPerLine, dwPALVBILines   // SIZE MinOutputSize;         // smallest bitmap stream can produce
      },
      {
         dwPALVBISamplesPerLine, dwPALVBILines   // SIZE MaxOutputSize;         // largest  bitmap stream can produce
      },
      1,          // int OutputGranularityX;     // granularity of output bitmap size
      2,          // int OutputGranularityY;
      0,          // StretchTapsX  (0 no stretch, 1 pix dup, 2 interp...)
      0,          // StretchTapsY
      0,          // ShrinkTapsX
      0,          // ShrinkTapsY
      dwPALFrameDuration,     // LONGLONG MinFrameInterval;  // 100 nS units
      dwPALFrameDuration,     // LONGLONG MaxFrameInterval;  // 100 nS units
      dwPALVBISamplesPerLine * dwPALFramesPerSecond * dwPALVBILines * 2 * 8, // LONG MinBitsPerSecond;
      dwPALVBISamplesPerLine * dwPALFramesPerSecond * dwPALVBILines * 2 * 8  // LONG MaxBitsPerSecond;
   },

   // KS_VBIINFOHEADER (default format)
   {
      dwPALVBIStart,      // StartLine  -- inclusive
      dwPALVBIEnd,        // EndLine    -- inclusive
      dwPALSamplingFrequency,
      dwPALVBIMinLineStartTime,    // (uS past HR LE) * 100
      dwPALVBIMaxLineStartTime,    // (uS past HR LE) * 100
      dwPALVBIActualLineStartTime, // (uS past HR LE) * 100
      dwPALVBIActualLineEndTime,   // (uS past HR LE) * 100
      KS_AnalogVideo_25fps,
      dwPALVBISamplesPerLine,
      dwPALVBIStrideInBytes,
      dwPALVBIBufferSize
   }
};


//---------------------------------------------------------------------------
//  VBI Stream Formats
//---------------------------------------------------------------------------

PKSDATAFORMAT BtVBIStream::Formats[] =
{
	(PKSDATAFORMAT) &FormatNTSCVBI,
	(PKSDATAFORMAT) &FormatPALVBI
};

// Number of formats!
#define NUM_VBI_FORMATS (sizeof (BtVBIStream::Formats) / sizeof (PKSDATAFORMAT))

// ------------------------------------------------------------------------
// Property set for all video capture/preview streams
// ------------------------------------------------------------------------

const KSPROPERTY_ITEM BtVBIStream::Connection[] = 
{
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_CONNECTION_ALLOCATORFRAMING,
      TRUE,                                   // GetSupported or Handler
      sizeof( KSPROPERTY ),                   // MinProperty
      sizeof( KSALLOCATOR_FRAMING ),          // MinData
      FALSE,                                  // SetSupported or Handler
      NULL,                                   // Values
      0,                                      // RelationsCount
      NULL,                                   // Relations
      NULL,                                   // SupportHandler
      sizeof( ULONG )                         // SerializedSize
   )
};

const KSPROPERTY_ITEM BtVBIStream::DroppedFrames[] = 
{
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_DROPPEDFRAMES_CURRENT,
      TRUE,                                   // GetSupported or Handler
      sizeof(KSPROPERTY_DROPPEDFRAMES_CURRENT_S),// MinProperty
      sizeof(KSPROPERTY_DROPPEDFRAMES_CURRENT_S),// MinData
      FALSE,                                  // SetSupported or Handler
      NULL,                                   // Values
      0,                                      // RelationsCount
      NULL,                                   // Relations
      NULL,                                   // SupportHandler
      0                                       // SerializedSize
   )
};

const KSPROPERTY_ITEM BtVBIStream::Allocator[] = 
{
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_ALLOCATOR_CONTROL_HONOR_COUNT,
      TRUE,                                   // GetSupported or Handler
      sizeof(KSPROPERTY),                     // MinProperty
      sizeof(BOOL), //BOOL                   // MinData 
      FALSE,                                  // SetSupported or Handler
      NULL,                                   // Values
      0,                                      // RelationsCount
      NULL,                                   // Relations
      NULL,                                   // SupportHandler
      0                                       // SerializedSize
   ),
};

// ------------------------------------------------------------------------
// Array of all of the property sets supported by video capture/preview streams
// ------------------------------------------------------------------------

KSPROPERTY_SET BtVBIStream::Properties[] =
{
    DEFINE_KSPROPERTY_SET
    (
        &KSPROPSETID_Connection,                          // Set
        SIZEOF_ARRAY( BtVBIStream::Connection ),      // PropertiesCount
        BtVBIStream::Connection,					  // PropertyItem
        0,                                                // FastIoCount
        NULL                                              // FastIoTable
    ),
    DEFINE_KSPROPERTY_SET
    ( 
        &PROPSETID_VIDCAP_DROPPEDFRAMES,				    // Set
        SIZEOF_ARRAY( BtVBIStream::DroppedFrames ),		// PropertiesCount
        BtVBIStream::DroppedFrames,                     // PropertyItem
        0,                                                  // FastIoCount
        NULL                                                // FastIoTable
    ),
    DEFINE_KSPROPERTY_SET
    ( 
        &PROPSETID_ALLOCATOR_CONTROL,						// Set
        SIZEOF_ARRAY( BtVBIStream::Allocator ),			// PropertiesCount
        BtVBIStream::Allocator,							// PropertyItem
        0,													// FastIoCount
        NULL												// FastIoTable
    )
};

// -----------------------------------------------------------------
// VBI Stream 
// ----------------------------------------------------------------

// HW_STREAM_INFORMATION -------------------------------------------
HW_STREAM_INFORMATION BtVBIStream::StreamInfo = 
{
	1,                                              // NumberOfPossibleInstances
	KSPIN_DATAFLOW_OUT,                             // DataFlow
	TRUE,                                           // DataAccessible
	NUM_VBI_FORMATS,                                // NumberOfFormatArrayEntries
	BtVBIStream::Formats,                           // StreamFormatsArray
	{
		0,                                          // ClassReserved[0]
		0,                                          // ClassReserved[1]
		0,                                          // ClassReserved[2]
		0                                           // ClassReserved[3]
	},
	SIZEOF_ARRAY(BtVBIStream::Properties),          // NumStreamPropArrayEntries
	(PKSPROPERTY_SET)BtVBIStream::Properties,		// StreamPropertiesArray
	0,                                              // NumStreamEventArrayEntries;
	0,                                              // StreamEventsArray;
	(GUID *) &PINNAME_VIDEO_VBI,                    // Category
	(GUID *) &PINNAME_VIDEO_VBI,                    // Name
	1,                                              // MediumsCount
	&BtDigitizer::StandardMedium,                   // Mediums
	FALSE
};

// HW_STREAM_OBJECT ------------------------------------------------
HW_STREAM_OBJECT BtVBIStream::StreamObject = 
{
	sizeof (HW_STREAM_OBJECT),						 // SizeOfThisPacket
	STREAM_IDX_VBI,									 // StreamNumber
	0,											     // HwStreamExtension
	BtDigitizer::cbReceiveDataPacket,				 // HwReceiveDataPacket
	BtDigitizer::cbReceiveCtrlPacket,				 // HwReceiveControlPacket
	{ NULL, 0 },									 // HW_CLOCK_OBJECT
	TRUE,											 // Dma
	TRUE,											 // Pio
	0,												 // HwDeviceExtension
	sizeof (KS_VBI_FRAME_INFO),						 // StreamHeaderMediaSpecific
	0,												 // StreamHeaderWorkspace 
	TRUE,											 // Allocator 
	NULL,											 // HwEventRoutine
	{ 0, 0 },										 // Reserved[2]
};

BtVBIStream::BtVBIStream(BtDigitizer *dev) :
	BtDigitizerStream(dev) ,
	bIsDirty(FALSE)
{
	TRACE(("BtVBIStream::BtVBIStream"));

	// Reset Counters
	BtVBIStream::ResetCounters();
}

BtVBIStream::~BtVBIStream()
{
	TRACE(("BtVBIStream::~BtVBIStream"));
}

// Reset Statistic counters
void BtVBIStream::ResetCounters()
{
	TRACE(("BtVBIStream::ResetCounters"));
	FrameInfo.PictureNumber = 0;
	FrameInfo.DropCount = 0;
	bDataDiscontinuity = FALSE; // Reset Flag
}

void BtVBIStream::IncPictureNumber()
{
	TRACE(("BtVBIStream::IncPictureNumber"));
	FrameInfo.PictureNumber++;
}

void BtVBIStream::IncDropCount()
{
	TRACE(("BtVBIStream::IncDropCount"));
	FrameInfo.DropCount++;
	bDataDiscontinuity = TRUE; // Remember we dropped a frame!
}

// VideoChannel events.
NTSTATUS BtVBIStream::Destroy()
{
	TRACE(("BtVBIStream::Destroy"));

	// Stop videoChannel and Close It!
	if (vc) { // If videoChannel already created ....
		digitizer->GetBtCard().CloseVideoChannel(vc); 
		vc = NULL;
	}

	return STATUS_SUCCESS;
}

NTSTATUS BtVBIStream::Create()
{
	TRACE(("BtVBIStream::Create"));

	// Create VideoChannel if not created.
	if (!vc) {

		// Create video Channel if possible!
		dwStride = VBIHeader.StrideInBytes;
		vc = digitizer->GetBtCard().OpenVideoChannel(
				VCM_VBI,
				this,
				MRect( 
					VBIHeader.ActualLineStartTime,
					VBIHeader.StartLine,
					VBIHeader.ActualLineEndTime ,
					VBIHeader.EndLine
					),
				MRect( 
					VBIHeader.ActualLineStartTime,
					VBIHeader.StartLine,
					VBIHeader.ActualLineEndTime ,
					VBIHeader.EndLine
					),
					CF_VBI	);
		if (!vc) {

			return STATUS_IO_DEVICE_ERROR;
		}
	}

	return STATUS_SUCCESS;
}


/*
** BtVBIStream::GetDroppedFramesProperty
**
**    Gets dropped frame information
**
** Arguments:
**
**    pSrb - pointer to the stream request block for properties
**
** Returns:
**
** Side Effects:  none
*/
NTSTATUS BtVBIStream::GetDroppedFramesProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtVBIStream::GetDroppedFramesProperty"));

	PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;
	ULONG Id = pSPD->Property->Id;              // index of the property

	switch (Id) {

		case KSPROPERTY_DROPPEDFRAMES_CURRENT: {

				DUMP(("KSPROPERTY_DROPPEDFRAMES_CURRENT\n"));
				PKSPROPERTY_DROPPEDFRAMES_CURRENT_S pDroppedFrames = (PKSPROPERTY_DROPPEDFRAMES_CURRENT_S) pSPD->PropertyInfo;
	
				pDroppedFrames->PictureNumber	 = FrameInfo.PictureNumber;
				pDroppedFrames->DropCount		 = FrameInfo.DropCount;

				// Get the size of the VBI in bytes
				pDroppedFrames->AverageFrameSize = VBIHeader.BufferSize;
	
				DUMP(("KSPROPERTY_DROPPEDFRAMES_CURRENT:\n"));
				DUMP(("   PicNum(%d)\n", (ULONG)pDroppedFrames->PictureNumber));
				DUMP(("   DropCnt(%d)\n", (ULONG)pDroppedFrames->DropCount));
				DUMP(("   FrameSize(%d)\n", pDroppedFrames->AverageFrameSize));
	
				pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_DROPPEDFRAMES_CURRENT_S);
				return STATUS_SUCCESS;
		   }
		default:

			ERROR(("KSPROPERTY_DROPPEDFRAMES_???(%d)\n", Id));
	}
	return STATUS_INVALID_PARAMETER;
}

/*
** BtVBIStream::VerifyFormat()
**
**   Checks the validity of a format request by walking through the
**       array of supported KSDATA_RANGEs for a given stream.
**
** Arguments:
**
**   pKSDataFormat - pointer of a KSDATAFORMAT structure.
**
** Returns:
** 
**   TRUE if the format is supported
**   FALSE if the format cannot be suppored
**
** Side Effects:  none
*/

BOOL BtVBIStream::VerifyFormat( PKSDATAFORMAT pKSDataFormatToVerify )
{
    TRACE(("BtVBIStream::VerifyFormat"));

	BOOL                        fOK = FALSE;
    ULONG                       j;

	// Get the pointer to the array of available formats
    PKSDATAFORMAT *pAvailableFormats =  Formats;

    DUMP(("FormatSize=%d\n",  pKSDataFormatToVerify->FormatSize));
    DUMP(("MajorFormat=%x\n", pKSDataFormatToVerify->MajorFormat));

    //
    // Walk the formats supported by the stream
    //

    for (j = 0; j < NUM_VBI_FORMATS; j++, pAvailableFormats++) {

        // Check for a match on the three GUIDs and format size
        if (!CompareGUIDsAndFormatSize(
                        pKSDataFormatToVerify, 
                        *pAvailableFormats,
                        FALSE)) {
            continue;
        }

        // Now that the three GUIDs match, switch on the Specifier
        // to do a further type-specific check

        // -------------------------------------------------------------------
        // Specifier FORMAT_VideoInfo for VIDEOINFOHEADER
        // -------------------------------------------------------------------
#if 0
        DUMP(("Try to match specifier\n"));
        DumpGUID(pKSDataFormatToVerify->Specifier);
        DUMP(("to KSDATAFORMAT_SPECIFIER_VIDEOINFO\n"));
        DumpGUID(KSDATAFORMAT_SPECIFIER_VIDEOINFO);
        DUMP(( "or KSDATAFORMAT_SPECIFIER_VIDEOINFO2\n"));
        DumpGUID(KSDATAFORMAT_SPECIFIER_VIDEOINFO2);
        DUMP(( "or KSDATAFORMAT_SPECIFIER_ANALOGVIDEO\n"));
        DumpGUID(KSDATAFORMAT_SPECIFIER_ANALOGVIDEO);
#endif
        // -------------------------------------------------------------------
        // Specifier FORMAT_VBIInfo for VBI
        // -------------------------------------------------------------------

        if ( pKSDataFormatToVerify->Specifier == KSDATAFORMAT_SPECIFIER_VBI ) {
                
            KS_DATAFORMAT_VBIINFOHEADER*  pDataFormatVideoInfoHeader = (KS_DATAFORMAT_VBIINFOHEADER*) pKSDataFormatToVerify;
            KS_VBIINFOHEADER*  pVideoInfoHdrToVerify = (KS_VBIINFOHEADER*) &pDataFormatVideoInfoHeader->VBIInfoHeader;
            PKS_DATARANGE_VIDEO_VBI pKSDataRangeVideo = (PKS_DATARANGE_VIDEO_VBI) *pAvailableFormats;
            KS_VIDEO_STREAM_CONFIG_CAPS *pConfigCaps = &pKSDataRangeVideo->ConfigCaps;

            DUMP(("AdapterVerifyFormat - VBIINFOHEADER\n"));
            DUMP(("pVideoInfoHdrToVerify=%x\n", pVideoInfoHdrToVerify));
            DUMP(("KS_VBIINFOHEADER size=%d\n", sizeof(KS_VBIINFOHEADER)));

            //
            // Perform all other verification tests here!!!
            //

#ifdef VBI_ORIGINAL_WAY
			// Make sure that the required videonorm for this open 
			// is active! - Here we must accept all 25fps formats as 
			// the same, or all 30 fps formats as the same

			// Get the VideoStandard of the requested format and the actual VideoStandard used in the chip.
            ULONG fVideoStandard = (ULONG) (pConfigCaps->VideoStandard);
			ULONG currVideoStandard = (ULONG) digitizer->GetBtCard().VideoGetCurrentStandard();
			// If format proposed does not have the Right VideoStandard, don't accept it.
			if ( 
					((fVideoStandard & KS_AnalogVideo_25fps) != 0 && (currVideoStandard & KS_AnalogVideo_25fps) == 0)
				||
					((fVideoStandard & KS_AnalogVideo_30fps) != 0 && (currVideoStandard & KS_AnalogVideo_30fps) == 0)
				) {
				continue;
			}


#else		// This is the method used by Conexant's drivers... (seems to work better!)

            ULONG VideoStandard = (ULONG) digitizer->GetBtCard().VideoGetCurrentStandard();

            if((VideoStandard & KS_AnalogVideo_30fps) &&
               !((pVideoInfoHdrToVerify->StartLine == dwNTSCVBIStart) &&
                 (pVideoInfoHdrToVerify->EndLine   == dwNTSCVBIEnd) &&
                 (pVideoInfoHdrToVerify->SamplesPerLine == dwNTSCVBISamplesPerLine))) {
               continue;   // fail
            }

            if((VideoStandard & KS_AnalogVideo_25fps) &&
               !((pVideoInfoHdrToVerify->StartLine == dwPALVBIStart) &&
                 (pVideoInfoHdrToVerify->EndLine   == dwPALVBIEnd) &&
                 (pVideoInfoHdrToVerify->SamplesPerLine == dwPALVBISamplesPerLine))) {
               continue;   // fail
            }

#endif
            //
            // HOORAY, the format passed all of the tests, so we support it
            //

            fOK = TRUE;
            break;

        } // End of VBIINFOHEADER specifier

    } // End of loop on all formats for this stream
    
    return fOK;
}

/*
** BtVBIStream::Open()
**
**   This routine is called when an OpenStream SRB request is received
**
** Arguments:
**
**   pSrb - pointer to stream request block for the Open command
**
** Returns:
**
** Side Effects:  none
*/
/* Notes: async call entry point */

NTSTATUS BtVBIStream::Open( IN PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtVBIStream::Open"));

	// If already open, stop and close channel first!
	NTSTATUS ret = Destroy(); /* (modifies videoChannel) */
	if (ret != STATUS_SUCCESS) {
		return ret;
	}

	//
	// Check the validity of the format being requested
	//
	if (!VerifyFormat (pSrb->CommandData.OpenFormat)) {
		ERROR(("BtVBIStream::Open() Exit (failed)\n"));
		return STATUS_INVALID_PARAMETER;
	}

	// Always open VBI stream as Alternating fields
	const KS_DATAFORMAT_VBIINFOHEADER &rKSVBIDataFormat = *(PKS_DATAFORMAT_VBIINFOHEADER) pSrb->CommandData.OpenFormat;
	if ( rKSVBIDataFormat.DataFormat.Specifier == KSDATAFORMAT_SPECIFIER_VBI ) {

		DUMP(("VBI\n"));

		// Remember the format of this Stream!
		VBIHeader = rKSVBIDataFormat.VBIInfoHeader;

#ifdef __FIXVIDEOSTD
		// But use the current video standard as the standard...
		VBIHeader.VideoStandard = (ULONG) digitizer->GetBtCard().VideoGetCurrentStandard();
#endif

	} else {

		ERROR(("Invalid video format header\n"));
		return STATUS_INVALID_PARAMETER;
	}

	// Set to Stop State
	KSState = KSSTATE_STOP;

	// the structure of the driver is such that a single callback could be used
	// for all stream requests. But the code below could be used to supply
	// different entry points for different streams

	pSrb->StreamObject->ReceiveDataPacket			= StreamObject.ReceiveDataPacket;
	pSrb->StreamObject->ReceiveControlPacket		= StreamObject.ReceiveControlPacket;
	pSrb->StreamObject->Dma							= StreamObject.Dma;
	pSrb->StreamObject->Allocator					= StreamObject.Allocator;

	// The PIO flag must be set when the mini driver will be accessing the data
	// buffers passed in using logical addressing

	pSrb->StreamObject->Pio							= StreamObject.Pio;
	pSrb->StreamObject->StreamHeaderMediaSpecific	= StreamObject.StreamHeaderMediaSpecific;
	pSrb->StreamObject->StreamHeaderWorkspace		= StreamObject.StreamHeaderWorkspace;
	pSrb->StreamObject->HwClockObject				= StreamObject.HwClockObject;

	// Now Stream is open!
	bStreamOpened = TRUE;

	DUMP(("Created!\n"));
	return STATUS_SUCCESS;
}

// Set video format
NTSTATUS BtVBIStream::SetFormat( PHW_STREAM_REQUEST_BLOCK pSrb ) 
{
	TRACE(("BtVBIStream::SetFormat"));

	if (!VerifyFormat (pSrb->CommandData.OpenFormat ) ) {
		ERROR(("SRB_SET_DATA_FORMAT FAILED\n"));
		return STATUS_NO_MATCH;
	} 

	// Get pointers to very used data!
	const KS_DATAFORMAT_VBIINFOHEADER &rKSVBIDataFormat = *(PKS_DATAFORMAT_VBIINFOHEADER) pSrb->CommandData.OpenFormat;
	if ( rKSVBIDataFormat.DataFormat.Specifier == KSDATAFORMAT_SPECIFIER_VBI ) {

		DUMP(("VBI\n"));

		// If no videochannel, only remember new format
		if (!vc) {

			DUMP(("No VideoChannel created - Only remember new format\n"));

			// Remember the format of this Stream!
			VBIHeader = rKSVBIDataFormat.VBIInfoHeader;

		} else {

			// There are 2 problems here: If we want to switch between alternating 
			// and normal mode, we MUST destroy videochannel. But, if only changing size,
			// perhaps we can do it without destroying it. The only problem is that
			// the creation of a VideoChannel can fail, and SetFormat shouldn't destroy
			// the videoChannel. So, get first the original parameters of VideoChannel.
			// destroy VideoChannel, try to create with the new parameters, if it fails,
			// Recreate with the old Parameters!

			// Destroy VideoChannel! - Hope to get success !
			NTSTATUS ret = Destroy();	/* (modifies videoChannel) */
			if (ret != STATUS_SUCCESS) {
				return ret;
			}

			// Try to reopen with the new parameters : 
			dwStride = rKSVBIDataFormat.VBIInfoHeader.StrideInBytes;
			vc = digitizer->GetBtCard().OpenVideoChannel(
					VCM_VBI,
					this,
					MRect( 
						rKSVBIDataFormat.VBIInfoHeader.ActualLineStartTime,
						rKSVBIDataFormat.VBIInfoHeader.StartLine,
						rKSVBIDataFormat.VBIInfoHeader.ActualLineEndTime ,
						rKSVBIDataFormat.VBIInfoHeader.EndLine
					),
					MRect( 
						rKSVBIDataFormat.VBIInfoHeader.ActualLineStartTime,
						rKSVBIDataFormat.VBIInfoHeader.StartLine,
						rKSVBIDataFormat.VBIInfoHeader.ActualLineEndTime ,
						rKSVBIDataFormat.VBIInfoHeader.EndLine
					),
					CF_VBI	);

			if (vc) {

				// Success ... Remember the new format!
				VBIHeader = rKSVBIDataFormat.VBIInfoHeader;

				// If we were running, restart stream!
				if (KSState == KSSTATE_RUN) {
					vc->Start();
				}

			} else {

				// We failed, recreate channel with the old parameters!
				NTSTATUS ret = Create();
				if (ret != STATUS_SUCCESS) {
					return ret;
				}

				// If we were running, restart stream!
				if (KSState == KSSTATE_RUN) {
					vc->Start();
				}
				return STATUS_NO_MATCH;
			}

		}

	} else {

		ERROR(("Invalid video header\n"));
		return STATUS_INVALID_PARAMETER;
	}

	// Everything OK
	return STATUS_SUCCESS;
}


/* Method: BtVBIStream::GetConnectionProperty
 * Purpose: Obtains allocator and state properties
 */
NTSTATUS BtVBIStream::GetConnectionProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtVBIStream::GetConnectionProperty"));

	PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;
	ULONG Id = pSPD->Property->Id;              // index of the property

	switch ( Id ) {
		case KSPROPERTY_CONNECTION_ALLOCATORFRAMING: {

			PKSALLOCATOR_FRAMING Framing = (PKSALLOCATOR_FRAMING) pSPD->PropertyInfo;
			Framing->RequirementsFlags   =
	            /*KSALLOCATOR_REQUIREMENTF_MUST_ALLOCATE |*/
				KSALLOCATOR_REQUIREMENTF_PREFERENCES_ONLY | 
				KSALLOCATOR_REQUIREMENTF_INPLACE_MODIFIER |
				KSALLOCATOR_REQUIREMENTF_SYSTEM_MEMORY;

				Framing->PoolType = NonPagedPool;

			Framing->Frames = 8;
			Framing->FrameSize = VBIHeader.BufferSize;
			Framing->FileAlignment = 0;
			Framing->Reserved = 0;

			pSrb->ActualBytesTransferred = sizeof( KSALLOCATOR_FRAMING );
			return STATUS_SUCCESS;
		}
	}
	ERROR(("Unsupported property"));
	return STATUS_INVALID_PARAMETER;
}

NTSTATUS BtVBIStream::GetAllocatorProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtVBIStream::GetAllocatorProperty"));

	// This whole stupid function/property is to get the overlay mixer to work again
	// with Windows 98 SP1

    PSTREAM_PROPERTY_DESCRIPTOR pSpd = pSrb->CommandData.PropertyInfo;
    ULONG Id  = pSpd->Property->Id;              // index of the property
    ULONG nS  = pSpd->PropertyOutputSize;        // size of data supplied

    ENSURE( nS >= sizeof( ULONG));

	switch (Id) {

		case KSPROPERTY_ALLOCATOR_CONTROL_HONOR_COUNT :

			*( PBOOL ) ( pSpd->PropertyInfo) = 0;
			pSrb->ActualBytesTransferred = sizeof( BOOL); //BOOL
			return STATUS_SUCCESS;
    }
	ERROR(("Unsupported property"));
	return STATUS_INVALID_PARAMETER;
}

/* 
 * Method: BtVBIStream::ChangeNotification
 * Purpose: Called to save off the tv tuner change notification
 * Input: pSrb
 */

/* Note: This is NOT synchronized to the processing of the other streams
 *	and can be called concurrently.   */

void BtVBIStream::ChangeNotification( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtVBIStream::ChangeNotification")); 
	const KSSTREAM_HEADER &DataPacket = *pSrb->CommandData.DataBufferArray;
	RtlCopyMemory( &TvTunerChangeInfo, DataPacket.Data, sizeof( KS_TVTUNER_CHANGE_INFO ) );
	bIsDirty = true; // From here , it will be taken by the main thread of the stream
}

/*
** BtVBIStream::FormatFromRange()
**
**   Returns a DATAFORMAT from a DATARANGE
**
** Arguments:
**
**         IN PHW_STREAM_REQUEST_BLOCK pSrb
**
** Returns:
**
**   TRUE if the format is supported
**   FALSE if the format cannot be suppored
**
** Side Effects:  none
*/
NTSTATUS BtVBIStream::FormatFromRange( IN PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtVBIStream::FormatFromRange"));

	PSTREAM_DATA_INTERSECT_INFO IntersectInfo = pSrb->CommandData.IntersectInfo;
	PKSDATARANGE                DataRange = IntersectInfo->DataRange;
	ULONG                       FormatSize;
	ULONG                       j;
	PKSDATAFORMAT               *pAvailableFormats;

	//
	// Get the pointer to the array of available formats
	//

	pAvailableFormats = Formats;

	//
	// Is the caller trying to get the format, or the size of the format?
	//

	const bool OnlyWantsSize = (IntersectInfo->SizeOfDataFormatBuffer == sizeof( ULONG ) ) ||
							   (IntersectInfo->SizeOfDataFormatBuffer == 0 )
								;

	//
	// Walk the formats supported by the stream searching for a match
	// of the three GUIDs which together define a DATARANGE
	//

	for ( j = 0; j < NUM_VBI_FORMATS; j++, pAvailableFormats++ ) {

		if ( !CompareGUIDsAndFormatSize( DataRange, *pAvailableFormats, TRUE )) {
			continue;
		}

		//	
		// Now that the three GUIDs match, switch on the Specifier
		// to do a further type specific check
		//

		// -------------------------------------------------------------------
		// Specifier FORMAT_VBI for KS_VBIINFOHEADER
		// -------------------------------------------------------------------

		if ( DataRange->Specifier == KSDATAFORMAT_SPECIFIER_VBI ) 
		{
			PKS_DATARANGE_VIDEO_VBI DataRangeVBIToVerify = (PKS_DATARANGE_VIDEO_VBI)DataRange;
			PKS_DATARANGE_VIDEO_VBI DataRangeVBI = (PKS_DATARANGE_VIDEO_VBI)*pAvailableFormats;

			//
			// Check that the other fields match: We must NOT let the caller pick a format unsupported
			// by the current videonorm
         
			ULONG vs = digitizer->GetBtCard().VideoGetCurrentStandard();

#ifdef VBI_ORIGINAL_WAY
			if ( !(
				/*vs */ DataRangeVBIToVerify->ConfigCaps.VideoStandard & DataRangeVBI->ConfigCaps.VideoStandard) )
			{
				DUMP(("BtVBIStream::FormatFromRange(): Wrong VBI VideoStandard: Reqd: %x , CurrentCmp = %x , Cur = %x \n",
					DataRangeVBIToVerify->ConfigCaps.VideoStandard,
					DataRangeVBI->ConfigCaps.VideoStandard ,
					vs ));
				continue;
			}

#else // The way Conexant's drivers work
			if ( (vs & DataRangeVBI->ConfigCaps.VideoStandard) == 0)
			{
				DUMP(("BtVBIStream::FormatFromRange(): Wrong VBI VideoStandard\n" ));
				continue;
			}
#endif

			FormatSize = sizeof( KS_DATAFORMAT_VBIINFOHEADER );

			if ( OnlyWantsSize )
			{
				break;
			}

			// Caller wants the full data format
			if ( IntersectInfo->SizeOfDataFormatBuffer < FormatSize ) 
			{
            
				DUMP(("Buffer too small\n"));
				return STATUS_BUFFER_TOO_SMALL;
			}

			// Copy over the KSDATAFORMAT, followed by the
			// actual VideoInfoHeader
			PKS_DATAFORMAT_VBIINFOHEADER InterVBIHdr =
				(PKS_DATAFORMAT_VBIINFOHEADER)IntersectInfo->DataFormatBuffer;

			RtlCopyMemory( &InterVBIHdr->DataFormat,
				&DataRangeVBIToVerify->DataRange, sizeof( KSDATARANGE ) );

			((PKSDATAFORMAT)IntersectInfo->DataFormatBuffer)->FormatSize = FormatSize;

			RtlCopyMemory( &InterVBIHdr->VBIInfoHeader,
				&DataRangeVBIToVerify->VBIInfoHeader, sizeof( KS_VBIINFOHEADER ) );

			break;
		} 

		// -------------------------------------------------------------------
		// No MATCH!
		// -------------------------------------------------------------------

		else 
		{
			ERROR(("STATUS_NO_MATCH\n" ));
			return STATUS_NO_MATCH;
		}

	} // End of loop on all formats for this stream

	if ( OnlyWantsSize ) {

        // Is the return buffer size = 0 ?
        if(IntersectInfo->SizeOfDataFormatBuffer == 0) {
			DUMP(("Proxy wants to know the required buffer size\n"));
            // the proxy wants to know the actuall buffer size to allocate.
            pSrb->ActualBytesTransferred = FormatSize;
            return STATUS_BUFFER_OVERFLOW;
        }


		DUMP(("only wants size\n" ));
		*(PULONG) IntersectInfo->DataFormatBuffer = FormatSize;
		pSrb->ActualBytesTransferred = sizeof( ULONG );
		return STATUS_SUCCESS;	
	}

	pSrb->ActualBytesTransferred = FormatSize;
	DUMP(("done\n" ));

	return STATUS_SUCCESS;
}

/*
 * Method: BtVBIStream::StreamGetNextBuffer
 * Purpose: Get the next databuffer to use (called back from the videochannel
 *		    class.
 * Notes: Xface to VideoChannel!
 */

/* Note: This can be called async to the others entry points of the stream */

/* Note: this is called at IRQL = DISPATCH_LEVEL */
const VideoDataBuf BtVBIStream::StreamGetNextBuffer()
{
	TRACE(("BtVideoStream::StreamGetNextBuffer"));

    // Only capture in the RUN state
    if (KSState == KSSTATE_RUN /* Atomic READ, does not hurt */
		) {

		// maintain statistics
		IncPictureNumber();

		// Try to get a new Data Buffer
		PHW_STREAM_REQUEST_BLOCK pSrb = NULL;
		BufferLock.Acquire();
		if (!BufferQueue.Remove(pSrb)) {
			pSrb = NULL;
		}
		BufferLock.Release();

		if (pSrb) {

			return VideoDataBuf(this,pSrb,dwStride);
	
		} else
		// If buffer unavailable, mark it as dropped and remember this data discontinuity
		{
			IncDropCount();
		}

	}	

	return VideoDataBuf();
}

/* 
 * Method: BtVBIStream::StreamBufferDone
 * Purpose: Called by the videoChannel Class when the adquisition of a Videoframe
 *			has finished. Notifies the Stream Class this frame is done.
 * Input:  pSrb, Vidfield
 * Output: None
 * Notes:  An stream can be opened in 2 ways, 
 *			a- 1 field per buffer , both fields
 *			b- 1 frame per buffer , (1 or both fields)
 *
 *			Depending on that, we must return to the Streaming class the field we have 
 *			streaming or not
 *
 * Notes2: Xface to VideoChannel!
 */

/* Note: This can be called async respect the others entry points of the stream */

/* Note: this is called at IRQL = DISPATCH_LEVEL */
void BtVBIStream::StreamBufferDone( const VideoDataBuf& pBuf , const VidField vf )
{
	TRACE(("BtVideoStream::StreamBufferDone"));

	// Get the pointer to the Srb
	PHW_STREAM_REQUEST_BLOCK pSrb = pBuf.GetSrb();
	
	PKSSTREAM_HEADER  pDataPacket = pSrb->CommandData.DataBufferArray;

	// Avoid returning data in the Stopped or cancelled State
	if (KSState == KSSTATE_STOP /* atomic READ, does not hurt */
		|| pSrb->Status == STATUS_CANCELLED ) {

		pDataPacket->DataUsed = 0;

	} else {

		pDataPacket->PresentationTime.Numerator = 1;
		pDataPacket->PresentationTime.Denominator = 1;
		pDataPacket->DataUsed = VBIHeader.BufferSize;

		DUMP(("DataUsed = %d\n", pDataPacket->DataUsed));

		// Don't timestamp 
		pDataPacket->OptionsFlags &= ~KSSTREAM_HEADER_OPTIONSF_DURATIONVALID;
		pDataPacket->OptionsFlags &= ~KSSTREAM_HEADER_OPTIONSF_TIMEVALID;
		pDataPacket->PresentationTime.Time = 0;
		pDataPacket->Duration = 0;

		// now gather the statistics
		PKS_VBI_FRAME_INFO pFrameInfo = (PKS_VBI_FRAME_INFO) ( pSrb->CommandData.DataBufferArray + 1 );

		// copy the information to the outbound buffer
		pFrameInfo->ExtendedHeaderSize = sizeof( KS_VBI_FRAME_INFO );
		pFrameInfo->PictureNumber =      FrameInfo.PictureNumber;
		pFrameInfo->DropCount =          FrameInfo.DropCount;
		pFrameInfo->dwSamplingFrequency = VBIHeader.SamplingFrequency;

		// propagate the tv tuner change notification
		if (bIsDirty) { 
	
			bIsDirty = false;
			pFrameInfo->TvTunerChangeInfo = TvTunerChangeInfo;
			pFrameInfo->dwFrameFlags      |= KS_VBI_FLAG_TVTUNER_CHANGE;
			pFrameInfo->VBIInfoHeader     = VBIHeader;
			pFrameInfo->dwFrameFlags      |= KS_VBI_FLAG_VBIINFOHEADER_CHANGE;

		} else {

			pFrameInfo->dwFrameFlags &= ~KS_VBI_FLAG_VBIINFOHEADER_CHANGE;
			pFrameInfo->dwFrameFlags &= ~KS_VBI_FLAG_TVTUNER_CHANGE;
			
		}

		if ( bDataDiscontinuity) {
			bDataDiscontinuity = FALSE; // Reset Flag
			pSrb->CommandData.DataBufferArray->OptionsFlags |= KSSTREAM_HEADER_OPTIONSF_DATADISCONTINUITY;
		} else {
			pSrb->CommandData.DataBufferArray->OptionsFlags &= ~KSSTREAM_HEADER_OPTIONSF_DATADISCONTINUITY;
		}

		// Every frame we generate is a key frame (aka SplicePoint)
		// Delta frames (B or P) should not set this flag

		pDataPacket->OptionsFlags |= KSSTREAM_HEADER_OPTIONSF_SPLICEPOINT;

		// don't forget to report our field type !
   
		pFrameInfo->dwFrameFlags &= ~3;
		pFrameInfo->dwFrameFlags |= ( (vf == VF_Odd) ? KS_VBI_FLAG_FIELD1 : KS_VBI_FLAG_FIELD2 ) ;
	}

	// Completing the request!
	pSrb->Status = STATUS_SUCCESS;
	CompleteDataPacket( pSrb );

}


HW_STREAM_INFORMATION& BtVBIStream::Info( void )
{
	TRACE(("BtVBIStream::Info"));

	return StreamInfo;
}

ULONG BtVBIStream::GetVideoControlCaps( void )
{
	TRACE(("BtVBIStream::GetVideoControlCaps"));
	return 0;
}

ULONG BtVBIStream::GetVideoControlMode( void )
{
	TRACE(("BtVBIStream::GetVideoControlMode"));
	return 0;
}

NTSTATUS BtVBIStream::SetVideoControlMode( ULONG mode )
{
	TRACE(("BtVBIStream::SetVideoControlMode"));
	return STATUS_NOT_SUPPORTED;
}

LONGLONG BtVBIStream::GetActualFramerateInNs( void )
{
	TRACE(("BtVBIStream::GetActualFramerateInNs"));
	// Always return number of fields per second in ns
	return digitizer->GetBtCard().TimeBetweenFields();
}

LONGLONG BtVBIStream::GetMaxAvailableFramerateInNs( void )
{
	TRACE(("BtVBIStream::GetMaxAvailableFramerateInNs"));
	// Always return number of fields per second in ns
	return digitizer->GetBtCard().TimeBetweenFields();
}

NTSTATUS BtVBIStream::GetAnalogWindow( RECT& rc )
{
	TRACE(( "BtVBIStream::GetAnalogWindow" ));

	if (vc) {
		rc = vc->GetAnalogWindow();
	} else {
		rc.top =	VBIHeader.ActualLineStartTime;
		rc.left =	VBIHeader.StartLine;
		rc.right =  VBIHeader.ActualLineEndTime;
		rc.bottom = VBIHeader.EndLine;
	}

	return STATUS_SUCCESS;
}

NTSTATUS BtVBIStream::SetAnalogWindow( const RECT& rc )
{
	TRACE(( "BtVBIStream::SetAnalogWindow" ));
	return STATUS_NOT_SUPPORTED;
}

NTSTATUS BtVBIStream::GetDigitalWindow( RECT& rc )
{
	TRACE(( "BtVBIStream::GetDigitalWindow" ));
	
	if (vc) {
		rc = vc->GetDigitalWindow();
	} else {
		rc.top =	VBIHeader.ActualLineStartTime;
		rc.left =	VBIHeader.StartLine;
		rc.right =  VBIHeader.ActualLineEndTime;
		rc.bottom = VBIHeader.EndLine;
	}

	return STATUS_SUCCESS;
}
