/*
	BtVBIStream.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef BTVBISTREAM_HPP
#define BTVBISTREAM_HPP

#include "BtDigitizerStream.hpp"

// This class represents the VBI stream
class BtVBIStream : 
	public BtDigitizerStream,
	public VideoChanIface	// And we are also a DataBuffer provider and consumer
{
	public:

		static KS_DATARANGE_VIDEO_VBI FormatNTSCVBI;
		static KS_DATARANGE_VIDEO_VBI FormatPALVBI;
		static PKSDATAFORMAT Formats[];
		static HW_STREAM_INFORMATION	StreamInfo;
		static HW_STREAM_OBJECT			StreamObject;
		static const KSPROPERTY_ITEM Connection[];
		static const KSPROPERTY_ITEM DroppedFrames[];
		static const KSPROPERTY_ITEM Allocator[];
		static KSPROPERTY_SET Properties[];

	protected:

		// Formato utilizado actualmente por el Stream
		KS_VBIINFOHEADER			VBIHeader;

		// Frame info (Stats)
		KS_VBI_FRAME_INFO			FrameInfo;	// Frame Info

		// To notify next filter of a Channel Change
		KS_TVTUNER_CHANGE_INFO		TvTunerChangeInfo;
		BOOL						bIsDirty;			// A notification has arrived

	public:

		BtVBIStream(BtDigitizer *dev);
		virtual ~BtVBIStream();

		// Reset Statistic counters
		virtual void ResetCounters();
		virtual void IncPictureNumber();
		virtual void IncDropCount();

		// Events on channels
		virtual NTSTATUS Create() ;
		virtual NTSTATUS Destroy() ;

	public:
		virtual NTSTATUS Open( IN PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS FormatFromRange( IN PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual HW_STREAM_INFORMATION& Info( void ); 
		virtual ULONG GetVideoControlCaps( void );
		virtual ULONG GetVideoControlMode( void );
		virtual NTSTATUS SetVideoControlMode( ULONG mode );
		virtual LONGLONG GetActualFramerateInNs( void );
		virtual LONGLONG GetMaxAvailableFramerateInNs( void );
		virtual NTSTATUS GetAnalogWindow( RECT& rc );
		virtual NTSTATUS SetAnalogWindow( const RECT& rc );
		virtual NTSTATUS GetDigitalWindow( RECT& rc );


	protected:
		// Called when user program supplies a buffer to fill in		
		virtual NTSTATUS SetFormat( PHW_STREAM_REQUEST_BLOCK pSrb ) ;

		// Functions
		virtual NTSTATUS GetConnectionProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) ;
		virtual NTSTATUS GetAllocatorProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) ;
		virtual NTSTATUS GetDroppedFramesProperty( PHW_STREAM_REQUEST_BLOCK pSrb ); // Here!

		virtual BOOL VerifyFormat( PKSDATAFORMAT pKSDataFormatToVerify ); // Here!

	public:
		void ChangeNotification( PHW_STREAM_REQUEST_BLOCK pSrb ); // VBI needs to track Channel Change info

	public:
		// VideoChanIfaca : Xface to VideoChannel
		virtual const VideoDataBuf StreamGetNextBuffer();
		virtual void StreamBufferDone( const VideoDataBuf& pBuf , const VidField vf ); 

};

#endif

