/*
	BtCard.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000-2002 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _BTCARD_HPP_
#define _BTCARD_HPP_

#include "custprop.h"
#include "Debug.hpp"
#include "WDMHeaders.hpp"
#include "CppRt.hpp"
#include "MyTypes.hpp"
#include "HwTuner\HwTuner.hpp"
#include "HwAudio\HwAudio.hpp"
#include "HwVideo\HwVideo.hpp"
#include "HwIR\HwIR.hpp"
#include "HwCapture\HwCapture.hpp"

// TunerChips
#include "HwTuner\I2CTuner.hpp"
#include "HwTuner\TEA5757.hpp"
#include "HwTuner\MT2032.hpp"
#include "HwTuner\MT2050.hpp"

// AudioChips
#include "HwAudio\PV951.hpp"
#include "HwAudio\AvermediaTvPhone.hpp"
#include "HwAudio\AvermediaTvPhoneOld.hpp"
#include "HwAudio\GvbcTv3.hpp"
#include "HwAudio\Lt9415.hpp"
#include "HwAudio\MSP3400.hpp"
#include "HwAudio\TDA7432.hpp"
#include "HwAudio\TDA8425.hpp"
#include "HwAudio\TDA9840.hpp"
#include "HwAudio\TDA9850.hpp"
#include "HwAudio\TDA9855.hpp"
#include "HwAudio\TDA9873.hpp"
#include "HwAudio\TDA9874.hpp"
#include "HwAudio\TDA9875.hpp"
#include "HwAudio\TEA6300.hpp"
#include "HwAudio\TEA6320.hpp"
#include "HwAudio\TEA6420.hpp"
#include "HwAudio\TerraTV.hpp"
#include "HwAudio\PT2254.hpp"
#include "HwAudio\WinFast2000.hpp"
#include "HwAudio\pvbt878p9b.hpp"
#include "HwAudio\Tekram205.hpp"
#include "HwAudio\DPL3518.hpp"
#include "HwAudio\Hoontech.hpp"
#include "HwAudio\WINDVR.hpp"

// VideoChips
#include "HwVideo\TDA8540.hpp"
#include "HwVideo\RSBT.hpp"
#include "HwVideo\SILK200.hpp"
#include "HwVideo\PCF8575A.hpp"
#include "HwVideo\GRANDTEC16.hpp"
#include "HwVideo\TRUST814PCI.hpp"
#include "HwVideo\SAA7118.hpp"

// IR chips
#include "HwIR\GPIOIR.hpp"
#include "HwIR\GPIO_INTIR.hpp"
#include "HwIR\HauppaugeIR.hpp"
#include "HwIR\PV951IR.hpp"
#include "HwIR\TEKRAMIR.hpp"
#include "HwIR\PixelViewIR.hpp"
#include "HwIR\KNC1IR.hpp"

// IF Dem chips
#include "HwIFDem\TDA9887.hpp"
#include "HwIFDem\TDA988xPinnacle.hpp"
#include "HwIFDem\VoodooTV.hpp"


#define MAX_VIDEOINS 5
#define NONE		0xFF

// Stores differences between cards
class BtCard {

	public:

		typedef enum {
			No_Digital = 0,			// No digital xface
			CCIR656_Digital,		// Generic CCIR656 (allows NTSC and PAL)
			SVI_Digital,			// Silicon Vision Digital camera
			LVM_Digital,			// Logitech Videoman Digital camera
			RQS_Digital				// Rockwell QuartzSight Digital camera
		} DigitalXface;

		typedef enum {
			PLL_NONE = 0,
			PLL_28   ,
			PLL_35   ,
			PLL_14 /*amazing, isn't it ?*/	} PLLType;

		typedef struct {
			/* char *name; */

			// Tuners (4)
			// SVideo (4)
			// Composite (32)
			// Serial Digital (4)
			// Parallel Digital (4)
			// RGB (4)
			// Component Video (4)

			BYTE analogvideoins;
			BYTE audioins;
			BYTE digitalvideoins;

			DigitalXface digxface;

			BYTE  tuner0;
			BYTE  svhs0;
			DWORD gpoe;		// Mask of bits that must be outputs to control audio sources
			BYTE videomux[MAX_VIDEOINS];
			DWORD audiomux[5];  // AudioConTuner ,	AudioConRadio, 	AudioConLineIn, AudioConLineOut ,AudioConMute 

			/* look for these i2c audio chips */
			int msp34xx:1;
			int tvaudio:1;

			/* if card has radio */
			int has_radio:1;

			// other settings
			PLLType pll;

			// Tuner type, if known
			HwTuner::Tuner tuner_type;
		} CInfo;

		// Info on every card we support
		static CInfo Info[];

		typedef enum {
			UNKNOWN				= 0x00, // -> This one forces autodetect (The others NO!)
			MIRO				= 0x01,
			HAUPPAUGE			= 0x02,
			STB					= 0x03,
			INTEL				= 0x04,
			DIAMOND				= 0x05,
			AVERMEDIA			= 0x06, 
			MATRIX_VISION		= 0x07,
			FLYVIDEO			= 0x08,
			TURBOTV				= 0x09,
			HAUPPAUGE878		= 0x0a,
			MIROPRO				= 0x0b,
			ADSTECH_TV			= 0x0c,
			AVERMEDIA98			= 0x0d,
			VHX					= 0x0e,
			ZOLTRIX				= 0x0f,
			PIXVIEWPLAYTV		= 0x10,
			WINVIEW_601			= 0x11,
			AVEC_INTERCAP		= 0x12,
			LIFE_FLYKIT			= 0x13,
			CEI_RAFFLES			= 0x14,
			CONFERENCETV		= 0x15,
			PHOEBE_TVMAS		= 0x16,
			MODTEC_205			= 0x17,
			MAGICTVIEW061		= 0x18,
			VOBIS_BOOSTAR		= 0x19,
			HAUPPAUG_WCAM		= 0x1a,
			MAXI				= 0x1b,
			TERRATV				= 0x1c,
			PXC200				= 0x1d,
			FLYVIDEO_98			= 0x1e,
			IPROTV				= 0x1f,
			INTEL_C_S_PCI		= 0x20,
			TERRATVALUE			= 0x21,
			WINFAST2000			= 0x22,
			CHRONOS_VS2			= 0x23,
			TYPHOON_TVIEW		= 0x24,
			PXELVWPLTVPRO		= 0x25,
			MAGICTVIEW063		= 0x26,
			PINNACLE			= 0x27,
			STB2				= 0x28,
			AVPHONE98			= 0x29,
			PV951				= 0x2a,
			ONAIR_TV			= 0x2b,
			SIGMA_TVII_FM		= 0x2c,
			MATRIX_VISION2		= 0x2d,
			ZOLTRIX_GENIE		= 0x2e,
			TERRATVRADIO		= 0x2f,
			DYNALINK			= 0x30,
			GVBCTV3PCI			= 0x31,
			PXELVWPLTVPAK		= 0x32,
			EAGLE				= 0x33,
			PINNACLEPRO			= 0x34,
			TVIEW_RDS_FM		= 0x35,
			LIFETEC_9415		= 0x36,
			BESTBUY_EASYTV		= 0x37,
			FLYVIDEO_98FM		= 0x38,
			GRANDTEC			= 0x39,
			PHOEBETVMO			= 0x3a,
			TVCAPTURER			= 0x3b,
			MM100PCTV			= 0x3c,
			GMV1				= 0x3d,
			BESTBUY_EASYTV2		= 0x3e,
			ATI_TVWONDER		= 0x3f,
			ATI_TVWONDERVE		= 0x40,
			FLYVIDEO2000		= 0x41,
			TERRATVALUER		= 0x42,
			GVBCTV4PCI			= 0x43,
			VOODOOTV_FM			= 0x44,
			AIMMS				= 0x45,
			PV_BT878P_PLUS		= 0x46,
			FLYVIDEO98EZ		= 0x47,
			PV_BT878P_9B		= 0x48,
			SENSORAY311			= 0x49,
			CONEXANTNTSCXEVK	= 0x4a,
			ROCKWELLNTSCXEVK	= 0x4b,
			CONEXANTFOGHORNREVA = 0x4c,
			CONEXANTFOGHORNREVB = 0x4d,
			CONEXANTFOGHORNREVC = 0x4e,
			RS_BT				= 0x4f,
			TEKRAMM205			= 0x50,
			BT848AWITH14XTAL	= 0x51,
			BT8XXWITH28XTAL		= 0x52,
			FLYVIDEO2000S		= 0x53,
			HOONTECH            = 0x54,
			KWORLD878			= 0x55,
			GALLANTIV550		= 0x56,
			MACHTV878			= 0x57,
			SKYEYE878			= 0x58,
			JETWAY				= 0x59,
			BOEDER				= 0x5A,
			SPIRIT				= 0x5B,
			AVERTVOLD			= 0x5C,
			SDISILK100_S		= 0x5D,
			SDISILK100_C		= 0x5E,
			SASEM_S				= 0x5F,
			TYPHOONV2			= 0x60,
			VC_3CAM				= 0x61,
			CUCUS				= 0x62,
			HSPEED				= 0x63,
			SKYWELL				= 0x64,
			SDISILK200_S		= 0x65,
			SDISILK200_C		= 0x66,
			WINDVR				= 0x67,
			PRIMAVISION			= 0x68,
			GRANDTEC16			= 0x69,
			TRUST814PCI			= 0x6A,
			SWEETSPOTPDI		= 0x6B,
			VHXDELUXE			= 0x6C,
			CUSTOM				= 0x6D	// Should be last!
		} Model;

		typedef struct {
			unsigned int id;
			Model cardnr;
			/*char *name;*/
		} CARD;
		
		// Mapping from E2PROM Id to card
		static CARD Card[];

		// Mapping from MIRO tuner ID to our Tuner ID
		static HwTuner::Tuner miro_tunermap[];

		// Array to know if MIRO Card supports Radio tuner based on TEA5757
		static bool miro_fmtuner[];

		// Mapping from HAUPPAUGE tuner ID to our Tuner ID
		typedef struct {
			HwTuner::Tuner  id;
			/*char *name; */
		} hauppauge_tunerentry;

		static hauppauge_tunerentry hauppauge_tuner[];
	
		// Avermedia Mappings
		static HwTuner::Tuner avermedia_tuner_0_table[];
		static HwTuner::Tuner avermedia_tuner_1_table[];

	protected:

		// Timing generator tables required for Digital Input
		static BYTE			SRAMTable[][ 60 ];

		// Capture Hardware detected
		BtPisces *			Pisces;

		// Tuners Detected
		DWORD				TunerCount;
		HwTuner *			TunerChip[16];

		// Audio Chips detected
		DWORD				AudioCount;
		HwAudio *			AudioChip[16];

		// Video Chips detected
		DWORD				VideoCount;
		HwVideo *			VideoChip[16];

		// IR Chips detected
		DWORD				IRCount;
		HwIR *				IRChip[16];

		// IFDem Chips detected
		DWORD				IFDemCount;
		HwIFDem *			IFDemChip[4];

		// The audio source selection Stuff
		AudioConnector		TunerConnector;
		AudioConnector		AudioMux;
		BOOL				bAudioMuted;

		// The video source selection Stuff
		VideoConnector		VideoMux;

		// Model of the card
		Model				dwCard;

		// Model of the chipset
		DWORD				dwDeviceId;
		DWORD				dwVendorId;
		DWORD				dwRevision;
		DWORD				dwSubsystemId;
		DWORD				dwSubsystemVendorId;

		// The crystal frequencies autodetected on this board
		ULONG				m_xtal0freq;
		ULONG				m_xtal1freq;

		// Pointer to the device object
		const PDEVICE_OBJECT PDO;

		// Hardware Device extension
		const PVOID			HwDeviceExtension;

		// Base address of the Card
		const DWORD_PTR			dwBaseAddress;

		// Current AudioMode
		ULONG				m_AudioMode;

		// Current Video Decoder Standard
		KS_AnalogVideoStandard   m_ksCurrentVideoStandard;

		// Current video input mode
		bool				m_IsDigitalVideo;

		// Current Tuner mode
		HwTuner::TunerMode	m_CurrentTunerMode;

		// Current Video Tuner Standard
		KS_AnalogVideoStandard m_ksCurrentTunerStandard;

		// Current Tuner Frecuency
		ULONG				m_CurrentTunerFrequency;

		// Last tuning method
		HwTuner::TuningMethod m_TunerLastMethod;
		KS_TUNER_TUNING_FLAGS m_TunerCurrMethod;

		// TRUE if last tuning operation was successful
		BOOL				m_TunerSignalDetected;

		// If we are using the TV audio circuit to get FM audio
		bool				m_bTVAudioUsedForFM;

		// The audio settings!
		int					m_VolumeLeft;
		int					m_VolumeRight;
		int					m_Loudness;
		int					m_Bass;
		int					m_Treble;
		LONG				m_EqBands[MAX_EQUALIZER_BANDS];
		DOLBY_PARAMS		m_DolbyParams;

		// Count of current open video channels
		int					m_VCOpened;

		// If everything Ok
		bool				m_InitedOK;

		// If Triton detected
		BOOL				Triton848;

		// If inside ISR
		LONG				ISRLock;

	public:

		// Functions to ID card and all its componentx
		BOOL		IsBt848( void ) const { return ( ( dwDeviceId == 0x0350 ) &&    // AND
										( ( dwRevision == 0x00 ) ||  // 0x00 or
										( dwRevision & 0x01 ) ) ); } // ends with 0x01
		BOOL		IsBt848A( void ) const {	return ( ( dwDeviceId == 0x0350 ) && 
										  ( dwRevision & 0x02 ) ); }   // ends with 0x2
		BOOL		IsBt849A( void ) const { return ( dwDeviceId == 0x0351 ); }
		BOOL		IsBt848Family( void ) const { return ( IsBt848() || IsBt848A() || IsBt849A() ); }
		BOOL		IsBt878( void ) const { return ( dwDeviceId == 0x036E ); }
		BOOL		IsBt879( void ) const { return ( dwDeviceId == 0x036F ); }
		BOOL		IsBt880( void ) const { return ( dwDeviceId == 0x0370 ); }
		BOOL		IsGeminiFamily( void ) const { return ( IsBt878() || IsBt879() || IsBt880() ); }
		BOOL		IsGeminiFamilyRevD( void ) const { return ( IsGeminiFamily() && dwRevision >= 0x02 ); }
		BOOL		IsBt878Family( void ) const { return ( IsBt878() || IsBt879() || IsBt880() ); }
		BOOL		HasPLL( void ) const { return ( IsBt848A() || IsBt849A() || IsGeminiFamily() ); }
		BOOL		Has28Xtal( void ) const { return (m_xtal0freq == 28636363) || (m_xtal1freq == 28636363); }
		BOOL		Has35Xtal( void ) const { return (m_xtal0freq == 35468950) || (m_xtal1freq == 35468950); }
		BOOL		HasPAL_NC( void ) const { return ( IsBt848A() || IsBt849A() || IsGeminiFamily() ); }
		BOOL		HasDigitalIn( void ) const { return ( IsBt848A() || IsBt849A() || IsGeminiFamily() ); }
		BOOL		HasFCNTR( void ) const { return ( IsBt848A() || IsBt849A() || IsGeminiFamily() ); }
		BOOL		HasPEAK( void ) const { return ( IsBt848A() || IsBt849A() || IsGeminiFamily() ); }
	protected:
		CInfo *		GetCardInfo( void ) const { return Info + dwCard ; }
	public:
		BOOL		IsTriton848( void ) const { return Triton848; }

		// Card topology info
		DWORD		AudioInputs( void ) const { return Info[dwCard].audioins; }
		bool		HasTunerInput( void ) const;
		int			SVideoInputCount( void ) const;
		int			CompositeInputCount( void ) const;
		int			SerialDigitalInputCount( void ) const;
		int			ParallelDigitalInputCount( void ) const;
		int			RGBInputCount( void ) const;
		int			ComponentVideoInputCount( void ) const;

		// Decoder functions
		bool		IsHLocked()			{ return GetBtPisces().IsDeviceInHLock(); }
		bool		Is525LinesVideo()	{ return GetBtPisces().Is525LinesVideo(); }
		REFERENCE_TIME TimeBetweenFields() { return GetBtPisces().TimeBetweenFields(); }
		ULONG		FieldsPerSecondX100()	{ return GetBtPisces().FieldsPerSecondX100(); }

		// decoder 'set' group
		NTSTATUS	SetGamma( DWORD value )			{ return (GetBtPisces().SetGamma(value) == Success ) ? STATUS_SUCCESS : STATUS_INVALID_PARAMETER; }
		NTSTATUS	SetBrightness( DWORD value )	{ return (GetBtPisces().SetBrightness( value ) == Success ) ? STATUS_SUCCESS : STATUS_INVALID_PARAMETER; }
		NTSTATUS	SetSaturation( DWORD value )	{ return (GetBtPisces().SetSaturation( value ) == Success ) ? STATUS_SUCCESS : STATUS_INVALID_PARAMETER; }
		NTSTATUS	SetContrast( DWORD value )		{ return (GetBtPisces().SetContrast( value ) == Success ) ? STATUS_SUCCESS : STATUS_INVALID_PARAMETER; }
		NTSTATUS	SetHue( DWORD value )			{ return (GetBtPisces().SetHue( value ) == Success ) ? STATUS_SUCCESS : STATUS_INVALID_PARAMETER; }
		NTSTATUS	SetWhiteBalance( DWORD Data )	{ return (GetBtPisces().SetWhiteBalance( Data ) == Success ) ? STATUS_SUCCESS : STATUS_INVALID_PARAMETER; }
		NTSTATUS	SetBacklightCompensation( DWORD Data ) { return (GetBtPisces().SetBacklightCompensation( Data ) == Success ) ? STATUS_SUCCESS : STATUS_INVALID_PARAMETER; }
		NTSTATUS	SetColorEnable( BOOL Data )		{ return (GetBtPisces().SetColorEnable( Data != 0 ) == Success ) ? STATUS_SUCCESS : STATUS_INVALID_PARAMETER; }
		NTSTATUS	SetSharpness( DWORD Data )		{ return (GetBtPisces().SetSharpness( Data ) == Success ) ? STATUS_SUCCESS : STATUS_INVALID_PARAMETER; }
		NTSTATUS	SetAutoSaturation(BOOL Auto)	{ return (GetBtPisces().SetAutoSaturation( Auto != 0) == Success ) ? STATUS_SUCCESS : STATUS_INVALID_PARAMETER; }
		NTSTATUS	SetAutoGamma(BOOL Auto)			{ return (GetBtPisces().SetAutoGamma( Auto != 0 ) == Success ) ? STATUS_SUCCESS : STATUS_INVALID_PARAMETER; }
		NTSTATUS	SetAutoSharpness(BOOL Auto)		{ return (GetBtPisces().SetAutoSharpness( Auto != 0 ) == Success ) ? STATUS_SUCCESS : STATUS_INVALID_PARAMETER; }

		// decoder 'get' group
		DWORD		GetGamma()			{ return GetBtPisces().GetGamma(); }
		DWORD		GetBrightness()		{ return GetBtPisces().GetBrightness(); }
		DWORD		GetSaturation()		{ return GetBtPisces().GetSaturation(); }
		DWORD		GetContrast  ()		{ return GetBtPisces().GetContrast(); }
		DWORD		GetHue       ()		{ return GetBtPisces().GetHue(); }
		DWORD		GetWhiteBalance()	{ return GetBtPisces().GetWhiteBalance(); }
		DWORD		GetBacklightCompensation() { return GetBtPisces().GetBacklightCompensation(); }
		DWORD		GetColorEnable()	{ return GetBtPisces().GetColorEnable(); }
		DWORD		GetSharpness()		{ return GetBtPisces().GetSharpness(); }
		BOOL		GetAutoSaturation() { return GetBtPisces().GetAutoSaturation();  }
		BOOL		GetAutoGamma()		{ return GetBtPisces().GetAutoGamma(); }
		BOOL		GetAutoSharpness()	{ return GetBtPisces().GetAutoSharpness();  }

		// GPIO Functions
		void		SetGPCLKMODE( State s) { GetBtPisces().SetGPCLKMODE(s); }
		int			GetGPCLKMODE( void ) { return GetBtPisces().GetGPCLKMODE(); }
		void		SetGPIOMODE( GPIO::GPIOMode m) { GetBtPisces().SetGPIOMODE(m); }
		int			GetGPIOMODE( void ) { return GetBtPisces().GetGPIOMODE(); }
		void		SetGPWEC( State s) { GetBtPisces().SetGPWEC(s); }
		int			GetGPWEC( void ) { return GetBtPisces().GetGPWEC(); }
		void		SetGPINTI( State s) { GetBtPisces().SetGPINTI(s); }
		int			GetGPINTI( void ) { return GetBtPisces().GetGPINTI(); }
		void		SetGPINTC( State s) { GetBtPisces().SetGPINTC(s); }
		int			GetGPINTC( void ) { return GetBtPisces().GetGPINTC(); }
		bool		SetGPOE( int x, State s) { return GetBtPisces().SetGPOE(x,s); }
		void		SetGPOE( DWORD d) { GetBtPisces().SetGPOE(d); }
		int			GetGPOE( int i) { return GetBtPisces().GetGPOE(i); }
		DWORD		GetGPOE( void ) { return GetBtPisces().GetGPOE(); }
		bool		SetGPIE( int x, State s) { return GetBtPisces().SetGPIE(x,s); }
		bool		SetGPIE( DWORD d) { return GetBtPisces().SetGPIE(d); }
		int			GetGPIE( int x) { return GetBtPisces().GetGPIE(x); }
		DWORD		GetGPIE( void ) { return GetBtPisces().GetGPIE(); }
		bool		SetGPDATA( DWORD * d, int x , int offset = 0 ) { return GetBtPisces().SetGPDATA(d,x,offset); }
		bool		GetGPDATA( DWORD * d, int x , int offset = 0 ) { return GetBtPisces().GetGPDATA(d,x,offset); }
		bool		SetGPDATA( int x, int y, DWORD z, int offset = 0 ) { return GetBtPisces().SetGPDATA(x,y,z,offset); }
		bool		GetGPDATA( int x, int y, DWORD *z, int offset = 0 ) { return GetBtPisces().GetGPDATA(x,y,z,offset); }

		// I2C Functions
		bool		I2CSetFreq(const DWORD freq ) { return GetBtPisces().I2CSetFreq(freq); }
		DWORD		I2CGetFreq() { return GetBtPisces().I2CGetFreq(); }	
		bool		I2CRead(const BYTE address, BYTE *value ) { return GetBtPisces().I2CRead( address, value ); }
		bool		I2CWrite(const BYTE address,const BYTE value1 ) { return GetBtPisces().I2CWrite( address, value1 ); }
		bool		I2CWrite(const BYTE address,const BYTE value1,const BYTE value2 ) { return GetBtPisces().I2CWrite( address, value1, value2 ); }
		bool		I2CWR(const BYTE address,const BYTE* pOut,const UINT OutLen , BYTE* pIn,const UINT InLen,const bool DontWaitACK = false ) { return GetBtPisces().I2CWR(address,pOut,OutLen,pIn,InLen,DontWaitACK); }
		int			I2CGetLastError( void ) { return GetBtPisces().I2CGetLastError(); }

#ifdef __FULLDECODERACCESS__ // NOT RECOMMENDED AT ALL
		// Misc functions... Should NEVER be used
		DWORD		ReadDecoder(const DWORD dwAddress);
		void		WriteDecoder(const DWORD dwAddress,const DWORD dwValue);
#endif

		// Audio functions
		void		AudioMute( void );
		void		AudioUnmute( void );
		bool		AudioSetInput( AudioConnector audioConnector );

		ULONG		AudioGetCaps();				// Capabilities of Hardware
		ULONG		AudioGetSupportedModes();	// All the modes that can be selected right now
		ULONG		AudioGetCurrentMode() { return m_AudioMode; }
		NTSTATUS	AudioSetMode(ULONG New);

		void		AudioSetVolume(int Left,int Right);
		void		AudioSetLoudness(int Loudness);
		void		AudioSetBassTreble(int Bass,int Treble);
		int			AudioGetLeftVolume() const { return m_VolumeLeft; }
		int			AudioGetRightVolume() const { return m_VolumeRight; }
		int			AudioGetLoudness() const { return m_Loudness; }
		int			AudioGetBass() const { return m_Bass; }
		int			AudioGetTreble() const { return m_Treble; }
		void		AudioSetEqualizer(LONG* bands);
		void		AudioGetEqualizer(LONG* bands);
		void		AudioSetDolby(DOLBY_PARAMS& params);
		void		AudioGetDolby(DOLBY_PARAMS& params);
		void		AudioControlCaps(DWORD& Caps,DWORD& Bands, DWORD* BandFreq);

		// Video Functions
		void		VideoSetInput( VideoConnector conn );
		LONG		VideoGetSupportedStandards();
		KS_AnalogVideoStandard VideoGetCurrentStandard() { return m_ksCurrentVideoStandard; }
		NTSTATUS	VideoSetCurrentStandard( KS_AnalogVideoStandard ksStandard );
		NTSTATUS	VideoSetVideoKind( bool IsDigital );

		// Tuner Functions
		LONG		TunerGetSupportedModes();
		LONG		TunerGetSupportedStandards();
		KS_AnalogVideoStandard TunerGetCurrentStandard() { return m_ksCurrentTunerStandard; }

		NTSTATUS	TunerSetCurrentStandard( KS_AnalogVideoStandard ksStandard );
		NTSTATUS	TunerSetMode( ULONG Mode );
		NTSTATUS	TunerSetFrequency(DWORD Frecuency, KS_TUNER_TUNING_FLAGS tf );			// Frecuency of channel
		ULONG		TunerGetFrequency() { return m_CurrentTunerFrequency; }
		KS_TUNER_TUNING_FLAGS TunerGetMethod() { return m_TunerCurrMethod; }
		BOOL		TunerWasSignalDetected() { return m_TunerSignalDetected; }
		ULONG		TunerGetMode();
		NTSTATUS	TunerGetModeCaps( PKSPROPERTY_TUNER_MODE_CAPS_S pCaps);

		// IR functions
		bool		IRAvailable() const { return IRCount > 0; }
		DWORD		IRGetCode();

		// Power management
		NTSTATUS	PowerDown();
		NTSTATUS	PowerUp();
		void		ReinitIfIdle();

		// VideoChannel Creation / Destruction
		VideoChannel* OpenVideoChannel(
			const VCMode mode,			// Mode of opening
			VideoChanIface * xface,		// Data buffer provider - consumer
			const MRect& analogwin,		// AnalogWindow
			const MRect& digitalwin,	// DigitalWindow
			const ColFmt ColorFormat	// Color format
			);

		void		CloseVideoChannel( VideoChannel * chan );

		// Misc info
		void		VideoGetMaxFrame(RECT& r) { GetBtPisces().GetMaxFrame(r); }

		// Must be called when a Hardware Int occurs!
		BOOLEAN		HwInterrupt();

		// Interfase exports for those who need it!
		I2C&		GetI2C()	{ return GetBtPisces().GetI2C(); }
		GPIO&		GetGPIO()	{ return GetBtPisces().GetGPIO(); }
		BtPisces&	GetBtPisces() { return *Pisces; } 

		// Some misc fns
		bool		IsDigitalSignal() const { return m_IsDigitalVideo; }

		// Constructors
		BtCard(
			const PDEVICE_OBJECT pPDO,
			const PVOID HwDeviceExtension_,
			const DWORD_PTR dwBase );

		bool Init();
		void Uninit();
		
		~BtCard();

	private:
		void SetupLatencyTimers(void);
		void SetChipsetCompatibility(bool s);
		void InitCard();

	protected:
		void AcquireISR( void );
		void ReleaseISR( void );

};

#endif

