/*
	Enums.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000-2002 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _ENUMS_HPP_
#define _ENUMS_HPP_


/* Type: VideoConnector
 * Purpose: Defines a video source
 */
typedef enum { 
	// Tuner (only ONE)
	VideoConTuner = 0, 
	// SVideo
	VideoConSVideo0, 
	VideoConSVideo1, 
	VideoConSVideo2, 
	VideoConSVideo3, 
	// Composite
	VideoConComposite0,
	VideoConComposite1,
	VideoConComposite2,
	VideoConComposite3,
	VideoConComposite4,
	VideoConComposite5,
	VideoConComposite6,
	VideoConComposite7,
	VideoConComposite8,
	VideoConComposite9,
	VideoConComposite10,
	VideoConComposite11,
	VideoConComposite12,
	VideoConComposite13,
	VideoConComposite14,
	VideoConComposite15,
	VideoConComposite16,
	VideoConComposite17,
	VideoConComposite18,
	VideoConComposite19,
	VideoConComposite20,
	VideoConComposite21,
	VideoConComposite22,
	VideoConComposite23,
	VideoConComposite24,
	VideoConComposite25,
	VideoConComposite26,
	VideoConComposite27,
	VideoConComposite28,
	VideoConComposite29,
	VideoConComposite30,
	VideoConComposite31, // (Some cards HAVE 31 composites!)
	// Serial Digital
	VideoConSDI0,
	VideoConSDI1,
	VideoConSDI2,
	VideoConSDI3,
	// Parallel Digital
	VideoConPDI0,
	VideoConPDI1,
	VideoConPDI2,
	VideoConPDI3,
	// RGB
	VideoConRGB0,
	VideoConRGB1,
	VideoConRGB2,
	VideoConRGB3,
	// Component Video
	VideoConComponent0,
	VideoConComponent1,
	VideoConComponent2,
	VideoConComponent3
} VideoConnector;

/* Type: AudioConnector
 * Purpose: Defines an audio source
 */
typedef enum { 
	AudioConTuner = 0, 
	AudioConRadio,
	AudioConLineIn,
	AudioConLineOut,
	AudioConMute/* off */
} AudioConnector;

/* Type: Audiomode
 * Purpose: Defines if audio will be in Mono or Stereo or some other (mono) language
 * Note this can be used as a bitfield... To report currently available modes.
 */
typedef enum {
	Am_Mono		= 0x0001,
	Am_Stereo	= 0x0002,
	Am_Lang1	= 0x0004,
	Am_Lang2	= 0x0008
} AudioMode;

typedef unsigned int AudioModes; // Just a bitfield of AudioMode

/* Dolby Processor modes */
typedef enum {
	DPL_Prologic = 0,	/* Dolby prologic mode */
	DPL_Noise,			/* Noise mode */
	DPL_Through			/* Pass through mode */
} DPLMode;

/* dolby prologic modes */
typedef enum {
	DPLMPl_Normal = 0,	/* 0 = Normal */
	DPLMPl_Phantom,		/* 1 = Phantom */
	DPLMPl_Wide,		/* 2 = Wide */
	DPLMPl_Dolby3Stereo,/* 3 = Dolby 3 Stereo */
	DPLMPl_CenterOff,	/* 4 = Center Off */
	DPLMPl_Panorama,	/* 5 = Panorama */
	DPLMPl_3DPanorama	/* 6 = 3D-Panorama (virtual dolby) */
} DPLPlMode;

#endif