/*
	DPL3518.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "DPL3518.hpp"
#include "..\HwCapture\Pisces.hpp"


/* ----------------------------------------------------------------------- */
/* functions for talking to the DPL3518 Sound processor                   */

bool DPL3518::Reset()
{
	static BYTE reset_off[3] = {0x00, 0x80, 0x00 };
	static BYTE reset_on[3]  = {0x00, 0x00, 0x00 };

	i2c.WR(i2caddr,reset_off,3,NULL,0,true);  /* XXX ignore errors here */
	if (!i2c.WR(i2caddr,reset_on, 3,NULL,0)) {
		ERROR(("DPL3518: chip Reset failed\n"));
		return false;
	}
    return true;
}

// Read address of the DPL3518
// dev = subaddr
WORD DPL3518::Read(BYTE subaddr, WORD addr)
{
	BYTE writebf[3];
	BYTE readbf[2];

	writebf[0] = subaddr + 1;
	writebf[1] = addr >> 8;
	writebf[2] = addr & 0xff;

	int err;
	for (err = 0; err < 3;) {
		if (i2c.WR(i2caddr,writebf,3,readbf,2)) 
			break;
		
		ERROR(("msp3DPL35184xx: I/O error #%d (Read 0x%02x/0x%02x)\n", err, subaddr, addr));
		err ++;
	}
	if (3 == err) {
		ERROR(("DPL3518: giving up, reseting chip. Sound will go off,\n"));
		Reset();
		return 0;
	}
	return ( WORD(readbf[0]) << 8) | readbf[1];
}

// Write to address of the msp
bool DPL3518::Write(BYTE subaddr, WORD addr, WORD val)
{
	BYTE buffer[5];

	buffer[0] = subaddr;
	buffer[1] = addr >> 8;
	buffer[2] = addr &  0xff;
	buffer[3] = val  >> 8;
	buffer[4] = val  &  0xff;

	int err;
	for (err = 0; err < 3;) {
		if (i2c.WR(i2caddr,buffer, 5,NULL,0))
			break;
		err++;
		ERROR(("DPL3518: I/O error #%d (Write 0x%02x/0x%02x)\n",err, subaddr, addr));
	}
	if (3 == err) {
		ERROR(("DPL3518: giving up, reseting chip. Sound will go off,\n"));
		Reset();
		return false;
	}
	return true;
}


void DPL3518::Ioc_init()
{
	DUMP(("dpl3518 init\n"));
	Write( I2C_DPL3518_MR , 0x0083, 0x0008); /* mode_reg */
	Write( I2C_DPL3518_DFP, 0x000b, 0x0320); /*I2S1 src: RL */
	Write( I2C_DPL3518_DFP, 0x0012, 0x0000); /*prescale2: off */
	Write( I2C_DPL3518_DFP, 0x0016, 0x1000); /*prescale: 0 dB */
	Write( I2C_DPL3518_DFP, 0x0038, 0x0420); /*I2S2 src: CS */
	Write( I2C_DPL3518_DFP, 0x0040, 0x0000); /*adaptive,surround */
	Write( I2C_DPL3518_DFP, 0x0041, 0x0520); /*i2s1, stereo */
	Write( I2C_DPL3518_DFP, 0x0042, 0x1400); /*delay: 20ms */
	Write( I2C_DPL3518_DFP, 0x0043, 0x0000); /*inp. Bal. auto */
	Write( I2C_DPL3518_DFP, 0x0044, 0x4000); /*spatial 50% */
	Write( I2C_DPL3518_DFP, 0x0045, 0x0000); /*panorama off */
	//Write(I2C_DPL3518_DFP,0x0045,0x5400);  /*panorama 66%*/
	Write( I2C_DPL3518_DFP, 0x0046, 0x0000); /*reverb off */
}

void DPL3518::Prologic_init()
{
	DUMP(("dpl3518 prologic_init\n"));
	Write( I2C_DPL3518_DFP, 0x000b, 0x0320); /*I2S1 src: RL */
	Write( I2C_DPL3518_DFP, 0x0038, 0x0420); /*I2S2 src: CS */
	Write( I2C_DPL3518_DFP, 0x0041, 0x0520); /*i2s1, stereo */
	m_mode = DPL_Prologic;
}

void DPL3518::Setmode(const DPLPlMode mode)
{
	DUMP(("dpl3518 setmode: %i\n", mode));
	Write( I2C_DPL3518_DFP, 0x0040, mode << 4);
	m_plmode = mode;
}

void DPL3518::Delay(const BYTE delay)
{
	DUMP(("dpl3518 setdelay: %i\n", delay));
	Write( I2C_DPL3518_DFP, 0x0042, WORD(delay) << 8);
	m_delay = delay;
}

void DPL3518::Reverb(const BYTE rev)
{
	DUMP(("dpl3518 setreverb: %i\n", rev));
	Write( I2C_DPL3518_DFP, 0x0046, WORD(rev) << 8);
	m_rev = rev;
}

void DPL3518::Panorama(const BYTE pan)
{
	DUMP(("dpl3518 panorama: %i\n", pan));
	Write( I2C_DPL3518_DFP, 0x0045, WORD(pan) << 8);
	m_pan = pan;
}

void DPL3518::Spatial(const BYTE spatial)
{
	DUMP(("dpl3518 setspatial: %i\n", spatial));
	Write( I2C_DPL3518_DFP, 0x0044, WORD(spatial) << 8);
	m_spatial = spatial;
}

void DPL3518::Noise_init()
{
	DUMP(("dpl3518 noise_init\n"));
	Write( I2C_DPL3518_DFP, 0x000b, 0x0320); /*I2S1 src: RL */
	Write( I2C_DPL3518_DFP, 0x0038, 0x0420); /*I2S2 src: CS */
	Write( I2C_DPL3518_DFP, 0x0041, 0x0100); /*NOISE */
	m_mode = DPL_Noise;
}

void DPL3518::Setnoisemode(const BYTE mode)
{
	static WORD noise_modes[5] = { 0x0000, 0x01a0, 0x01b0, 0x01c0, 0x01d0 };
	BYTE tmode = mode;
	if (tmode > 4) tmode = 4;
	DUMP(("dpl3518 setnoisemode: %i\n", tmode));
	Write( I2C_DPL3518_DFP, 0x0041, noise_modes[tmode]);
	m_noise = tmode;
}

void DPL3518::Through_init()
{
	DUMP(("dpl3518 through_init\n"));
	Write( I2C_DPL3518_DFP, 0x000b, 0x0520); /*I2S1 src: S1 */
	Write( I2C_DPL3518_DFP, 0x0038, 0x0220); /*I2S2 src: scart*/
	Write( I2C_DPL3518_DFP, 0x0041, 0x0520); /*i2s1, stereo */
	m_mode = DPL_Through;
}

// To support Hibernate mode
void DPL3518::Restart( void )
{
	// Revision code
	WORD rev1,rev2;

	if (!Reset()) {
		ERROR(("DPL3518: no chip found\n"));
	}

	// Init IOC
    Ioc_init();

	// Based upon mode, set it
	switch(m_mode) {
		case DPL_Prologic:		/* Dolby prologic mode */
			Prologic_init();
			break;

		case DPL_Noise:			/* Noise mode */
			Noise_init();			// Set noise mode
			Setnoisemode(m_noise);	// And set the kind!
			break;

		case DPL_Through:		/* Pass through mode */
			Through_init();
			break;
	}

	// Now set all the other parameters!
	Setmode(m_plmode);
	Delay(m_delay);
	Reverb(m_rev);
	Panorama(m_pan);
	Spatial(m_spatial);
}

// Initialization!
DPL3518::DPL3518(I2C& bus,const BYTE Addr) :
	i2c(bus) , i2caddr(Addr) , 
	m_mode(DPL_Through),			// Current processor mode
	m_plmode(DPLMPl_Wide),			// Current Prologic mode
	m_delay(0x14),					// Current delay (20ms)
	m_rev(0),						// Current reverb (off)
	m_pan(0),						// Current Panorama (off)
	m_spatial(0x40),				// Current spatial (50%)
	m_noise(0)						// Current noise mode
{
	// And reinit the chip
	Restart();
}

DPL3518::~DPL3518()
{
}

void DPL3518::SetDolby(DOLBY_PARAMS& params)
{

	// Based upon mode, set it
	switch(params.mode) {
		case DPL_Prologic:		/* Dolby prologic mode */
			Prologic_init();
			break;

		case DPL_Noise:			/* Noise mode */
			Noise_init();			// Set noise mode
			Setnoisemode(params.noise);	// And set the kind!
			break;

		case DPL_Through:		/* Pass through mode */
			Through_init();
			break;
	}

	// Now set all the other parameters!
	Setmode(DPLPlMode(params.plmode));
	Delay(params.delay);
	Reverb(params.rev);
	Panorama(params.pan);
	Spatial(params.spatial);
}