/*
	DPL3518.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
 * programming the DPL3518 sound processor 
 * ---
 * DOLBY and PRO LOGIC are trademarks of 
 * Dolby Laboratories Licensing Corporation.
 *
 * ---
 * This is based on linux driver:
 *   Copyright (C) 1999 Roland Jansen <roland@lut.rwth-aachen.de>
 */

#include "HwAudio.hpp"

#define I2C_DPL3518       0x84

#define I2C_DPL3518_MR    0x10	/* write address MODE_REG */
#define I2C_DPL3518_DFP   0x12	/* write address DFP */

class I2C;

class DPL3518 : public HwAudio {

	private: 

		// I2C Bus and Address
		I2C&					i2c;				// Bus
		const BYTE				i2caddr;			// Direccin I2C
		DPLMode					m_mode;				// Current processor mode
		DPLPlMode				m_plmode;			// Current Prologic mode
		BYTE					m_delay;			// Current delay
		BYTE					m_rev;				// Current reverb
		BYTE					m_pan;				// Current Panorama
		BYTE					m_spatial;			// Current spatial
		BYTE					m_noise;			// Current noise mode

	private:

		// Private functions
		bool Reset();
		WORD Read(BYTE subaddr, WORD addr);
		bool Write(BYTE subaddr, WORD addr, WORD val);

		// IOC init
		void Ioc_init();

		// Prologic mode
		void Prologic_init();
		void Setmode(const DPLPlMode mode);
		void Delay(const BYTE delay);		// 0-FF
		void Reverb(const BYTE rev);		// 0-7F
		void Panorama(const BYTE pan);		// 0-7F 
		void Spatial(const BYTE spatial);	// 0-7F

		// Noise mode 
		void Noise_init();
		void Setnoisemode(const BYTE mode);

		// Through mode
		void Through_init();

	public:

		// Initialization!
		DPL3518(I2C& bus,const BYTE Addr);
		virtual ~DPL3518();

		// To support Hibernate mode
		virtual void Restart( void );

		// Yes! - We are a Dolby chip
		virtual bool IsDolbyAvailable()		{ return true; }

		// And the fn we need to set those params!
		virtual void SetDolby(DOLBY_PARAMS& params);
};

