/*
	GPIOAudio.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#ifndef _GPIOAUDIO_HPP_
#define _GPIOAUDIO_HPP_

#include "HwAudio.hpp"
#include "..\HwCapture\Pisces.hpp"

class GPIOAudio : public HwAudio {

	private:

		// GPIO Bus associated to the Audio Hardware
		GPIO& gpio_;
		const DWORD GPOE;			// Bits to set as outputs that actually control audio mode
		const DWORD GPDATA_Mono;	// GPDATA register for mono   sound
		const DWORD GPDATA_Stereo;	// GPDATA register for stereo sound
		const DWORD GPDATA_SAP;		// GPDATA register for sap    sound

	public:

		GPIOAudio(GPIO& gpio,const DWORD gpoe,const DWORD stereo,const DWORD mono,const DWORD sap) 
			: GPOE(gpoe), GPDATA_Mono(mono) , GPDATA_Stereo(stereo), GPDATA_SAP(sap), gpio_(gpio) {}

		virtual void SetAudioMode(AudioMode audio);

		// Audiomodes
		virtual AudioModes AudioModesAvailable() { return (Am_Mono | Am_Stereo | Am_Lang1); }	
		virtual AudioModes AudioModesDetected()  { return (Am_Mono | Am_Stereo | Am_Lang1); }

};



//   GPOE      ST       MO       SAP
// 0x000013 0x000002 0x000012 0x000001  = AverMedia OLD
// 0x000003 0x000001 0x000000 0x000002  = AverMedia NEW
// 0x000300 0x000200 0x000000 0x000300  = GVBCTV3
// 0x000880 0x000880 0x000000 0x000080  = LT9415
// 0x000003 0x000002 0x000001 0x000000  = pvbt878p9b
// 0x180000 0x180000 0x000000 0x080000  = TerraTV+
// 0x430000 0x020000 0x420000 0x410000  = WinFast2000
// 0x140000 0x000000 0x040000 0x100000  = WINDVR 


#endif


