/*
	HwAudio.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
 * Class that represents the Hardware associated to the TVAudio (and more)
 */

#ifndef _HWAUDIO_HPP_
#define _HWAUDIO_HPP_

// To get all the required things!
#include "Debug.hpp"
#include "CppRt.hpp"
#include "MyTypes.hpp"
#include "custprop.h"

#include "..\Enums.hpp"

class HwAudio {

	public:

		// constructors and destructors... Need to be declared here as we want the 'virtual' keyword for destructor
		HwAudio() {}
		virtual ~HwAudio() {}

		// Called when videoChannel has changed
		virtual void ChannelChanged() {}

		// Called when VideoNorm changed!
		virtual void VideoStandardChanged(KS_AnalogVideoStandard vf) {}

		// Selection of Audio Connections
		virtual void SetAudioConnector(AudioConnector conn) {}

		// -32767 , 0 = 0Db, 32767
		virtual void SetVolume(const int Left,const int Right) {}

		// -32767 , 0 = 0Db, 32767
		virtual void SetBassTreble(const int Bass,const int Treble) {}

		// -32767 , 0 = 0Db ,32767
		virtual void SetLoudness(const int Loudness) {}

		// To select between Mono and Stereo, and Language
		virtual void SetAudioMode(AudioMode audio) {}

		// To set the equalizer
		virtual void SetEqualizer(LONG* Bands) {}

		// To set the Dolby mode
		virtual void SetDolby(DOLBY_PARAMS& params) {}

		// Capabilities...
		virtual bool  IsVolumeAvailable()	{ return false; }
		virtual bool  IsBassTrebleAvailable(){ return false; }
		virtual bool  IsLoudnessAvailable()	{ return false; }
		virtual DWORD EqualizerBandsAvailable(DWORD* freqs) { return 0; }
		virtual bool  IsDolbyAvailable()	{ return false; }

		// Audiomodes
		virtual AudioModes AudioModesAvailable() { return 0; }	// No audiomodes available 
		virtual AudioModes AudioModesDetected()  { return 0; }  // No audiomodes detected by default.

		// To support Hibernate mode
		virtual void Restart( void ) {}
		virtual void Suspend( void ) {}

};


/* AudioConTuner ,	AudioConRadio, 	AudioConLineIn, AudioConLineOut ,AudioConMute */



#endif