/*
	PT2254.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "PT2254.hpp"
#include "..\HwCapture\Pisces.hpp"

#define PT2254_L_CHANEL			0x10		/* 00 0000 0000 0001 0000 */
#define PT2254_R_CHANEL			0x08		/* 00 0000 0000 0000 1000 */
#define PT2254_DBS_IN_2			0x400		/* 00 0000 0100 0000 0000 */
#define PT2254_DBS_IN_10		0x20000		/* 10 0000 0000 0000 0000 */

#if 0
#define WINVIEW_PT2254_CLK_BIT			6
#define WINVIEW_PT2254_DATA_BIT			5
#define WINVIEW_PT2254_STROBE_BIT		7

#define TEKRAMM205_PT2254_CLK_BIT		0
#define TEKRAMM205_PT2254_DATA_BIT		1
#define TEKRAMM205_PT2254_STROBE_BIT	2
#endif


PT2254::PT2254(GPIO& bus,int clkbit,int databit,int strobebit) :
	gpio_(bus), clk(clkbit) , data(databit), strobe(strobebit)
{
}
	
	/* -32767 , 0 = 0Db, 32767 */
void PT2254::SetVolume(const int Left,const int Right) 
{
	int att = -((Left + Right) / (2 * 482));	// Can attenuate 68 Db
	if (att < 0) att = 0;						// Only allow attenuation in DB

	/* PT2254A programming Jon Tombs, jon@gte.esi.us.es, fixed by ejt */
	ULONG bits_out, vol;
	

	/* units */
	bits_out  = (PT2254_DBS_IN_2  >> ((att % 10) >> 1) );
	/* tens */
	bits_out |= (PT2254_DBS_IN_10 >> (att / 10) );
	/* for both channels */
	bits_out |= PT2254_L_CHANEL | PT2254_R_CHANEL;

	// Set all lines to 0 and set them to outputs!
	gpio_.SetGPOE(clk,On);
	gpio_.SetGPOE(data,On);
	gpio_.SetGPOE(strobe,On);
	gpio_.SetGPDATA(clk,clk,0,0);
	gpio_.SetGPDATA(data,data,0,0);
	gpio_.SetGPDATA(strobe,strobe,0,0);

	// Just give some time ...
	udelay(20);

	int repeat;
	for (repeat = 0; repeat < 2 ; repeat++) {
		int loops;
		for (loops = 17; loops >= 0 ; loops--) {

			// Set the data line...
			gpio_.SetGPDATA(data,data, (bits_out & (1<<loops)) ? 1 : 0 , 0 );

			// And send a Clock pulse!
			udelay(20);
			gpio_.SetGPDATA(clk,clk, 1,0);
			udelay(20);
			gpio_.SetGPDATA(clk,clk, 0,0);
			udelay(20);
		}
		// Now we must set Data to 0, and send an Strobe.
		gpio_.SetGPDATA(data,data, 0 , 0 );
		udelay(20);
		gpio_.SetGPDATA(strobe,strobe, 1,0);
		mdelay(10);
		gpio_.SetGPDATA(strobe,strobe, 0,0);
		mdelay(10);
	}
}

