/*
	PV951.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "PV951.hpp"
#include "..\HwCapture\Pisces.hpp"

/* ---------------------------------------------------------------------- */
/* audio chip descriptions - defines+functions for pic16c54 (PV951)       */

/* the registers of 16C54, I2C sub address. */
#define PIC16C54_REG_KEY_CODE     0x01	       /* Not use. */
#define PIC16C54_REG_MISC         0x02

/* bit definition of the RESET register, I2C data. */
#define PIC16C54_MISC_RESET_REMOTE_CTL 0x01 /* bit 0, Reset to receive the key */
                                            /*        code of remote controller */
#define PIC16C54_MISC_MTS_MAIN         0x02 /* bit 1 */
#define PIC16C54_MISC_MTS_SAP          0x04 /* bit 2 */
#define PIC16C54_MISC_MTS_BOTH         0x08 /* bit 3 */
#define PIC16C54_MISC_SND_MUTE         0x10 /* bit 4, Mute Audio(Line-in and Tuner) */
#define PIC16C54_MISC_SND_NOTMUTE      0x20 /* bit 5 */
#define PIC16C54_MISC_SWITCH_TUNER     0x40 /* bit 6	, Switch to Tuner */
#define PIC16C54_MISC_SWITCH_LINE      0x80 /* bit 7	, Switch to Line-In */

void PV951AC::Restart()
{
	// Set i2c Speed to ~6Khz because the PIC cannot handle more than this
	i2c.SetFreq(6000);
	i2c.Write(i2caddr,PIC16C54_REG_MISC,mode);
#if 0
	// Select tuner source
	i2c.Write(i2caddr,PIC16C54_REG_MISC,PIC16C54_MISC_SWITCH_TUNER);
	// Select main audio
	i2c.Write(i2caddr,PIC16C54_REG_MISC,PIC16C54_MISC_MTS_MAIN);
	// Mute
	i2c.Write(i2caddr,PIC16C54_REG_MISC,PIC16C54_MISC_SND_MUTE);
#endif
}

PV951AC::PV951AC(I2C& bus,const BYTE Addr) :
	i2c(bus) , i2caddr(Addr) , mode(/*PIC16C54_MISC_MTS_BOTH |*/ PIC16C54_MISC_SND_MUTE | PIC16C54_MISC_SWITCH_TUNER)
{
	PV951AC::Restart();
}

void PV951AC::SetAudioConnector(AudioConnector conn)
{
	// Set i2c Speed to ~6Khz because the PIC cannot handle more than this
	i2c.SetFreq(6000);
	switch(conn) {
		case AudioConTuner:
		case AudioConRadio:
			mode &= 0x0F;	// Clean mute and unmute and source
			mode |= (PIC16C54_MISC_SND_NOTMUTE | PIC16C54_MISC_SWITCH_TUNER);
			i2c.Write(i2caddr,PIC16C54_REG_MISC,mode);
#if 0
			// unmute
			i2c.Write(i2caddr,PIC16C54_REG_MISC,PIC16C54_MISC_SND_NOTMUTE);
			// Select tuner source
			i2c.Write(i2caddr,PIC16C54_REG_MISC,PIC16C54_MISC_SWITCH_TUNER);
#endif
			break;
		case AudioConLineIn:
			mode &= 0x0F;	// Clean mute and unmute and source
			mode |= (PIC16C54_MISC_SND_NOTMUTE | PIC16C54_MISC_SWITCH_LINE);
			i2c.Write(i2caddr,PIC16C54_REG_MISC,mode);
#if 0
			// unmute
			i2c.Write(i2caddr,PIC16C54_REG_MISC,PIC16C54_MISC_SND_NOTMUTE);
			// Select line-in source
			i2c.Write(i2caddr,PIC16C54_REG_MISC,PIC16C54_MISC_SWITCH_LINE);
#endif
			break;
		case AudioConMute:
			mode &= 0xCF;	// Clean mute and unmute and source
			mode |= PIC16C54_MISC_SND_MUTE;
			i2c.Write(i2caddr,PIC16C54_REG_MISC,mode);
#if 0
			// mute
			i2c.Write(i2caddr,PIC16C54_REG_MISC,PIC16C54_MISC_SND_MUTE);
#endif
			break;
	}
}
		
void PV951AC::SetAudioMode(AudioMode audio) 
{
#if 0
	i2c.SetFreq(6000);
	if (audio == Am_Lang2) {
		mode &= 0xF0;	// Clean lang bits
		mode |= PIC16C54_MISC_MTS_BOTH;
		i2c.Write(i2caddr,PIC16C54_REG_MISC,mode);
#if 0
		// Select both languages
		i2c.Write(i2caddr,PIC16C54_REG_MISC,PIC16C54_MISC_MTS_BOTH);
#endif
	} else 
	if (audio == Am_Lang1) {
		mode &= 0xF0;	// Clean lang bits
		mode |= PIC16C54_MISC_MTS_SAP;
		i2c.Write(i2caddr,PIC16C54_REG_MISC,mode);
#if 0
		// Select sap audio
		i2c.Write(i2caddr,PIC16C54_REG_MISC,PIC16C54_MISC_MTS_SAP);
#endif
	} else {
		mode &= 0xF0;	// Clean lang bits
		mode |= PIC16C54_MISC_MTS_MAIN;
		i2c.Write(i2caddr,PIC16C54_REG_MISC,mode);
#if 0
		// Select main audio
		i2c.Write(i2caddr,PIC16C54_REG_MISC,PIC16C54_MISC_MTS_MAIN);
#endif
	}
#endif
}
