/*
	TDA7432.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _TDA7432_HP_
#define _TDA7432_HP_

#include "HwAudio.hpp"

/*
 * For the STS-Thompson TDA7432 audio processor chip
 */
#define I2C_TDA7432        0x8a

class I2C;
class TDA7432 : public HwAudio {

	private:

		int  MainVol;	// Master volume
		int  Loudness;	// Loudness
		bool Muted;		// If the chip is muted...

		// bus
		I2C& i2c;
		const BYTE i2caddr;

	public:

		TDA7432(I2C& bus,const BYTE Addr);
		virtual void SetVolume(const int Left,const int Right); /* 0-65535 */
		virtual void SetBassTreble(const int Bass,const int Treble); /* -128 a +127 */
		
		/* -32767 , 0 = 0Db ,32767 */
		virtual void SetLoudness(const int Loudness_);
		virtual bool IsVolumeAvailable()	{ return true; }
		virtual bool IsBassTrebleAvailable(){ return true; }
		virtual bool IsLoudnessAvailable()	{ return true; }

		// Selection of Audio Connections
		virtual void SetAudioConnector(AudioConnector conn);

		// To support Hibernate mode
		virtual void Restart( void );

};

// Can be read
#endif