/*
	TDA8425.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/
/*
 * for the TDA8425 chip (I don't know which cards have this)
 * WARNING: THIS DRIVER WILL LOAD WITHOUT COMPLAINTS EVEN IF A DIFFERENT
 * CHIP IS AT ADDRESS 0x82 (it relies on i2c to make sure that there is a
 * device acknowledging that address)
 */

// TODO: Perhaps the Mono mode ?

#include "TDA8425.hpp"
#include "..\HwCapture\Pisces.hpp"

#define TDA8425_VL         0x00  /* volume left */
#define TDA8425_VR         0x01  /* volume right */
#define TDA8425_BA         0x02  /* bass */
#define TDA8425_TR         0x03  /* treble */
#define TDA8425_S1         0x08  /* switch functions */
                                 /* values for those registers: */
#define TDA8425_S1_OFF     0xEE  /* audio off (mute on) */
#define TDA8425_S1_ON      0xCE  /* audio on (mute off) - "linear stereo" mode */

void TDA8425::Restart()
{
	i2c.Write(i2caddr, TDA8425_VL, 0x3C |0xC0);
	i2c.Write(i2caddr, TDA8425_VR, 0x3C |0xC0);
	i2c.Write(i2caddr, TDA8425_BA, 0x6  |0xF0);
	i2c.Write(i2caddr, TDA8425_TR, 0x6  |0xF0);
	i2c.Write(i2caddr, TDA8425_S1, TDA8425_S1_OFF); /* mute */
}

TDA8425::TDA8425(I2C& bus, const BYTE Addr) :
	i2c(bus) , i2caddr(Addr) 
{
	TDA8425::Restart();
}

void TDA8425::SetAudioConnector(AudioConnector conn) 
{
	switch(conn) {
		case AudioConTuner:
		case AudioConRadio:
		case AudioConLineIn:
			i2c.Write(i2caddr, TDA8425_S1, TDA8425_S1_ON);
			break;
		case AudioConMute:
			i2c.Write(i2caddr, TDA8425_S1, TDA8425_S1_OFF);
			break;
	}
}

void TDA8425::SetVolume(const int Left,const int Right) /* 0x3C = 0dB */
{
	int AdjLeft  = (Left /546) + 0x3C;
	int AdjRight = (Right/546) + 0x3C;
	AdjLeft = (AdjLeft > 0x3F) ? 0x3F: AdjLeft;
	AdjRight = (AdjRight > 0x3F) ? 0x3F: AdjRight;

	i2c.Write(i2caddr, TDA8425_VL, AdjLeft  | 0xC0);
	i2c.Write(i2caddr, TDA8425_VR, AdjRight | 0xC0);
} 

void TDA8425::SetBassTreble(const int Bass,const int Treble) 
{
	int AdjBass   = (Bass   / 3641) + 0x6;
	int AdjTreble = (Treble / 3641) + 0x6;
	AdjBass   = (AdjBass   < 0) ? 0: AdjBass;
	AdjTreble = (AdjTreble < 0) ? 0: AdjTreble;

	i2c.Write(i2caddr, TDA8425_BA, AdjBass   |0xF0); 
	i2c.Write(i2caddr, TDA8425_TR, AdjTreble |0xF0); 
}



