/*
	TDA9840.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "TDA9840.hpp"
#include "..\HwCapture\Pisces.hpp"

#define TDA9840_SW         0x00
#define TDA9840_LVADJ      0x02
#define TDA9840_STADJ      0x03
#define TDA9840_TEST       0x04

#define TDA9840_MONO       0x10
#define TDA9840_STEREO     0x2a
#define TDA9840_DUALA      0x12
#define TDA9840_DUALB      0x1e
#define TDA9840_DUALAB     0x1a
#define TDA9840_DUALBA     0x16
#define TDA9840_EXTERNAL   0x7a

#define TDA9840_DS_DUAL    0x20 /* Dual sound identified          */
#define TDA9840_ST_STEREO  0x40 /* Stereo sound identified        */
#define TDA9840_PONRES     0x80 /* Power-on reset detected if = 1 */

void TDA9840::Restart()
{
	// Just set the audio mode to the one selected...
	TDA9840::SetAudioMode(m_mode);
}

TDA9840::TDA9840(I2C& bus,const BYTE Addr) :
	m_mode(Am_Stereo) , m_muted(false) ,
	i2c(bus) , i2caddr(Addr)
{
	TDA9840::Restart();
}

void TDA9840::SetAudioMode(AudioMode audio) 
{
	m_mode = audio;
	if (m_muted) {
		i2c.Write(i2caddr, TDA9840_SW, 0x00);  /* sound mode switching */
	} else {
		if (audio == Am_Lang2) {
			i2c.Write(i2caddr, TDA9840_SW, TDA9840_DUALB);
		} else 
		if (audio == Am_Lang1) {
			i2c.Write(i2caddr, TDA9840_SW, TDA9840_DUALA);
		} else 
		if (audio == Am_Stereo) {
			i2c.Write(i2caddr, TDA9840_SW, TDA9840_STEREO);
		} else {
			i2c.Write(i2caddr, TDA9840_SW, TDA9840_MONO);
		}
	}
}

void TDA9840::SetAudioConnector(AudioConnector conn) 
{
	
	switch(conn) {
		case AudioConTuner:
		case AudioConRadio:
		case AudioConLineIn:
			m_muted = false;	// Unmute
			break;
		case AudioConMute:
			m_muted = true;		// Mute
			break;
	}
	// Mute or unmute if we need to...
	TDA9840::SetAudioMode(m_mode);
}

AudioModes TDA9840::AudioModesDetected()
{
	BYTE Stat = 0;
	i2c.Read(i2caddr,&Stat);

	return (Am_Mono |
			(((Stat & TDA9840_ST_STEREO) != 0) ? Am_Stereo : 0) |
			(((Stat & TDA9840_DS_DUAL  ) != 0) ? (Am_Lang1 | Am_Lang2) : 0)
			);
}

