/*
	TDA9874.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _TDA9874_HPP_
#define _TDA9874_HPP_

#include "HwAudio.hpp"
#include "SyncEvent.hpp"
#include "NotifyEvent.hpp"

#define I2C_TDA9874_0					0xb0
#define I2C_TDA9874_1_NOTUSEDBYANY      0xb2
#define I2C_TDA9874_2_NOTUSEDBYANY      0xb4
#define I2C_TDA9874_3_NOTUSEDBYANY      0xb6

class I2C;
class TDA9874 : public HwAudio {

	protected:
		typedef enum {
			Autodetect,				// Fire autodetection
			SuspendAutodetect,		// Thread suspension
			Exit					// Exit thread
		} Action;

		typedef struct {
			DWORD Norms;	// Videonorms that use this
			DWORD c1,c2;	// Carrier 1 and Carrier 2 frequencies
			BYTE  DCR;		// Demodulator control register
			BYTE  FMER;		// FM deemphasis	(only for FM, NICAM ignores it)
			BYTE  FMMR;		// FM dematrix		(only for FM, NICAM ignores it). User can override it
			BOOL  Nicam;	// Is it is NICAM mode
			BOOL  AGCOff;	// If AGC must be turned off
		} ANInfo;

	private:
		I2C&		i2c;
		const BYTE	i2caddr;
		int			m_SIF ;
		BYTE		std;				// Current standard number
		volatile KS_AnalogVideoStandard  currVideoStandard;	// The current video Standard we are using

		// For thread communication
		SyncEvent				t_exec;				// Used to signal thread that must do something
		volatile LONG			t_pendingcmds;		// Number of Pending commands
		volatile Action			t_action;			// Action thread must do
		NotifyEvent				t_suspended;		// Used to signal thread is suspended
		NotifyEvent				t_ended;			// Used to signal thread is ended

		// Supported audio norms
		static ANInfo Info[];

		// The desired Audio mode
		AudioMode	desiredam;

	protected:
		static VOID ThreadProc(IN PVOID ctx);
		void AutodetectProc();
		void SetChipStandard(bool FastCheck);
		void SetChipMode(AudioMode audio);

	public:
		TDA9874(I2C& bus,const BYTE Addr) ;
		virtual ~TDA9874();

		// Audiomodes
		virtual AudioModes AudioModesAvailable() { return (Am_Mono | Am_Stereo | Am_Lang1 | Am_Lang2); }	
		virtual AudioModes AudioModesDetected();

		virtual void SetAudioMode(AudioMode audio) ;

		// Called when videoChannel has changed
		virtual void ChannelChanged();

		// Called when VideoNorm changed!
		virtual void VideoStandardChanged(KS_AnalogVideoStandard vf);

		// Selection of Audio Connections
		virtual void SetAudioConnector(AudioConnector conn);

		// To support Hibernate mode
		virtual void Restart( void );
		virtual void Suspend( void );

};

#endif