/*
	TEA6300.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "TEA6300.hpp"
#include "..\HwCapture\Pisces.hpp"

/*
 * for the TEA6300 chip (only found on Gateway STB TV/FM cards to the best
 * of my knowledge)
 */

#define TEA6300_VL         0x00  /* volume left */
#define TEA6300_VR         0x01  /* volume right */
#define TEA6300_BA         0x02  /* bass */
#define TEA6300_TR         0x03  /* treble */
#define TEA6300_FA         0x04  /* fader control */
#define TEA6300_S          0x05  /* switch register */
                                 /* values for those registers: */
#define TEA6300_S_SA       0x01  /* stereo A input */
#define TEA6300_S_SB       0x02  /* stereo B */
#define TEA6300_S_SC       0x04  /* stereo C */
#define TEA6300_S_GMU      0x80  /* general mute */

void TEA6300::Restart()
{
	i2c.Write(i2caddr, TEA6300_FA, 0x3f); /* fader off */
	i2c.Write(i2caddr, TEA6300_S , TEA6300_S_GMU); /* mute */
}

TEA6300::TEA6300(I2C& bus,const BYTE Addr) :
	i2c(bus) , i2caddr(Addr)
{
	TEA6300::Restart();
}

void TEA6300::SetAudioConnector(AudioConnector conn)
{
	switch(conn) {
		case AudioConTuner:
			i2c.Write(i2caddr, TEA6300_S, TEA6300_S_SA);
			break;
		case AudioConRadio:
			i2c.Write(i2caddr, TEA6300_S, TEA6300_S_SB);
			break;
		case AudioConLineIn:
			i2c.Write(i2caddr, TEA6300_S, TEA6300_S_SC);
			break;
		case AudioConMute:
			i2c.Write(i2caddr, TEA6300_S, TEA6300_S_GMU);
			break;
	}
}
		
void TEA6300::SetVolume(const int Left,const int Right) /* 0 = Gain 1 */
{
	// 54
	int AdjLeft  = 54 + Left  / 607;
	int AdjRight = 54 + Right / 607;
	AdjLeft  = (AdjLeft  > 63) ? 63 : AdjLeft ;
	AdjRight = (AdjRight > 63) ? 63 : AdjRight;
	i2c.Write(i2caddr, TEA6300_VL, BYTE(AdjLeft) );
	i2c.Write(i2caddr, TEA6300_VR, BYTE(AdjRight));
}

void TEA6300::SetBassTreble(const int Bass,const int Treble) 
{
	i2c.Write(i2caddr, TEA6300_BA, (Bass >> 12) + 7);
	i2c.Write(i2caddr, TEA6300_TR, (Treble >> 12) + 7);
}

// Note : No Reading available!!!!
