/*
	Tekram205.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
	TekramM205 has an I2C device ( a PIC16c54) at 0xAA address... Seems to be
	responsible	of Audio routing.
	Also, it has a programmable attenuator connected to GPIO (a PT2254) ...Argh!!!
	
	This was taken by reverse engineering the apps supplied with Tekram, so no promises at all!

I2C Chip:

	Write regs:
		reg 0x04: : Default 0x20
			bits 3-4 = Audio source select
				2 = Tuner
				3 = Mute
			bits 5-6 = MTS select: 

	Read Regs:
		reg 0x01
			remote control code... Read twice to get the code (seems as a 16 bit register)

		reg 0x02
			bit 0-4 = Tuner Manufacturer / System
			bit 5   = MTS det - All the other bits should be preserved ?

		reg 0x03
			bit 6 = AFC det
			Bit 1 = ST det
			bit 0 = AL det

	I2C bus should run at 20Khz. (from Tekram apps)

Audio chip:
	A GPIO controlled programmable attenuator
	68 db maximum attenuation

 */

#include "Tekram205.hpp"
#include "..\HwCapture\Pisces.hpp"

Tekram205::Tekram205(I2C& i2cbus,const BYTE addr) :
	i2c_(i2cbus) , i2caddr(addr)
{
	// Restart card
	Restart();
}
	
void Tekram205::SetAudioConnector(AudioConnector conn)
{
	// Set I2C frequency to 20 KHz (needed by Tekram I2C chip)
	i2c_.SetFreq(20000);
	// And select the audio Source using the pic16c54 I2C chip.
	if (conn == AudioConTuner) {
		i2c_.Write(i2caddr,0x20 | (2 << 3));
		i2c_.Write(i2caddr,0x20 | (2 << 3));
	} else {
		i2c_.Write(i2caddr,0x20 | (3 << 3));
		i2c_.Write(i2caddr,0x20 | (3 << 3));
	}
}

void Tekram205::Restart()
{
	BYTE In[1];
	BYTE Out[1];

	// Set I2C frequency to 20 KHz (needed by Tekram I2C chip)
	i2c_.SetFreq(20000);
	// Read register 0x02 twice (as Tekram APP does)
	Out[0] = 0x02;
	i2c_.WR(i2caddr,Out,1,In,1);
	Out[0] = 0x02;
	i2c_.WR(i2caddr,Out,1,In,1);

	// Remember the important bits as a shadow! - Seemes to be Manufacturer/system?
	shadow = In[0] & 0x1F;
}

