/*
	CompReg.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "Debug.hpp"
#include "compreg.hpp"

/* Method: CompositeReg::operator DWORD()
 * Purpose: Performs the read from a composite register
*/
CompositeReg::operator DWORD()
{
	// obtain the low and high values
	DWORD dwLowBits  = (DWORD)LSBPart_;
	DWORD dwHighBits = (DWORD)MSBPart_;

	// put high part to the proper place
	dwHighBits <<= LowPartWidth_;

	// done !
	return dwHighBits | dwLowBits;
}


/* Method: CompositeReg::operator=
 * Purpose: performs the assignment to a composite register
*/
DWORD CompositeReg::operator=( DWORD dwValue )
{
	// if a register is read-only nothing is done. This is an error
	ENSURE( GetRegisterType() != RO );

	// compute the mask to apply to the passed value, so it can be...
	DWORD dwMask = ::MakeAMask( LowPartWidth_ );

	// ... assigned to the low portion register
	LSBPart_ = dwValue & dwMask;

	// shift is enough to get the high part
	MSBPart_ = ( dwValue >> LowPartWidth_ );
	return dwValue;
}
