/*
	Defaults.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __DEFAULTS_HPP
#define __DEFAULTS_HPP

static const int DefWidth = 320;
static const int DefHeight = 240;

static const int MaxInWidth = 924;
static const int MinInWidth = 80;

static const int MaxInHeight = 576;
static const int MinInHeight = 60;

static const int MaxOutWidth = 924;
static const int MinOutWidth = 80;

static const int MaxOutHeight = 576;
static const int MinOutHeight = 60;


//  NTSC
static const DWORD dwNTSCVBIStart  =  10; // sync looks like this 1:PRE,2:PRE,3:PRE,4:SYNC,5:SYNC,6:SYNC,7:POST 8:POST 9:POST, 10: First video line
static const DWORD dwNTSCVBIEnd    =  21; // This is line of CC
static const DWORD dwNTSCVBILines  = dwNTSCVBIEnd - dwNTSCVBIStart + 1;
static const DWORD dwNTSCSamplingFrequency  = 28636363;
static const DWORD dwNTSCVBIMinLineStartTime    =  454; // in 10-nanosecond units
static const DWORD dwNTSCVBIMaxLineStartTime    =  900; // in 10-nanosecond units
static const DWORD dwNTSCVBIActualLineStartTime =  820; // in 10-nanosecond units
static const DWORD dwNTSCVBIActualLineEndTime   = 5942; // in 10-nanosecond units
static const DWORD dwNTSCFrameDuration        = 333667;
static const DWORD dwNTSCFramesPerSecond        =   30;
static const DWORD dwNTSCVBISamplesPerLine   = 800 * 2;
static const DWORD dwNTSCVBIStrideInBytes    = 800 * 2;
static const DWORD dwNTSCVBIBufferSize = dwNTSCVBIStrideInBytes * dwNTSCVBILines;

//  PAL
static const DWORD dwPALVBIStart   =  7;  // 5 pre, 5 sync, 5 post (total 15, to get nbr of lines = 15/2 =7.5
static const DWORD dwPALVBIEnd     =  22; // This is line of CC
static const DWORD dwPALVBILines   = dwPALVBIEnd - dwPALVBIStart + 1;
static const DWORD dwPALSamplingFrequency  = 35468950;
static const DWORD dwPALVBIMinLineStartTime    =  454; // in 10-nanosecond units
static const DWORD dwPALVBIMaxLineStartTime    =  900; // in 10-nanosecond units
static const DWORD dwPALVBIActualLineStartTime =  820; // in 10-nanosecond units
static const DWORD dwPALVBIActualLineEndTime   = 6342; // in 10-nanosecond units
static const DWORD dwPALFrameDuration        = 400000;
static const DWORD dwPALFramesPerSecond          = 25;
static const DWORD dwPALVBISamplesPerLine  = 1000 * 2;
static const DWORD dwPALVBIStrideInBytes  =  1000 * 2;
static const DWORD dwPALVBIBufferSize = dwPALVBIStrideInBytes * dwPALVBILines;

// margin (separation between programs) to be able to distinguish them by the RISC PC
static const DWORD RISCMargin		 = 16; /* 16 bytes sounds like a good one number */


// the max size of a video program is for PAL and PLANAR programs
static const DWORD MaxVidProgSize   = 288 * 5 * sizeof( DWORD );

// worst case buffer layout is with RGB32: This should be the biggest buffer we could receive
static const DWORD MaxVidCrossings  = MaxOutWidth * 288 * 4 / PAGE_SIZE; 

// SYNC + PROG + JUMP(CHAIN) + [JUMP(LOOPONITSELF)

static const DWORD MaxVidSizeRaw    = RISCMargin + MaxVidProgSize + MaxVidCrossings * 5 * sizeof( DWORD )
								+ (2 + 5 + 5) * sizeof(DWORD) ; // To account for JUMPS and SYNC

// Make it a multiple of 256 bytes to align to cache
static const DWORD MaxVidSize		 = ((MaxVidSizeRaw + 0xFF) & (~(0xFF)));

// the max size of a VBI program is PACKED 
static const DWORD MaxVBIProgSize   = max( dwNTSCVBILines, dwPALVBILines ) * 2 * sizeof( DWORD );

// worst case buffer layout
static const DWORD MaxVBICrossings  = max( dwNTSCVBISamplesPerLine, dwPALVBISamplesPerLine ) 
                                    * max( dwNTSCVBILines, dwPALVBILines ) / PAGE_SIZE;
//
static const DWORD MaxVBISizeRaw    = RISCMargin + MaxVBIProgSize + MaxVBICrossings * 5 * sizeof( DWORD )
								+  (2 + 5 + 5) * sizeof(DWORD) ; // To account for JUMPS and SYNC;

// Make it a multiple of 256 bytes to align to cache
static const DWORD MaxVBISize		 = ((MaxVBISizeRaw + 0xFF) & (~(0xFF)));

// 2 fields, 2 VBI/Sync programs per field , DoubleBuffering
static const DWORD VideoOffset      = MaxVBISize * 8;

// total memory needed for all risc programs
static const DWORD RISCProgramsSize = VideoOffset + MaxVidSize * 4 + 0xFF /* to align start of buffer */ ;

#endif

