/*
	FieldDataBuf.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef FIELDDATABUF_HPP_
#define FIELDDATABUF_HPP_

#include "debug.hpp" 
#include "CppRt.hpp"
#include "MyTypes.hpp"
#include "VideoDataBuf.hpp"

/*
 * FieldDataBuf: Class used to pass Buffer info to the Field Class
 */

class FieldDataBuf {

	private:
		
		VideoDataBuf				pBuf;			// Video Buffer 
		DWORD_PTR					dwOffset;		// Offset to the start of buffer
		DWORD						dwPitch;		// Pitch of buffer

	public:
		// Default constructor
		FieldDataBuf() :
			pBuf() , dwOffset(0) , dwPitch(0) {}

		// Constructor
		FieldDataBuf(const VideoDataBuf& pBuf_ , DWORD_PTR dwOffset_ , DWORD dwPitch_ ) :
			pBuf(pBuf_) , dwOffset(dwOffset_) , dwPitch(dwPitch_) {}

		// Copy constructor
		FieldDataBuf(const FieldDataBuf& src) : 
			pBuf(src.pBuf) , dwOffset(src.dwOffset) , dwPitch(src.dwPitch) {}

		// Copy operator
		FieldDataBuf& operator=(const FieldDataBuf& src) 
		{
			pBuf = src.pBuf;
			dwOffset = src.dwOffset;
			dwPitch = src.dwPitch;
			return *this;
		}

		// Access Members
		
		// To get the linear addr of the databuffer
		PVOID GetBufAddr(LONG_PTR lOffset = 0 ) const { return pBuf.GetBufAddr(lOffset); }

		// To get the physical addr of the databuffer
		DWORD_PTR GetBufPhysAddr(const LONG_PTR lOffset, ULONG& len) const { return pBuf.GetBufPhysAddr(lOffset,len); }

		// To get buffer offset from start
		DWORD_PTR GetBufOffset() const { return dwOffset; }

		// To get buffer pitch
		DWORD GetBufPitch()  const { return dwPitch; }

		// To get the Buffer size
		DWORD GetBufSize() const { return pBuf.GetBufSize(); }

		// To know if buffer is a real one
		bool  RealBuffer() const { return pBuf.RealBuffer(); }

		// To get the internal VideoBuffer
		const VideoDataBuf&	GetBuf() const { return pBuf; } 
};



#endif