/*
	Hw.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __HW_H
#define __HW_H

/**************************************************************************\
* Macros
\**************************************************************************/

#define MASK(msb,lsb)			         ((0xFFFFFFFF>>(32-msb+lsb+1))<<(lsb))
#define SETFIELD(preg,msb,lsb,val)  	(*preg=((*preg&~MASK(msb,lsb))|(val<<lsb)))
#define GETFIELD(reg,msb,lsb) 			((MASK(msb,lsb)&reg)>>(lsb))

/**************************************************************************\
* Register Offset Definitions
\**************************************************************************/

#define BT848_REG_DSTATUS				0x000    // Device Status
#define BT848_REG_IFORM 				0x004    // Input Format
#define BT848_REG_TDEC   				0x008    // Temporal Decimation

#define BT848_REG_E_CROP  				0x00C    // MSB Cropping Even Field
#define BT848_REG_O_CROP 				0x08C    // MSB Cropping Odd Field

#define BT848_REG_E_VDELAY_LO			0x090    // LSB Vertical Delay Even Field
#define BT848_REG_O_VDELAY_LO			0x010    // LSB Vertical Delay Odd Field

#define BT848_REG_E_VACTIVE_LO		0x014    // LSB Vertical Active Even Field
#define BT848_REG_O_VACTIVE_LO		0x094    // LSB Vertical Active Odd Field

#define BT848_REG_E_HDELAY_LO			0x018    // LSB Horizontal Delay Even Field
#define BT848_REG_O_HDELAY_LO			0x098    // LSB Horizontal Delay Odd Field

#define BT848_REG_E_HACTIVE_LO		0x01C    // LSB Horizontal Active Even Field
#define BT848_REG_O_HACTIVE_LO		0x09C    // LSB Horizontal Active Odd Field

#define BT848_REG_E_HSCALE_HI			0x020    // MSB Horizontal Scaling Even Field
#define BT848_REG_O_HSCALE_HI			0x0A0    // MSB Horizontal Scaling Odd Field
#define BT848_REG_E_HSCALE_LO			0x024    // LSB Horizontal Scaling Even Field
#define BT848_REG_O_HSCALE_LO			0x0A4    // LSB Horizontal Scaling Odd Field

#define BT848_REG_BRIGHT 				0x028    // Brightness Control
#define BT848_REG_E_CONTROL			0x02C    // Miscellaneous Control Even Field
#define BT848_REG_O_CONTROL			0x0AC    // Miscellaneous Control Odd Field

#define BT848_REG_CONTRAST_LO			0x030    // LSB Luma Gain
#define BT848_REG_SAT_U_LO				0x034    // LSB Chroma (U) Gain
#define BT848_REG_SAT_V_LO				0x038    // LSB Chroma (V) Gain
#define BT848_REG_HUE   				0x03C    // Hue Control

#define BT848_REG_E_SCLOOP				0x040    // SC Loop Control Even Field
#define BT848_REG_O_SCLOOP				0x0C0    // SC Loop Control Odd Field

#define BT848_REG_WC_UP 				0x044    // White Crush Up
#define BT848_REG_OFORM 				0x048    // Output Format

#define BT848_REG_E_VSCALE_HI			0x04C    // MSB Vertical Scaling Even Field
#define BT848_REG_O_VSCALE_HI			0x0CC    // MSB Vertical Scaling Odd Field
#define BT848_REG_E_VSCALE_LO			0x050    // LSB Vertical Scaling Even Field
#define BT848_REG_O_VSCALE_LO			0x0D0    // LSB Vertical Scaling Odd Field

#define BT848_REG_TEST         		0x054    // Test Control Register
#define BT848_REG_ADELAY     			0x060    // AGC Delay
#define BT848_REG_BDELAY     			0x064    // Burst Delay
#define BT848_REG_ADC          		0x068    // ADC Interface

#define BT848_REG_E_VTC      			0x06C    // Video Timing Control Even Field
#define BT848_REG_O_VTC      			0x0EC    // Video Timing Control Odd Field

#define BT848_REG_SRESET     			0x07C    // Software Reset

#define BT848_REG_WC_DOWN				0x078    // White Crush Down

#define BT848_REG_TGLB       			0x080    // Timing Generator Load Byte
#define BT848_REG_TGCTRL      		0x084    // Timing Generator Control

#define BT848_REG_VTOTAL_LO  			0x0B0    // LSB Total Line Count
#define BT848_REG_VTOTAL_HI  			0x0B4    // MSB Total Line Count

#define BT848_REG_COLOR_FMT  			0x0D4    // Color Format
#define BT848_REG_COLOR_CTL  			0x0D8    // Color Control
#define BT848_REG_CAPTURE_CTL  		0x0DC    // Capture Control

#define BT848_REG_VBI_PACK_SIZE		0x0E0    // VBI Packet Size
#define BT848_REG_VBI_PACK_DEL		0x0E4    // VBI Packet Size / Delay

#define BT848_REG_FCAP       			0x0E8    // Field Capture Counter

#define BT848_REG_PLL_F_LO   			0x0F0    // LSB PLL Reference Multiplier
#define BT848_REG_PLL_F_HI   			0x0F4    // MSB PLL Reference Multiplier
#define BT848_REG_PLL_XCI    			0x0F8    // PLL Integer

#define BT848_REG_DVSIF      			0x0FC    // Digital Video Signal Interface Format

#define BT848_REG_INT_STAT   			0x100    // Interrupt Status
#define BT848_REG_INT_MASK   			0x104    // Interrupt Mask
#define BT848_REG_GPIO_DMA_CTL		0x10C    // GPIO and DMA Control
#define BT848_REG_I2C_DATA_CTL		0x110    // I2C Data / Control 

#define BT848_REG_RISC_START_ADDR	0x114    // Risc Program Start Address
#define BT848_REG_GPIO_OUT_EN   		0x118    // GPIO Output Enable Control
#define BT848_REG_RISC_COUNT    		0x120    // Risc Program Counter

#define BT848_REG_GPIO_DATA  			0x200    // GPIO Data I/O

#endif
