/*
	Pisces.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
 Log:
	05/10/02  Torbjrn Jansson <torbjorn.jansson@mbox200.swipnet.se>
			   -Fixed the compatibility issues with DScaler and all programs
			    that access the hardware directly... Thanks a lot!
*/


#include "CppRt.hpp"		// To get new and delete ops
#include "pisces.hpp"
#include "Hw.h"
#include "PCI.hpp"
#include "Registry.hpp"
#include "..\BtCard.hpp"

/* Power state definitions */
#define BT878_POWER_STATE_D3  3
#define BT878_POWER_STATE_D0  0

// I know what i am doing!
#pragma warning( disable: 4355 )

/*
 * BtPisces constructor
 */
BtPisces::BtPisces(
	BtCard*	cd,
	const PDEVICE_OBJECT pPDO,
	const PVOID HwDeviceExtension_,
	const DWORD_PTR dwBase) :

	PDO(pPDO),
	Card(cd),

#if DBG
	Signature(PISCES_SIGNATURE),
#endif

	// Powerup/down
	bPoweredDown(false),

	// All the parts a Bt848 has
	Engine_( HwDeviceExtension_,dwBase),
	Even_( this, VF_Even, dwBase, &COLOR_EVEN, &WSWAP_EVEN, &BSWAP_EVEN ),
	Odd_( this, VF_Odd, dwBase, &COLOR_ODD, &WSWAP_ODD, &BSWAP_ODD ),
	VBIE_( this, VF_Even ,dwBase ), 
	VBIO_( this, VF_Odd ,dwBase ), 
	PLL_( dwBase ),
	TimGen_( dwBase ),
	Decoder_( this, dwBase ) ,
	gpio_( this ,dwBase) ,			// Init GPIO 
	i2c_( this ,dwBase) ,		// Init I2C Bus

	ColorControl( 0x00D8 + dwBase , RW), 
	BSWAP_EVEN( ColorControl, 0, 1, RW), 
	BSWAP_ODD( ColorControl, 1, 1, RW), 
	WSWAP_EVEN( ColorControl, 2, 1, RW),
	WSWAP_ODD( ColorControl, 3, 1, RW), 

	InterruptStatus( 0x0100 + dwBase, RW), 
	FMTCHG( InterruptStatus, 0, 1, RR), 
	VSYNC( InterruptStatus, 1, 1, RR), 
	HSYNC( InterruptStatus, 2, 1, RR), 
	OFLOW( InterruptStatus, 3, 1, RR), 
	HLOCK( InterruptStatus, 4, 1, RR), 
	VPRES( InterruptStatus, 5, 1, RR), 
	I2CDONE( InterruptStatus, 8, 1, RR), 
	GPINT( InterruptStatus, 9, 1, RR), 
	RISCI( InterruptStatus, 11, 1, RR), 
	FBUS( InterruptStatus, 12, 1, RR), 
	FTRGT( InterruptStatus, 13, 1, RR), 
	FDSR( InterruptStatus, 14, 1, RR), 
	PPERR( InterruptStatus, 15, 1, RR), 
	RIPERR( InterruptStatus, 16, 1, RR), 
	PABORT( InterruptStatus, 17, 1, RR), 
	OCERR( InterruptStatus, 18, 1, RR), 
	SCERR( InterruptStatus, 19, 1, RR), 
	FIELD( InterruptStatus, 24, 1, RO), 
	RACK( InterruptStatus, 25, 1, RO), 
	V5IO( InterruptStatus, 26, 1, RO), 
	RISC_EN( InterruptStatus, 27, 1, RO), 
	RISCS( InterruptStatus, 28, 4, RO),

	InterruptMask( 0x0104 + dwBase , RW), 
	IMASK_FMTCHG( InterruptMask, 0, 1, RW), 
	IMASK_VSYNC( InterruptMask, 1, 1, RW), 
	IMASK_HSYNC( InterruptMask, 2, 1, RW), 
	IMASK_OFLOW( InterruptMask, 3, 1, RW), 
	IMASK_HLOCK( InterruptMask, 4, 1, RW), 
	IMASK_VPRES( InterruptMask, 5, 1, RW), 
	IMASK_I2CDONE( InterruptMask, 8, 1, RW), 
	IMASK_GPINT( InterruptMask, 9, 1, RW), 
	IMASK_RISCI( InterruptMask, 11, 1, RW), 
	IMASK_FBUS( InterruptMask, 12, 1, RW), 
	IMASK_FTRGT( InterruptMask, 13, 1, RW), 
	IMASK_FDSR( InterruptMask, 14, 1, RW), 
	IMASK_PPERW( InterruptMask, 15, 1, RW), 
	IMASK_RIPERW( InterruptMask, 16, 1, RW), 
	IMASK_PABORT( InterruptMask, 17, 1, RW), 
	IMASK_OCERW( InterruptMask, 18, 1, RW), 
	IMASK_SCERW( InterruptMask, 19, 1, RW), 

	ColorFormat( 0x00D4 + dwBase, RW), 
	COLOR_EVEN( ColorFormat, 0, 4, RW), 
	COLOR_ODD( ColorFormat, 4, 4, RW),

	CaptureControl( 0x00DC + dwBase, RW), 
	CAPTURE_EVEN( CaptureControl, 0, 1, RW), 
	CAPTURE_ODD( CaptureControl, 1, 1, RW), 
	CAPTURE_VBI_EVEN( CaptureControl, 2, 1, RW), 
	CAPTURE_VBI_ODD( CaptureControl, 3, 1, RW), 
	DITH_FRAME( CaptureControl, 4, 1, RW), 

	FCNTR(0xE8 + dwBase , RW ), // Field counter, only bt848A or better

	dwLastExecutedGroup(3) ,	// The last executed group...

	// Gamma
	dwGamma(1),

	// Store base address!
	dwBaseAddress(dwBase),

	i_lPendingEntries(0),		//
	i_MergedInts(0),			// No merged ints by default
	i_Dpc(PKDEFERRED_ROUTINE(cbDpcFunc),this) ,

	dwFields(0) ,
	RISC_Lock(0L),				// RISC Lock...

	dwEnabledInts(0),			// No extra enabled ints by now

	viCurrent(&VideoInfo[0])	// Current video information
{
	TRACE(("BtPisces::BtPisces"));

	// Make sure Bt878 is powered up...
	if (Card->IsBt878Family()) {
		SetBt878PowerState( BT878_POWER_STATE_D0 );
		// Let the hardware init!
		mdelay(500);
	}

	// Now, RESET adapter ... 
	Decoder_.SoftwareReset();

	// Software work-around for BANDGAP start-up yield problem:
	// Register XXX bit 7 powers up low. it must be turned on - a one -
	// then reset back to a low in order to properly start-up the bandgap. 
	{      
		const DWORD Reg_XXX_Offset = 0x058;

		DWORD regXXX = READ_REGISTER_ULONG(PULONG(Reg_XXX_Offset + dwBase));
		regXXX |= 0x80;   // turn on bit 7
		WRITE_REGISTER_ULONG(PULONG(Reg_XXX_Offset + dwBase),regXXX);
		regXXX &= ~0x80;  // turn off bit 7
		WRITE_REGISTER_ULONG(PULONG(Reg_XXX_Offset + dwBase),regXXX);
	}

	// mask all interrupts (but keep the "FIX" bit
	InterruptMask = dwEnabledInts = (Card->IsTriton848() ? EN_TRITON1_BUG_FIX : 0) ; 

	// And clear all Interrupt Status
	InterruptStatus = AllFs; 

	// Stop all!!! - But it shouldn't be necessary as the Bt8xx defaults to Stopped
	Engine_.Stop();

	// No dithering by default!
	DITH_FRAME = 0;

	//
	// Program layout:
	//
	// #  | Type           |Ints?|When |
	//----+----------------+-----+-----+
	// 0  | VBIO Data      | no  |IOdd |\
	// 1  | ODD  Data      | no  |VOdd | | Group 0
	// 2  | Sync VRO       | Yes |*Odd |/
	//----+----------------+-----+-----+
	// 3  | VBIE Data      | no  |IEven|\
	// 4  | EVEN Data      | no  |VEven| | Group 1
	// 5  | Sync VRE       | Yes |*Even|/
	//----+----------------+-----+-----+
	// 6  | VBIO Data      | no  |IOdd |\
	// 7  | ODD  Data      | no  |VOdd | | Group 2 
	// 8  | Sync VRO       | Yes |*Odd |/
	//----+----------------+-----+-----+
	//  9 | VBIE Data      | no  |IEven|\
	// 10 | EVEN Data      | no  |VEven| | Group 3
	// 11 | Sync VRE       | Yes |*Even|/
	//----+----------------+-----+-----+

	// Remember that the RISC Status is set at the END of the execution of each program! 
	// Note i use the RISCS to know the last executed group

	// Create all the RISC programs we could use.
	RISCProg[ 0]  = Engine_.CreateProgram(&VBIO_,RISCProgram::Sm_None  ,false);
	RISCProg[ 1]  = Engine_.CreateProgram(&Odd_ ,RISCProgram::Sm_None  ,false);
	RISCProg[ 2]  = Engine_.CreateProgram(SC_VRO,RISCProgram::Sm_SetTo0,true ); /* Sync to start of even field */

	RISCProg[ 3]  = Engine_.CreateProgram(&VBIE_,RISCProgram::Sm_None  ,false);
	RISCProg[ 4]  = Engine_.CreateProgram(&Even_,RISCProgram::Sm_None  ,false);
	RISCProg[ 5]  = Engine_.CreateProgram(SC_VRE,RISCProgram::Sm_SetTo1,true ); /* Sync to start of odd field */

	RISCProg[ 6]  = Engine_.CreateProgram(&VBIO_,RISCProgram::Sm_None  ,false);
	RISCProg[ 7]  = Engine_.CreateProgram(&Odd_ ,RISCProgram::Sm_None  ,false);
	RISCProg[ 8]  = Engine_.CreateProgram(SC_VRO,RISCProgram::Sm_SetTo2,true ); /* Sync to start of even field */

	RISCProg[ 9]  = Engine_.CreateProgram(&VBIE_,RISCProgram::Sm_None  ,false);
	RISCProg[10]  = Engine_.CreateProgram(&Even_,RISCProgram::Sm_None  ,false);
	RISCProg[11]  = Engine_.CreateProgram(SC_VRE,RISCProgram::Sm_SetTo3,true ); /* Sync to start of odd field */

	// By default, set normal mode for GPIO
	gpio_.SetGPIOMODE(GPIO::GPIO_NORMAL);

	// Instead of using default values, set some registers fields to optimum values
	Decoder_.SetOptimumDefaults();
	Decoder_.SetChromaAGC( true );
	Decoder_.SetLowColorAutoRemoval( false ); // Due to some requests, i am disabling low color autoremoval...
	Decoder_.SetAdaptiveAGC( false ); // Prevent Overflow CADC off

	// Set by default mode for I2C
	i2c_.SetFreq(100000);

	// And enable the I2C interrupts so we can use them!
	EnableUserInt(I2CDONE_I);
}

BtPisces::~BtPisces()
{
	TRACE(("BtPisces::~BtPisces"));

	// Prepare to be paged-out (disables ints) if we need to (we could be already powered down)
	DisableInts();
	
	// Destroy all the generated programs!
	for (int i=11 ; i >= 0; i--) {
		Engine_.DestroyProgram( RISCProg[i] ) ;
	}
}

// User interrupts access: 
// Notes: MUST BE CALLED WHEN VIDEO IS NOT STREAMING (At initialization time!)
void BtPisces::EnableUserInt(const DWORD nbr)
{
	// If all the requested ints are already enabled, just return!
	if ((dwEnabledInts & nbr) == nbr) {
		return;
	}

	// Now enable the requested ints! - Will be enabled as soon as 
	dwEnabledInts |= nbr;

	// Only access hardware if we are not powered down
	if (!bPoweredDown) {
		InterruptStatus = InterruptStatus | nbr;
		InterruptMask = InterruptMask | nbr;
	}
}

// Notes: MUST BE CALLED WHEN VIDEO IS NOT STREAMING (At initialization time!)
void BtPisces::DisableUserInt(const DWORD nbr)
{
	// If all the requested ints are already disabled, just return!
	if ((dwEnabledInts & nbr) == 0) {
		return;
	}

	// Now disable the requested ints! - Will be disabled as soon as 
	// a new video channel is created/destroyed!
	dwEnabledInts &= (~nbr);

	// Only access hardware if not powered down
	if (!bPoweredDown) {
		InterruptStatus = InterruptStatus & (~nbr);
		InterruptMask = InterruptMask & (~nbr);
	}

	// Be careful here,  because interrupts are posted DPC calls, so
	// we must be sure there are no pending nterrupts
	// to avoid getting a queued (and now invalid) int.
	while (i_lPendingEntries) {
		DUMP (( "Waiting until all pending ints are gone!\n"));
		// Let ints run while we wait.
		mdelay(10);
	};
}

// To get the number of fields 
BYTE BtPisces::GetFieldCount()
{
	if (Card->HasFCNTR()) {
		return BYTE(FCNTR);
	} 
	return BYTE(dwFields);
}

/* Note: This is called at DIRQL level */
BOOLEAN BtPisces::HwInterrupt( void )
{
	TRACE(("BtPisces::HwInterrupt"));

	// If we are Powered Down, we can't be source of Ints! (and all local registers were saved)
	if (bPoweredDown) {
		DUMP(("Got an int while powered down...\n"));

#if 0 // Not needed... But, who knows - better safe than to lock system...!
		// Just to be sure... If we suspect something...

		// Reset part
		Decoder_.SoftwareReset();

		// And disable ints (just to be sure)
		InterruptStatus = AllFs;
		InterruptMask = (Card->IsTriton848() ? EN_TRITON1_BUG_FIX : 0);	// mask all interrupts (but keep the "FIX" bit
#endif

		// As we must assume that register writes were not successfull (we were turned off), just let Windows handle it!
		return FALSE; // (and NO register reading is allowed!)
	}

	// Check if we are the responsables of this ... Remember we can share ints!
	DWORD dwIntStatus = InterruptStatus;

	// Get mask of interrupts enabled
	DWORD dwIntMask   = InterruptMask;

	DUMP(("IntrStatus = %x, IntrMask = %x\n", dwIntStatus , dwIntMask));

	// Did we generate that interrupt ?
	BOOLEAN b = ( (dwIntStatus & dwIntMask & 0xFFB3F) != 0 ) ? true : false;

	/* 1111 1111 1011 0011 1111 = FFB3F */

	// If we did that ...
	if( b ) { 

		// Acknowledge all the interrupts we have pending (means ALL)
		InterruptStatus = dwIntStatus;

		//  Queue is being used in DPC. We cant wait here (in the ISR) 
		// until queue is freed again, so we realize that , at least, 
		// HwInterrupt can't be called recursively. 
		//  Thanks to this, we can use the Insert method from the circular
		// Queue (FirstFree pointer of Queue is only touched here, FirstUsed
		// pointer of Queue is only touched in DPC.

		// Now queue the int (we will drop ints if Queue is not enough... Won't be so bad...
		i_MergedInts |= dwIntStatus;		// Merge all the int sources accumulated until now...

		DUMP(("Got a REAL interrupt: Status = 0x%08x\n",i_MergedInts));

		if (i_Queue.Insert(i_MergedInts)) {	// Try to insert interrupt sources...
			i_MergedInts = 0;				// If we were able to insert them, start from no int sources for next time.

			// We were able to queue the int, keep track of the number of pending ints..
			if (InterlockedIncrement((LONG*)&i_lPendingEntries) == 1) {

				// Post the DPC to process the int if no DPC is executing...
				ENSURE(KeGetCurrentIrql() >= DISPATCH_LEVEL );
				bool posted = i_Dpc.Post();
				ENSURE(posted);

			}
		}

	}
	return b;
}


/*
   BtPisces::cbDpcFunc
   Purpose: Callback to Schedule the DPC associated to the ISR
 */
 /* Note: this is called at IRQL = DISPATCH_LEVEL */
VOID BtPisces::cbDpcFunc (IN PKDPC Dpc , IN PVOID DeferredContext, IN PVOID SystemArgument1, IN PVOID SystemArgument2 )
{
	TRACE_EP(("BtDigitizer::cbDpcFunc"));

	// Make sure we have the correct this pointer!
	ENSURE((((BtPisces*)DeferredContext)->Signature == PISCES_SIGNATURE));

	// Call our Delayed procedure call (to avoid to be called at Raised IRQ level)
	((BtPisces*)DeferredContext)->DpcFunc();
}


// This is called to serve an interrupt generated by our board, but
// as a delayed Procedure call.
/* Note: this is called at IRQL = DISPATCH_LEVEL */
void BtPisces::DpcFunc()
{
	TRACE(("BtPisces::DpcFunc"));

	do {

		// Start without int sources...
		DWORD dwStatus = 0;

		// Get an int source... and process it.
		if (i_Queue.Remove(dwStatus)) {
			ENSURE(dwStatus != 0);

			// Execute interrupt handler
			Interrupt( dwStatus );
		}

	// Loop while there are requests to process ...
	} while (InterlockedDecrement((LONG*)&i_lPendingEntries) > 0);
}

ErrorCode BtPisces::DisableInts()
{
	TRACE(("BtPisces::DisableInts"));

	// Only access hardware if we are not powered down...
	if (!bPoweredDown) {

#if DBG
		if (Engine_.IsRunning()) {
			ERROR(("BT848 is capturing data, cancelling anyway!!!\n"));
		}
#endif

		// Stop execution... No videochannels should be open right now
		// or something is VERY bad!
		Engine_.Stop();				
	
		// Disable all interrupts (needed to page-out safely)
		UINT Times = 10000;
		while (InterruptMask & 0xFFB3F  && Times ) {
			DUMP (( "Trying to turn off ints!\n"));
			InterruptMask = (Card->IsTriton848() ? EN_TRITON1_BUG_FIX : 0);	// mask all interrupts (but keep the "FIX" bit
			Times--;
		};

		// Not able to turn off ints!
		if (!Times) {
			return Fail;
		}

		// Clear status
		InterruptStatus = AllFs;

		// Wait until all ints are done.
		while (i_lPendingEntries) {
			DUMP (( "Waiting until all pending ints are gone!\n"));
			mdelay(10); // Let other people run while we wait...
		};
	}

	// Prepared!
	return Success;
}

// Powerup/Down implementations
void BtPisces::SaveRegisterState( )
{
	TRACE(("BtPisces::SaveRegisterState"));

	// GPIO related stuff
	SaveRegister( BT848_REG_GPIO_OUT_EN );
	SaveRegister( BT848_REG_GPIO_DATA );

	// Save general regs.
	SaveRegister( BT848_REG_E_CROP );
	SaveRegister( BT848_REG_O_CROP );
	SaveRegister( BT848_REG_E_VDELAY_LO );
	SaveRegister( BT848_REG_O_VDELAY_LO );
	SaveRegister( BT848_REG_E_VACTIVE_LO );
	SaveRegister( BT848_REG_O_VACTIVE_LO );
	SaveRegister( BT848_REG_E_HDELAY_LO );
	SaveRegister( BT848_REG_O_HDELAY_LO );
	SaveRegister( BT848_REG_E_HACTIVE_LO );
	SaveRegister( BT848_REG_O_HACTIVE_LO );
	SaveRegister( BT848_REG_E_HSCALE_HI );
	SaveRegister( BT848_REG_O_HSCALE_HI );
	SaveRegister( BT848_REG_E_HSCALE_LO );
	SaveRegister( BT848_REG_O_HSCALE_LO );
	SaveRegister( BT848_REG_E_SCLOOP );
	SaveRegister( BT848_REG_O_SCLOOP );
	SaveRegister( BT848_REG_E_VSCALE_HI );
	SaveRegister( BT848_REG_O_VSCALE_HI );
	SaveRegister( BT848_REG_E_VSCALE_LO );
	SaveRegister( BT848_REG_O_VSCALE_LO );
	SaveRegister( BT848_REG_E_VTC );
	SaveRegister( BT848_REG_O_VTC );
	SaveRegister( BT848_REG_E_CONTROL );
	SaveRegister( BT848_REG_O_CONTROL );
	SaveRegister( BT848_REG_CONTRAST_LO );
	SaveRegister( BT848_REG_SAT_U_LO );
	SaveRegister( BT848_REG_SAT_V_LO );
	SaveRegister( BT848_REG_BRIGHT );
	SaveRegister( BT848_REG_HUE );

	SaveRegister( BT848_REG_DSTATUS );
	SaveRegister( BT848_REG_VTOTAL_LO );
	SaveRegister( BT848_REG_VTOTAL_HI );
	SaveRegister( BT848_REG_OFORM );
	SaveRegister( BT848_REG_TDEC );

	SaveRegister( BT848_REG_WC_UP );
	SaveRegister( BT848_REG_WC_DOWN );

	SaveRegister( BT848_REG_TEST );
	SaveRegister( BT848_REG_ADELAY );
	SaveRegister( BT848_REG_BDELAY );
	SaveRegister( BT848_REG_ADC );
		
	SaveRegister( BT848_REG_COLOR_FMT );
	SaveRegister( BT848_REG_COLOR_CTL );
	SaveRegister( BT848_REG_VBI_PACK_SIZE );
	SaveRegister( BT848_REG_VBI_PACK_DEL );
	SaveRegister( BT848_REG_FCAP ); 
	SaveRegister( BT848_REG_DVSIF );

	SaveRegister( BT848_REG_IFORM );
	SaveRegister( BT848_REG_PLL_F_LO );
	SaveRegister( BT848_REG_PLL_F_HI );
	SaveRegister( BT848_REG_PLL_XCI );
	SaveRegister( BT848_REG_TGCTRL ); 

	SaveRegister( BT848_REG_CAPTURE_CTL );
	SaveRegister( BT848_REG_INT_MASK );
	SaveRegister( BT848_REG_I2C_DATA_CTL );

	// Read the SRAM of the timing generator and save the program
	TimGen_.ReadSRAM(SavedSRAM,SIZEOF_ARRAY(SavedSRAM));
	
	// This is the last thing to do
	SaveRegister( BT848_REG_RISC_START_ADDR );
	SaveRegister( BT848_REG_GPIO_DMA_CTL );

#ifdef SAVE_PCI_CONFIG
	// Now remember the PCI configuration... Some Stupid drivers don't do it!
	PCI pci(PDO);
	pci.ReadConfigDWORD(0x04 ,dwSavedPCIConfig[0]); // Command and Status Register...
	pci.ReadConfigDWORD(0x0C ,dwSavedPCIConfig[1]); // Latency Timer register...
	pci.ReadConfigDWORD(0x10 ,dwSavedPCIConfig[2]); // Base Address 0 Registers...
	pci.ReadConfigDWORD(0x3C ,dwSavedPCIConfig[3]); // Interrupt Line, Interrupt Pin, Min_Gnt, Max_Lat Register...
	pci.ReadConfigDWORD(0x40 ,dwSavedPCIConfig[4]); // Device Control Register...
#endif
}

void BtPisces::RestoreRegisterState( void )
{
	TRACE(("BtPisces::RestoreRegisterState"));

#ifdef SAVE_PCI_CONFIG
	// Restore the PCI configuration (just in case they were not restored!...)
	PCI pci(PDO);
	pci.WriteConfigDWORD(0x04 ,dwSavedPCIConfig[0]); // Command and Status Register...
	pci.WriteConfigDWORD(0x0C ,dwSavedPCIConfig[1]); // Latency Timer register...
	pci.WriteConfigDWORD(0x10 ,dwSavedPCIConfig[2]); // Base Address 0 Registers...
	pci.WriteConfigDWORD(0x3C ,dwSavedPCIConfig[3]); // Interrupt Line, Interrupt Pin, Min_Gnt, Max_Lat Register...
	pci.WriteConfigDWORD(0x40 ,dwSavedPCIConfig[4]); // Device Control Register...
#endif

	// Reset part (Just to be sure we start fresh!)
	Decoder_.SoftwareReset();

	// Software work-around for BANDGAP start-up yield problem:
	// Register XXX bit 7 powers up low. it must be turned on - a one -
	// then reset back to a low in order to properly start-up the bandgap. 
	{      
		const DWORD Reg_XXX_Offset = 0x058;
		DWORD regXXX = READ_REGISTER_ULONG(PULONG(Reg_XXX_Offset + dwBaseAddress));
		regXXX |= 0x80;   // turn on bit 7
		WRITE_REGISTER_ULONG(PULONG(Reg_XXX_Offset + dwBaseAddress),regXXX);
		regXXX &= ~0x80;  // turn off bit 7
		WRITE_REGISTER_ULONG(PULONG(Reg_XXX_Offset + dwBaseAddress),regXXX);
	}

	// Now restore all saved registers

	// GPIO related stuff...
	RestoreRegister( BT848_REG_GPIO_OUT_EN );
	RestoreRegister( BT848_REG_GPIO_DATA );

	RestoreRegister( BT848_REG_E_CROP );
	RestoreRegister( BT848_REG_O_CROP );
	RestoreRegister( BT848_REG_E_VDELAY_LO );
	RestoreRegister( BT848_REG_O_VDELAY_LO );
	RestoreRegister( BT848_REG_E_VACTIVE_LO );
	RestoreRegister( BT848_REG_O_VACTIVE_LO );
	RestoreRegister( BT848_REG_E_HDELAY_LO );
	RestoreRegister( BT848_REG_O_HDELAY_LO );
	RestoreRegister( BT848_REG_E_HACTIVE_LO );
	RestoreRegister( BT848_REG_O_HACTIVE_LO );
	RestoreRegister( BT848_REG_E_HSCALE_HI );
	RestoreRegister( BT848_REG_O_HSCALE_HI );
	RestoreRegister( BT848_REG_E_HSCALE_LO );
	RestoreRegister( BT848_REG_O_HSCALE_LO );
	RestoreRegister( BT848_REG_E_SCLOOP );
	RestoreRegister( BT848_REG_O_SCLOOP );
	RestoreRegister( BT848_REG_E_VSCALE_HI );
	RestoreRegister( BT848_REG_O_VSCALE_HI );
	RestoreRegister( BT848_REG_E_VSCALE_LO );
	RestoreRegister( BT848_REG_O_VSCALE_LO );
	RestoreRegister( BT848_REG_E_VTC );
	RestoreRegister( BT848_REG_O_VTC );
	RestoreRegister( BT848_REG_E_CONTROL );
	RestoreRegister( BT848_REG_O_CONTROL );
	RestoreRegister( BT848_REG_CONTRAST_LO );
	RestoreRegister( BT848_REG_SAT_U_LO );
	RestoreRegister( BT848_REG_SAT_V_LO );
	RestoreRegister( BT848_REG_BRIGHT );
	RestoreRegister( BT848_REG_HUE );

	RestoreRegister( BT848_REG_DSTATUS ); 
	RestoreRegister( BT848_REG_VTOTAL_LO );
	RestoreRegister( BT848_REG_VTOTAL_HI );
	RestoreRegister( BT848_REG_OFORM );
	RestoreRegister( BT848_REG_TDEC );

	RestoreRegister( BT848_REG_WC_UP );
	RestoreRegister( BT848_REG_WC_DOWN );

	RestoreRegister( BT848_REG_TEST );
	RestoreRegister( BT848_REG_ADELAY );
	RestoreRegister( BT848_REG_BDELAY );
	RestoreRegister( BT848_REG_ADC );
		
	RestoreRegister( BT848_REG_COLOR_FMT );
	RestoreRegister( BT848_REG_COLOR_CTL );
	RestoreRegister( BT848_REG_VBI_PACK_SIZE );
	RestoreRegister( BT848_REG_VBI_PACK_DEL );
	RestoreRegister( BT848_REG_FCAP );
	RestoreRegister( BT848_REG_DVSIF );

	RestoreRegister( BT848_REG_IFORM );		// We select the proper Xtal (very important!)
	mdelay(100);							// Leave some time for stabilization purposes (Xt0 is selected as a clock source when the Reset() is applied)
	RestoreRegister( BT848_REG_PLL_F_LO );  // Prepare the PLL
	RestoreRegister( BT848_REG_PLL_F_HI );
	RestoreRegister( BT848_REG_PLL_XCI );	// And activate PLL if needed... We will have to wait until PLL locks... 
	mdelay(100);							// Leave some time for PLL lock (as this setting was used before, PLL should lock)
	RestoreRegister( BT848_REG_TGCTRL );

	RestoreRegister( BT848_REG_CAPTURE_CTL );
	RestoreRegister( BT848_REG_I2C_DATA_CTL );
	
	//Restore the SRAM of the timing generator
	TimGen_.LoadSRAM(SavedSRAM,SIZEOF_ARRAY(SavedSRAM));

	// Clear Interrupt status (start fresh)
	InterruptStatus = AllFs;

	// The following restoring ops must be the last so RISC engine is restarted properly
	RestoreRegister( BT848_REG_RISC_START_ADDR );
	RestoreRegister( BT848_REG_GPIO_DMA_CTL ); // So we reenable RISC engine when everything is up and running!

	// Enable ints last!
	RestoreRegister( BT848_REG_INT_MASK );		// Torbjrn Jansson <torbjorn.jansson@mbox200.swipnet.se>
}


/**************************************************************************\
* bool BtPisces::SetBt878PowerState
*
* Puts the PCI device into the given power state.
*
* Must be called at low priority
*
* History:
*  27-Oct-1998 -by- Tom Zakrajsek [tomz]
*   Wrote it.
*
\**************************************************************************/

ErrorCode BtPisces::SetBt878PowerState(DWORD dwNewPowerState )
{
	TRACE(("BtPisces::SetBt878PowerState"));

	PCI pci(PDO);

	DWORD dwVal;
	pci.ReadConfigDWORD(0x50 ,dwVal);

	DUMP(( "PCI: read state (%d)\n", dwVal & 3));
	dwVal &= ~3;            // Preserve all bits but 1:0
	dwVal |= dwNewPowerState;

	DUMP(( "PCI: wrote state (%d)\n", dwVal & 3));

	pci.WriteConfigDWORD(0x50 , dwVal);

	return Success;
}

/**************************************************************************\
* NTSTATUS BtPisces::PowerDown
*
* Powers down the PCI device.
*
* Must be called at low priority
*
* History:
*  10-Oct-1998 -by- Tom Zakrajsek [tomz]
*   Wrote it.
*
\**************************************************************************/

ErrorCode BtPisces::PowerDown( void )
{
	TRACE(("BtPisces::PowerDown"));
	ErrorCode ret = Success;

	// Save Bt8xx register set
	SaveRegisterState( );

	// Turn off all Bt action!
	DisableInts();

	/////////////////////////////////////////////////////////////////////
	// System Clock off, Sleep Y ADC, Sleep C ADC
	DWORD ulADCOnPDown = READ_REGISTER_ULONG( PULONG(dwBaseAddress + 0x68) );
	WRITE_REGISTER_ULONG( PULONG(dwBaseAddress + 0x68) , ulADCOnPDown | 0x0e) ;

	/////////////////////////////////////////////////////////////////////
	// ACPI - PLL fix
	// Appearant bug in Bt878A requires us to turn off the PLL before
	// powering down, otherwise our power consumption actually goes up!

	if (Card->HasPLL()) {
		WRITE_REGISTER_ULONG( PULONG(dwBaseAddress + BT848_REG_TGCTRL), 0 );
	}

	if (Card->IsBt878Family()) {
		ret = SetBt878PowerState( BT878_POWER_STATE_D3 );
	} 

	// Remember we are now Powered Down
	bPoweredDown = true; // (now we can't be a source of ints)

	return ret;
}

/**************************************************************************\
* NTSTATUS BtPisces::PowerUp
*
* Powers up the PCI device.
*
* Must be called at low priority
*
* History:
*  10-Oct-1998 -by- Tom Zakrajsek [tomz]
*   Wrote it.
*
\**************************************************************************/

ErrorCode BtPisces::PowerUp( void )
{
	TRACE(("BtPisces::PowerUp"));

	// Power up and then restore registers
	ErrorCode n;
	if (Card->IsBt878Family()) {
		n = SetBt878PowerState( BT878_POWER_STATE_D0 );
		// Let the hardware init!
		mdelay(500);
	} else {
		n = Success;
	}

	// Remember we are already Powered Up! (So we can service all the Pending ints when we enable them!)
	bPoweredDown = false;

	// Now restore registers (This will restore ints and powerup Device parts if required!
	RestoreRegisterState();

	return n;
}


// Returns the maximum lines per field of the current videoformat
DWORD BtPisces::MaxLinesPerField( )
{
	DWORD lines = viCurrent->Max_Lines;
    return lines;
}

// Returns time between fields in 100 ns units
LONGLONG BtPisces::TimeBetweenFields()
{
	LONGLONG timebf = viCurrent->TimeBetweenFields;
    return timebf;
}

// Returns the number of fields per second
ULONG BtPisces::FieldsPerSecondX100()
{
	ULONG fps = viCurrent->FieldsPerSecondX100;
    return fps;
}


// Returns the maximum frame for the current videoformat
void BtPisces::GetMaxFrame(RECT& r)
{
	r.top  = 0;
	r.left = 0;
	r.right = viCurrent->StdX;
	r.bottom = 2* (	viCurrent->Max_Lines  ); 	// To get Frame lines
}

/*
Top field in PAL is field 1, top field in NTSC is field 2. 
Odd = 1, Even = 2
-> Top field in PAL  is Odd field
-> Top field in NTSC is Even Field

  But, always fields are paired Odd-Even!!
*/

VidField BtPisces::GetTopField()
{
	TRACE(("BtPisces::GetTopField"));
    VidField fld = viCurrent->TopField;
    return fld;
}

/* ************************************************************************************* */
/* Method: BtPisces::OpenVideoChannel
 * Purpose: This function opens a VideoChannel with the given parameters
 */

VideoChannel* BtPisces::OpenVideoChannel
	( 
		const VCMode mode,			// Mode of opening
		VideoChanIface * xface,		// Data buffer provider - consumer
		const MRect& analogwin,		// AnalogWindow
		const MRect& digitalwin,	// DigitalWindow
		const ColFmt ColorFormat	// Color format
	)
{
	TRACE(("BtPisces::OpenVideoChannel"));

	DUMP(("Mode = %d, AnalogWin(%d,%d) DigitalWin(%d,%d)\n",mode,analogwin.right,analogwin.bottom,
		digitalwin.right,digitalwin.bottom));

	VideoChannel* vc = NULL;

	// Based on mode, try ...
	switch (mode) {

		case VCM_AutoOdd: 
			{
				// Do we need to interlace ?
				const int threshold = MaxLinesPerField( );
				if ( digitalwin.Height() > threshold ) {

					// Create a new interlaced channel.
					vc = new InterlacedVideoChannel( xface, mode, Odd_ , Even_ , this);

				} else {

					// No need for interlaced, open the odd field
					vc = new SingleVideoChannel( xface, mode, Odd_ );

				}

			}
			break;

		case VCM_AutoEven:
			{
				// Do we need to interlace ?
				const int threshold = MaxLinesPerField( );
				if ( digitalwin.Height() > threshold ) {

					// Create a new interlaced channel.
					vc = new InterlacedVideoChannel( xface, mode, Odd_ , Even_ , this);

				} else {

					// No need for interlaced, open the even field
					vc = new SingleVideoChannel( xface, mode, Even_ );

				}

			}
			break;

		case VCM_Odd: 
			{
				// Open the odd field
				vc = new SingleVideoChannel( xface, mode, Odd_ );

			}
			break;

		case VCM_Even:
			{
				// Open the even field
				vc = new SingleVideoChannel( xface, mode, Even_ );
			}
			break;

		case VCM_Interlaced: 
			{
				// Create a new interlaced channel.
				vc = new InterlacedVideoChannel( xface, mode, Odd_ , Even_ , this);
			}
			break;

		case VCM_Alternating:
			{
				// Open an alternating video channel
				vc = new AlternatingVideoChannel( xface, mode, Odd_ , Even_ , this);
			}
			break;

		case VCM_VBI:
			{
				// Have a problem Here: Forget analog window, it is unused. But 
				// Digital window specifies Timings instead of pixels, so we must 
				// convert that! - This is done in the VBIField class

				// VBI is always alternating!
				vc = new AlternatingVideoChannel( xface, mode, VBIO_ , VBIE_ , this);
			}
			break;

		case VCM_Virtual: // The virtual channel!
			{
				vc = new VirtualChannel( xface, mode );
			}
			break;

		default:
			ERROR(("Invalid Mode for BtPisces::OpenVideoChannel(%x)\n",mode));

	}

	// If successfull, try to set all parameters required (Channel is stopped)
	if (vc) {

		// Only if we were able to open the fields required ...
		ErrorCode ret = Fail;
		if (vc->WasOpen()) {
			// Try to set the right format for this : The order of calls is VERY important
			if (vc->SetAnalogWindow(analogwin) == Success) {
				if (vc->SetDigitalWindow(digitalwin) == Success) {
					if (vc->SetColorFormat(ColorFormat) == Success) {
						DUMP(("VideoChannel Opened Successfully\n"));
						ret = Success;
					}
				}
			}
		}

		// If some error, don't open channel!
		if (ret != Success) {
			DUMP(("VideoChannel couldnt be opened!\n"));
			delete vc;
			return NULL;
		}

	}

	return vc;
}

/* Method: BtPisces::CloseVideoChannel
 * Purpose: Stops video and closes that channel
 */

void BtPisces::CloseVideoChannel( VideoChannel * chan )
{
	TRACE(("BtPisces::CloseVideoChannel"));

	ENSURE(chan);

	// Stop channel first - This is a MUST!
	chan->Stop();

	// delete video channel!
	delete chan;
}

/* Method: BtPisces::SetPll
 * Purpose:
 * Input:   value: DWORD
 * Output:
 */

/* Frequency = (F_input / PLL_X) * PLL_I.PLL_F/PLL_C 
   PLL_X = Reference pre-divider (0=1, 1=2) 
   PLL_C = Post divider (0=6, 1=4)
   PLL_I = Integer input 
   PLL_F = Fractional input 
   
   F_input = 28.636363 MHz: 
   PAL (CLKx2 = 35.46895 MHz): PLL_X = 1, PLL_I = 0x0E, PLL_F = 0xDCF9, PLL_C = 0
*/

void BtPisces::SetPll(unsigned int fin, unsigned int fout)
{
	const int MAX_PLL_WAIT = 100;    // max wait is 100ms

	unsigned char fl, fh, fi;

	// No need for PLL if same frecuency!
	if (fin == fout) {
		return;
	}

    /* prevent overflows */
    fin/=4;
    fout/=4;

    fout*=12;
    fi=fout/fin;

    fout=(fout%fin)*256;
    fh=fout/fin;

    fout=(fout%fin)*256;
    fl=fout/fin;

	PLL_.SetPLL(fl,fh,fi,false,true);

	// After initialized PLL, need to read and clear PLL out of lock bit until
	// it is no longer set (may take ~100ms)
	DWORD dwTickCount = 0;
	while ( dwTickCount < MAX_PLL_WAIT ) {
		if ( Decoder_.IsPLLOutOfLock() )
			Decoder_.ResetPLLOutOfLock();
		else
			break;

		// we have to wait approx. 1ms between polls
		mdelay(1);

		dwTickCount++;

	};
}

void BtPisces::SetPllToSleep( )
{
	PLL_.SetPLL(0,0,0,false,false);
}

ErrorCode BtPisces::SetVideoFormat(const DWORD xtal0freq, const DWORD xtal1freq,const VideoFormat vf)
{
	TRACE(("BtPisces::SetVideoFormat"));
   
	// Set the current video standard pointer struct
	viCurrent= &VideoInfo[vf];

	// Find out required frequency to support norm
	DWORD f_req = viCurrent->FscX4;

	// Verify, based on chip type, the availability of a PLL and crystals.
	
	// Start without a PLL ( very important: Avoids decoder lockups!)
	TimGenSetDecInputClk( DecClk_XTAL );

	// If we have a PLL, use it! - Will give all the videoformats to a card if it has Bt848A or better!
	if (Card->HasPLL()) {

		// If the Crystal 0 can provide the required frequency, use it instead of PLL
		if ( abs( int(xtal0freq - f_req) ) < 1500 ) {

			// Just use the Crystal 0 frequency
			Decoder_.SetCrystal(Crystal_XT0);

			// And put the PLL to sleep!
			SetPllToSleep();

		} else
		// If the Crystal 1 can provide the required frequency, use it instead of PLL
		if ( abs( int(xtal1freq - f_req) ) < 1500 ) {

			// Just use the Crystal 1 frequency
			Decoder_.SetCrystal(Crystal_XT1);

			// And put the PLL to sleep!
			SetPllToSleep();

		} else
		// We need the PLL, set up and use it!
		{

			// Use the Crystal 0 frequency as the reference
			Decoder_.SetCrystal(Crystal_XT0);

			// Set up PLL
			SetPll(xtal0freq,f_req);

			// And set the PLL as the input clock!
			TimGenSetDecInputClk( DecClk_PLL );
		}

	} else {

		// Don't use PLL, cause we dont have it!
		SetPllToSleep(); // Disables PLL

		if ( (!xtal1freq) || 
			 abs( int (xtal0freq - f_req) ) <= abs( int(xtal1freq - f_req) ) ) { // prefer the XT0 if both apply

			// The XT0 is the one we need
			Decoder_.SetCrystal(Crystal_XT0);

		} else {

			// The XT1 is the one we need
			Decoder_.SetCrystal(Crystal_XT1);

		}

	}

	// If we are going to get digital video, prepare the Timing generator
	if (viCurrent->DigitalSignal && viCurrent->TGB_Table != TGB_None ) {

		// Load the required Timing generator table
		TimGenLoadSRAM(SRAMTable[viCurrent->TGB_Table]+1,SRAMTable[viCurrent->TGB_Table][0]);

		// We will use the required pixels as HReset
		Decoder_.SetHReset( viCurrent->HReset );

		// Enable the GPCLOCK as the decoder input clock
		TimGenSetDecInputClk( DecClk_GPCLK );

		// Set the Sync video reference
		Decoder_.SetSyncVideoRef( VRef_alignedCb );

		// Disable Sync Video Output
		Decoder_.EnableSyncOutput( false );

		// Set Video signal format to CCIR digital
		Decoder_.SetVideoSignalFormat( Signal_CCIR656 );

	} else {
		// We are trying to display analog Video. Select the analog video input

		// We will use 64 pixels as HReset
		Decoder_.SetHReset( 64 );

		// Set Video signal format to analog
		Decoder_.SetVideoSignalFormat( Signal_Analog );

	}

	// Disable luma full output range
	Decoder_.SetFullOutputRange( false );

	// Set the video format parameters...
	Decoder_.SetLineCount(	viCurrent->VTotal);
	Decoder_.SetAGCDelay(	viCurrent->ADelay);
	Decoder_.SetBurstDelay(	viCurrent->BDelay);
	Decoder_.SetChromaComb(	viCurrent->ChromaComb );

	// Set the analog Video Format
	if (Decoder_.SetVideoFormat( viCurrent->VF ) != Success) {
		// Won't happen
		return Fail;
	}
		
	// let the scaler/s know format has changed
	Even_.VideoFormatChanged();
	Odd_.VideoFormatChanged();
	VBIE_.VideoFormatChanged();
	VBIO_.VideoFormatChanged();

	return Success;
}

ErrorCode BtPisces::SetGamma( DWORD Data )
{ 
	// Ensure parameters are bounded
	if (Data > 280 || Data < 100) {
		return Fail;
	}
	dwGamma = Data; 
	return Success; 
}

/* Method: BtPisces::SetSharpness
 * Purpose:
 * Input:
 * Output: None
 */
ErrorCode BtPisces::SetSharpness( DWORD Data )
{
	Even_.SetSharpness( Data );
	Odd_.SetSharpness( Data );
	return Success;
}


ErrorCode BtPisces::SetAutoSharpness(bool Auto)
{
	if (Even_.SetAutoSharpness(Auto) != Success ||
		Odd_.SetAutoSharpness(Auto) != Success ) {
		return Fail;
	}
	return Success;
}


void BtPisces::TurnVFilter( State s )
{
   TRACE(("BtPisces::TurnVFilter"));
   Even_.TurnVFilter( s );
   Odd_.TurnVFilter( s );
}

// RISC programs
// This fns are called by the Field class to enable/disable field capture

void BtPisces::SetFieldEnableState(const VidField evenodd, bool isVBI, bool Enable)
{
	TRACE(("BtPisces::SetFieldEnableState"));

	// Get the mutex to modify RISC programs (this let us modify/use RISC programs in a
	// safe way and to modify the interrupt mask in a safe way)
	while (InterlockedIncrement((LONG*) &RISC_Lock) > 1) { // Means interrupt is using resources...
		DUMP(("Waiting to acquire RISC lock...\n"));

		InterlockedDecrement((LONG*) &RISC_Lock);
		// Leave some time to the other processes...
		mdelay(10);
	}

	// We were granted the RISC chain modify, stop engine to be able to do it.
	Engine_.Stop();

	// Now redo the interrupt mask setting, so we always set the mask we want (even
	// if restart has been called, as if we own the lock, the HwInt can't set the mask)
	InterruptMask = dwEnabledInts;

	// Now set the field capture register that we were asked to.
	if (isVBI) {
		if (evenodd == VF_Even) {
			// Even VBI field
			CAPTURE_VBI_EVEN = Enable;
		} else {
			// Odd VBI field
			CAPTURE_VBI_ODD = Enable;
		}
	} else {
		if (evenodd == VF_Even) {
			// Even Video field
			CAPTURE_EVEN = Enable;
		} else {
			// Odd Video field
			CAPTURE_ODD = Enable;
		}
	}

	// Now we must relink programs based upon our needs:
	//
	// Program layout:
	//
	// #  | Type           |Ints?|When |
	//----+----------------+-----+-----+
	// 0  | VBIO Data      | no  |IOdd |\
	// 1  | ODD  Data      | no  |VOdd | | Group 0
	// 2  | Sync VRO       | Yes |*Odd |/
	//----+----------------+-----+-----+
	// 3  | VBIE Data      | no  |IEven|\
	// 4  | EVEN Data      | no  |VEven| | Group 1
	// 5  | Sync VRE       | Yes |*Even|/
	//----+----------------+-----+-----+
	// 6  | VBIO Data      | no  |IOdd |\
	// 7  | ODD  Data      | no  |VOdd | | Group 2 
	// 8  | Sync VRO       | Yes |*Odd |/
	//----+----------------+-----+-----+
	//  9 | VBIE Data      | no  |IEven|\
	// 10 | EVEN Data      | no  |VEven| | Group 3
	// 11 | Sync VRE       | Yes |*Even|/
	//----+----------------+-----+-----+

	// Note: If a field has it's capture disabled, the associated programs MUST be skipped

	// Explanations:
	// We use a Quad Buffer Method to achieve Full frame capturing in all cases. Thats why we have 4
	// groups of programs ( 2 programs associated to the same data field , 2 video fields)
	// Each program is linked to the next one by a jump, except the last that is linked to the 
	// first one. 
	// If we don't execute the ISR fast enough, simply buffers are refilled more than one time. That's
	// not a problem, as we have not returned buffers to the stream class.

	// At the end of a group execution, we fire a RISC interrupt. The firing of
	// this RISC int is done at the end of execution of each program that wants to. This is used
	// to supply all those programs of the previous group that have already executed, a new buffer
	// to fill. Using only 1 int per group, we try to minimize the number of ints required to handle
	// the capturing process. (Should be only 50 per second for PAL, or 60 per second for NTSC.

	// Here it is the array of conditions to enable each program
	typedef struct {
		RegField* fc;	// must be enabled.
		RegField* sc;	// If not NULL, must be enabled.
	} RPEnabler;
	
	// This is the array of fields that must be enabled to include that program
	RPEnabler ea[12] = {
		{ &CAPTURE_VBI_ODD	, NULL			},
		{ &CAPTURE_ODD		, NULL			},
		{ &CAPTURE_VBI_ODD	, &CAPTURE_ODD	},
		{ &CAPTURE_VBI_EVEN	, NULL			},
		{ &CAPTURE_EVEN		, NULL			},
		{ &CAPTURE_VBI_EVEN	, &CAPTURE_EVEN },
		{ &CAPTURE_VBI_ODD	, NULL			},
		{ &CAPTURE_ODD		, NULL			},
		{ &CAPTURE_VBI_ODD	, &CAPTURE_ODD	},
		{ &CAPTURE_VBI_EVEN	, NULL			},
		{ &CAPTURE_EVEN		, NULL			},
		{ &CAPTURE_VBI_EVEN	, &CAPTURE_EVEN }
	};
	
	// Be careful here,  because interrupts are posted DPC calls, so
	// we must be sure there are no pending (queued) RISC interrupts
	// to avoid getting a queued (and now invalid) RISC int.
	while (i_lPendingEntries) {
		DUMP (( "Waiting until all pending (RISC) ints are gone!\n"));
		// Let ints run while we wait.
		mdelay(10);
	};

	// Eventually, taking into consideration that user ints are
	// sporadic, we will get out of this loop

	// Update the array of RISC programs enabled and unlink then
	ULONG f;
	for (f=0 ; f < 12 ; f++) {

		// Unlink all the programs first as a security measure
		Engine_.Unlink(RISCProg[f]);

		// And now update the array of enabled programs!
		RISCProgEnabled[f] = ( *ea[f].fc || (ea[f].sc && *ea[f].sc) );
	}

	// Find the first enabled program...
	for (f = 0; f < 12; f++) {
		if (RISCProgEnabled[f]) break;
	}

	// If there are no programs enabled, just go out.
	if (f >= 12) {

		DUMP(("No channels open, Stopping Engine\n"));
	
		// We take this opportunity to initialize the first group that should be considered to be executed.
		dwLastExecutedGroup = 3;

		// We don't need now any access to modify RISC chains, so release it!
		InterlockedDecrement(&RISC_Lock);

		// Go out!
		return;
	}

	
	// Rechain all the needed programs between them in the required order!
	RiscPrgHandle start = RISCProg[f]; 
	RiscPrgHandle curr  = start;
	RiscPrgHandle prev  = NULL;

	do {

		// Now the current program becomes the previous one
		prev = curr;

		// Get the next enabled program...
		do {
			f = (f + 1) % 12;
		} while (!RISCProgEnabled[f]); // We are absolutely sure at least we have 1 program enabled, so this loop will always find an enabled program...
		curr = RISCProg[f];

		// Chain the previous to the current program
		Engine_.Chain(prev,curr);

		// While we don't get again to the first program, go on!
	} while (curr != start);

	// From now on, this is more like a restart process!

	// We have right now all the required programs chained. We must find out 
	// the right program to start execution and we must handle some interesting cases:
	//	1) Maybe, the last program being executed does not exist anymore
	//	   because it is no more active. Simply select the first active
	//	   one from that position and start from it.
	//	2) Maybe, the last program being executed still is one of the active 
	//	   programs: Simply restart it!

	// Get the last group executed at the last int and estimate the right sync
	// program to call to restart the group that should be executing.
	// (from this program onwards, buffers were not returned to the kernel)
	DWORD dwCurrent = ((dwLastExecutedGroup * 3) + 2) % 12; 

	// We should try to resync to the start of a field. This is to avoid
	// syncronization failure interrupts. We can take as an advantage that we complete buffers
	// on a group basis (we also get new buffers on a group basis) and, due to this completion
	// on a group basis, we are able to resync to the start of each field without problems!

	// If the last program of the last executed group (a sync program) is NOT active 
	// (because we don't need it anymore),then it's a MUST that the other
	// previous sync one is. (because we must be capturing something if
	// we have reached this point)
	if ( !RISCProgEnabled[dwCurrent] )  {

		// Select the previous one as our sync.
		dwCurrent = (dwCurrent + 9) % 12;
		ENSURE( RISCProgEnabled[dwCurrent] );

		// Remember that this will be the new first program to start, and remember
		// the group that will start execution after it...
		dwLastExecutedGroup = dwCurrent / 3; // (Here we use truncation)
	}

	DUMP(("Going to (re)start from %d program...",dwCurrent));

	// Read the fields that are running.
	DWORD OldCap = CaptureControl;

	// No more fields to capture (this will restart process)
	CaptureControl = 0;

	// Recreate all programs only if enabled as a safe measure
	// (the disabled programs are not linked properly, so they shouldn't/need not to be recreated)
	for (f=0; f < 12 ; f++ ) {
		if ( RISCProgEnabled[f] ) {
			RISCProg[f]->Recreate();
		}
	}

#if DBG
	// Dump all the enabled programs to assist in debugging...
	for (f=0; f < 12 ; f++ ) {
		DUMP(("Dumping %s program #%d\n",RISCProgEnabled[f] ? "enabled" : "disabled" , f));
		RISCProg[f]->Dump();
	}
#endif

	// Reenable all fields that must be captured
	CaptureControl = OldCap;

	// Start with a clean status!
	InterruptStatus = RISC_I | RIPERR_I | PABORT_I | OCERR_I | SCERR_I | FBUS_I;

	// And allow ints to proceed as chaining is done - Note that 
	// Int_RISCInt and Int_Restart can't be called, as the RISC engine is
	// currently stopped, so no race conditions between Restart() and the following
	// set of the interrupt mask!
	InterlockedDecrement(&RISC_Lock);

	// Enable all the used interrupts before starting execution
	InterruptMask =   RISC_I | RIPERR_I | PABORT_I | OCERR_I | dwEnabledInts
					// | SCERR_I -> Do not enable it at the beginning...
					// | VSYNC_I -> Not needed 
					// | FBUS_I
					;

	// Start execution of the sync program of the last executed group to restart capture... (The last program of all 
	// groups always ReSyncs.
	Engine_.Start( RISCProg[ dwCurrent ] ); 

}
/*
	RIPERR_I and PABORT is not an enabled int, but it is used to restart
	OCERR_I and SCERR_I are enabled ints, and are used to erestart

 */

/* Method:  BtPisces::int_RISCInt
 * Purpose: Called when a RISC hardware interrupt has occurred. 
 *			We get the current executing group and we proceed.
 * Input:	
 * Output:	
 * Notes: Can be called ONLY from interrupt handler (Won't work from other places!)
 * Notes:	USED on hardware interrupt context!
 */
/* Note: this is called at IRQL = DISPATCH_LEVEL */
void BtPisces::int_RISCInt(void) // 
{
	TRACE(("BtPisces::int_RISCInt"));

	// Get the first program of the group of the last executing group at the last int.
	DWORD dwFrom = ((dwLastExecutedGroup * 3) + 3) % 12;

	// RISCS gives the number of the last ALREADY EXECUTED GROUP.
	dwLastExecutedGroup = (RISCS & 3);

	// Get the number of the first program of the currently executing group, so we acknowledge
	// up TO the first program of the current group, without including it
	DWORD dwTo   = ((dwLastExecutedGroup * 3) + 3) % 12;

	// Now:
	// dwFrom	points to the first program of the last executing Group at the last int
	// dwTo		points to the first program of the now executing Group - Go on program by program informing them!
	DUMP(("Previous program executing: %d, Current: %d\n", dwFrom, dwTo ));

	// Allow SCERR ints only if some programs executed and we have detected a valid
	// Video signal (so we will avoid firing lots of unuseful interrupts) !
#if 0
	if (dwFrom != dwTo && Decoder_.IsDeviceInHLock() ) { // Something executed and we have a valid video signal ...
		// If sync ints not already enabled... Enable them.
		if (!IMASK_SCERW) {	
			// Only modify if required (avoids rewriting if possible)
			IMASK_SCERW = 1;
		} 
	} else { // We assume an invalid signal is present...
		// If sync ints already enabled... Disable them.
		if (IMASK_SCERW) {	
			// Only modify if required (avoids rewriting if possible)
			IMASK_SCERW = 0;
		} 
	}
#endif

	// Now dispatch the interrupt events - if no program executed, don't execute it!
	while (dwFrom != dwTo) {

		// Only update enabled programs
		if (RISCProgEnabled[ dwFrom ]) {

			// Get a pointer to the program
			RiscPrgHandle prg = RISCProg[dwFrom];
	
			// Inform to this (already executed) program it has already been executed!
			DUMP(("RISC Program %d has executed!\n",dwFrom));
			prg->Interrupt();
		}

		// Go to the next program
		dwFrom = (dwFrom + 1) % 12;
	};
}

/* Method:  BtPisces::Int_Restart
 * Purpose: Called when a fault has occurred. Tries to restart
 * Input:	
 * Output:	
 * Notes: It MUST be called only from interrupt Handler (It won't work otherwise)
 * Notes:	USED on hardware interrupt context!
 * Notes: Access to InterruptMask is serialized by mLock, so no race conditions here!
 */

void BtPisces::Int_Restart(void)
{
	DUMP(("BtPisces::Int_Restart"));

	// Stop engine
	Engine_.Stop();

	// Read the fields that are running.
	DWORD OldCap = CaptureControl;

	// No more fields to capture
	CaptureControl = 0;

	// Get the last group executing at the last int and calculate the first
	// program that allows us to resync to the start of that group 
	// (from this program onwards, buffers were not returned to the kernel)
	DWORD dwCurrent = ((dwLastExecutedGroup * 3) + 2) % 12; 

	// We should try to resync to the start of a field if it is possible. This is to avoid
	// syncronization failure interrupts. We can take as an advantage that we complete buffers
	// on a group basis (we also get new buffers on a group basis) and, due to this completion
	// on a group basis, we are able to resync to the start of each field without problems!

	// If the first program of the group (a sync program) is NOT active 
	// (because we don't need it anymore),then it's a MUST that the other
	// previous sync one is. (because we must be capturing something if
	// we have reached this point)
	if ( !RISCProgEnabled[dwCurrent] )  {

		// Select the previous one as our sync.
		dwCurrent = (dwCurrent + 9) % 12;
		ENSURE( RISCProgEnabled[dwCurrent] );

		// Remember that this will be the new first program to start, and remember
		// the group it belongs to.
		dwLastExecutedGroup = dwCurrent / 3; // (Here we use truncation)
	}

	// Recreate all enabled programs
	for (DWORD i=0; i < 12 ; i++ ) {
		if ( RISCProgEnabled[i] ) {
			DUMP(("Recreating enabled program #%d\n",i));
			RISCProg[i]->Recreate();
		}
	}

	// Reenable all fields that must be captured
	CaptureControl = OldCap;

	// Start with a clean status!
	InterruptStatus = RISC_I | RIPERR_I | PABORT_I | OCERR_I | SCERR_I | FBUS_I;

	// Enable all the used interrupts before starting execution
	InterruptMask = RISC_I | RIPERR_I | PABORT_I | OCERR_I | dwEnabledInts
				  // | SCERR_I -> Do not enable it as the first measure
				  // | VSYNC_I -> Not needed
				  // | FBUS_I 
				  ;

	// Start execution of right program to sync and go on to the last executing program
	// at the last int.
	Engine_.Start( RISCProg[ dwCurrent ] ); 

}

/* Method:  BtPisces::Interrupt
 * Purpose: Called when a hardware interrupt has occurred
 * Input:	Status at interrupt
 * Output:	
 * Notes:	USED on hardware interrupt context!
 */

/* Note: this is called at IRQL = DISPATCH_LEVEL */
void BtPisces::Interrupt(DWORD status)
{
	TRACE(("BtPisces::Interrupt"));
	DUMP (("Status = %08x\n",status));

	// If vertical int, count fields
	if (status & VSYNC_I) {
		DUMP(("VSYNC\n"));

		// Count fields!
		dwFields++;
	}

	// If I2C int is received, signal it!
	if (status & I2CDONE_I) {
		DUMP(("I2CDONE\n"));
		i2c_.IntReceived();
	}

	// If GPIO int is received, signal it!
	if (status & GPINT_I) {
		DUMP(("GPINT\n"));
		gpio_.IntReceived();
	}

#if DBG

	// If some RISC program/s has executed, and we have got an int, process it!
	// (Should never happen ... We are not using RISC ints!
	if ( status & RISC_I )		DUMP(("RISC_I\n"));
	if ( status & FBUS_I )		ERROR(("FBUS\n"));
	if ( status & FTRGT_I )		ERROR(("FTRGT\n"));
	if ( status & FDSR_I )		ERROR(("FDSR\n"));
	// Parity
	if ( status & PPERR_I )		ERROR(("PPERR\n"));
	if ( status & RIPERR_I )	ERROR(("RIPERR\n"));
	// 
	if ( status & SCERR_I )		ERROR(("SCERR\n"));
	if ( status & PABORT_I )	ERROR(("PABORT\n"));
	if ( status & OCERR_I ) {

		ERROR(("OCERR\n"));
		// Invalid code, Dump programs!
		DUMP(("Status = %08x\n",status));

		// Dump all the enabled programs...
		for (DWORD id=0; id < 12 ; id++ ) {
			DUMP(("Dumping %s program #%d\n",RISCProgEnabled[id] ? "enabled" : "disabled" , id));
			RISCProg[id]->Dump();
		}
	}
#endif

	// If something RISC - related....
	if (status & ( RIPERR_I | PABORT_I | OCERR_I /*| SCERR_I*/ | RISC_I ) ) {

		// Only touch RISC things if we are allowed to
		if (InterlockedIncrement((LONG*)&RISC_Lock) == 1) { // We are exclusive owners...

			// Only if executing RISC code...
			if (Engine_.MustBeRunning()) {

				// If some of these errors,
				// must recreate programs and restart them!
				if ( status & (RIPERR_I | PABORT_I | OCERR_I /*| SCERR_I*/) ) {	

					ERROR(("Restarting Capture process\n"));

					// restart recreating programs if we have to
					Int_Restart();

				} else

				// RISC interrupt
				if (status & RISC_I) {

					// Keep all our tasks running!
					int_RISCInt();
				}

			}
		}
		InterlockedDecrement((LONG*)&RISC_Lock);
	}
}

bool BtPisces::IsDigitalSignal() const 
{ 
	return Card->IsDigitalSignal(); 
}

bool BtPisces::IsGeminiFamily() const 
{ 
	return Card->IsGeminiFamily() == TRUE; 
}

bool BtPisces::IsGeminiFamilyRevD() const 
{ 
	return Card->IsGeminiFamilyRevD() == TRUE; 
}

bool BtPisces::IsBt878Family() const
{
	return Card->IsBt878Family() == TRUE;
}

bool BtPisces::HasPEAK() const
{
	return Card->HasPEAK() == TRUE;
}


// This fn measures the time between to fields. This MUST NOT be called 
// when digitizing, and it is allowed to call it only at initialization time
// Returns the time in ms * 10: Don't try to get more resolution , as Bt8xx 
// won't give us much more than that as it is NOT locked to any video signal,
// so it is drifting. (approx +/- 0.1 ms)
ULONG BtPisces::TimeDefaultField(Crystal cr)
{
	TRACE(("BtPisces::TimeDefaultField"));

	// Setup for test...
	TimGenSetDecInputClk( DecClk_XTAL );		// Start without a PLL
	Decoder_.SetCrystal( cr );					// Crystal 0/1
	SetPllToSleep();
	Decoder_.SetVideoSignalFormat( Signal_Analog ); 	// Set Video signal format to analog
	Decoder_.SetFullOutputRange( false );				// Disable luma full output range
	SetVideoFormat( 28636363, 35468950, (cr == Crystal_XT0) ? VFormat_NTSC : VFormat_PAL_BDGHI);		// Now set the videoMode to NTSC/PAL
	
	// Calculate the addresses of the InterruptMask and the InterruptStatus
	// so we don't calculate them in the inner loops... We don't use the register classes
	// to avoid the performance hit related to it's use (minimal, but here it counts)
	PULONG INT_MASK_ADDR = (PULONG) (dwBaseAddress + 0x104);
	PULONG INT_STAT_ADDR = (PULONG) (dwBaseAddress + 0x100);

	// Get the interrupt mask
	DWORD OldMask = InterruptMask;

	// Disable all ints
	InterruptMask = 0;

	// Set the current Video mux to an unconnected entry if possible: Just try it
	ULONG Mux;
	for (Mux = 0; Mux < 4 ; Mux++) {

		// Try this Video mux
		Decoder_.SetVideoInput((3 - Mux),false); // Try to leave last the TV tuner

		// Let the Chip detect if there is video present here
		mdelay(20);

		// If video is not here, we found our mux!
		if (!Decoder_.IsVideoPresent()) {
			DUMP(("Found Video mux %d without video signal\n",3-Mux));
			break;
		}
	}

	// Disable locking to video signal.
	Decoder_.SetTEST(TEST_DisableLockToSignal);

	// Leave some time for stabilization purposes
	mdelay(50);

	// Get number of ticks per second
	LARGE_INTEGER TicksPerSecond;
	KeQueryPerformanceCounter(&TicksPerSecond);

	// Calculate the timeout of the loop (1/10 of a second)
	LARGE_INTEGER TimeOut;
	TimeOut.QuadPart = (TicksPerSecond.QuadPart / 10);

	// Repeat this timing 11 times to get a mean value
	UINT idx;
	LARGE_INTEGER PeriodDiffs[11]; // The period differences

	// Start timing!
	for (idx = 0; idx < 11; idx++) {

		// Get the current time to estimate the ending time
		LARGE_INTEGER endTime = KeQueryPerformanceCounter(NULL);
	
		// Allow 1/8 of a second as the timeout
		endTime.QuadPart += TimeOut.QuadPart;

		// Reserve the following variables here (outside of loop, so timimg loop is as fast as possible)
		LARGE_INTEGER startPeriod;
		LARGE_INTEGER endPeriod;

		// Raise IRQL so we don't get any kind of interrupts that might interfere with our timing
		KIRQL OldIrql;
		KeRaiseIrql(POWER_LEVEL,&OldIrql);

		// Clear the VSYNC bit on Interrupt status
		WRITE_REGISTER_ULONG(INT_STAT_ADDR,VSYNC_I);

		// #### Now wait until a VSYNC starts... to avoid lockups if failure, timeout
		while (1) {

			// Get the current time
			startPeriod = KeQueryPerformanceCounter(NULL);

			// If VSYNC bit is now set, VSYNC is starting: Start measuring
			if (READ_REGISTER_ULONG(INT_STAT_ADDR) & VSYNC_I) {

				// Clear the VSYNC bit on Interrupt status as it is set now
				WRITE_REGISTER_ULONG(INT_STAT_ADDR,VSYNC_I);

				break;
			}

			// If we have timed-out, exit
			if (startPeriod.QuadPart > endTime.QuadPart) {
	
				// Lower the IRQ level (unmasking ints)
				KeLowerIrql(OldIrql);

				// Reenable all the disabled ints
				InterruptMask = OldMask;
	
				// Set decoder to normal operation
				Decoder_.SetTEST(TEST_None);
	
				// Return : we were not able to measure 
				return 0;
			}
		};


		//## now measure the time between two VSYNC interrupts
		while (1) {

			// Get the current time
			endPeriod = KeQueryPerformanceCounter(NULL);

			// If VSYNC bit is now set, VSYNC is starting: Start measuring
			if (READ_REGISTER_ULONG(INT_STAT_ADDR) & VSYNC_I) {

				// Clear the VSYNC bit on Interrupt status as it is set now
				WRITE_REGISTER_ULONG(INT_STAT_ADDR,VSYNC_I);

				break;
			}

			// If we have timed-out, exit
			if (endPeriod.QuadPart > endTime.QuadPart) {
	
				// Lower the IRQ level (unmasking ints)
				KeLowerIrql(OldIrql);

				// Reenable all the disabled ints
				InterruptMask = OldMask;
	
				// Set decoder to normal operation
				Decoder_.SetTEST(TEST_None);
	
				// Return : we were not able to measure 
				return 0;
			}
		};

		// Lower the IRQ level (unmasking ints) just to give Win2000 a break.
		KeLowerIrql(OldIrql);

		// Leave some time so other threads under Win2000 can execute
		mdelay(1);

		// Calculate the period between VSYNCs
		PeriodDiffs[idx].QuadPart = endPeriod.QuadPart - startPeriod.QuadPart;
	};

	// Reenable all the disabled ints
	InterruptMask = OldMask;
	
	// Set decoder to normal operation
	Decoder_.SetTEST(TEST_None);

	ULONG Period[10];				// An array of each result
	

	// We estimate the time of each field: Note we ignore the firat one, as that one was used to load
	// loop code/data into the processor cache
	for (idx = 0; idx < 10; idx++) {
		Period[idx] = ULONG( (PeriodDiffs[idx+1].QuadPart * 10000i64) / TicksPerSecond.QuadPart);
		DUMP(( "Partial field timing = %d/10 ms x field\n", Period[idx] ));
	}

	// Go on suppresing the maximum deviation values until just one value remains
	ULONG Count;
	for (Count = 10; Count > 1; Count--) {

		// Estimate the standard deviation of each value
		ULONG MaxStdDev = 0;
		ULONG MaxStdDevIdx = 0;
		for (idx = 0; idx < Count ; idx ++) {
			// Get the mean value without including the current one
			LARGE_INTEGER mean; 
			UINT x;
			mean.QuadPart = 0;
			for (x = 0; x < Count ; x++) {
				if (x != idx) {
					mean.QuadPart += Period[x];
				}
			}
			mean.QuadPart /= (Count-1);

			// Remember the difference between this and the media
			ULONG StdDev = ULONG(abs( int(ULONG(mean.QuadPart) - Period[idx]) ));

			// If this item has more standard deviation than the others, remember it
			if (StdDev > MaxStdDev) {
				MaxStdDev = StdDev;
				MaxStdDevIdx = idx;
			}
		}

		// Suppress the item that has maximum std deviation
		RtlMoveMemory(Period + MaxStdDevIdx,
				Period + MaxStdDevIdx + 1, 
				(Count - MaxStdDevIdx - 1) * sizeof(ULONG) );
	}

	// Now get the value field timing in ms * 10
	ULONG rmean = Period[0];

	DUMP(( "Field timing = %d ms/10 x field\n", rmean ));
	return rmean;
}

#define VIDEOINFO_ENTRY( stdx, clkx1_hactive , clkx1_hdelay, max_pix, alpf, VDelay ,Fsc , vf , vtotal, adelay, bdelay, chromacomb , tbf , tf, fpsx100, dsig, tgb ,hreset,autohf ) \
{ \
	stdx,				/* standarized x resolution */ \
	clkx1_hactive,		/* Clkx1_HACTIVE          = 746 */ \
	clkx1_hdelay,		/* Clkx1_HDELAY           = 140 */ \
	(WORD) ( ((float)(clkx1_hactive)) / 16.999755859375 + 0.9999),  /* Min_Pixels             =  44 */ \
	max_pix ,			/* Max_Pixels             = ((Clkx1_HACTIVE < 774) ? Clkx1_HACTIVE - 6 : 768) */ \
	(WORD)((float)(alpf) / 17.01953125),	/* Min_Lines              = (Active_lines_per_field / 16 + 1) * 2 */ \
	alpf,				/* Max_Lines              = Active_lines_per_field */ \
	((clkx1_hactive > 796) ? 384 : (clkx1_hactive * 14 / 29)),     /* Max_VFilter1_Pixels    = ((Clkx1_HACTIVE > 796) ? 384 : (Clkx1_HACTIVE * 14 / 29)) */ \
	clkx1_hactive * 8 / 33,     /* Max_VFilter2_Pixels    = Clkx1_HACTIVE * 8 / 33 */ \
	clkx1_hactive * 8 / 33,     /* Max_VFilter3_Pixels    = Clkx1_HACTIVE * 8 / 33 */ \
	alpf,     /* Max_VFilter1_Lines     = Active_lines_per_field */ \
	alpf / 2, /* Max_VFilter2_Lines     = Active_lines_per_field / 2 */ \
	alpf * 2 / 5,      /* Max_VFilter3_Lines     = Active_lines_per_field * 2 / 5 */ \
	VDelay, /* VDelay */ \
	Fsc,		/* FscX4 */ \
	vf,		/* Real VideoFormat to use */ \
	vtotal, /* VTotal (if used) */ \
	adelay, /* ADelay */ \
	bdelay, /* BDelay */ \
	chromacomb, /* Chroma comb filter */ \
	tbf,			/* Time Between Fields */ \
	tf,				/* Top Field */ \
	fpsx100,			/* fields per second * 100 */ \
	dsig,				/* Digital Signal */ \
	tgb,				/* Time Generator Table */ \
	hreset,				/* HReset pulse */ \
	autohf				/* Auto HFiltering */ \
}

/*Min_lines = (WORD)((float)(alpf) / 17.01953125) */

/*
  General Info:
				lines       active      vertical    aspect      horizontal  frame rate  
							lines       resolution  ratio       resolution              
	NTSC        525         484         242         4/3         427         29.94       
	PAL         625         575         290         4/3         425         25          
	SECAM       625         575         290         4/3         465         25          

  NTSC Timings:
	Line period 63.5 us 
	Line blanking 10.7 +.7/-.2 uS
	Line sync 4.7 .1 uS
	Front porch 1.40 uS
	Burst start 5.08 +- 0.1 us after sync start.
	Active Video 52.8 us

  PAL Timings:
	Line period 64 us 
	Line blanking 12.05 +- 0.25 us
	Line sync 4.7 +- 0.1 us
	Front porch: 1.65 +- 0.1 us
	Burst start 5.6 +- 0.1 us after sync start.
	Active Video 52 us

  Bt8xx Calculations:
	HTOTAL  = XTal[Hz] * LinePeriod[s] / 2
	HACTIVE = XTal[Hz] * ActiveVideo[s] / 2
	HDELAY  = XTal[Hz] * (LineBlanking-FrontPorch)[s] / 2
    ADELAY  = ( 6.8 uS * 4*Fsc) + 7 (15 for Bt878)
    BDELAY  = ( 6.5 uS * 4*Fsc)


	This assumes that the falling edge of HRESET is syncronized to
  the falling edge of the sync signal of analog video.	

  Norm		HTotal	HActive HDelay	ADelay	BDelay
	NTSC:	 909	 756	133
	PAL_NC:  916	 745	149
	PAL:    1135	 922	184

  XTals
	28.636360 NTSC , PAL-M and NTSC-50
	35.468950 PAL (all the others)

 */

/* Note:
	StdX should be a multiple of 4 to get Cropping restrictions to be hold !
	*/
/*
 Previous values for VDelay: 0x16 = NTSC
							 0x20 = PAL
 New Values: (Assuming that the first line that starts the VSync is the line #1)
							 0x1A = NTSC (assumes video starts at 10+ 0x1a/2 = line 23.
							 0x22 = PAL (assumes video starts at 7+ 0x22/2   = line 24
 These new values are due to useful discussions with _Demo_
 */

// This is the video information table
BtPisces::VideoInfoStruct BtPisces::VideoInfo[] = {
 VIDEOINFO_ENTRY(752,756,133,756,240,0x1a,28636363,BtVFormat_NTSC      ,0  ,0x68,0x5c,true ,333667/2,VF_Odd ,5994 ,false, TGB_None   ,64,true ), /* NTSC */
 VIDEOINFO_ENTRY(752,756,133,756,240,0x1a,28636363,BtVFormat_NTSC_J    ,0  ,0x70,0x60,true ,333667/2,VF_Odd ,5994 ,false, TGB_None   ,64,true ), /* NTSC Japan */
 VIDEOINFO_ENTRY(768,922,184,768,288,0x22,35468950,BtVFormat_PAL_BDGHI ,0  ,0x7F,0x72,true ,400000/2,VF_Odd ,5000 ,false, TGB_None   ,64,true ), /* PAL BDGHI */
 VIDEOINFO_ENTRY(752,756,133,752,240,0x1a,28636363,BtVFormat_PAL_M     ,0  ,0x68,0x5c,true ,333667/2,VF_Odd ,5994 ,false, TGB_None   ,64,true ), /* PAL M */
 VIDEOINFO_ENTRY(768,922,184,768,288,0x22,35468950,BtVFormat_PAL_N     ,0  ,0x7F,0x72,true ,400000/2,VF_Odd ,5000 ,false, TGB_None   ,64,true ), /* PAL N */
 VIDEOINFO_ENTRY(768,922,184,768,288,0x22,35468950,BtVFormat_SECAM     ,0  ,0x7F,0xb0,false,400000/2,VF_Odd ,5000 ,false, TGB_None   ,64,false), /* SECAM */
 VIDEOINFO_ENTRY(736,745,149,736,288,0x22,28636363,BtVFormat_PAL_N_COMB,0  ,0x68,0x5c,true ,400000/2,VF_Odd ,5000 ,false, TGB_None   ,64,true ), /* PAL NC */
 VIDEOINFO_ENTRY(768,922,184,768,240,0x22,35468950,BtVFormat_PAL_BDGHI ,525,0x7F,0x72,true ,333667/2,VF_Odd ,5994 ,false, TGB_None   ,64,true ), /* PAL 60 */
 VIDEOINFO_ENTRY(752,756,133,752,288,0x22,28636363,BtVFormat_NTSC      ,625,0x68,0x5c,true ,400000/2,VF_Odd ,5000 ,false, TGB_None   ,64,true ), /* NTSC 50 */
 VIDEOINFO_ENTRY(720,720,128,720,240,0x1a,28636363,BtVFormat_NTSC	   ,0  ,0x68,0x5c,false,333667/2,VF_Odd ,5994 ,true , TGB_NTSC858,64,true ), /* NTSC Digital */
 VIDEOINFO_ENTRY(720,720,128,720,288,0x22,35468950,BtVFormat_PAL_BDGHI ,0  ,0x7F,0x72,false,400000/2,VF_Odd ,5000 ,true , TGB_PAL858 ,64,true ), /* PAL Digital */
 VIDEOINFO_ENTRY(512,512,100,512,240,0x13,19636363,BtVFormat_NTSC      ,0  ,0x68,0x5c,false,333667/2,VF_Odd ,5994 ,true , TGB_NTSC624,64,true ), /* SVI Digital */
 VIDEOINFO_ENTRY(640,640,128,640,240,0x1A,24545454,BtVFormat_NTSC      ,0  ,0x68,0x5c,false,333667/2,VF_Odd ,5994 ,true , TGB_NTSC780,64,true ), /* LVM Digital */
 VIDEOINFO_ENTRY(352,352, 32,352,288,0x08,28636363,BtVFormat_NTSC      ,607,0x7F,0x72,false,400000/2,VF_Odd ,5000 ,true , TGB_NTSC392,16,true ), /* RQS Digital */
};


// Note that a VCR requires LESS lines to be stable... Just REDUCE the number of available 


// Original Conexant settings were:

// NTSC / NTSC_J / PAL_M
// ADELAY     = 0x68;
// BDELAY     = 0x5D;
// VDELAY     = 0x1A;
// VLINES     = 240
// HACTIVE    = 730
// HDELAY     = 148
// MAX_Pixels = 724

// PAL_BDGHI / PAL_N / PAL_N_COMB -
// ADELAY     = 0x7F
// BDELAY     = 0x72
// VDELAY     = 0x22
// VLINES     = 284
// HACTIVE    = 914
// HDELAY     = 190
// MaxPixels  = 768

// PAL_N_COMB -
// ADELAY     = 0x7F
// BDELAY     = 0x72
// VDELAY     = 0x22
// VLINES     = 284
// HACTIVE    = 744
// HDELAY     = 150
// MaxPixels  = 738

// SECAM
// ADELAY     = 0x7F
// BDELAY     = 0xA0
// VDELAY     = 0x22
// VLINES     = 284
// HACTIVE    = 914
// HDELAY     = 190
// MaxPixels  = 768


//===========================================================================
// Timing generator SRAM table values
//===========================================================================

BYTE BtPisces::SRAMTable[][ 52 ] =
{
   // TGB_NTSC624
   {
      0x33, // size of table = 51
      0x0E, 0xC0, 0x00, 0x00, 0x90, 0x02, 0x03, 0x10, 0x03, 0x06,
      0x10, 0x04, 0x12, 0x12, 0x05, 0x02, 0x13, 0x04, 0x19, 0x00,
      0x04, 0x39, 0x00, 0x06, 0x59, 0x08, 0x03, 0x85, 0x08, 0x07,
      0x03, 0x50, 0x00, 0x91, 0x40, 0x00, 0x11, 0x01, 0x01, 0x4D,
      0x0D, 0x02, 0x03, 0x11, 0x01, 0x05, 0x37, 0x00, 0x37, 0x21, 0x00
   },
   // TGB_NTSC780
   {
      0x33, // size of table = 51
      0x0e, 0xc0, 0x00, 0x00, 0x90, 0xe2, 0x03, 0x10, 0x03, 0x06,
      0x10, 0x34, 0x12, 0x12, 0x65, 0x02, 0x13, 0x24, 0x19, 0x00,
      0x24, 0x39, 0x00, 0x96, 0x59, 0x08, 0x93, 0x85, 0x08, 0x97,
      0x03, 0x50, 0x50, 0xaf, 0x40, 0x30, 0x5f, 0x01, 0xf1, 0x7f,
      0x0d, 0xf2, 0x03, 0x11, 0xf1, 0x05, 0x37, 0x30, 0x85, 0x21, 0x50
   },
   // TGB_NTSC858 (NTSC CCIR656)
   {
      0x33, // size of table = 51
      0x0c, 0xc0, 0x00, 0x00, 0x90, 0xc2, 0x03, 0x10, 0x03, 0x06,
      0x10, 0x34, 0x12, 0x12, 0x65, 0x02, 0x13, 0x24, 0x19, 0x00,
      0x24, 0x39, 0x00, 0x96, 0x59, 0x08, 0x93, 0x83, 0x08, 0x97,
      0x03, 0x50, 0x30, 0xc0, 0x40, 0x30, 0x86, 0x01, 0x01, 0xa6,
      0x0d, 0x62, 0x03, 0x11, 0x61, 0x05, 0x37, 0x30, 0xac, 0x21, 0x50
   },
   // TGB_NTSC392
   // This table has been modified to be used for Fusion Rev D
   {
      0x2A, // size of table = 42
      0x06, 0x08, 0x04, 0x0a, 0xc0, 0x00, 0x18, 0x08, 0x03, 0x24,
      0x08, 0x07, 0x02, 0x90, 0x02, 0x08, 0x10, 0x04, 0x0c, 0x10,
      0x05, 0x2c, 0x11, 0x04, 0x55, 0x48, 0x00, 0x05, 0x50, 0x00,
      0xbf, 0x0c, 0x02, 0x2f, 0x3d, 0x00, 0x2f, 0x3f, 0x00, 0xc3,
      0x20, 0x00
   },
   // TGB_PAL858 (PAL CCIR656)
   {
    
	  0x2D, // size of table = 45
      0x36, 0x11, 0x01, 0x00, 0x90, 0x02, 0x05, 0x10, 0x04, 0x16, 
	  0x14, 0x05, 0x11, 0x00, 0x04, 0x12, 0xc0, 0x00, 0x31, 0x00, 
	  0x06, 0x51, 0x08, 0x03, 0x89, 0x08, 0x07, 0xc0, 0x44, 0x00,
      0x81, 0x01, 0x01, 0xa9, 0x0d, 0x02, 0x02, 0x50, 0x03, 0x37, 
	  0x3d, 0x00, 0xaf, 0x21, 0x00
   }
};

