/*
	PLL.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/////////////////////////////////////////////////////////////////////////////
//
//  Module:
//
//    PLL.cpp
//
//  Abstract:
//
//    Bt848A/Bt849 PLL Class API
//
//  DESIGN NOTES
//
//
/////////////////////////////////////////////////////////////////////////////

#include "pll.hpp"


//===========================================================================
// Bt848A/Bt849 PLL Class Implementation
//===========================================================================


/////////////////////////////////////////////////////////////////////////////
// Constructor
/////////////////////////////////////////////////////////////////////////////

PLL::PLL(const DWORD_PTR dwBase ):
	pllRegPLL_F_LO( ( dwBase + (0x3C + 0) * 4) + 0, RW ) ,
	pllRegPLL_F_HI( ( dwBase + (0x3D + 0) * 4) + 0, RW ) ,
	pllRegPLL_XCI ( ( dwBase + (0x3E + 0) * 4) + 0, RW ) ,
	pllFieldPLL_X ( pllRegPLL_XCI, 7, 1, RW) ,
	pllFieldPLL_C ( pllRegPLL_XCI, 6, 1, RW) ,
	pllFieldPLL_I ( pllRegPLL_XCI, 0, 6, RW)
{
}


/////////////////////////////////////////////////////////////////////////////
// Destructor
/////////////////////////////////////////////////////////////////////////////
PLL::~PLL()
{
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void PLL::SetPLL( BYTE f_lo, BYTE f_hi, BYTE i, bool c, bool x )
// Purpose: Set PLL register values to synthesize desired frequency
// Input:   BYTE f_lo - value for PLL_F_LO
//          BYTE f_hi - value for PLL_F_HI
//          BYTE i    - value for PLL_I
//          bool c    - true to set PLL_C; else clear PLL_C
//          bool x    - true to set PLL_X; else clear PLL_X
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void PLL::SetPLL(const BYTE f_lo,const BYTE f_hi,const BYTE i,const bool c,const bool x )
{
   pllRegPLL_F_LO = f_lo;
   pllRegPLL_F_HI = f_hi;
   pllFieldPLL_I  = i;
   pllFieldPLL_C  = ( c == false ) ? 0 : 1;
   pllFieldPLL_X  = ( x == false ) ? 0 : 1;
}




