/*
	Pll.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/
/////////////////////////////////////////////////////////////////////////////
//
//  Module:
//
//    pll.h
//
//  Abstract:
//
//    Header file Bt849 PLL Registers
//
/////////////////////////////////////////////////////////////////////////////


#ifndef __PLL_HPP
#define __PLL_HPP

#include "debug.hpp"
#include "regField.hpp"
#include "mytypes.hpp"

/////////////////////////////////////////////////////////////////////////////
// CLASS PLL
//
// Description:
//    This class encapsulates the PLL register fields in the Bt848A/Bt849
//		Bt878/Bt879/Bt880
//
// Methods:
//    // set various PLL registers
//    void SetPLL( BYTE f_lo, BYTE f_hi, BYTE i, bool c, bool x );
//
//    // get contents of PLL registers
//    BYTE GetPLL_F_LO( void );
//    BYTE GetPLL_F_HI( void );
//    BYTE GetPLL_I( void );
//    bool GetPLL_C( void );
//    bool GetPLL_X( void );
//
// Note:
//
/////////////////////////////////////////////////////////////////////////////

class PLL
{
	private:

		RegisterB pllRegPLL_F_LO;
		RegisterB pllRegPLL_F_HI;
		RegisterB pllRegPLL_XCI;
		RegField  pllFieldPLL_X;
		RegField  pllFieldPLL_C;
		RegField  pllFieldPLL_I;

	public:

		PLL(const DWORD_PTR dwBase );
		~PLL();

		// set various PLL registers
		void SetPLL(const BYTE f_lo,const BYTE f_hi,const BYTE i,const bool c,const bool x );

		// get contents of PLL registers
		BYTE GetPLL_F_LO( void );
		BYTE GetPLL_F_HI( void );
		BYTE GetPLL_I( void );
		bool GetPLL_C( void );
		bool GetPLL_X( void );
};

/////////////////////////////////////////////////////////////////////////////
// Method:  BYTE PLL::GetPLL_F_LO( void )
// Purpose: Obtain PLL_F_LO register value
// Input:   None
// Output:  None
// Return:  Content of PLL_F_LO
/////////////////////////////////////////////////////////////////////////////
inline BYTE PLL::GetPLL_F_LO( void )
{
   return BYTE(pllRegPLL_F_LO);
}

/////////////////////////////////////////////////////////////////////////////
// Method:  BYTE PLL::GetPLL_F_HI( void )
// Purpose: Obtain PLL_F_HI register value
// Input:   None
// Output:  None
// Return:  Content of PLL_F_HI
/////////////////////////////////////////////////////////////////////////////
inline BYTE PLL::GetPLL_F_HI( void )
{
   return BYTE(pllRegPLL_F_HI);
}

/////////////////////////////////////////////////////////////////////////////
// Method:  BYTE PLL::GetPLL_I( void )
// Purpose: Obtain PLL_I register value
// Input:   None
// Output:  None
// Return:  Content of PLL_I
/////////////////////////////////////////////////////////////////////////////
inline BYTE PLL::GetPLL_I( void )
{
   return BYTE(pllFieldPLL_I);
}

/////////////////////////////////////////////////////////////////////////////
// Method:  bool PLL::GetPLL_C( void )
// Purpose: Obtain PLL_C register value
// Input:   None
// Output:  None
// Return:  true if PLL_C is set; else false
/////////////////////////////////////////////////////////////////////////////
inline bool PLL::GetPLL_C( void )
{
  return (pllFieldPLL_C == 1);
}

/////////////////////////////////////////////////////////////////////////////
// Method:  bool PLL::GetPLL_X( void )
// Purpose: Obtain PLL_X register value
// Input:   None
// Output:  None
// Return:  true if PLL_X is set; else false
/////////////////////////////////////////////////////////////////////////////
inline bool PLL::GetPLL_X( void )
{
  return (pllFieldPLL_X == 1);
}


#endif __PLL_HPP








