/*
	PsColSpc.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "Debug.hpp"
#include "CppRt.hpp"
#include "MyTypes.hpp"
#include "PsColSpc.hpp"

/* Method: PsColorSpace::SetColorFormat
 * Purpose: Sets BtPisces color space converter to the given color
 * Input: eaColor: enum of type ColFmt
 * Output: None
 * Note: No error checking is done ( enum is checked by the compiler during compile
 *   The function writes to the XXXX register
*/

bool PsColorSpace::VerifyColorFormat( ColFmt val )
{
   // [WRK] - use constants here for valid formats
   switch( val ) {
      case 0:
      case 1:
      case 2:
      case 3:
      case 4:
      case 5:
      case 6:
      case 7:
      case 8:
      case 9:
      case 0xe:
         return( TRUE );
      default:
         ERROR(("Caught bad write to ColorFormat (0x%x)\n", val));
         return( FALSE );
   }
}   

void PsColorSpace::SetColorFormat( ColFmt eColor )
{
	// save for later use...
	ColorSpace::SetColorFormat( eColor );
	ByteSwap_ = 0;
 
	switch ( eColor ) {
	case CF_YUV9:
		eColor = CF_PL_411;
		break;
	case CF_YUV12:
	case CF_I420:
		eColor = CF_PL_422;
		break;
	case CF_UYVY:
		eColor = CF_YUY2;
		ByteSwap_ = 1;
		break;
	}

	// set the hardware
	if ( VerifyColorFormat(eColor) ) {
		Format_ = eColor;
	} else {
		Format_ = 7;
		ERROR(("Forcing invalid color format to 0x%x\n", Format_));
	}
}

