/*
	PsColSpc.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __PSCOLSPC_H
#define __PSCOLSPC_H

#include "CompReg.hpp"
#include "ColSpace.hpp"


/* Class: PsColorSpace
 * Purpose: This class interfaces to the Pisces HW. That is the only difference
 *    from its base class ColorSpace. Such division was created so ring3 capture
 *    driver can utilize ColorSpace's functionality
 * Attributes: Format_: Register & - reference to the color format register
 * Operations:
 *      void SetColorFormat( ColorFormat aColor );
 */
class PsColorSpace : public ColorSpace
{
	private:

		RegBase &Format_;
		RegBase &WordSwap_;
		RegBase &ByteSwap_;

	public:

		void SetColorFormat( ColFmt aColor );

		PsColorSpace( ColFmt aColForm, RegBase &aFormatReg, RegBase &WS, RegBase &BS ) :
			ColorSpace( aColForm ), Format_( aFormatReg ), WordSwap_( WS ), ByteSwap_( BS ) 
		{
		}

	protected:

		bool VerifyColorFormat( ColFmt val );
};

#endif
