/*
	RegField.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "debug.hpp"
#include "regfield.hpp"

/* Method: RegField::operator DWORD()
 * Purpose: Performs the read from a field of register
*/
RegField::operator DWORD()
{
   // for RO and RW do the actual read
   // get the register data and move it to the right position
   return ( Owner_ & Mask_) >> StartBit_ ;
}


/* Method: RegField::operator=
 * Purpose: performs the assignment to a field of register
 * Note:
   This function computes the mask to apply to the owner register to reset
   all bits that are part of a field. Mask is based on the start position and size
   Then it calculates the proper value from the passed argument ( moves the size
   number of bits to the starting position ) and ORs these bits in the owner register.
*/
DWORD RegField::operator=( DWORD dwValue )
{
	// if a register is read-only nothing is done. This is an error
	ENSURE ( GetRegisterType() != RO );

	// calculate the proper value from the passed argument ( move the size
	// number of bits to the starting position )
	DWORD dwFieldValue = (dwValue << StartBit_) & Mask_;

	// do not perform intermediate steps on the owner; rather use a temp and update
	// the owner at once
	DWORD dwRegContent = 0; // Assume a RR register

	// reset the relevant bits
	if ( GetRegisterType() != RR ) {
		dwRegContent = Owner_ & (~Mask_);
	}

	// OR these bits in the owner register.
	dwRegContent |= dwFieldValue;

	// And write it...
	Owner_ = dwRegContent;
	return dwValue;
}
