/*
	Register.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "debug.hpp"
#include "Register.hpp"

/* Method: RegisterB::operator DWORD()
 * Purpose: Performs the read from a byte register
*/
RegisterB::operator DWORD()
{
	// for RO and RW do the actual read
	return READ_REGISTER_UCHAR( LPBYTE(GetAddr()) );
}


/* Method: RegisterB::operator=
 * Purpose: performs the assignment to a byte register
*/
DWORD RegisterB::operator=( DWORD dwValue )
{
	// if a register is read-only nothing is done. This is an error
	ENSURE ( GetRegisterType() != RO );

	WRITE_REGISTER_UCHAR( LPBYTE(GetAddr()) , (BYTE)dwValue );
	return dwValue;
}

/* Method: RegisterW::operator DWORD()
 * Purpose: Performs the read from a word register
*/
RegisterW::operator DWORD()
{
	// for RO and RW do the actual read
	return READ_REGISTER_USHORT( LPWORD (GetAddr()) );
}


/* Method: RegisterW::operator=
 * Purpose: performs the assignment to a word register
*/
DWORD RegisterW::operator=( DWORD dwValue )
{
	// if a register is read-only nothing is done. This is an error
	ENSURE ( GetRegisterType() != RO );

	WRITE_REGISTER_USHORT( LPWORD( GetAddr() ), (WORD)dwValue );
	return dwValue;
}

/* Method: RegisterDW::operator DWORD()
 * Purpose: Performs the read from a dword register
*/
RegisterDW::operator DWORD()
{
	// for RO and RW do the actual read
	return READ_REGISTER_ULONG( LPDWORD( GetAddr() ) );
}


/* Method: RegisterDW::operator=
 * Purpose: performs the assignment to a dword register
*/
DWORD RegisterDW::operator=( DWORD dwValue )
{
	// if a register is read-only nothing is done. This is an error
	ENSURE ( GetRegisterType() != RO );

	WRITE_REGISTER_ULONG( LPDWORD( GetAddr() ), dwValue );
	return dwValue;
}
