/*
	Scaler.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#ifndef __SCALER_HPP
#define __SCALER_HPP

#include "debug.hpp"
#include "mytypes.hpp"
#include "Decoder.hpp"
#include "CompReg.hpp"

// Forward declarations
class BtPisces;

/////////////////////////////////////////////////////////////////
//
// for pisces, instantiate as ...
//
// Scaler evenScaler( VF_Even );
// Scaler oddScaler( VF_Odd );
//
/////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
// CLASS Scaler
//
// Description:
//    This class encapsulates the register fields in the scaler portion of
//    the Bt848.
//    A complete set of functions are developed to manipulate all the
//    register fields in the scaler registers for the Bt848.
//
// Methods:
//    See below
//
// Note:
//    For Bt848, instantiate as ...
//       Scaler evenScaler(dwBase, VF_Even );
//       Scaler oddScaler(dwBase, VF_Odd );
//
/////////////////////////////////////////////////////////////////////////////

class Scaler
{
	public:

		Scaler(BtPisces* p,const VidField,const DWORD_PTR);
		~Scaler();

		void			VideoFormatChanged( void );
                             
		ErrorCode		SetAnalogWin( const MRect & );
		const MRect&	GetAnalogWin() const { return AnalogWin_; }
		ErrorCode		SetDigitalWin( const MRect & );
		const MRect&	GetDigitalWin() const { return DigitalWin_; }

		void			TurnVFilter( const State st ) { VFilterFlag_ = st; }

		ErrorCode		SetSharpness( const ULONG Sharpness);
		ULONG			GetSharpness() const { return m_sharpness; }
		ErrorCode		SetAutoSharpness(const bool Auto);
		bool			GetAutoSharpness() const { return m_sharpauto; }

	protected:

		//===========================================================================
		// Scaler registers
		//===========================================================================
		RegisterB regCROP;
		RegField  fieldVDELAY_MSB;
		RegField  fieldVACTIVE_MSB;
		RegField  fieldHDELAY_MSB;
		RegField  fieldHACTIVE_MSB;
		RegisterB regVDELAY_LO;
		RegisterB regVACTIVE_LO;
		RegisterB regHDELAY_LO;
		RegisterB regHACTIVE_LO;
		RegisterB regHSCALE_HI;
		RegField  fieldHSCALE_MSB;
		RegisterB regHSCALE_LO;
		RegisterB regSCLOOP;
		RegField  fieldHFILT;
		RegField  fieldPEAK;
		RegisterB regVSCALE_HI;
		RegField  fieldVSCALE_MSB;
		RegField  fieldCOMB;
		RegisterB regVSCALE_LO;
		RegisterB regVTC;
		RegField  fieldVFILT;
		CompositeReg regVDelay;
		CompositeReg regVActive;
		CompositeReg regVScale;
		CompositeReg regHDelay;
		CompositeReg regHActive;
		CompositeReg regHScale;
		RegisterB regCONTROL;
		RegField  fieldLNOTCH;
		RegField  fieldLDEC;
		RegField  fieldCBSENSE;
		RegisterB regReverse_CROP;    // see comment below

		// From above:  since VDelay register in hardware is reversed;
		// i.e. odd reg is really even field and vice versa, need an
		// extra cropping reg for the opposite field

		// Rectangles
		MRect	   AnalogWin_;
		MRect	   DigitalWin_;


		// member functions to set scaling registers
		void Scale( void );

		// Tell us if we must try to suppress junk lines at start
		State			VFilterFlag_;

		// Store the Settings
		bool			m_sharpauto;
		ULONG			m_sharpness;

		// Pisces pointer
		BtPisces*		pisces;
};


#endif __SCALER_H








