/*
	VideoDataBuf.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "Debug.hpp"
#include "VideoDataBuf.hpp"

/*
 * Function: GetPhysAddr
 * Purpose: To get the physical address of an offset in the Data buffer. 
 */
DWORD_PTR VideoDataBuf::GetBufPhysAddr(
	const LONG_PTR lOffset,					// Offset dentro del Buffer
	ULONG& len							// Lenght remaining
) const
{
	// TRACE(("VideoDataBuf::GetPhysAddr"));
#if 0

	// I think this is the safest way to proceed, but the other way will work also and will be faster,
	const PVOID Addr = PVOID(lOffset + LONG_PTR(GetSrb()->CommandData.DataBufferArray->Data));
	const DWORD_PTR ret = StreamClassGetPhysicalAddress(
				GetSrb()->HwDeviceExtension,
				GetSrb(),
				Addr, // Virtual Address to translate
				SRBDataBuffer,
				&len ).LowPart;

	// DUMP(("Linear Address: %x , Physical Address: %x, Len= %x\n",Addr,ret,len));
	return ret;

#else

	// This is our attempt to speed up translation
#if DBG
	const PVOID dAddr = GetBufAddr(lOffset);
#endif

	PHW_STREAM_REQUEST_BLOCK  pSrb = GetSrb();
	const DWORD_PTR BufferOffset = lOffset; // Calculate offset into buffer

	// Avoid getting out of the buffer!
	if (lOffset >= LONG(GetBufSize()) ) { /* LONG maximum is far more than enough for databuffer size! */
		ERROR(("Offset outside of buffer!\n"));

		// Invalid offset, no physical addr!
		len = 0;
		return 0;
	}

	DWORD_PTR EntryOffset		= 0;
	DWORD_PTR Entries			= pSrb->NumberOfPhysicalPages;
	PKSSCATTER_GATHER Entry	= pSrb->ScatterGatherBuffer;
	while (Entries) {
		if ( BufferOffset >= EntryOffset &&
			BufferOffset < (EntryOffset + Entry->Length) ) { // Found it!
	
			len = ULONG(Entry->Length) - ULONG(BufferOffset - EntryOffset);
			const DWORD_PTR ret = DWORD_PTR(Entry->PhysicalAddress.LowPart) + (BufferOffset - EntryOffset);
			// DUMP(("Linear Address: %x , Physical Address: %x, Len= %x\n",dAddr,ret,len));

			return ret;
		}
		EntryOffset += Entry->Length;
		Entry++;
		Entries--;
	};

	// Failed, revert to StreamClassGetPhysicalAddr (Give class a chance to allocate!)
	const PVOID Addr = PVOID(lOffset + LONG_PTR(GetSrb()->CommandData.DataBufferArray->Data));
	const DWORD_PTR ret = StreamClassGetPhysicalAddress(
				GetSrb()->HwDeviceExtension,
				GetSrb(),
				Addr, // Virtual Address to translate
				SRBDataBuffer,
				&len ).LowPart;

	// DUMP(("Linear Address: %x , Physical Address: %x, Len= %x\n",Addr,ret,len));
	if (!ret) {
		len = 0;
	}

#if DBG
	if (!ret) {
	ERROR(("No Physical -> Linear Mapping for this DataBuf?\n"));
	EntryOffset		= 0;
	Entries			= GetSrb()->NumberOfPhysicalPages;
	Entry	= GetSrb()->ScatterGatherBuffer;
	DUMP(("lOffset = %08x , Linear Address = %08x\n",lOffset,Addr));
	DUMP(("Offset To Translate: %08x\nDumping Table (%d entries):\n",BufferOffset,Entries));
	while (Entries) {
		DUMP(("EntryOffset = %08x, Physical Address = %08x, Length = %d (%08x)\n", EntryOffset, Entry->PhysicalAddress.LowPart, Entry->Length , Entry->Length ));
		if ( BufferOffset >= EntryOffset &&
			BufferOffset < (EntryOffset + Entry->Length) ) { // Found it!
			len = ULONG(Entry->Length) - ULONG(BufferOffset - EntryOffset);
			const DWORD ret=DWORD (Entry->PhysicalAddress.LowPart) + (BufferOffset - EntryOffset);
			DUMP(("Got it ...Linear Address: %x , Physical Address: %x, Len= %x\n",Addr,ret,len));
		}
		EntryOffset += Entry->Length;
		Entry++;
		Entries--;
	};
	}
#endif

	return ret;

#endif
}
