/*
	VideoDataBuf.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef VIDEODATABUF_HPP_
#define VIDEODATABUF_HPP_

#include "debug.hpp" 
#include "CppRt.hpp"
#include "MyTypes.hpp"

// Forward decls
class VideoChanIface;

/*
 * VideoDataBuf: Class used to pass Buffer info to the other 
 *			classes from the VideoChannel
 */

class VideoDataBuf {

	private:
		
		PHW_STREAM_REQUEST_BLOCK	pSrb;			// Pointer to the Srb that holds the buffer
		DWORD						dwPitch;		// Pitch of buffer
		VideoChanIface*				vcOwner;		// Pointer to the Stream owner of the Buffer

	public:
		// Default constructor
		VideoDataBuf() :
			pSrb(NULL) , dwPitch(0) , vcOwner(NULL) {}

		// Constructor
		VideoDataBuf(VideoChanIface* vcOwner_, const PHW_STREAM_REQUEST_BLOCK pSrb_ , DWORD dwPitch_ ) :
			pSrb(pSrb_) , dwPitch(dwPitch_) , vcOwner(vcOwner_) {}

		// Copy constructor
		VideoDataBuf(const VideoDataBuf& src) : 
			pSrb(src.pSrb) , dwPitch(src.dwPitch) , vcOwner(src.vcOwner) {}

		// Copy operator
		VideoDataBuf& operator=(const VideoDataBuf& src) 
		{
			pSrb = src.pSrb;
			dwPitch = src.dwPitch;
			vcOwner = src.vcOwner;
			return *this;
		}

		// To get the linear addr of the databuffer
		PVOID GetBufAddr(LONG_PTR lOffset = 0 ) const { return PVOID(lOffset + LONG_PTR(GetSrb()->CommandData.DataBufferArray->Data)); }

		// To get the physicxal addr of the databuffer
		DWORD_PTR GetBufPhysAddr(
			const LONG_PTR lOffset,					// Offset dentro del Buffer
			ULONG& len							// Lenght remaining
			) const;

		// To get the Buffer size
		DWORD GetBufSize() const {
			return GetSrb()->CommandData.DataBufferArray->FrameExtent;
		}

		// Access Members
		PHW_STREAM_REQUEST_BLOCK	GetSrb() const { return pSrb; } 
		DWORD						GetPitch() const { return dwPitch; }
		bool						RealBuffer() const { return pSrb != NULL; }
		VideoChanIface*				GetOwner() const { return vcOwner; }

};



#endif