/*
	MyTypes.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __MYTYPES_H
#define __MYTYPES_H

#include "WDMHeaders.hpp"
#include "CppRt.hpp"	// To get new and delete ops

#ifndef FIELDOFFSET
   #define  FIELDOFFSET(type, field)   ((int)(&((type *)1)->field)-1)
#endif

#define KS_SIZE_PREHEADER2 (FIELD_OFFSET(KS_VIDEOINFOHEADER2,bmiHeader))
#define KS_SIZE_VIDEOHEADER2(pbmi) ((pbmi)->bmiHeader.biSize + KS_SIZE_PREHEADER2)


// Needed for VC4.x compile (_MSC_VER is defined as 1100 for MSVC 5.0) 
#if _MSC_VER < 1100
//  #pragma message  ("***  MSVC 4.x build ***")

  #define bool  BOOL
  #define true  TRUE
  #define false FALSE
  #define VC_4X_BUILD 
#else
//  #pragma message  ("***  MSVC 5.0 or better build ***")
  #undef  VC_4X_BUILD
#endif

typedef enum { Success, Fail } ErrorCode;

// copied from mmsystem.h
#define mmioFOURCC( ch0, ch1, ch2, ch3 )                                \
                ( (DWORD)(BYTE)(ch0) | ( (DWORD)(BYTE)(ch1) << 8 ) |    \
                ( (DWORD)(BYTE)(ch2) << 16 ) | ( (DWORD)(BYTE)(ch3) << 24 ) )


#define FCC_YUY2    mmioFOURCC( 'Y', 'U', 'Y', '2' )
#define FCC_Y41P    mmioFOURCC( 'Y', '4', '1', 'P' )
#define FCC_Y8      mmioFOURCC( 'Y', '8', ' ', ' ' )
#define FCC_422     mmioFOURCC( '4', '2', '2', ' ' )
#define FCC_411     mmioFOURCC( '4', '1', '1', ' ' )
#define FCC_YVU9    mmioFOURCC( 'Y', 'V', 'U', '9' )
#define FCC_YV12    mmioFOURCC( 'Y', 'V', '1', '2' )
#define FCC_VBI     0xf72a76e0L  //mmioFOURCC( 'V', 'B', 'I', ' ' )
#define FCC_UYVY    mmioFOURCC( 'U', 'Y', 'V', 'Y' )
#define FCC_RAW     mmioFOURCC( 'R', 'A', 'W', ' ' )
#define FCC_I420    mmioFOURCC( 'I', '4', '2', '0' )

#define abs( x ) \
	( ( (x) < 0 ) ? -(x) : (x) ) 

class MSize;
class MRect;

class  MPoint : public tagPOINT {
	public:
		// Constructors
		MPoint()				{ x = 0; y = 0;}
		MPoint(int x_,int y_)	{ x = x_; y = y_; }
		MPoint(const POINT& p)	{ x = p.x; y = p.y; }
		MPoint(const MPoint& m) { x=m.x; y=m.y;}
		// Operators
		MPoint& operator=( const MPoint& m) { x = m.x; y = m.y; return *this; }
		MPoint& operator=( const POINT& m) { x = m.x; y = m.y; return *this; }
};

//
// class MSize
// ----- -----
//
class  MSize : public tagSIZE {
	public:
		// Constructors
		MSize() { cx = 0; cy = 0;}
		MSize(const SIZE& s) { cx = s.cx; cy = s.cy; }
		MSize(const MSize& s) { cx = s.cx; cy = s.cy; }
		MSize(int dx, int dy) {cx = dx; cy = dy;}
		void Set( int dx, int dy ) { cx = dx; cy = dy; }
		// Operators
		MSize& operator=( const SIZE& s) { cx = s.cx; cy = s.cy; return *this; }
		MSize& operator=( const MSize& s) { cx = s.cx; cy = s.cy; return *this; }
};


class  MRect : public tagRECT {
	public:
		// Constructors
		MRect() { right = left = top = bottom = 0; }
		MRect( int _left, int _top, int _right, int _bottom );
		MRect( const MPoint& origin, const MSize& extent );
		MRect( const struct tagRECT &orgn );

		void Set( int _left, int _top, int _right, int _bottom );

		// Information/access functions(const and non-const)
		const MPoint& TopLeft() const {return *(MPoint*)&left;}
		int           Width() const {return right-left;}
		int           Height() const {return bottom-top;}
		const MSize   Size() const  {return MSize( Width(), Height() ) ;}
		bool IsEmpty() const;
		bool IsNull() const;

		// Operators
		MRect& operator=( const RECT& s) { top = s.top; bottom = s.bottom; left = s.left; right = s.right; return *this; }
		MRect& operator=( const MRect& s) { top = s.top; bottom = s.bottom; left = s.left; right = s.right; return *this; }

		// Comparison ops
		friend bool operator==(const MRect& o, const MRect& s);
		friend bool operator!=(const MRect& o, const MRect& s);

};

inline bool operator==(const MRect& o, const MRect& s) { return ((s.right==o.right) && (s.left==o.left) && (s.top==o.top) && (s.bottom==o.bottom)); }
inline bool operator!=(const MRect& o, const MRect& s) { return ((s.right!=o.right) || (s.left!=o.left) || (s.top!=o.top) || (s.bottom!=o.bottom)); }

//----------------------------------------------------------------------------
// Inlines
//----------------------------------------------------------------------------

inline void MRect::Set(int _left, int _top, int _right, int _bottom) {
  left = _left;
  top = _top;
  right = _right;
  bottom = _bottom;
}

inline MRect::MRect(int _left, int _top, int _right, int _bottom) {
  Set(_left, _top, _right, _bottom);
}
inline MRect::MRect(const MPoint& orgn, const MSize& extent) {
  Set(orgn.x, orgn.y, orgn.x+extent.cx, orgn.y+extent.cy);
}

inline MRect::MRect( const struct tagRECT &orgn )
{
   Set( orgn.left, orgn.top, orgn.right, orgn. bottom );
}

//
// Return true if the rectangle is empty.
//
inline bool MRect::IsEmpty() const
{
  return bool( left >= right || top >= bottom );
}

//
// Return true if all of the points on the rectangle is 0.
//
inline bool MRect::IsNull() const
{
  return bool( !left && !right && !top && !bottom );
}


#endif
