/*
	TDA988xPinnacle.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#include "TDA988xPinnacle.hpp"
#include "..\HwCapture\Pisces.hpp"

 /*
Norm:		Audio Carrier	Video Bandwidth

SECAM_DKK1L	6.5MHz			6.0Mhz
PAL_D		6.5MHz			6.0MHz 

PAL_I		6.0Mhz			5.5MHz

PAL_BGH		5.5Mhz			5.0MHz
PAL_N		5.5MHz			5.0MHz
SECAM_BGH	5.5MHz			5.0MHz

NTSC_M		4.5Mhz			4.2MHz
PAL_M		4.5Mhz			4.2MHz
PAL_NC		4.5Mhz			4.2MHz

 */

/// TDA9885 (PAL, NTSC)
/// TDA9886 (PAL, SECAM, NTSC)
/// TDA9887 (PAL, SECAM, NTSC, FM Radio)

//
// TDA defines
//

//// first reg
#define TDA9887_VideoTrapBypassOFF     0x00    // bit b0
#define TDA9887_VideoTrapBypassON      0x01    // bit b0

#define TDA9887_AutoMuteFmInactive     0x00    // bit b1
#define TDA9887_AutoMuteFmActive       0x02    // bit b1

#define TDA9887_Intercarrier           0x00    // bit b2
#define TDA9887_QSS                    0x04    // bit b2

#define TDA9887_PositiveAmTV           0x00    // bit b3:4
#define TDA9887_FmRadio                0x08    // bit b3:4
#define TDA9887_NegativeFmTV           0x10    // bit b3:4

#define TDA9887_ForcedMuteAudioON      0x20    // bit b5
#define TDA9887_ForcedMuteAudioOFF     0x00    // bit b5

#define TDA9887_OutputPort1Active      0x00    // bit b6
#define TDA9887_OutputPort1Inactive    0x40    // bit b6
#define TDA9887_OutputPort2Active      0x00    // bit b7
#define TDA9887_OutputPort2Inactive    0x80    // bit b7


//// second reg
#define TDA9887_DeemphasisOFF          0x00    // bit c5
#define TDA9887_DeemphasisON           0x20    // bit c5
#define TDA9887_Deemphasis75           0x00    // bit c6
#define TDA9887_Deemphasis50           0x40    // bit c6
#define TDA9887_AudioGain0             0x00    // bit c7
#define TDA9887_AudioGain6             0x80    // bit c7


//// third reg
#define TDA9887_AudioIF_4_5             0x00    // bit e0:1
#define TDA9887_AudioIF_5_5             0x01    // bit e0:1
#define TDA9887_AudioIF_6_0             0x02    // bit e0:1
#define TDA9887_AudioIF_6_5             0x03    // bit e0:1


#define TDA9887_VideoIF_58_75           0x00    // bit e2:4
#define TDA9887_VideoIF_45_75           0x04    // bit e2:4
#define TDA9887_VideoIF_38_90           0x08    // bit e2:4
#define TDA9887_VideoIF_38_00           0x0C    // bit e2:4
#define TDA9887_VideoIF_33_90           0x10    // bit e2:4
#define TDA9887_VideoIF_33_40           0x14    // bit e2:4
#define TDA9887_RadioIF_45_75           0x18    // bit e2:4
#define TDA9887_RadioIF_38_90           0x1C    // bit e2:4


#define TDA9887_TunerGainNormal         0x00    // bit e5
#define TDA9887_TunerGainLow            0x20    // bit e5

#define TDA9887_Gating_18               0x00    // bit e6
#define TDA9887_Gating_36               0x40    // bit e6

#define TDA9887_AgcOutON                0x80    // bit e7
#define TDA9887_AgcOutOFF               0x00    // bit e7


void TDA988xPinnacle::SetMode(const bool EnableTuner)
{
	BYTE bData[5];
	
	BYTE   bVideoIF     = 0;
	BYTE   bAudioIF     = 0;
	BYTE   bDeEmphasis  = 0;
	BYTE   bDeEmphVal   = 0;
	BYTE   bModulation  = 0;
	BYTE   bCarrierMode = 0;
	BYTE   bOutPort1    = TDA9887_OutputPort1Inactive;
	BYTE   bOutPort2    = 0; //TDA9887_OutputPort2Inactive
	BYTE   bVideoTrap   = TDA9887_VideoTrapBypassOFF;
	// Noise fix supplied by anonymous
	BYTE   bTopAdjust   = 0x0e; //mbAGC;
	
	//if (mParams.fVideoTrap)
	//{
	//	bVideoTrap   = cVideoTrapBypassON;
	//}
	
	if (m_tm == HwTuner::Tm_Radio_FM || 
		m_tm == HwTuner::Tm_Radio_AM) //FM Radio
	{     
		bDeEmphasis  = TDA9887_DeemphasisON;	
		if (id == 3) {
			bDeEmphVal = TDA9887_Deemphasis75;
		} 
		else {
			bDeEmphVal = TDA9887_Deemphasis50;
		}
		bModulation  = TDA9887_FmRadio;
		bCarrierMode = TDA9887_QSS;		
		bOutPort1    = TDA9887_OutputPort1Inactive;
		
		bVideoTrap   = TDA9887_VideoTrapBypassOFF;
	}
	else
	if (m_vf & (KS_AnalogVideo_PAL_B | KS_AnalogVideo_PAL_D |
				KS_AnalogVideo_PAL_G | KS_AnalogVideo_PAL_H |
				KS_AnalogVideo_PAL_I | KS_AnalogVideo_PAL_M |
				KS_AnalogVideo_PAL_N | KS_AnalogVideo_PAL_60 )) {

		bDeEmphasis  = TDA9887_DeemphasisON;
		bDeEmphVal   = TDA9887_Deemphasis50;        
		bModulation  = TDA9887_NegativeFmTV;		
		if (id == 1) {
			bCarrierMode = TDA9887_Intercarrier;
		} 
		else  // stereo boards
		{			
			bCarrierMode = TDA9887_QSS;
		}
		bOutPort1    = TDA9887_OutputPort1Inactive;
		
		if ( m_vf & (KS_AnalogVideo_PAL_M|KS_AnalogVideo_PAL_N) ) {
			bDeEmphVal = TDA9887_Deemphasis75;
			if ((id == 5) || (id == 6))
			{
				bCarrierMode = TDA9887_Intercarrier;
			} 
			else  // stereo boards
			{			
				bCarrierMode = TDA9887_QSS;
			}
		}
	} 
	else if (m_vf & ( KS_AnalogVideo_SECAM_B | KS_AnalogVideo_SECAM_D |
					  KS_AnalogVideo_SECAM_G | KS_AnalogVideo_SECAM_H | 
					  KS_AnalogVideo_SECAM_K | KS_AnalogVideo_SECAM_K1 |
					  KS_AnalogVideo_SECAM_L | KS_AnalogVideo_SECAM_L1)) {     

		bDeEmphasis  = TDA9887_DeemphasisON;
		bDeEmphVal   = TDA9887_Deemphasis50;
		bModulation  = TDA9887_NegativeFmTV;
		bCarrierMode = TDA9887_QSS;
		bOutPort1    = TDA9887_OutputPort1Inactive;
		
		if ( m_vf & (KS_AnalogVideo_SECAM_L | KS_AnalogVideo_SECAM_L1)) {
			bDeEmphasis  = TDA9887_DeemphasisOFF;
			bDeEmphVal   = TDA9887_Deemphasis75;
			bModulation  = TDA9887_PositiveAmTV;
		}
	}
	else if (m_vf & (KS_AnalogVideo_PAL_NC | KS_AnalogVideo_NTSC_M |
		KS_AnalogVideo_NTSC_M_J | KS_AnalogVideo_NTSC_50)) {

		bDeEmphasis  = TDA9887_DeemphasisON;
		bDeEmphVal   = TDA9887_Deemphasis75;
		bModulation  = TDA9887_NegativeFmTV;                                
		if ((id == 5) || (id == 6))
		{
			bCarrierMode = TDA9887_Intercarrier;
		} 
		else 
		{
			bCarrierMode = TDA9887_QSS;
		}
		bOutPort1    = TDA9887_OutputPort1Inactive;
	}
	
	//Set bandwidths
	
	bVideoIF     = TDA9887_VideoIF_38_90;
	bAudioIF     = TDA9887_AudioIF_5_5;
	
	if (m_tm == HwTuner::Tm_Radio_FM || 
		m_tm == HwTuner::Tm_Radio_AM) //FM Radio
	{     
		if (id == 3) {
			// NTSC
			bVideoIF     = TDA9887_RadioIF_45_75;
			bAudioIF     = TDA9887_AudioIF_4_5;			
		} 
		else {
			// PAL
			bVideoIF     = TDA9887_RadioIF_38_90;			
			bAudioIF     = TDA9887_AudioIF_5_5;			
		}
	} else
	if (m_vf & (KS_AnalogVideo_PAL_B | KS_AnalogVideo_PAL_G | KS_AnalogVideo_PAL_H | KS_AnalogVideo_PAL_60)) {
		bVideoIF     = TDA9887_VideoIF_38_90;
		bAudioIF     = TDA9887_AudioIF_5_5;
	} else
	if (m_vf & (KS_AnalogVideo_PAL_D | KS_AnalogVideo_SECAM_D)) {
		bVideoIF     = TDA9887_VideoIF_38_00;
		bAudioIF     = TDA9887_AudioIF_6_5;
	} else
	if (m_vf & KS_AnalogVideo_PAL_I) {
		bVideoIF     = TDA9887_VideoIF_38_90;
		bAudioIF     = TDA9887_AudioIF_6_0;
	} else
	if (m_vf & (KS_AnalogVideo_NTSC_M | KS_AnalogVideo_NTSC_M_J | KS_AnalogVideo_PAL_M | KS_AnalogVideo_PAL_NC | KS_AnalogVideo_NTSC_50)) {
		bVideoIF     = TDA9887_VideoIF_45_75;
		bAudioIF     = TDA9887_AudioIF_4_5;
	} else 
	if (m_vf & (KS_AnalogVideo_SECAM_K | KS_AnalogVideo_SECAM_K1 | KS_AnalogVideo_SECAM_L)) {
		bVideoIF     = TDA9887_VideoIF_38_90;
		bAudioIF     = TDA9887_AudioIF_6_5;        
	} else 
	if (m_vf & KS_AnalogVideo_SECAM_L1) {
		bVideoIF     = TDA9887_VideoIF_33_90;
		bAudioIF     = TDA9887_AudioIF_6_5;        
	}
		
		
	bData[0] =  0;
	bData[1] =  bVideoTrap                   |  // B0: video trap bypass
		TDA9887_AutoMuteFmInactive   |  // B1: auto mute
		bCarrierMode                 |  // B2: InterCarrier for PAL else QSS 
		bModulation                  |  // B3 - B4: positive AM TV for SECAM only
		TDA9887_ForcedMuteAudioOFF   |  // B5: forced Audio Mute (off)
		bOutPort1                    |  // B6: Out Port 1 
		bOutPort2;                      // B7: Out Port 2 
	bData[2] =  bTopAdjust                   |  // C0 - C4: Top Adjust 0 == -16dB  31 == 15dB
		bDeEmphasis                  |  // C5: De-emphasis on/off
		bDeEmphVal                   |  // C6: De-emphasis 50/75 microsec
		TDA9887_AudioGain0;             // C7: normal audio gain
	bData[3] =  bAudioIF                     |  // E0 - E1: Sound IF
		bVideoIF                     |  // E2 - E4: Video IF
		TDA9887_TunerGainNormal      |  // E5: Tuner gain (normal)
		TDA9887_Gating_18            |  // E6: Gating (18%)
		TDA9887_AgcOutOFF;              // E7: VAGC  (off)
	
    if (EnableTuner) {
        bData[1] |= TDA9887_OutputPort2Inactive;
    } else {
        bData[1] &= ~TDA9887_OutputPort2Inactive;    
    }
	
	DUMP(("TDA9885/6/7 Pinnacle: 0x%02x 0x%02x 0x%02x", bData[1],bData[2],bData[3]));
    
	// Init TDA... 
	i2c.WR(i2caddr,bData,4,NULL,0); 
}

// We provide AFC status report
bool TDA988xPinnacle::AFCProvider() 
{
	return true;
}

// To get the AFC information...
int TDA988xPinnacle::GetAFC()
{
	// Read current status
	BYTE ret = 0;

	// Try to read AFC
	if (!i2c.Read(i2caddr,&ret)) return 0; // No AFC on failure...

	// Convert to an offset
	int val = ((ret >> 1) & 0xF); if (val & 8) val |= (-8);

	// Return the frequency delta...
	return -1 * ((val * 25000) + 12500);
}

void TDA988xPinnacle::OnPreTunerSet(KS_AnalogVideoStandard vf,HwTuner::TunerMode tm)
{
	m_vf = vf;
	m_tm = tm;
	SetMode(true); // Leave Tuner enabled...
	
}

void TDA988xPinnacle::OnPostTunerSet()
{
	SetMode(false); // Disable Tuner access ...
}

TDA988xPinnacle::TDA988xPinnacle(I2C& bus, const BYTE Addr, const int ID) :
	i2c(bus) , i2caddr(Addr) 
	, m_vf(KS_AnalogVideoStandard(0)) , m_tm(HwTuner::Tm_TV) , id(ID)
{
	SetMode(true); // Leave Tuner enabled...
}

	
