/*
	TDA988xPinnacle.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
	Used on latest Miro PCTV cards eith MT2032 tuners...
	-Is also used as an AFC ... (this code is based on DScaler Pinnacle Support)
 */

#ifndef _TDA988xPinnacle_HPP_
#define _TDA988xPinnacle_HPP_

#include "HwIFDem.hpp"

#define I2C_TDA988x_0				0x86 /* Used by pinnacle */
#define I2C_TDA988x_1_NOTUSEDBYANY	0x84 /* unused */
#define I2C_TDA988x_2_NOTUSEDBYANY	0x96 /* unused */
#define I2C_TDA988x_3_NOTUSEDBYANY	0x94 /* unused */

class I2C;
class TDA988xPinnacle : public HwIFDem {

	private:
		I2C& i2c;
		const BYTE i2caddr;
		const int id;

		HwTuner::TunerMode m_tm;
		KS_AnalogVideoStandard	m_vf;

	public:
		TDA988xPinnacle(I2C& bus, const BYTE Addr, const int ID);

		// Called before a tuning setting frequency...
		virtual void OnPreTunerSet(
			KS_AnalogVideoStandard	vf,			// VideoFormat of Signal (For TV mode)
			HwTuner::TunerMode		tm);		// Tuner Mode (TV/FM/AM)

		// Called after a tuner set ...
		virtual void OnPostTunerSet(void);

		// Called to know if an internal AFC will be used...
		virtual bool AFCProvider(void);

		// Called to get AFC status...
		virtual int  GetAFC(void);

	private:

		void SetMode(const bool EnableTuner);
};

#endif