/*
	VoodooTV.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#include "VoodooTV.hpp"
#include "..\HwCapture\Pisces.hpp"

 /*
Norm:		Audio Carrier	Video Bandwidth

SECAM_DKK1L	6.5MHz			6.0Mhz
PAL_D		6.5MHz			6.0MHz 

PAL_I		6.0Mhz			5.5MHz

PAL_BGH		5.5Mhz			5.0MHz
PAL_N		5.5MHz			5.0MHz
SECAM_BGH	5.5MHz			5.0MHz

NTSC_M		4.5Mhz			4.2MHz
PAL_M		4.5Mhz			4.2MHz
PAL_NC		4.5Mhz			4.2MHz

 */


// Called before a tuning setting frequency...
void VoodooTV::OnPreTunerSet(
	KS_AnalogVideoStandard	vf,			// VideoFormat of Signal (For TV mode)
	HwTuner::TunerMode		tm)			// Tuner Mode (TV/FM/AM)
{
	// Set the demodulator TV standard using the GPIO port (Only for VOODOO cards)
	ULONG Val;
	gpio.SetGPOE( gpio.GetGPOE() | (1<<16) );
	gpio.GetGPDATA(&Val,1,0);
	if (vf & 
		(	KS_AnalogVideo_NTSC_M | 
			KS_AnalogVideo_NTSC_M_J | 
			KS_AnalogVideo_PAL_NC |
			KS_AnalogVideo_NTSC_50) ) {
		Val |=   (1<<16);	// NTSC
	} else {
		Val &= ~ (1<<16);	// PAL
	}
	gpio.SetGPDATA(&Val,1,0);
}

VoodooTV::VoodooTV(GPIO& bus) :
	gpio(bus)
{
}

