/*
	GPIOIR.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/



#include "GPIOIR.hpp"
#include "..\HwCapture\Pisces.hpp"

GPIOIR::GPIOIR(GPIO& bus,const DWORD Mask_,const DWORD Lock_,const DWORD Xor_) :
	gpio(bus) , Mask(Mask_), Lock(Lock_), Xor(Xor_)
{
	// Start polling!
	StartPolling();
}

GPIOIR::~GPIOIR()
{
	// Finish IR Polling
	EndPolling();
}


// To get the IR code received
DWORD GPIOIR::PeekCurrCode()
{
	ULONG out = gpio.GetGPOE();

	// Set to inputs the appropiate lines 
	gpio.SetGPOE( out & (~Mask) );
		
	// Read the GPIO port
	ULONG Val;
	gpio.GetGPDATA(&Val,1,0);

	// Invert the required bits
	Val ^= Xor;
	
	// If the lock mask was given, and any of those bits are set, lockout the code
	if (Lock && (Val & Lock)) {
		return 0; // No code
	}

	// Get all the meaning bits
	DWORD code = 0;
	int shift = 0;
	DWORD TMask = Mask;
	while (TMask) {
		if (TMask & 1) { // If this is a meaning bit...
			code |= (Val & 1) << shift;
			shift++;
		}
		TMask >>= 1;
		Val >>= 1;
	};

	// Return the received code
	return (0x10000 | code);
}
