/*
	GPIOIR.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

// This class implements a simple polled IR receiver

/*
Pixel View TV pro
	Mask:0x1f00		, Lock:0x8000	,Xor:0
AverMedia - AverMedia98
	Mask:0x0f88000	, Lock:0x010000	,Xor:0x010000

  typedef struct
{
	char *name;
	DWORD mask, enable, checkmask, xormask; // el xormask a usar es realmente xormask ^ checkmask
	RKEY *keytable;
} REMINFO;
  REMINFO RemoteTable[] = {
	"AverMedia 98",					0x003b8000, 0x00004000, 0x00800000,          0, KeyTableAvermedia98,
	"Avermedia TVCapture 95",		0x00f88000,          0, 0x00010000,          0, KeyTableAvermedia95,
//	"Avermedia TVPhone 98",			0x00f88000,          0, 0x00010000,          0, NULL,
	"Avermedia TVPhone 98 w/VCR",	0x00f88000,          0, 0x00010000,          0, KeyTableAverPhone98VCR,
	"Avermedia TVStudio",			0x00f88000,          0, 0x00010000,          0, KeyTableAverPhone98VCR,
	"Apac ViewComp 878",			0x00001f00,          0, 0x00008000, 0x00008000, KeyTableApacViewComp,
	"Art TV Tuner",					0x00001f00,          0, 0x00004000, 0x00004000, KeyTableArtTVTuner,
	"Askey Magic TView 99 CPH053",	0x0028e000,          0, 0x00020000, 0x00020000, KeyTableAskeyTView,
	"Askey Magic TViewer CPH031",	0x00001f00,          0, 0x00004000, 0x00004000, KeyTableAskeyCPH031,
	"BT848 Capture",				0x00001f00,          0, 0x00004000, 0x00004000, KeyTableBt848Capture,
	"Dynalink TV",					0x007cf000,          0, 0x00800000, 0x00800000, KeyTableDynalink,
	"FlyVideo II",					0x000000f8,          0, 0x000000f8,          0, KeyTableFlyVideo,
	"KWorld TV-FM",					0x00001f00,          0, 0x00004000, 0x00004000, KeyTableArtTVTuner,
	"Leadtek WinView 601 TV/FM (*)",0x00001f00,			 0, 0x00000000, 0x00000000, KeyTableWinview601,
	"Lenco MXR 9577",				0x00001f00,          0, 0x00008000, 0x00008000, KeyTablePixelViewPro,
	"Mach TV",						0x00001f00,          0, 0x00004000, 0x00004000, KeyTableMachTV,
//	"Miro PCTV",					0x00001f00,          0, 0x00004000, 0x00004000, NULL,
	"PixelView PlayTV Pro",			0x00001f00,          0, 0x00008000, 0x00008000, KeyTablePixelViewPro,
	"PixelView PlayTV Pak",			0x00003e00,			 0,	0x00010000,	0x00010000, KeyTablePixelViewPak,
	"TTT Tritan Technology",		0x00aaf000,			 0, 0x00aaf000,          0, KeyTableTritan1,
	"TTT Tritan Technology (*)",	0x00001f00,			 0, 0x00000000, 0x00000000, KeyTableTritan2,
	"TV Capturer",					0x00001f00,          0, 0x00008000, 0x00008000, KeyTableTVCapturer,

/// Interrupt driven...
	"FlyVideo '98",					0x000000f8,          0, 0x00000100, 0x00000100, KeyTableFlyVideo,
	"Series TV Capture BCM",		0x000000f8,          0, 0x00000100, 0x00000100, KeyTableFlyVideo,
	"Chronos Video Shuttle II",		0x000000f8,          0, 0x00000100, 0x00000100, KeyTableGeniusVideoWonder,
	"Dart TV Live",					0x000000f8,          0, 0x00000100, 0x00000100, KeyTableFlyVideo,
	"FlyVideo 2000S",				0x0000001f,          0, 0x00000100, 0x00000100, KeyTableFlyVideo2000,
	"Genius VideoWonder Pro2",		0x000000f8,          0, 0x00000100, 0x00000100, KeyTableGeniusVideoWonder,
	"Leadtek WinFast TV 2000",		0x000000f8,          0, 0x00000100, 0x00000100, KeyTableWinFast2000,	
	"Typhoon TView",				0x000000f8,          0, 0x00000100, 0x00000100, KeyTableFlyVideo,
};


 */

#ifndef _GPIOIR_HPP_
#define _GPIOIR_HPP_

#include "PolledIR.hpp"

class GPIO;
class GPIOIR : public PolledIR {


	private:
		GPIO& gpio;

		const DWORD Mask;		// The mask of meaningful bits
		const DWORD Lock;		// The mask of bits that invalidates codes
		const DWORD Xor;		// The mask of bits to invert before processing

	public:
		GPIOIR(GPIO& bus,const DWORD Mask_,const DWORD Lock_,const DWORD Xor_);
		virtual ~GPIOIR();

	protected: // Protected interfases that derived classes must implement
		virtual DWORD PeekCurrCode();

};

#endif