/*
	GPIO_INTIR.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/



#include "GPIO_INTIR.hpp"
#include "..\HwCapture\Pisces.hpp"

GPIO_INTIR::GPIO_INTIR(GPIO& bus,const DWORD Mask_) :
	gpio(bus) , Mask(Mask_)
{
	// Set to inputs the appropiate lines 
	gpio.SetGPOE( gpio.GetGPOE() & (~Mask) );

	// set rising edge of inverted GPINTR pin as irq trigger
	gpio.SetGPINTC( On );
	gpio.SetGPINTI( On );

	// And register ourselves as the object to be called on GPIO ints
	gpio.RegisterIntCallback( this );
}

GPIO_INTIR::~GPIO_INTIR()
{
	// Unregister ourselves as the GPIO callback
	gpio.RegisterIntCallback();
}


// To get the IR code received: This is called when an int happens
void GPIO_INTIR::cbInt()
{
	ULONG out = gpio.GetGPOE();

	// Set to inputs the appropiate lines 
	gpio.SetGPOE( out & (~Mask) );
		
	// Read the GPIO port
	ULONG Val;
	gpio.GetGPDATA(&Val,1,0);

	// Get all the meaningful bits
	DWORD code = 0;
	int shift = 0;
	DWORD TMask = Mask;
	while (TMask) {
		if (TMask & 1) { // If this is a meaning bit...
			code |= (Val & 1) << shift;
			shift++;
		}
		TMask >>= 1;
		Val >>= 1;
	};

	// Add the code to the Queue ( will be dropped of full)
	Codes.Insert((0x10000 | code));
}
