/*
	GPIO_INTIR.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

// This class implements an interrupt driven IR receiver

/*

Leadtek Winfast tv2000
 	Mask:0x1F8	, and an GPIO int when a new code arrives
	Uses GPIO interrupts to get codes!
	the 9:th bit in gpio output enable must be 0 or there will be no gpio interupt

FlyVideo 98
	Mask:0x1F8	, and an GPIO int when a new code arrives
	Uses GPIO interrupts to get codes!

	"FlyVideo '98",					0x000000f8,          0, 0x00000100, 0x00000100, KeyTableFlyVideo,
	"Series TV Capture BCM",		0x000000f8,          0, 0x00000100, 0x00000100, KeyTableFlyVideo,
	"Chronos Video Shuttle II",		0x000000f8,          0, 0x00000100, 0x00000100, KeyTableGeniusVideoWonder,
	"Dart TV Live",					0x000000f8,          0, 0x00000100, 0x00000100, KeyTableFlyVideo,
	"FlyVideo 2000S",				0x0000001f,          0, 0x00000100, 0x00000100, KeyTableFlyVideo2000,
	"Genius VideoWonder Pro2",		0x000000f8,          0, 0x00000100, 0x00000100, KeyTableGeniusVideoWonder,
	"Leadtek WinFast TV 2000",		0x000000f8,          0, 0x00000100, 0x00000100, KeyTableWinFast2000,	
	"Typhoon TView",				0x000000f8,          0, 0x00000100, 0x00000100, KeyTableFlyVideo,

 */



#ifndef _GPIO_INTIR_HPP_
#define _GPIO_INTIR_HPP_

#include "HwIR.hpp"
#include "..\HwCapture\IntCallback.hpp"

class GPIO;
class GPIO_INTIR : public HwIR, public IntCallback {


	private:
		GPIO& gpio;

		const DWORD Mask;		// The mask of meaningful bits

	public:
		GPIO_INTIR(GPIO& bus,const DWORD Mask_);
		virtual ~GPIO_INTIR();

	public: // The GPIO int callback
		virtual void cbInt( void ); 

};

#endif