/*
	HauppaugeIR.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

// This is for the Hauppauge IR receiver

#include "HauppaugeIR.hpp"
#include "..\HwCapture\Pisces.hpp"


/*
 * If this key changes, a new key was pressed.
 */
#define REPEAT_TOGGLE_0      192
#define REPEAT_TOGGLE_1      224

HauppaugeIR::HauppaugeIR(I2C& bus, const BYTE addr) :
	i2c(bus) , address(addr)
{
	// Now start polling
	StartPolling();
}

HauppaugeIR::~HauppaugeIR()
{
	// Finish polling
	EndPolling();
}
	
// To get the IR code received
DWORD HauppaugeIR::PeekCurrCode()
{
	// We have to read 3 Bytes!
	BYTE in[3];

	// Poll the I2C IR receiver
	i2c.WR(address,NULL,0,in,3);

	// key pressed ? 
	if (in[0] != REPEAT_TOGGLE_0 && 
		in[0] != REPEAT_TOGGLE_1) {
		// No, return no key
		return 0;
	}

	// look what we have 
	DUMP(("key (0x%02x/0x%02x)\n", in[0], in[1]));

	// bit 5   of in[0] means repeat
	// bit 2-7 of in[1] contains code

	// repeat_bit= ((in[0]&0x20) ? 0x800 : 0)
	DWORD code = (0x10000 | (in[1]>>2) );

	return code;
}
