/*
	KNC1IR.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* Note: Information was taken from LIRC...
 * modified for PixelView (BT878P+W/FM) by
 
 *      Michal Kochanowicz <mkochano@pld.org.pl>
 
 *      Christoph Bartelmus <lirc@bartelmus.de>
 
 * modified for KNC ONE TV Station/Anubis Typhoon TView Tuner by
 
 *      Ulrich Mueller <ulrich.mueller42@web.de>
 
 */

#include "KNC1IR.hpp"
#include "..\HwCapture\Pisces.hpp"

KNC1IR::KNC1IR(I2C& bus, const BYTE addr) :
	i2c(bus) , address(addr) , klast(0)
{
	// Start polling
	StartPolling();
}

KNC1IR::~KNC1IR()
{
	// Finish polling
	EndPolling();
}

// To get the IR code received
DWORD KNC1IR::PeekCurrCode()
{
	// We have to read 1 Byte!
	BYTE in[1];

	// Poll the I2C IR receiver
	i2c.WR(address,NULL,0,in,1);

	// it seems that 0xFE indicates that a button is still hold

	// down, while 0xFF indicates that no button is hold

	// down. 0xFE sequences are sometimes interrupted by 0xFF

	if (in[0] == 0xFF) {
		// No, return no key
		return 0;
	}

	if (in[0] == 0xFE) {
		// Return the last received key.
		in[0] = klast; 
	} else {
		// Remember key as last received
		klast = in[0];
	}

	// look what we have 
	DUMP(("key (0x%02x)\n", in[0]));

	DWORD code = (0x10000 | (in[0]) );

	return code;
}
