/*
	PolledIR.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

// This class implements the thread and queuing required to
// handle an IR receiver that does not generate any kind of
// interrupt. We use a thread to poll the IR chip 10 times 
// per second, to get every keypress

#ifndef _POLLEDIR_HPP_
#define _POLLEDIR_HPP_

#include "MyTypes.hpp"
#include "HwIR.hpp"
#include "SyncEvent.hpp"
#include "NotifyEvent.hpp"


class PolledIR : public HwIR {

	private:

		typedef enum {
			Checking,			// We are checking remote codes
			SuspendChecking,	// Thread suspension
			Exit				// Exit thread
		} Action;

	private:

		// For thread communication
		SyncEvent				t_exec;				// Used to signal thread that must do something
		volatile Action			t_action;			// Action thread must do
		NotifyEvent				t_suspended;		// Used to signal thread is suspended
		NotifyEvent				t_ended;			// Used to signal thread is ended

		// To handle codes
		DWORD					LastCode;			// Last received code
		bool					bSuspended;			// If the code poller is suspended

	private: // Private fns

		static VOID ThreadProc(IN PVOID ctx);
		void PollProc();

	public: // Public interfases to HwIR
		
		PolledIR() : t_action(SuspendChecking) , bSuspended(false) {}

		// To get the next received code.
		virtual DWORD GetCode();

		// To support Hibernate mode
		virtual void Restart( void );
		virtual void Suspend( void );


	protected: // Protected interfases that derived classes must implement

		virtual DWORD PeekCurrCode() = 0;

	protected: // Functions that must be called by derived class to start and end polling thread

		void StartPolling();
		void EndPolling();

};

#endif