/*
	MT2032.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* 
	Voodoo TV seems to use internal AFC
	Miro      seems to use External AFC (TDA9887)
 */

#ifndef _MT2032_HPP_
#define _MT2032_HPP_

#include "HwTuner.hpp"
#include "..\HwIFDem\HwIFDem.hpp"
#include "..\HwCapture\Pisces.hpp"

// This class represents an MT2032 Tuner
class MT2032 : public HwTuner {

		TunerMode	m_mode;				// Active mode for tuner
		KS_AnalogVideoStandard m_std;	// Active std
		BYTE		m_xogc;
		const bool	m_optimize_vco;
		HwIFDem*	m_ifdem;			// AFC provider or NULL if internal used.

		// Interfase
		I2C&		i2c;
		const BYTE	i2caddr;

	private:

		static bool SpurCheck(int f1, int f2, int spectrum_from,int spectrum_to);
		bool SetLORegs(int rfin, int if1, int if2, int spectrum_from, int spectrum_to); //all in Hz
		bool CheckLOLock();
		bool OptimizeVCO();
		void SetIfFreq(int rfin, int if1, int if2, int from, int to);
		DWORD SetTvFreq(DWORD freq,  KS_AnalogVideoStandard vf /* VideoFormat */);
		void SetRadioFreq(DWORD freq);

	public:
		
		MT2032(I2C& ibus, const BYTE Addr,HwIFDem* dem = NULL);

		// Called to Set Frecuency: Returns the Real Frecuency tuned
		virtual DWORD SetFrequency(
			KS_AnalogVideoStandard	vf,			// VideoFormat of Signal (For TV mode)
			TunerMode				tm,			// Tuner Mode (TV/FM/AM)
			DWORD					Frequency,	// Frequency of channel
			TuningMethod			method);	

		// To get capacities of Tuner
		virtual bool CanTuneTV();
		virtual bool TVTunerCaps(
			ULONG& MinFrecuency,
			ULONG& MaxFrecuency,
			ULONG& TuningGranularity,
			ULONG& SettlingTime
			);
		virtual bool TVStereoAvailable();
		virtual bool TVStereoDetected();
		virtual ULONG TVTunerVideoStandardsSupported();

		virtual bool CanTuneFM();
		virtual bool FMTunerCaps(
			ULONG& MinFrecuency,
			ULONG& MaxFrecuency,
			ULONG& TuningGranularity,
			ULONG& SettlingTime
			);
		virtual bool FMStereoAvailable();
		virtual bool FMStereoDetected();

		// To support Hibernate mode
		virtual void Restart( void );

};

#endif