/*
	TEA5757.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

// TEA5757n Radio Tuner : Used in MIRO and in VHX 
#include "HwTuner.hpp"
#include "..\HwCapture\Pisces.hpp"

class GPIO;

class TEA5757 : public HwTuner {
	public:

		typedef enum {
			WE = 0, 
			CLK , 
			DATA , 
			MOST
		} Line;

	private:

		GPIO&		gpio;


		BYTE		m_bus[4];			// IO Pins used for the functions
		bool		m_bStereoDetected;	// Mono / Stereo indicator
		bool		m_bForceMono;		// Mono Forced!
		TunerMode	m_Tm;				// Tuner Mode
		DWORD		m_dwFrequency;		// Frecuency
		

	private:

		// Low level functions
		void BUS_LOW(Line l) { gpio.SetGPDATA(m_bus[l],m_bus[l],0); }
		void BUS_HIGH(Line l) { gpio.SetGPDATA(m_bus[l],m_bus[l],1); }
		ULONG BUS_IN(Line);
		void BUS_SETTOOUT(Line l) { gpio.SetGPOE(m_bus[l],On); }
		void BUS_SETTOIN(Line l) { gpio.SetGPOE(m_bus[l],Off); }
		
		// Medium level functions
		ULONG Read(void);
		void  Write(const ULONG);

	public:

		TEA5757(GPIO& bus,
				const BYTE bWE,
				const BYTE bCLK,
				const BYTE bDATA,
				const BYTE bMOST);

		// Called to Set Frecuency
		virtual DWORD SetFrequency(
			KS_AnalogVideoStandard vf,	// VideoFormat of Signal (For TV mode)
			TunerMode	tm,				// Tuner Mode (TV/FM/AM)
			DWORD Frequency,			// Frecuency of channel
			TuningMethod	method);	

		// Called to set audio mode (Some tuners need this cause they handle FM Stereo Decoding)
		virtual bool SetAudioMode(
			AudioMode audio
			);


		// To get capacities of Tuner
		virtual bool CanTuneAM();
		virtual bool AMTunerCaps(
			ULONG& MinFrecuency,
			ULONG& MaxFrecuency,
			ULONG& TuningGranularity,
			ULONG& SettlingTime
			);
		virtual bool AMStereoAvailable();
		virtual bool AMStereoDetected();

		virtual bool CanTuneFM();
		virtual bool FMTunerCaps(
			ULONG& MinFrecuency,
			ULONG& MaxFrecuency,
			ULONG& TuningGranularity,
			ULONG& SettlingTime
			);
		virtual bool FMStereoAvailable();
		virtual bool FMStereoDetected();

		// To support Hibernate mode
		virtual void Restart( void );


};
