/*
	HwVideo.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
 * Class that represents the Hardware associated to the TVAudio (and more)
 */

#ifndef _HWVIDEO_HPP_
#define _HWVIDEO_HPP_

// To get all the required things!
#include "Debug.hpp"
#include "CppRt.hpp"
#include "MyTypes.hpp"

#include "..\Enums.hpp"
#include "..\HwCapture\Pisces.hpp"

// Type describing a given connector
typedef signed long condesc_t;

// Macros to access the fields
#define CONDESC(mux,svideo,isdigital) ((mux) & 3) | (((svideo)&1) << 2) | (((isdigital)&1) << 3)
#define CONDESC_MUX(x) ((x)&3)
#define CONDESC_SVIDEO(x) ((x)&4)
#define CONDESC_DIGITAL(x) ((x)&8)

class HwVideo {

	public:

		// Types of video chip controls
		typedef enum {
			VC_NONE = 0,
			VC_TDA8540
		} VideoChip;

	public:

		// constructors and destructors... Need to be declared here as we want the 'virtual' keyword for destructor
		HwVideo() {}
		virtual ~HwVideo() {}

		// Connector information
		// Tuners (1)
		// SVideo (4)
		// Composite (32)
		// Serial Digital (4)
		// Parallel Digital (4)
		// RGB (4)
		// Component Video (4)

		virtual bool HasTunerInput( void ) const { return false; }
		virtual int	SVideoCount( void ) const { return 0; }
		virtual int	CompositeCount( void ) const { return 0; }
		virtual int	SerialDigitalCount( void ) const { return 0; }
		virtual int	ParallelDigitalCount( void ) const { return 0; }
		virtual int	RGBCount( void ) const { return 0; }
		virtual int	ComponentVideoCount( void ) const { return 0; }

		// Selection of Video Connections: Returns kind of connector selected
		virtual condesc_t SetConnector(VideoConnector conn) {return -1;}

		// Selection of video Format
		virtual void SetFormat(VideoFormat v) {}

		// To support Hibernate mode
		virtual void Restart( void ) {}
		virtual void Suspend( void ) {}

};

#endif