/*
	PCF8575A.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
Technical Details of card (Primavision):
 
After some reverse engineering on the card I can say the following points about it:
 
The card has the bt878 + PCF8574A (I2C Port Expander) + MAX455 (8 Channel Video Mux) + GAL16V8 for external digital IO's.
 
The video input from the MAX455 goes into the 3rd video input channel when using the generic 4 channel driver.
 
The card has an expantion header (IDC20) for 4 additional video input channels (Channels 0-3 on the MAX455) with another card.
 
The cards video channels are 4 to 7 on the MAX455.  The A2 line on the MAX455 is pulled high when no expansion card is available, and I don't have one anyway as Tenestra didn't use it in their new systems since it reduced the capture rate.
 
The A0 and A1 pins of the MAX455 come from the PCF8574A pins IO6 and IO7 respectively.  The other lines goto the GAL which handled external inputs.
 
The card also has built-in compression with some ICs, but this is not populated on the cards that I have available, so it's nothing to worry about.
 
*/


#include "PCF8575A.hpp"
#include "..\HwCapture\Pisces.hpp"

PCF8575A::PCF8575A(I2C& bus, const BYTE Addr) :
	i2c(bus) , i2caddr(Addr)
{
}

int PCF8575A::CompositeCount( void ) const
{ 
	return 4;
}
	
condesc_t PCF8575A::SetConnector(VideoConnector conn)
{
	switch (conn) {
		default:
			return -1;
		case VideoConComposite0: 
			i2c.Write(i2caddr,0x00);
			break;
		case VideoConComposite1: 
			i2c.Write(i2caddr,0x40);
			break;
		case VideoConComposite2: 
			i2c.Write(i2caddr,0x80);
			break;
		case VideoConComposite3: 
			i2c.Write(i2caddr,0xC0);
			break;
	}
	return CONDESC(2,0,0); // MUX,svideo,isdigital
}

