/*
	RSBT.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#include "RSBT.hpp"
#include "..\HwCapture\Pisces.hpp"

RSBT::RSBT(GPIO& bus) :
	gpio(bus)
{
	Restart();
}
	
int RSBT::SVideoCount( void ) const
{
	return 1;
}

int RSBT::CompositeCount( void ) const
{
	return 1;
}

int RSBT::SerialDigitalCount( void ) const
{
	return 4;
}

void RSBT::Restart( )
{
	DWORD Val1 = 0xFFFFFFFF;
	DWORD Val0 = 0;
	gpio.SetGPOE(Val1);
	gpio.SetGPDATA(&Val1,1,0);
	mdelay(50);
	gpio.SetGPDATA(&Val0,1,0);
	mdelay(50);
	gpio.SetGPDATA(&Val1,1,0);
	mdelay(50);
	gpio.SetGPOE(0x1F800);
}


condesc_t RSBT::SetConnector(VideoConnector conn)
{
	DWORD Val = 0;
	switch(conn) {
		case VideoConComposite0:
			return CONDESC(3,0,0); // mux,svideo,digital
		case VideoConSVideo0:
			return CONDESC(1,1,0); // mux,svideo,digital
		default:
		case VideoConSDI0:
			gpio.GetGPDATA(&Val,1,0);
			Val &= ~0x1F800;
			Val |= 0x1000;
			gpio.SetGPDATA(&Val,1,0);
			break;
		case VideoConSDI1:
			gpio.GetGPDATA(&Val,1,0);
			Val &= ~0x1F800;
			Val |= 0x9000;
			gpio.SetGPDATA(&Val,1,0);
			break;
		case VideoConSDI2:
			gpio.GetGPDATA(&Val,1,0);
			Val &= ~0x1F800;
			Val |= 0x11000;
			gpio.SetGPDATA(&Val,1,0);
			break;
		case VideoConSDI3:
			gpio.GetGPDATA(&Val,1,0);
			Val &= ~0x1F800;
			Val |= 0x19000;
			gpio.SetGPDATA(&Val,1,0);
			break;
	}
	mdelay(1);
	gpio.GetGPDATA(&Val,1,0);
	Val &= ~0x1000;
	gpio.SetGPDATA(&Val,1,0);

	return CONDESC(2,0,1); // mux,svideo,digital
}

