/*
	SAA7118.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#ifndef __SAA7118_H
#define __SAA7118_H


#include "HwVideo.hpp"

#define I2C_SAA7118_0    0x42 /* Default I2C address */

class I2C;
class SAA7118 : public HwVideo {

	private:
		I2C& i2c;
		const BYTE i2caddr;
		VideoFormat    vf;
		VideoConnector conn;

	public:

		SAA7118(I2C& bus, const BYTE Addr);

		// Information
		virtual int	SVideoCount( void ) const;
		virtual int	CompositeCount( void ) const;
		virtual int	SerialDigitalCount( void ) const;
		virtual int	ParallelDigitalCount( void ) const;
		virtual int	RGBCount( void ) const;
		virtual int	ComponentVideoCount( void ) const;

		// To Select the video Source
		virtual condesc_t SetConnector(VideoConnector c);

		// Selection of video Format
		virtual void SetFormat(VideoFormat v);

		// To support Hibernate mode
		virtual void Restart( void );

	protected:
		
		void SetRegister(BYTE Register, BYTE Value);
		BYTE GetRegister(BYTE Register);

};


#endif
