/*
	TRUST814PCI.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	This file was written Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
Technical Details of card:

Camera	MUX	GPOE		GPDATA
1		2	0x0000FF	0x00
2		2	0x0000FF	0xa0
3		2	0x0000FF	0x50
4		2	0x0000FF	0xf0

*/


#include "TRUST814PCI.hpp"
#include "..\HwCapture\Pisces.hpp"

CTRUST814PCI::CTRUST814PCI(GPIO& g) :
	gpio(g) 
{
}

int CTRUST814PCI::CompositeCount( void ) const
{ 
	return 4;
}
	
condesc_t CTRUST814PCI::SetConnector(VideoConnector conn)
{
	DWORD Val = 0;

	// Enable required outputs
	gpio.SetGPOE( gpio.GetGPOE() | 0x0000FF );

	// And select proper input
	gpio.GetGPDATA(&Val,1,0);
	DUMP(("Current Value = 0x%06X\n", Val));
	Val &= 0xFFFF00;

	DUMP(("selecting %d\n", conn));

	switch (conn) {
		default:
			return -1;

		case VideoConComposite0:
			Val |= 0x00;
			break;

		case VideoConComposite1:
			Val |= 0xa0;
			break;

		case VideoConComposite2:
			Val |= 0x50;
			break;

		case VideoConComposite3:
			Val |= 0xf0;
			break;
	}
	
	DUMP(("Setting GPDATA = 0x%02X\n", Val));
	gpio.SetGPDATA(&Val,1,0);

	return CONDESC(2,0,0); // MUX2, no digital, no SVideo
}

