/*
	PCI_IO.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000-2002 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#include "Debug.hpp"
#include "PCI_IO.hpp"

// Constructor
PCI_IO::PCI_IO() :
	Bus(0) , Device(0) , Function(0)
{
	TRACE(("PCI_IO::PCI_IO"));
}

bool PCI_IO::FindDevice(ULONG VendorID, ULONG DeviceID ) 
{
	DUMP(( "FindPciDevice" ));

	WORD V_Id;
	WORD D_Id;

    // Scan each bus.
    for ( Bus = 0;
            Bus < MAX_PCI_BUSES;
            Bus ++ ) {

        // Scan each device.
        for ( Device = 0;
                Device < MAX_PCI_DEVICES;
                Device++ ) {

			
			Function = 0;
			// Get Vendor ID...
			if	(!ReadConfigWORD(0,V_Id)) {
				continue;
			}
			// If no device present, don't try to scan functions...
			if (V_Id == 0 || V_Id == 0xFFFF ) {
				continue;
			}

            //
            // Scan each function.
            //
			ULONG MaxFunctions = 1;
            for ( Function = 0;
                    Function < MaxFunctions;
                    Function++ ) {


                // Check what's in the current slot.
                if	(!ReadConfigWORD(0,V_Id)) {

                    //
                    // The specified PCI bus does not exist.  We are done
                    // with this bus.  Set the device number to the maximum
                    // and break out of the function loop.  This will scan
                    // the next system bus.  There is no guarantee that the
                    // system buses are sequentially ordered.
                    //

                    Device = MAX_PCI_DEVICES;

                    break;

                }

				
				if (!ReadConfigWORD(2,D_Id) ) {
                    Device = MAX_PCI_DEVICES;
                    break;
				}
					
                if (V_Id == 0 || V_Id == 0xFFFF ) {

                    //
                    // Although the current function may show an invalid
                    // device, we really have to keep checking for functions
                    // on the current device.  This is required because the
                    // PCI specification is unclear whether function numbers
                    // in a PCI device must be numerically sequential.  For
                    // example, a particular PCI device could respond to
                    // functions 1, 3, and 4.
                    //

                    //
                    // Check next function.
                    //

                    continue;

                }

                DUMP(("PCI device found: bus = 0x%x  device = 0x%x  function = 0x%x\n", Bus, Device, Function ));

				// If we are dealing with Function 0, check if we can find more functions...
				if (Function == 0) {

					// Get Header type..
					BYTE hType = 0;
					ReadConfigBYTE(0x0e,hType);

					// Determine if we are dealing with a multifunction device...
					MaxFunctions = (hType & 0x80) ? MAX_PCI_FUNCTIONS : 1;
				}

				// Check if found requested device
                if ( V_Id != VendorID || D_Id != DeviceID ) {

                    DUMP(( "VendorId (0x%0x) or DeviceId (0x%0x) not as requested.\n", V_Id, D_Id ));

                    // Check next function.
                    continue;

                }

				DUMP(( "VendorId and DeviceId match!\n"));

                //
                // At this point, we've found a valid PCI device.
                //

                // Indicate at least one device found.
				return true;


            }   // Function
        }   // Device
    }   // Bus

    return false;
}   

