/*
	Registry.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef REGISTRY_H_
#define REGISTRY_H_

#include "WDMHeaders.hpp"

/* Note: This MUST be called at PASSIVE_LEVEL or will fail */

class Registry {

	private:

		HANDLE KeyHandle;

	private:

		static HANDLE OpenDriverKey( const PDEVICE_OBJECT PhysicalDeviceObject );
		static void PrepareKeyName( PUNICODE_STRING UKeyName, PCHAR name ) ;
		static NTSTATUS GetRegistryValue( IN HANDLE Handle, IN const PUNICODE_STRING KeyName,IN PCHAR Data, IN ULONG DataLength );
		static NTSTATUS SetRegistryValue( HANDLE, const PUNICODE_STRING, ULONG, PCHAR, ULONG );

	public:
	
		Registry( const PDEVICE_OBJECT PhysicalDeviceObject );
		~Registry();

		bool IsInitOK() const { return KeyHandle != NULL; }
		bool ReadKey(char* Key ,DWORD&, DWORD def = 0);
		bool WriteKey(char* Key ,DWORD);
		bool ReadKey(char* Key ,int& v, int def = 0, int idx = -1) { return ReadKey(Key,(DWORD&)v,DWORD(def)); }
		bool WriteKey(char* Key ,int v, int idx = -1) { return WriteKey(Key,DWORD(v)); }
		bool ReadKey(char* Key ,bool& v, bool def = false, int idx = -1) { return ReadKey(Key,(DWORD&)v,DWORD(def)); }
		bool WriteKey(char* Key ,bool v, int idx = -1) { return WriteKey(Key,DWORD(v)); }
		bool ReadKey(char* Key ,LONG& v, LONG def = 0, int idx = -1) { return ReadKey(Key,(DWORD&)v,DWORD(def)); }
		bool WriteKey(char* Key ,LONG v, int idx = -1) { return WriteKey(Key,DWORD(v)); }

};

#endif