/*
	SyncEvent.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _SYNCEVENT_HPP_
#define _SYNCEVENT_HPP_

#include "WDMHeaders.hpp"

// This represents a Syncronization event, with autoreset!
class SyncEvent {

	private:

		// The event
		KEVENT m_Event;	

	public:
		
		SyncEvent() 
		{
			KeInitializeEvent(&m_Event, SynchronizationEvent 
				/* This means the event is autoreset when wait exits and that a single thread is elegible for execution */
				, FALSE);
		}

		void WaitFor( void )
		{
			KeWaitForSingleObject(&m_Event, Executive, KernelMode, FALSE, NULL);
		}

		NTSTATUS WaitFor( DWORD dwTimeInMs ) // Wait with timeout
		{
			LARGE_INTEGER tm;
			tm.QuadPart = - int( dwTimeInMs * 10000 ); 
			return KeWaitForSingleObject(&m_Event, Executive, KernelMode, FALSE, &tm);
		}

		/* Note: this must be called at IRQL <= DISPATCH_LEVEL */
		void Signal( void )
		{
			KeSetEvent(&m_Event, 0, FALSE);
		}

};

#endif

