/*
	CustProp.h: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "WDMHeaders.hpp"

#ifndef __BT848PROP_H
#define __BT848PROP_H

// This DEFINE should NEVER be enabled for regular builds...
#if 0
//#define __FULLDECODERACCESS__ // NOT RECOMMENDED AT ALL!  
#endif


// Audio properties extension
extern  GUID PROPSETID_AUDIO;

#define KSPROPERTY_AUDIO_VOLUME				0x30000001	/* to get/Set audio volume */
#define KSPROPERTY_AUDIO_BASSTREBLE			0x30000002  /* to get/set audio bass/treble */
#define KSPROPERTY_AUDIO_LOUDNESS			0x30000003  /* to get/set audio loudness */
#define KSPROPERTY_AUDIO_EQUALIZER			0x30000004  /* to get/set equalizer settings */
#define KSPROPERTY_AUDIO_DOLBY				0x30000005  /* to get/set dolby parameters */

#define KSPROPERTY_AUDIO_CAPS				0x3000000f  /* to get audio caps */

typedef struct {
    KSPROPERTY Property;
    LONG       Left;						
	LONG	   Right;				
} KSPROPERTY_AUDIO_VOLUME_S, *PKSPROPERTY_AUDIO_VOLUME_S;

typedef struct {
    KSPROPERTY Property;
    LONG       Bass;						
	LONG	   Treble;				
} KSPROPERTY_AUDIO_BASSTREBLE_S, *PKSPROPERTY_AUDIO_BASSTREBLE_S;

typedef struct {
    KSPROPERTY Property;
    LONG       Loudness;						
} KSPROPERTY_AUDIO_LOUDNESS_S, *PKSPROPERTY_AUDIO_LOUDNESS_S;

#define MAX_EQUALIZER_BANDS			16  /* maximum number of equalizer bands */

typedef struct {
	KSPROPERTY Property;
	LONG	   Band[MAX_EQUALIZER_BANDS];			// Allows up to 16 bands for audio equalizers
} KSPROPERTY_AUDIO_EQUALIZER_S, *PKSPROPERTY_AUDIO_EQUALIZER_S;

/* Dolby Processor modes */
#define KSPROPERTY_AUDIO_DOLBY_MODE_PROLOGIC	0x0		/* Dolby prologic mode */
#define KSPROPERTY_AUDIO_DOLBY_MODE_NOISE		0x1		/* Noise mode */
#define KSPROPERTY_AUDIO_DOLBY_MODE_THROUGH		0x2		/* Pass through mode */

/* dolby prologic modes */
#define KSPROPERTY_AUDIO_DOLBY_PLMODE_NORMAL		0	/* 0 = Normal */
#define KSPROPERTY_AUDIO_DOLBY_PLMODE_PHANTOM		1	/* 1 = Phantom */
#define KSPROPERTY_AUDIO_DOLBY_PLMODE_WIDE			2	/* 2 = Wide */
#define KSPROPERTY_AUDIO_DOLBY_PLMODE_DOLBY3STEREO	3	/* 3 = Dolby 3 Stereo */
#define KSPROPERTY_AUDIO_DOLBY_PLMODE_CENTEROFF		4	/* 4 = Center Off */
#define KSPROPERTY_AUDIO_DOLBY_PLMODE_PANORAMA		5	/* 5 = Panorama */
#define KSPROPERTY_AUDIO_DOLBY_PLMODE_3DPANORAMA	6	/* 6 = 3D-Panorama (virtual dolby) */

typedef struct {
	BYTE mode;			// Current processor mode
	BYTE plmode;		// Current Prologic mode
	BYTE delay;			// Current delay	  (0 to 0xFF)
	BYTE rev;			// Current reverb	  (0 to 0x7F)
	BYTE pan;			// Current Panorama   (0 to 0x7F)
	BYTE spatial;		// Current spatial    (0 to 0x7F)
	BYTE noise;			// Current noise mode (0 to 5)
} DOLBY_PARAMS;

typedef struct {
	KSPROPERTY Property;
	DOLBY_PARAMS params;		// Dolby parameters
} KSPROPERTY_AUDIO_DOLBY_S, *PKSPROPERTY_AUDIO_DOLBY_S;

#define KSPROPERTY_AUDIO_CAPS_HASVOLUME		0x0001		/* Has volume controls */
#define KSPROPERTY_AUDIO_CAPS_HASBASSTREBLE 0x0002		/* Has Bass/Treble controls */
#define KSPROPERTY_AUDIO_CAPS_HASLOUDNESS   0x0004		/* Has Loudness control */
#define KSPROPERTY_AUDIO_CAPS_HASEQUALIZER  0x0008		/* Has Equalizer control */
#define KSPROPERTY_AUDIO_CAPS_HASDOLBY	    0x0010		/* Has Dolby control (maybe not all DOLBY modes are supported) */

typedef struct {
    KSPROPERTY Property;
    ULONG       Caps;				// Capabilities	(KSPROPERTY_AUDIO_CAPS_...)			
	ULONG       EqualizerBands;		// Number of bands for the Equalizer
	ULONG	    BandFrequency[MAX_EQUALIZER_BANDS];	// Frequency of each band
} KSPROPERTY_AUDIO_CAPS_S, *PKSPROPERTY_AUDIO_CAPS_S;


// IR remote control property extension
extern  GUID PROPSETID_IR;

#define KSPROPERTY_IR_GETCODE				0x20000001	/* to read the IR remote code */
#define KSPROPERTY_IR_CAPS					0x2000000f	/* to get IR caps */

typedef struct {
    KSPROPERTY Property;
    ULONG       Code;						
} KSPROPERTY_IR_GETCODE_S, *PKSPROPERTY_IR_GETCODE_S;

#define KSPROPERTY_IR_CAPS_AVAILABLE		0x0001		/* means that IR control is available */

typedef struct {
    KSPROPERTY  Property;
    ULONG       Caps;				// Capabilities (KSPROPERTY_IR_CAPS_...)
} KSPROPERTY_IR_CAPS_S, *PKSPROPERTY_IR_CAPS_S;


// Video properties extension
extern  GUID PROPSETID_VIDEOWINDOW;

#define KSPROPERTY_VIDEOWINDOW_ANALOG		0x41000001	/* Get/Set current analog video window for channel */
#define KSPROPERTY_VIDEOWINDOW_MAXANALOG	0x41000002  /* Get maximum analog video window for channel */
#define KSPROPERTY_VIDEOWINDOW_DIGITAL		0x41000003  /* Get the current digital video window */

typedef struct {
    KSPROPERTY Property;
	DWORD	   dwChannel;			// Video channel to query/modify
    RECT	   rcRect;				// Video Window
} KSPROPERTY_VIDEOWINDOW_S, *PKSPROPERTY_VIDEOWINDOW_S;

#define VIDEOWINDOW_CHANNEL_CAPTURE 0	/* Capture Channel */
#define VIDEOWINDOW_CHANNEL_PREVIEW 1	/* Preview Channel */


// Custom properties (only for special purposes)
extern  GUID PROPSETID_CUSTOMBT848;

#define BT848_CUSTPROP_GPIO_SETGPOEBIT		0x00000001
#define BT848_CUSTPROP_GPIO_SETGPOE			0x00000002
#define BT848_CUSTPROP_GPIO_SETGPDATA		0x00000003
#define BT848_CUSTPROP_GPIO_SETGPDATABITS	0x00000004
#define BT848_CUSTPROP_GPIO_GETGPOEBIT		0x10000001
#define BT848_CUSTPROP_GPIO_GETGPOE			0x10000002
#define BT848_CUSTPROP_GPIO_GETGPDATA		0x10000003
#define BT848_CUSTPROP_GPIO_GETGPDATABITS	0x10000004

typedef struct tagKSPROPERTY_CUSTOMBT848_GPIO_S {
	KSPROPERTY	Property;
	DWORD		dwOperation;		// The operation to perform (BT848_CUSTPROP_GPIO_...)
	DWORD		dwReturnValue;		// Return value
	DWORD		dwState;			//
	DWORD		dwBit;				//
	DWORD		dwValue;			//
	DWORD		dwSize;				//
	DWORD		dwOffset;			//
	DWORD		dwFromBit;			//
	DWORD		dwToBit;			//
	DWORD		adwData[ 64 ];		//
} KSPROPERTY_CUSTOMBT848_GPIO_S, *PKSPROPERTY_CUSTOMBT848_GPIO_S;

#define BT848_CUSTPROP_I2C_SETFREQ			0x11000001
#define BT848_CUSTPROP_I2C_GETFREQ			0x11000002
#define BT848_CUSTPROP_I2C_WR				0x11000003
#define BT848_CUSTPROP_I2C_GETLASTERROR		0x11000004

typedef struct tagKSPROPERTY_CUSTOMBT848_I2C_S {
	KSPROPERTY	Property;
	DWORD		dwOperation;
	DWORD		dwReturnValue;
	DWORD		dwFreq;
	BYTE		ucAddress;
	BYTE		ucOutBuf[32];
	DWORD		dwOutLen;
	BYTE		ucInBuf[32];
	DWORD		dwInLen;
	BOOL		bDontWaitACK;
} KSPROPERTY_CUSTOMBT848_I2C_S, *PKSPROPERTY_CUSTOMBT848_I2C_S;


#ifdef __FULLDECODERACCESS__ // NOT RECOMMENDED AT ALL!

#pragma message ("/**-W-A-R-N-I-N-G--**--W-A-R-N-I-N-G--**--W-A-R-N-I-N-G-**\\")
#pragma message ("|>>>>> This driver allows UNSAFE FULL DECODER ACCESS <<<<<|")
#pragma message ("\\*-*-*-*-*-####-V-E-R-Y-*-D-A-N-G-E-R-O-U-S-####-*-*-*-*-*/")
#pragma message ("       Will create a huge security hole if installed")

typedef struct tagKSPROPERTY_CUSTOMBT848_DECODER_S {
	KSPROPERTY	Property;
	DWORD		dwOperation;
	DWORD		dwAddress;
	DWORD		dwValue;
} KSPROPERTY_CUSTOMBT848_DECODER_S, *PKSPROPERTY_CUSTOMBT848_DECODER_S;

#define BT848_CUSTPROP_DECODER_READ			0x31000001
#define BT848_CUSTPROP_DECODER_WRITE		0x31000002

#endif

typedef enum { 
	KSPROPERTY_CUSTOMBT848_GPIO				= 1,
	KSPROPERTY_CUSTOMBT848_I2C				= 2,
#ifdef __FULLDECODERACCESS__
	,KSPROPERTY_CUSTOMBT848_DECODER			= 3
#endif
} KSPROPERTY_CUSTOMBT848_PROPS;

#endif	// __BT848PROP_H


