/*
	Debug.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _DEBUG_H_
#define _DEBUG_H_

#if DBG

// Our logging fn
void LogPrint(char* szFormat, ...);

// Our TimeStamping fn
void DbgTimeStamp();

// global Tracing facilities
extern char gTrace0[];
extern char gTrace1[];
extern char gTrace2[];
extern char gTrace3[];

class _Trace {

	public: 

		// a Debug Stack
		typedef struct {
			bool	MustTrace;		// If we must trace here
			int		Top;			// Top entry of the Stack
			char*	Stack[64];		// Stack
		} StackEntry;

	public:

		static int			DebugStack;	// Debug Stack to be used
		static StackEntry	Stack[16];	// Stacks

		bool bOrigMustTrace;			// Original MustTrace
		const int  iOrigDebugStack;		// Original Debug Stack

	public:
		_Trace(bool EntryPoint, char* Fn);
		~_Trace();

		static bool MustTrace( void ) { return Stack[DebugStack].MustTrace; }
		static void DumpFn( void ) { PrintIndent(DebugStack,Stack[DebugStack].Top); }
		static void TraceBack( void );
		static void PrintIndent(int stk, int i );
	
	private:
		bool Matchs(char* Fn,char* with);
		
};

// Trace EntryPoint
#define TRACE_EP(x)			_Trace __t(true,  (char*)x )

// Normal Trace
#define TRACE(x)			_Trace __t(false, (char*)x )

#define DUMP(x)  \
	if (_Trace::MustTrace()) { \
		DbgTimeStamp(); \
		_Trace::DumpFn(); \
		DbgPrint x ; \
		LogPrint x ; \
	}

#define ERROR(x) \
	do { \
		DbgTimeStamp(); \
		DbgPrint x ; \
		LogPrint x ; \
	} while(0)

#define ENSURE( exp ) \
	if ( !(exp) ) { \
		DbgTimeStamp(); \
		_Trace::TraceBack(); \
		DbgPrint("Assertion Failed (%s) at %s(%d) ", #exp , __FILE__ , __LINE__ ); \
		LogPrint("Assertion Failed (%s) at %s(%d) ", #exp , __FILE__ , __LINE__ ); \
		DbgBreakPoint(); \
	}

#define DEBUG_BREAK \
		DbgBreakPoint(); 

#else

#define TRACE_EP(x)
#define TRACE(x)
#define DUMP(x)
#define ERROR(x)
#define ENSURE( exp )
#define DEBUG_BREAK

#endif


#endif

