/*
	WinHalEmu.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

// NT Hardware Abstration Layer Emulation for WinME and Win98SE
// to be able to access PCI Config Space under Win98/Me for ANY
// device from a WDM driver. (Not neccesarily the device under 
// control by this device Driver)
// (NTKERN.DLL does not export this fns on WinME and Win98SE)
// The only way to implement this left to us is the Direct Hardware Access 
// (Beware Windows!)


#include "Debug.hpp"
#include <WDMHeaders.hpp>

//#define DirectAccessPCI

#ifdef DirectAccessPCI

bool pci_HalEmuInit();

// Kind of access method to read/write PCI config info.
int pci_CfgMech = 0; 

bool pci_ReadConfigBYTE(ULONG bus, ULONG device, ULONG func, ULONG reg, BYTE *ret)
{
	if (!pci_CfgMech) {
		pci_HalEmuInit();
	}

	BYTE value;
	if (pci_CfgMech == 1) {

		DWORD addr = 0x80000000L | (((DWORD)(bus & 0xFF)) << 16) |
				((((unsigned)device) & 0x1F) << 11) |
				((((unsigned)func) & 0x07) << 8) | (reg & 0xFC) ;

		_asm {
			push eax
			push edx
			mov edx,0CF8h
			pushfd						// Save flags
			cli							// Disable Ints (To avoid to be interrupted here)
			in eax,dx					// Get and save current State
			push eax
			mov eax,[addr]
			out dx,eax					// Set up addressing to config data
			mov dl,byte ptr [reg]
			and dl,3
			add dl,0FCh					// Access the proper offset
			in al,dx					// Get requested DWORD of config data
			mov [value],al
			mov dl,0F8h
			pop eax
			out dx,eax					// Restore configuration control
			popfd						// Restore flags
			pop edx
			pop eax
		}

	} else if (pci_CfgMech == 2) { // cfg_mech == 2

		if (device > 0x0F) {			// mech#2 only supports 16 devices per bus
			return false ;
		}

		DWORD addr = 0xC000 | ((device & 0x0F) << 8) | (reg & 0xFF) ;
		BYTE  fn = BYTE(((func & 7) << 1) | 0xf0);

		_asm {
			push eax
			push edx
			mov edx,0CF8h
			pushfd						// Save flags
			cli							// Disable Ints (To avoid to be interrupted here)
			in al,dx					// store current state of config space (enable)
			push eax
			mov dl,0FAh
			in al,dx					// store current state of config space (Bus)
			push eax
			mov al,byte ptr [bus]
			out dx,al					// Set bus
			mov dl,0F8h
			mov al,[fn]					
			out dx,al					// Set Fn
			mov edx,[addr]				// Set Device and register
			in al,dx					// Read value
			mov [value],al				// Store value
			mov edx,0CF8h
			xor al,al					// al = 0
			out dx,al
			mov dl,0FAh
			pop eax
			out dx,al					// Restore configuration Space (Bus)
			mov dl,0F8h
			pop eax
			out dx,al					// Restore configuration Space (Enable)
			popfd						// Restore flags
			pop edx
			pop eax
		}
	} else {
		return false;
	}

	// Success!
    *ret = value;
	return true ;
}

bool pci_ReadConfigWORD(ULONG bus, ULONG device, ULONG func, ULONG reg, WORD *ret)
{
	if (!pci_CfgMech) {
		pci_HalEmuInit();
	}

	WORD value;
	if (pci_CfgMech == 1) {

		DWORD addr = 0x80000000L | (((DWORD)(bus & 0xFF)) << 16) |
				((((unsigned)device) & 0x1F) << 11) |
				((((unsigned)func) & 0x07) << 8) | (reg & 0xFC) ;

		_asm {
			push eax
			push edx
			mov edx,0CF8h
			pushfd						// Save flags
			cli							// Disable Ints (To avoid to be interrupted here)
			in eax,dx					// Get and save current State
			push eax
			mov eax,[addr]
			out dx,eax					// Set up addressing to config data
			mov dl,byte ptr [reg]
			and dl,2
			add dl,0FCh					// Access the proper offset
			in ax,dx					// Get requested DWORD of config data
			mov [value],ax
			mov dl,0F8h
			pop eax
			out dx,eax					// Restore configuration control
			popfd						// Restore flags
			pop edx
			pop eax
		}
	
	} else if (pci_CfgMech == 2) { // cfg_mech == 2

		if (device > 0x0F) {			// mech#2 only supports 16 devices per bus
			return false ;
		}

		DWORD addr = 0xC000 | ((device & 0x0F) << 8) | (reg & 0xFF) ;
		BYTE  fn = BYTE(((func & 7) << 1) | 0xf0);

		_asm {
			push eax
			push edx
			mov edx,0CF8h
			pushfd						// Save flags
			cli							// Disable Ints (To avoid to be interrupted here)
			in al,dx					// store current state of config space (enable)
			push eax
			mov dl,0FAh
			in al,dx					// store current state of config space (Bus)
			push eax
			mov al,byte ptr [bus]
			out dx,al					// Set bus
			mov dl,0F8h
			mov al,[fn]					
			out dx,al					// Set Fn
			mov edx,[addr]				// Set Device and register
			in ax,dx					// Read value
			mov [value],ax				// Store value
			mov edx,0CF8h
			xor al,al					// al = 0
			out dx,al
			mov dl,0FAh
			pop eax
			out dx,al					// Restore configuration Space (Bus)
			mov dl,0F8h
			pop eax
			out dx,al					// Restore configuration Space (Enable)
			popfd						// Restore flags
			pop edx
			pop eax
		}

	} else {
		return false;
	}

	// Success!
    *ret = value;
	return true ;
}

bool pci_ReadConfigDWORD(ULONG bus, ULONG device, ULONG func, ULONG reg, DWORD *ret)
{
	if (!pci_CfgMech) {
		pci_HalEmuInit();
	}

	DWORD value;
	if (pci_CfgMech == 1) {

		DWORD addr = 0x80000000L | (((DWORD)(bus & 0xFF)) << 16) |
				((((unsigned)device) & 0x1F) << 11) |
				((((unsigned)func) & 0x07) << 8) | (reg & 0xFC) ;

		_asm {
			push eax
			push edx
			mov edx,0CF8h
			pushfd						// Save flags
			cli							// Disable Ints (To avoid to be interrupted here)
			in eax,dx					// Get and save current State
			push eax
			mov eax,[addr]
			out dx,eax					// Set up addressing to config data
			mov dl,0FCh					// Access the proper offset
			in eax,dx					// Get requested DWORD of config data
			mov [value],eax
			mov dl,0F8h
			pop eax
			out dx,eax					// Restore configuration control
			popfd						// Restore flags
			pop edx
			pop eax
		}

	} else if (pci_CfgMech == 2) { // cfg_mech == 2

		if (device > 0x0F) {			// mech#2 only supports 16 devices per bus
			return false ;
		}

		DWORD addr = 0xC000 | ((device & 0x0F) << 8) | (reg & 0xFF) ;
		BYTE  fn = BYTE(((func & 7) << 1) | 0xf0);

		_asm {
			push eax
			push edx
			mov edx,0CF8h
			pushfd						// Save flags
			cli							// Disable Ints (To avoid to be interrupted here)
			in al,dx					// store current state of config space (enable)
			push eax
			mov dl,0FAh
			in al,dx					// store current state of config space (Bus)
			push eax
			mov al,byte ptr [bus]
			out dx,al					// Set bus
			mov dl,0F8h
			mov al,[fn]					
			out dx,al					// Set Fn
			mov edx,[addr]				// Set Device and register
			in eax,dx					// Read value
			mov [value],eax				// Store value
			mov edx,0CF8h
			xor al,al					// al = 0
			out dx,al
			mov dl,0FAh
			pop eax
			out dx,al					// Restore configuration Space (Bus)
			mov dl,0F8h
			pop eax
			out dx,al					// Restore configuration Space (Enable)
			popfd						// Restore flags
			pop edx
			pop eax
		}

	} else {
		return false;
	}

	// Success!
    *ret = value;
	return true ;
}

bool pci_WriteConfigBYTE(ULONG bus, ULONG device, ULONG func, ULONG reg, BYTE value)
{
	if (!pci_CfgMech) {
		pci_HalEmuInit();
	}

	if (pci_CfgMech == 1) {

		DWORD addr = 0x80000000L | (((DWORD)(bus & 0xFF)) << 16) |
				((((unsigned)device) & 0x1F) << 11) |
				((((unsigned)func) & 0x07) << 8) | (reg & 0xFC) ;

		_asm {
			push eax
			push edx
			mov edx,0CF8h
			pushfd						// Save flags
			cli							// Disable Ints (To avoid to be interrupted here)
			in eax,dx					// Get and save current State
			push eax
			mov eax,[addr]
			out dx,eax					// Set up addressing to config data
			mov dl,byte ptr [reg]
			and dl,3
			add dl,0FCh					// Access the proper offset
			mov al,[value]
			out dx,al					// Write byte
			mov dl,0F8h
			pop eax
			out dx,eax					// Restore configuration control
			popfd						// Restore flags
			pop edx
			pop eax
		}

	} else if (pci_CfgMech == 2) { // cfg_mech == 2

		if (device > 0x0F) {			// mech#2 only supports 16 devices per bus
			return false ;
		}

		DWORD addr = 0xC000 | ((device & 0x0F) << 8) | (reg & 0xFF) ;
		BYTE  fn = BYTE(((func & 7) << 1) | 0xf0);

		_asm {
			push eax
			push edx
			mov edx,0CF8h
			pushfd						// Save flags
			cli							// Disable Ints (To avoid to be interrupted here)
			in al,dx					// store current state of config space (enable)
			push eax
			mov dl,0FAh
			in al,dx					// store current state of config space (Bus)
			push eax
			mov al,byte ptr [bus]
			out dx,al					// Set bus
			mov dl,0F8h
			mov al,[fn]					
			out dx,al					// Set Fn
			mov edx,[addr]				// Set Device and register
			mov al,[value]				// Get value
			out dx,al					// Write value
			mov edx,0CF8h
			xor al,al					// al = 0
			out dx,al
			mov dl,0FAh
			pop eax
			out dx,al					// Restore configuration Space (Bus)
			mov dl,0F8h
			pop eax
			out dx,al					// Restore configuration Space (Enable)
			popfd						// Restore flags
			pop edx
			pop eax
		}

	} else {
		return false;
	}

	// Success!
	return true ;
}

bool pci_WriteConfigWORD(ULONG bus, ULONG device, ULONG func, ULONG reg, WORD value)
{
	if (!pci_CfgMech) {
		pci_HalEmuInit();
	}

	if (pci_CfgMech == 1) {

		DWORD addr = 0x80000000L | (((DWORD)(bus & 0xFF)) << 16) |
				((((unsigned)device) & 0x1F) << 11) |
				((((unsigned)func) & 0x07) << 8) | (reg & 0xFC) ;

		_asm {
			push eax
			push edx
			mov edx,0CF8h
			pushfd						// Save flags
			cli							// Disable Ints (To avoid to be interrupted here)
			in eax,dx					// Get and save current State
			push eax
			mov eax,[addr]
			out dx,eax					// Set up addressing to config data
			mov dl,byte ptr [reg]
			and dl,2
			add dl,0FCh					// Access the proper offset
			mov ax,[value]
			out dx,ax					// Write byte
			mov dl,0F8h
			pop eax
			out dx,eax					// Restore configuration control
			popfd						// Restore flags
			pop edx
			pop eax
		}

	} else if (pci_CfgMech == 2) { // cfg_mech == 2

		if (device > 0x0F) {			// mech#2 only supports 16 devices per bus
			return false ;
		}

		DWORD addr = 0xC000 | ((device & 0x0F) << 8) | (reg & 0xFF) ;
		BYTE  fn = BYTE(((func & 7) << 1) | 0xf0);

		_asm {
			push eax
			push edx
			mov edx,0CF8h
			pushfd						// Save flags
			cli							// Disable Ints (To avoid to be interrupted here)
			in al,dx					// store current state of config space (enable)
			push eax
			mov dl,0FAh
			in al,dx					// store current state of config space (Bus)
			push eax
			mov al,byte ptr [bus]
			out dx,al					// Set bus
			mov dl,0F8h
			mov al,[fn]					
			out dx,al					// Set Fn
			mov edx,[addr]				// Set Device and register
			mov ax,[value]				// Get value
			out dx,ax					// Write value
			mov edx,0CF8h
			xor al,al					// al = 0
			out dx,al
			mov dl,0FAh
			pop eax
			out dx,al					// Restore configuration Space (Bus)
			mov dl,0F8h
			pop eax
			out dx,al					// Restore configuration Space (Enable)
			popfd						// Restore flags
			pop edx
			pop eax
		}

	} else {
		return false;
	}

	// Success!
	return true ;
}

bool pci_WriteConfigDWORD(ULONG bus, ULONG device, ULONG func, ULONG reg, DWORD value)
{
	if (!pci_CfgMech) {
		pci_HalEmuInit();
	}

	if (pci_CfgMech == 1) {

		DWORD addr = 0x80000000L | (((DWORD)(bus & 0xFF)) << 16) |
				((((unsigned)device) & 0x1F) << 11) |
				((((unsigned)func) & 0x07) << 8) | (reg & 0xFC) ;

		_asm {
			push eax
			push edx
			mov edx,0CF8h
			pushfd						// Save flags
			cli							// Disable Ints (To avoid to be interrupted here)
			in eax,dx					// Get and save current State
			push eax
			mov eax,[addr]
			out dx,eax					// Set up addressing to config data
			mov dl,0FCh					// Access the proper offset
			mov eax,[value]
			out dx,eax					// Write byte
			mov dl,0F8h
			pop eax
			out dx,eax					// Restore configuration control
			popfd						// Restore flags
			pop edx
			pop eax
		}
	} else if (pci_CfgMech == 2) { // cfg_mech == 2

		if (device > 0x0F) {			// mech#2 only supports 16 devices per bus
			return false ;
		}

		DWORD addr = 0xC000 | ((device & 0x0F) << 8) | (reg & 0xFF) ;
		BYTE  fn = BYTE(((func & 7) << 1) | 0xf0);

		_asm {
			push eax
			push edx
			mov edx,0CF8h
			pushfd						// Save flags
			cli							// Disable Ints (To avoid to be interrupted here)
			in al,dx					// store current state of config space (enable)
			push eax
			mov dl,0FAh
			in al,dx					// store current state of config space (Bus)
			push eax
			mov al,byte ptr [bus]
			out dx,al					// Set bus
			mov dl,0F8h
			mov al,[fn]					
			out dx,al					// Set Fn
			mov edx,[addr]				// Set Device and register
			mov eax,[value]				// Get value
			out dx,eax					// Write value
			mov edx,0CF8h
			xor al,al					// al = 0
			out dx,al
			mov dl,0FAh
			pop eax
			out dx,al					// Restore configuration Space (Bus)
			mov dl,0F8h
			pop eax
			out dx,al					// Restore configuration Space (Enable)
			popfd						// Restore flags
			pop edx
			pop eax
		}

	} else {
		return false;
	}

	// Success!
	return true ;
}

/*
 * (taken from Linux) Before we decide to use direct hardware access mechanisms, 
 * we try to do some trivial checks to ensure it at least _seems_ to be working
 * -- we just test whether bus 00 contains a host bridge.
 *
 * This should be close to trivial, but it isn't, because there are buggy
 * chipsets (yes, you guessed it, by Intel and Compaq) that have no class ID.
 */
#define PCI_CLASS_DEVICE        0x0a    /* Device class */
#define PCI_CLASS_BRIDGE_HOST	0x0600
#define PCI_CLASS_DISPLAY_VGA	0x0300
#define PCI_VENDOR_ID			0x00	/* 16 bits */
#define PCI_VENDOR_ID_INTEL		0x8086
#define PCI_VENDOR_ID_COMPAQ	0x0e11

bool pci_SanityCheck()
{
	WORD x;
	for(ULONG device = 0; device < 0x100; device++ )
		for (ULONG fn = 0; fn < 0x8; fn++ )
			if ((
					pci_ReadConfigWORD(0,device,fn, PCI_CLASS_DEVICE, &x) &&
					(x == PCI_CLASS_BRIDGE_HOST || x == PCI_CLASS_DISPLAY_VGA) 
				) || (
					pci_ReadConfigWORD(0,device,fn, PCI_VENDOR_ID, &x) &&
					(x == PCI_VENDOR_ID_INTEL || x == PCI_VENDOR_ID_COMPAQ)
				))
				return true;
	return false;
}

/*
 * Tries to Guess the right way to access PCI bus without rebooting the PC.
 */
bool pci_HalEmuInit()
{
	{
		// Check if configuration type 1 works.
		DWORD vret;
		_asm {
			push eax
			push edx
			pushfd
			cli
			mov al,001h
			mov edx,0CFBh
			out dx,al
			mov dl,0F8h
			in eax,dx
			push eax
			mov	eax,080000000h
			out dx,eax
			in eax,dx
			mov [vret],eax
			pop eax
			out dx,eax
			popfd
			pop edx
			pop eax
		}

		if (vret == 0x80000000) {
			pci_CfgMech = 1;
			if (pci_SanityCheck()) {
				return true;
			}
		}
	}

	{
		BYTE vret1;
		BYTE vret2;
		// Check if configuration type 2 works.
		_asm {
			push eax
			push edx
			pushfd
			cli
			xor al,al
			mov edx,0CFBh
			out dx,al
			mov dl,0F8h
			out dx,al
			mov dl,0FAh
			out dx,al
			mov dl,0F8h
			in al,dx
			mov [vret1],al
			mov dl,0FAh
			in al,dx
			mov [vret2],al
			popfd
			pop edx
			pop eax
		}

		if (vret1 == 0 && vret2 == 0) {
			pci_CfgMech = 2;
			if (pci_SanityCheck()) {
				return true;
			}
		}
	}

	// Failed!
	pci_CfgMech = 0;
	return false;
}

#else

// Emulation: Doesn't do anything
bool pci_ReadConfigBYTE(ULONG bus, ULONG device, ULONG func, ULONG reg, BYTE* value)
{
	value = 0;
	return true;
}

bool pci_ReadConfigWORD(ULONG bus, ULONG device, ULONG func, ULONG reg, WORD* value)
{
	value = 0;
	return true;
}

bool pci_ReadConfigDWORD(ULONG bus, ULONG device, ULONG func, ULONG reg, DWORD* value)
{
	value = 0;
	return true;
}

bool pci_WriteConfigBYTE(ULONG bus, ULONG device, ULONG func, ULONG reg, BYTE value)
{
	return true;
}

bool pci_WriteConfigWORD(ULONG bus, ULONG device, ULONG func, ULONG reg, WORD value)
{
	return true;
}

bool pci_WriteConfigDWORD(ULONG bus, ULONG device, ULONG func, ULONG reg, DWORD value)
{
	return true;
}

#endif


#if 0

extern "C" {
#include <ntddk.h>
}

/* WARNING: The following routines , despite that are the recommended way to access PCI space,
   have been declared obsolete by Microsoft, but they havn't provided a replacement.

   I have tested the following routines under WinME and WinXP, and they simply don't work.
   Under WinME, the driver does not load, and under WinXP the machine reboots, so don't use at all
   */

// Those definitions clash so we define them ourselves
typedef unsigned char		BYTE;
typedef unsigned short      WORD;
typedef unsigned long       DWORD;

	// WinNT uses the HAL to do this - (But Micro$oft documents them as 'obsolete, do not use')
	// In fact, there is no way (documented) to read the PCI config space of a device that is not
	// handled by this driver! ... Maybe above code will have to be used on WinXP also!

bool pci_ReadConfigBYTE(ULONG bus, ULONG device, ULONG func, ULONG reg, BYTE* value)
{

	PCI_SLOT_NUMBER slotNumber;
	slotNumber.u.AsULONG = 0;
	slotNumber.u.bits.DeviceNumber = device;
	slotNumber.u.bits.FunctionNumber = func;

	if (HalGetBusDataByOffset(PCIConfiguration,bus,slotNumber.u.AsULONG, value, reg, 1) == 1) {
		return true;
	}
	return false;
}

bool pci_ReadConfigWORD(ULONG bus, ULONG device, ULONG func, ULONG reg, WORD* value)
{

	PCI_SLOT_NUMBER slotNumber;
	slotNumber.u.AsULONG = 0;
	slotNumber.u.bits.DeviceNumber = device;
	slotNumber.u.bits.FunctionNumber = func;

	if (HalGetBusDataByOffset(PCIConfiguration,bus,slotNumber.u.AsULONG, value, reg, 2) == 2) {
		return true;
	}
	return false;
}

bool pci_ReadConfigDWORD(ULONG bus, ULONG device, ULONG func, ULONG reg, DWORD* value)
{

	PCI_SLOT_NUMBER slotNumber;
	slotNumber.u.AsULONG = 0;
	slotNumber.u.bits.DeviceNumber = device;
	slotNumber.u.bits.FunctionNumber = func;

	if (HalGetBusDataByOffset(PCIConfiguration,bus,slotNumber.u.AsULONG, value, reg, 4) == 4) {
		return true;
	}
	return false;
}

bool pci_WriteConfigBYTE(ULONG bus, ULONG device, ULONG func, ULONG reg, BYTE value)
{

	PCI_SLOT_NUMBER slotNumber;
	slotNumber.u.AsULONG = 0;
	slotNumber.u.bits.DeviceNumber = device;
	slotNumber.u.bits.FunctionNumber = func;

	if (HalSetBusDataByOffset(PCIConfiguration,bus,slotNumber.u.AsULONG, &value, reg, 1) == 1) {
		return true;
	}
	return false;
}

bool pci_WriteConfigWORD(ULONG bus, ULONG device, ULONG func, ULONG reg, WORD value)
{

	PCI_SLOT_NUMBER slotNumber;
	slotNumber.u.AsULONG = 0;
	slotNumber.u.bits.DeviceNumber = device;
	slotNumber.u.bits.FunctionNumber = func;

	if (HalSetBusDataByOffset(PCIConfiguration,bus,slotNumber.u.AsULONG, &value, reg, 2) == 2) {
		return true;
	}
	return false;
}

bool pci_WriteConfigDWORD(ULONG bus, ULONG device, ULONG func, ULONG reg, DWORD value)
{

	PCI_SLOT_NUMBER slotNumber;
	slotNumber.u.AsULONG = 0;
	slotNumber.u.bits.DeviceNumber = device;
	slotNumber.u.bits.FunctionNumber = func;

	if (HalSetBusDataByOffset(PCIConfiguration,bus,slotNumber.u.AsULONG, &value, reg, 4) == 4) {
		return true;
	}
	return false;
}

#endif


	 
