/*
    BtCard.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000-2002 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Log:
  
10/19/2001	ejtagle				As Andrew Miller suggested, now PV951 lets user choose any tuner 
11/20/2001  Kevin Liao			Fixed a silly bug in VideoSetConnector that didnt select properly Composite0-3
12/19/2001  Dima Kornev			AverMedia TVPhone uses PLL_35
01/21/2002  Sven Grothklags		Fixed Typhoon TView RDS card entry and TDA9873/74 code!
02/14/2002	Takahiro Sakamoto	Fixed GV-BCTV4/PCI SVHS input.
03/03/2002  ejtagle             I have used the code kindly provided by Torbjrn Jansson <torbjorn.jansson@mbox200.swipnet.se>
								to implement IR on WinFast2000 card. The code that was sent by him worked pretty well,
								but i had to modify it to adapt it to the new internal structure of the IR driver, 
								so, if it does not work now, it's my fault - Mail me to ejtagle@tutopia.com!
05/12/2002  Torbjrn Jansson	The was one who tracked and solved the Dscaler incompatibility problems.... Thanks a lot!
08/30/2002  Jan Nikitenko & ejt Fixed MSP34xx support. Now Simple mode should work... This fixes several audio problems related to the MSP34xx support. Thank Jan!
10/22/2002  Fred Hoogduin		Fixed the Audio TV norm problems due to a bug of TunerSetCurrentStandard() . Thanks a lot!
05/06/2003  John Newbigin <jn@it.swin.edu.au> Added support for the Grandtec16 video capture card and some other small fixes
*/


/*
	TO FIX_
		I2C in hardware for multiread/write is not working with MSP34xx

*/


/*

Card Database:
--------------

  Typhoon TView RDS
	TDA9873H at 0xb6
	SAA6588  for RDS
	Temic 4009


 */

#include "custprop.h"
#include "BtCard.hpp"
#include "CppRt.hpp"		// To get New and delete ops
#include "PCI.hpp"
#include "Registry.hpp"
#include "EEprom.hpp"
#include "HwCapture\Hw.h"

// To get the PCI Bus Handling class
#include "pci_io.hpp"

/* ------------------------------------------------------------------------- */
/* array with description for bt848 / bt878 tv/grabber cards sorted by model */

BtCard::CInfo BtCard::Info[] = 
{
	/* 0x00 */
        {/* " *** UNKNOWN *** ", */ 4, 1, 0, No_Digital,
			0, 2, 0, { 2, 3, 1, 0}, { 0, 0, 0, 0, 0},
			1,1,0, PLL_NONE, HwTuner::TC_Unknown },
        {/* "MIRO PCTV", */ 4, 1, 0, No_Digital,
			0, 2,15, { 2, 3, 1, 1}, { 2, 0, 0, 0,10},
			1,1,0, PLL_NONE, HwTuner::TC_Unknown },
        {/* "Hauppauge (bt84)", */ 4, 1, 0, No_Digital,
			0, 2, 7, { 2, 3, 1, 1}, { 0, 1, 2, 3, 4},
			1,1,0, PLL_NONE, HwTuner::TC_Unknown },
        {/* "STB", */ 3, 1, 0, No_Digital,
			0, 2, 7, { 2, 3, 1}, { 4, 0, 2, 3, 1},
			0,1,0, PLL_NONE, HwTuner::TC_Unknown }, // NEVER has an MSP34xx present!

        {/* "Intel", */ 3, 1, 0, No_Digital,
			0, NONE, 7, { 2, 3, 1}, { 0, 1, 2, 3, 4},
			1,1,0, PLL_NONE, HwTuner::TC_Unknown },
		{/* "Diamond DTV2000", */ 4, 1, 0, No_Digital,
			0, 2, 3, { 2, 3, 1, 0}, { 0, 1, 0, 1, 3},
			1,1,0, PLL_NONE, HwTuner::TC_Unknown },
		{/* "AVerMedia TVPhone", */ 3, 1, 0, No_Digital,
			0, 3,0x0c, { 2, 3, 1}, { 0x0c, 0x04, 0x08, 0x04, 0} , 
			1,1,0, PLL_35, HwTuner::TC_Unknown }, // Per recomendation of Dima Kornev
        {/* "MATRIX-Vision MV-Delta", */ 5, 1, 0, No_Digital,
			NONE, 3, 0, { 2, 3, 1, 0, 0},{0 },
			1,1,0, PLL_NONE, HwTuner::TC_Unknown },

	/* 0x08 */
        {/* "Fly Video II (Bt848)", */ 3, 1, 0, No_Digital,
			0, 2, 0xc00, { 2, 3, 1},{ 0, 0xc00, 0x800, 0x400, 0xc00},
			1,1,0, PLL_NONE, HwTuner::TC_Unknown }, // I have received reports that the TV audio is not working properly...
        {/* "TurboTV", */ 3, 1, 0, No_Digital,
			0, 2, 3, { 2, 3, 1}, { 1, 1, 2, 3, 0},
			1,1,0, PLL_NONE, HwTuner::TC_Unknown },
        {/* "Standard BT878 (Hauppauge (bt878))", */ 4, 1, 0, No_Digital,
			0, 2, 0x0f /* old:7*/ , { 2, 0, 1, 1}, { 0, 1, 2, 3, 4},
			1,1,0, PLL_28,   HwTuner::TC_Unknown },
        {/* "MIRO PCTV pro", */ 3, 1, 0, No_Digital,
			0, 2, 0x3000F, { 2, 3, 1}, {0x20001, 0x10001, 0, 0,10}, 
		//  0, 2, 0x3000F, { 2, 3, 1, 1}, {1, 0x10001, 0, 0,10}, 
			1,1,0, PLL_NONE, HwTuner::TC_Unknown },

		{/* "ADS Technologies Channel Surfer TV", */ 3, 1, 0, No_Digital,
			0, 2, 15, { 2, 3, 1}, { 13, 14, 11, 7, 0}, // dTV says TunerInput = 2
			1,1,0, PLL_NONE, HwTuner::TC_Unknown },
        {/* "AVerMedia TVCapture 98", */ 3, 4, 0, No_Digital,
			0, 2, 15, { 2, 3, 1}, { 13, 14, 11, 7, 0},
			1,1,0, PLL_28,   HwTuner::TC_Unknown },
        {/* "Aimslab VHX", */ 3, 1, 0, No_Digital,
			0, 2, 7, { 2, 3, 1}, { 0, 2, 1,/*1,2,*/ 3, 4}, // Works for me!
			1,1,0, PLL_NONE,  HwTuner::TC_Unknown  },
        {/* "Zoltrix TV-Max", */ 3, 1, 0,  No_Digital,
			0, 2,15, { 2, 3, 1}, {0 , 0, 1 , 0, 10},
			1,1,0, PLL_NONE,  HwTuner::TC_Unknown  },

	/* 0x10 */
        {/* "Pixelview PlayTV (bt878)", */ 3, 1, 0, No_Digital,
			0, 2, 0x01fe00, { 2, 3, 1}, { 0x01c000, 0, 0x018000, 0x014000, 0x002000 },
			1,1,0, PLL_28,    HwTuner::TC_Unknown  },
        {/* "Leadtek WinView 601", */ 3, 1, 0, No_Digital,
			0, 2, 0x830018, { 2, 3, 1}, { 0x4fa007,0xcfa007,0xcfa007,0xcfa007,0xcfa007},
			1,1,1, PLL_NONE,  HwTuner::TC_Unknown  }, 
        {/* "AVEC Intercapture", */ 3, 2, 0, No_Digital,
			0, 2, 0, {2, 3, 1}, {1, 0, 0, 0, 0},
			1,1,0, PLL_NONE,  HwTuner::TC_Unknown  },
		{/* "LifeView FlyKit w/o Tuner", */ 3, 1, 0, No_Digital,
			NONE, NONE, 0x8dff00, { 2, 3, 1}, { 0 },
			0,0,0, PLL_NONE,  HwTuner::TC_Unknown  },

        {/* "CEI Raffles Card", */ 3, 3, 0, No_Digital,
			0, 2, 0, {2, 3, 1}, {0, 0, 0, 0 ,0},
			0,1,0, PLL_NONE,  HwTuner::TC_Unknown  }, // NEVER has an MSP34xx present!
		{/* "Lucky Star Image World ConferenceTV", */ 3, 1, 0, No_Digital,
			0, 2, 0x00fffe07, { 2, 3, 1}, { 131072, 1, 1638400, 3, 4},
			1,1,0, PLL_28,   HwTuner::TC_PHILIPS_PAL_I },
		{/* "Phoebe Tv Master + FM (CPH050)", */ 3, 1, 0, No_Digital,
			0, 2, 0xc00, { 2, 3, 1},{0, 1, 0x800, 0x400, 0xc00},
			1,1,0, PLL_NONE,  HwTuner::TC_Unknown  },
        {/* "Modular Technology MM205 PCTV, bt878", */ 2, 1, 0, No_Digital,
			0, NONE, 7, { 2, 3 }, { 0, 0, 0, 0, 0 },
			0,1,0, PLL_28,  HwTuner::TC_ALPS_TSBB5_PAL_I  },

	/* 0x18 */
        {/* "[many vendors] CPH05X/06X (bt878)", */ 4, 1, 0, No_Digital,
			0, 2, 0xe00, { 2, 3, 1, 0 }, {0x400, 0x200, 0x000, 0x000, 0xc00},
			1,1,0, PLL_28,    HwTuner::TC_Unknown  },
        {/* "Terratec/Vobis TV-Boostar", */ 3, 1, 0, No_Digital,
			0, 2, 16777215 , { 2, 3, 1}, { 131072, 1, 1638400, 3,4},
			1,1,0, PLL_NONE,  HwTuner::TC_Unknown  },
        {/* "Newer Hauppauge WinCam (bt878)", */ 4, 1, 0, No_Digital,
			0, 3, 7, { 2, 0, 1, 1}, { 0, 1, 2, 3, 4},
			1,1,0, PLL_NONE,  HwTuner::TC_Unknown  },
        {/* "MAXI TV Video PCI2", */ 3, 1, 0, No_Digital,
			0, 2, 0xffff, { 2, 3, 1}, { 0, 1, 2, 3, 0xc00},
			1,1,0, PLL_NONE, HwTuner::TC_PHILIPS_SECAM},

        {/* "Terratec TerraTV+", */ 3, 1, 0, No_Digital,
			0, 2, 0x70000, { 2, 3, 1}, { 0x20000, 0x30000, 0x10000, 0x00000, 0x40000},
			1,1,0, PLL_NONE,  HwTuner::TC_PHILIPS_PAL},
        {/* "Imagenation PXC200", */ 5, 1, 0,  No_Digital,/* Jannik Fritsch <jannik@techfak.uni-bielefeld.de> */
			NONE, 1/* was: 4*/, 0, { 2, 3, 1, 0, 0}, { 0 }, 
			1,1,0, PLL_NONE,  HwTuner::TC_Unknown  },
        {/* "FlyVideo 98", */ 3, 1, 0, No_Digital,
			0, 2, 0x1800, {2, 3, 1}, { 0, 0x0800, 0x1000, 0x1000, 0x1800},
			1,1,0, PLL_NONE,  HwTuner::TC_Unknown  },
        {/* "iProTV", */ 3, 1, 0, No_Digital,
			0, 2, 1, { 2, 3, 1}, { 1, 0, 0, 0, 0 },
			1,1,0, PLL_NONE,  HwTuner::TC_Unknown  },

	/* 0x20 */
		{/* "Intel Create and Share PCI", */ 4, 1, 0, No_Digital,
			0, 2, 7, { 2, 3, 1, 1} /*{ 2, 3, 1, 1}*/ , { 4, 4, 4, 4, 4}, /* Per Christophe Ego suggestion/testing to get color image on composite input */
			1,1,0, PLL_NONE,  HwTuner::TC_Unknown /* No tuner by default! */ },
		{/* "Terratec TerraTValue", */ 3, 1, 0, No_Digital,
			0, 2, 0xffff00, { 2, 3, 1}, { 0x500, 0, 0x300, 0x900, 0x900},
			1,1,0, PLL_28,  HwTuner::TC_PHILIPS_PAL  }, 
		{/* "Leadtek WinFast 2000", */ 3, 1, 0, No_Digital,
			0, 2, 0x803000, { 2, 3, 1}, { 0x422000,0x001000,0x621100,0x620000,0x800000},
			1,1,0, PLL_28,    HwTuner::TC_Unknown  },
		{/* "Flyvideo 98 (LR50Q) / Chronos Video Shuttle II", */ 3, 3, 0, No_Digital,
			0, 2, 0x1800, { 2, 3, 1}, { 0, 0x800, 0x1000, 0x1000, 0x1800},
			1,1,0, PLL_28,    HwTuner::TC_Unknown  },
		{/* "Flyvideo 98FM (LR50Q) / Typhoon TView TV/FM Tuner", */ 3, 3, 0, No_Digital,
			0, 2, 0x1800, { 2, 3, 1}, { 0, 0x800, 0x1000, 0x1000, 0x1800},
			1,1,0, PLL_28,    HwTuner::TC_Unknown  },
		{/* "PixelView PlayTV pro", */ 3, 1, 0, No_Digital,
			0, 2, 0xff, { 2, 3, 1}, { 0x21, 0x20, 0x24, 0x2c, 0x29 }, 
			0,1,0, PLL_28,    HwTuner::TC_Unknown  },
		{/* "TView99 CPH06X", */ 4, 1, 0, No_Digital,
			0, 2, 0x551e00, { 2, 3, 1, 0}, { 0x551400, 0x551200, 0, 0,  0x551c00 }, 
			1,1,0, PLL_28,  HwTuner::TC_Unknown  },
		{/* "Pinnacle PCTV Studio/Rave", */ 3, 1, 0, No_Digital,
			0, 2, 0x03000F, { 2, 3, 1}, { 2, 0, 0, 0, 1},
			1,1,0, PLL_28,  HwTuner::TC_Unknown  },

	/* 0x28 */
        {/* "STB2", */ 3, 1, 0, No_Digital,
			0, 2, 7, { 2, 3, 1}, { 4, 0, 2, 3, 1},
			0,1,0, PLL_NONE,  HwTuner::TC_Unknown }, // NEVER has an MSP34xx present!
        {/* "AVerMedia TVPhone 98", */ 3, 4, 0, No_Digital,
			0, 2, 12, { 2, 3, 1}, { 13, 4, 11, 7, 0},
			1,1,1, PLL_28,    HwTuner::TC_Unknown },
        {/* "ProVideo PV951", */ 3, 1, 0, No_Digital,/* pic16c54 */
			0, 2, 0, { 2, 3, 1}, { 0, 0, 0, 0, 0},
			0,1,0, PLL_28,   HwTuner::TC_Unknown }, // Due to mods suggested by Andrew Miller, let user select tuner
		{/* "Little OnAir TV", */ 3, 1, 0, No_Digital,
			0, 2, 0xe00b, {2, 3, 1}, {0xff9ff6, 0xff9ff6, 0xff1ff7, 0, 0xff3ffc},
			0,1,0, PLL_NONE,  HwTuner::TC_Unknown  },

		{/* "Sigma TVII-FM", */ 2, 1, 0, No_Digital,
			0, NONE, 3, {2, 3}, {1, 1, 0, 2, 3},
			0,1,0, PLL_NONE,  HwTuner::TC_Unknown  },	
		{/* "MATRIX-Vision MV-Delta 2", */ 5, 1, 0, No_Digital,
			NONE, 3, 0, { 2, 3, 1, 0, 0},{0 }, 
			0,1,0, PLL_28,  HwTuner::TC_Unknown  },
		{/* "Zoltrix Genie TV/FM", */ 3, 1, 0, No_Digital,
			0, 2, 0xbcf03f, { 2, 3, 1}, { 0xbc803f, 0xbc903f, 0xbcb03f, 0, 0xbcb03f}, 
			0,1,0, PLL_28,  HwTuner::TC_TEMIC_4039FR5_NTSC },
		{/* "Terratec TV/Radio+", */ 3, 1, 0,  No_Digital,/* Radio ?? */
			0, 2, 0x1f0000, { 2, 3, 1}, { 0xe2ffff, 0xebffff, 0, 0, 0xe0ffff },
			0,1,1, PLL_35,  HwTuner::TC_PHILIPS_PAL_I },

	/* 0x30 */
		{/* "Dynalink Magic TView ", */ 3, 1, 0,  No_Digital,
			0, 2, 15, { 2, 3, 1}, {2,0,0,0,1},
			1,1,0, PLL_28,  HwTuner::TC_Unknown },
		{/* "GV-BCTV3" , */ 3, 1, 0, No_Digital,
			0, 2,0x010C00, {2,3,0} , {0x10000, 0, 0x10000, 0, 0},
			0,1,0, PLL_28,  HwTuner::TC_ALPS_TSHC6_NTSC } ,
		{/* "Prolink PV-BT878P+4E / PixelView PlayTV PAK / Lenco MXTV-9578 CP" ,*/ 4, 1, 1, RQS_Digital,
			0, 2,0xAA0000,{2,3,1,1} , { 0x20000, 0, 0x80000, 0x80000, 0xa8000},
			0,1,0, PLL_28,  HwTuner::TC_PHILIPS_PAL_I } ,
		{/* "Eagle Wireless Capricorn2 (bt878A)" ,*/ 4, 1, 0, No_Digital,
			0, 2, 7 , { 2, 0, 1, 1}, { 0, 1, 2, 3, 4}, 
			1,1,0, PLL_28,  HwTuner::TC_Unknown } ,

		{/* "Pinnacle PCTV Studio Pro", */ 3, 1, 0,  No_Digital,/* David Hrdeman <david@2gen.com> */
			0, 2, 0x03000F, { 2, 3, 1}, { 1, 0x10001, 0, 0, 10},  
			1,1,0, PLL_28,  HwTuner::TC_Unknown } ,
		{/* "Typhoon TView RDS + FM Stereo / KNC1 TV Station RDS", */ 3, 3, 0,  No_Digital,/* Claas Langbehn <claas@bigfoot.com>, Sven Grothklags <sven@upb.de> */
			0, 2, 0x1c, {2 , 3 , 1}, { 0, 0, 0x10, 8, 4 },  
			1,1,1, PLL_28,  HwTuner::TC_TEMIC_4009FR5_PAL } , /* has_radio = 1*/
		{/* "Livetec LT 9415 TV (LR90 Rev.F)", */ 4, 1, 0,  No_Digital,/* Tim Rstermundt <rosterm@uni-muenster.de>  */
			0, 2, 0x1060, { 2, 3, 1, 0},	{ 0x0000,0x0800,0x1000,0x1000,0x18e0 },  
			1,1,1, PLL_28,  HwTuner::TC_PHILIPS_PAL } , /* has_radio = 1*/
		{/* "BESTBUY Easy TV (model CPH031)", */ 4, 1, 0, No_Digital,/* Miguel Angel Alvarez <maacruz@navegalia.com> */
			0 ,2, 0xF , { 2, 3, 1, 0}, { 2, 0, 0, 0, 10}, 
			1,1,0, PLL_28, HwTuner::TC_TEMIC_PAL } ,

	/* 0x38 */
		{/* "FlyVideo '98/FM / CyberVideo Xtreme", */ 3, 3, 0,  No_Digital,/* Gordon Heydon <gjheydon@bigfoot.com ('98) */
			0 ,2, 0x1800, { 2, 3, 0}, { 0, 0x800, 0x1000, 0x1000, 0x1800},
			1,1,0, PLL_28, HwTuner::TC_Unknown } ,
		{/* "GrandTec 'Grand Video Capture'", */ 2, 0, 0,  No_Digital,/* Steve Hosgood <steve@equiinet.com> */
			NONE, 1 , 0, {3,1} , {0,0,0,0} , 
			0,0,0, PLL_35, HwTuner::TC_Absent } ,
		{/* "Phoebe TV Master Only (No FM) CPH060" , */ 3, 1, 0,  No_Digital,/* Daniel Herrington <daniel.herrington@home.com> */
			0, 2, 0x0e00, {2,3,1} , { 0x400, 0x400, 0x400, 0x400, 0x800 }, 
			1,1,0, PLL_NONE, HwTuner::TC_TEMIC_4036FY5_NTSC } ,
		{/* "TV Capturer (CPH03X)", */ 4, 1, 0,  No_Digital,/* Matti Mottus <mottus@physic.ut.ee> */
			0, 2, 0x03000F, { 2, 3, 1, 0}, { 2,0,0,0,1 }, 
			1,1,0, PLL_28, HwTuner::TC_TEMIC_PAL } ,

		{/* "MM100PCTV",*/ 3, 1, 0, No_Digital,// Fixed by ejt
			0, 2, /*11*/ 0xF, { 2, 3 ,1}, { 2, 0, /*0*/1, 1, /*8*/ 3}, 
			1,1,0, PLL_NONE, HwTuner::TC_TEMIC_PAL } ,
		{/* "AG Electronics GMV1", */ 2, 0, 0, No_Digital,/* Adrian Cox <adrian@humboldt.co.uk */
			NONE, 1, 0xF, { 2, 2}, 
			0,1,0, PLL_28, HwTuner::TC_Absent }, 
		{/* "BESTBUY Easy TV (bt878, model CPH061))", */ 3, 2, 0, No_Digital, /* Miguel Angel Alvarez <maacruz@navegalia.com> */
			0, 2, 0xFF, { 2, 3, 1}, { 1, 0, 4, 4, 9},
			1,1,0, PLL_28, HwTuner::TC_PHILIPS_PAL },
		{/* "ATI TV-Wonder", */ 3, 1, 0,  No_Digital,/* Lukas Gebauer <geby@volny.cz> */
			0, 2, 0xf03f, { 2, 3, 1}, { 0xbffe, 0, 0xbfff, 0, 0xbffe},
			1,1,0, PLL_28, HwTuner::TC_TEMIC_4006FN5_MULTI_PAL },

	/* 0x40 */
		{/* "ATI TV-Wonder VE", */ 2, 1, 0,  No_Digital,/* Peter Ploskov <asysta@mail.techno-link.com> */
			0, NONE, 0x300f, { 2, 3, 1}, {0x1002, 0x1002, 0x3003, 0x3003, 0x3003},
			0,1,0, PLL_28, HwTuner::TC_TEMIC_4006FN5_MULTI_PAL }, // TEMIC4006FN5
		{/* "FlyVideo 2000", */ 3, 3, 0,  No_Digital,/* DeeJay <deejay@westel900.net (2000S) */
			0, 2, 0x18e0, { 2, 3, 0}, { 0,0x18e0,0x1000,0x1000,0x1080 },
			1,1,0, PLL_28, HwTuner::TC_Unknown },
		{/* "Terratec TValueRadio", */ 3, 1, 0,  No_Digital,
			0, 2, 0xffff00, { 2, 3, 1}, { 0x500, 0x500, 0x300, 0x900, 0x900},
			1,1,1, PLL_28, HwTuner::TC_PHILIPS_PAL },
		{/* "GV-BCTV4/PCI", */ 3, 1, 0, No_Digital,
			0, 2, 0x010f00, { 2, 3, 1}, { 0x10000, 0, 0x10000, 0, 0}, // was {2,3,0} before, Takahiro Sakamoto <takas@yl.is.s.u-tokyo.ac.jp>
			0,1,0, PLL_28, HwTuner::TC_SHARP_2U5JF5540_NTSC },

		{/* "3Dfx VoodooTV FM (Euro), VoodooTV 200 (USA)", */ 4, 1, 0, No_Digital,
			0, NONE, 0x4e8a00, { 2, 3 ,0 ,1}, {0x947fff, 0x987fff,0x947fff,0x947fff, 0x947fff}, 
			1,1,1, PLL_28, HwTuner::TC_MT2032 }, /* has_radio */
		{/* "Active Imaging AIMMS", */ 3, 4, 0, No_Digital,
			0, 2, 15, { 2,3,1,1}, {0, 0,0,0, 0}, 
			1,1,0, PLL_28, HwTuner::TC_Unknown },
		{/* "PV-BT878P+", */ 1, 0, 0, No_Digital,
			NONE, 0, 0, { 2}, { 0, 0, 11, 7, 13},
			1,1,0, PLL_28, HwTuner::TC_LG_PAL_I_FM },
		{/* "Flyvideo 98EZ (capture only)", */ 4, 0, 0, No_Digital,
			NONE, 2, 0, { 2,3,1,1 }, { 0, 0, 11, 7, 13},
			0,1,0, PLL_28, HwTuner::TC_Unknown},

	/* 0x48 */
		{/* "Prolink PV-BT878P+9B (PlayTV Pro rev.9B FM+NICAM)", */ 3, 1, 0, No_Digital,
			0, 2, 0x3c, { 2,3,0,1 }, { 0x01, 0x00, 0x03, 0x03, 0x09},
			0,1,1, PLL_28, HwTuner::TC_Unknown},
		{/* "Sensoray 311", */ 5, 0, 0, No_Digital,
			NONE, 4, 0, { 2,3,1,0 }, { 0 },
			1,0,0, PLL_28, HwTuner::TC_Unknown},
		{/* "Conexant Bt878 NTSC XEVK", */ 3, 1, 1, No_Digital,
			0, 2, 0xFFFEFF, { 2, 3, 1}, { 0x001000, 0x001000, 0x000000, 0x000000, 0x003000},
			1,1,0, PLL_NONE, HwTuner::TC_PHILIPS_NTSC	} ,
		{/* "Rockwell Bt878 NTSC XEVK", */ 3, 1, 1, No_Digital,
			0, 2, 0xFFFEFF, { 2, 3, 1}, { 0x001000, 0x001000, 0x000000, 0x000000, 0x003000},
			1,1,0, PLL_NONE, HwTuner::TC_PHILIPS_NTSC } ,

		{/* "Conexant Foghorn NTSC/ATSC-A", */ 3, 1, 1, No_Digital,
			0, 2, 0xFF00F8, { 2, 3, 1}, { 0x000048, 0x000048, 0x000048, 0x000048, 0x000048},
			1,1,0, PLL_NONE, HwTuner::TC_PHILIPS_1236D_NTSC_INPUT1 } ,
		{/* "Conexant Foghorn NTSC/ATSC-B", */ 3, 1, 1, No_Digital,
			0, 2, 0xFF00F8, { 2, 3, 1}, { 0x000048, 0x000048, 0x000048, 0x000048, 0x000048},
			1,1,0, PLL_NONE, HwTuner::TC_PHILIPS_1236D_NTSC_INPUT1 } ,
		{/* "Conexant Foghorn NTSC/ATSC-C", */ 3, 1, 1, No_Digital,
			0, 2, 0xFF00F8, { 2, 3, 1}, { 0x000048, 0x000048, 0x000048, 0x000048, 0x000048},
			1,1,0, PLL_NONE, HwTuner::TC_PHILIPS_1236D_NTSC_INPUT1 } ,
		{/* "RS BT Card", */ 2, 1, 4, CCIR656_Digital,
			NONE, 0, 0, { 1, 3, 0, 0, 0/*and up...*/}, { 0,0,0,0,0 },
			1,1,0, PLL_28, HwTuner::TC_Absent },

	/* 0x50 */
        {/* "Tekram M205 Pro", */ 3, 1, 0,  No_Digital,/* Thai Khanh <thaivankhanh@yahoo.com> */
			0, 2, 0x000078, { 2, 3, 1}, {0x000068, 0x000068, 0x000061, 0x000061, 0x000060},
			1,1,0, PLL_NONE,  HwTuner::TC_Unknown },
		{/* "Bt848A based with 14Mhz ref. Xtal", */ 4, 0, 0, No_Digital,
			NONE, 3, 0,{0,3,2,1} , {0,0,0,0} , 
			0,0,0, PLL_14, HwTuner::TC_Absent } ,
		{/* "Bt8xx based with 28Mhz ref. Xtal and 4 Composite ins", */ 4, 0, 0, No_Digital,
			NONE, NONE, 0,{0,3,2,1} , {0,0,0,0} , 
			0,0,0, PLL_28, HwTuner::TC_Absent } ,
		{/* "FlyVideo 2000S", */ 3, 3, 0, No_Digital,
			0, 2, 0x18e0, { 2, 3, 0}, { 0x1080 , 0x18e0 , 0x1000 , 0x1000 , 0x1080 },
			1,1,0, PLL_28, HwTuner::TC_Unknown },

		{/* "Hoontech 878TV/FM", */ 3, 1, 0, No_Digital,
			0, 2, 0x0000300F, { 2, 3, 0}, { 0x1002 , 0x1002 , 0x4002 , 0x4002 , 0x3003 }, // Audio tuner uses 0x700F
			1,1,0, PLL_28, HwTuner::TC_Unknown },
		{/* "KWorld TV878RF-PRO", */ 3, 4, 0,  No_Digital,/* Contributed by Zvone <zdigas@public.srce.hr> */
			0, 2, 0x1c8007, { 2, 3, 1, 0}, { 0x1c0000 , 0x080009 , 0x080002 , 0x080002 , 0x080009 },
			1,1,0, PLL_28, HwTuner::TC_Unknown }, // (Philips BG+DK tuner)
		{/* "Gallant InterVisionTV IV550", */ 3, 3, 0,  No_Digital,/* Contributed by A. Jackson <ade.jackson@btopenworld.com> */
			0, 2, 0x0080ff, { 2, 3, 1, 0}, { 0x000001 , 0x000001 , 0x000041 , 0x000041 , 0x000040 },
			1,1,0, PLL_28, HwTuner::TC_Unknown }, 
		{/* "Mach TV 878", */ 3, 4, 0, No_Digital,
			1, 2, 0x1c800f, { 2, 3, 1, 0}, { 0x08000a , 0x080009 , 0x08000c , 0x08000c , 0x08000c },
			1,1,0, PLL_28, HwTuner::TC_Unknown }, 

	/* 0x58 */
		
		{/* "Asonic Skyeye BT878", */ 3, 3, 0, No_Digital,
			0, 2, 0x00000f, { 2, 3, 1, 0}, { 0x000001 , 0x000001 , 0x000002 , 0x000002 , 0x000009 },
			1,1,0, PLL_28, HwTuner::TC_Unknown }, 
		{/* "Jetway PciTV Station", */ 1, 1, 0, No_Digital,/* Justin Smith <moose_999@hotmail.com> PAL BGI RC */ 
			0, NONE, 0x1c8007, { 2, 3, 1, 0}, { 0x080000 , 0x080000 , 0x080003 , 0x080003 , 0x080003 },
			1,1,0, PLL_28, HwTuner::TC_Unknown }, 
		{/* "Boeder TV/FM Capture Card 68.404", */ 3, 4, 0, No_Digital,/* Tnu Janno <tjanno@neti.ee> */
			0, 2, 0xfff03f, { 2, 3, 1, 0}, { 0xebb002 , 0xebb003 , 0xebb002 , 0xebb002 , 0xebb003 },
			1,1,1, PLL_28, HwTuner::TC_Unknown }, 
		{/* "Spirit TV Tuner", */ 3, 3, 0, No_Digital,/* Metabolis <meta@wiw.org> */
			0, 2, 0x00000f, { 2, 3, 1, 0}, { 0x000002 , 0x000002 , 0x000001 , 0x000001 , 0x000003 },
			1,1,0, PLL_28, HwTuner::TC_Unknown }, 

		{/* "AVerMedia TVPhone (old)", */ 3, 1, 0, No_Digital,
			0, 3,0x00702c, { 2, 3, 1}, { 0x00600e, 0x006006, 0x00600a, 0x00600a, 0x006002} ,
			1,1,0, PLL_35, HwTuner::TC_Unknown },
		{/* "SDI Silk 100 (S-Video Jumper)", */ 4, 1, 1, CCIR656_Digital,
			NONE, 0,0, { 2, 0, 1, 3, 2}, { 0, 0, 0, 0, 0} ,
			0,0,0, PLL_28, HwTuner::TC_Absent },
		{/* "SDI Silk 100 (C-Video Jumper)", */ 4, 1, 1, CCIR656_Digital,
			NONE, NONE,0, { 0, 1, 2, 3}, { 0, 0, 0, 0, 0} ,
			0,0,0, PLL_28, HwTuner::TC_Absent },
		{/* "Sasem 4-Channel Dev Board (S-Video Jumper)", */ 4, 1, 1, CCIR656_Digital,
			NONE, 0,0, { 2, 2, 2,2,2}, { 0, 0, 0, 0, 0} ,
			0,0,0, PLL_28, HwTuner::TC_Absent },

	/* 0x60 */
		{/* "Typhoon TV-Tuner Art.Nr.: 50272", */ 3, 1, 0, No_Digital,
			0, 2, 0xffffff, { 2, 1, 3,0}, { 0xffff21, 0xffff21, 0xffff24, 0xffff24, 0xffff29} ,
			1,1,0, PLL_28, HwTuner::TC_Unknown },
		{/* "3cam PCI Video Capture Card", */ 3, 1, 0, No_Digital,
			NONE, 2, 0x00000f, { 2, 3, 1,0}, { 0x000001, 0x000001, 0x000002, 0x000002, 0x000001} ,
			1,1,0, PLL_28, HwTuner::TC_Absent },
		{/* "Cucus Video Capture Card", */ 1, 1, 0, No_Digital,
			NONE, NONE, 0x00000f, { 3, 0, 0,0}, { 0x000003, 0x000003, 0x000003, 0x000003, 0x000003} ,
			0,0,0, PLL_28, HwTuner::TC_Absent },
		{/* "High-speed Video Capture Card C8-T.N.C.", */ 2, 1, 0, No_Digital,
			NONE, 1,    0x00000f, { 3, 1, 0,0}, { 0x000001, 0x000001, 0x000001, 0x000001, 0x000001} ,
			0,0,0, PLL_28, HwTuner::TC_Absent },

		{/* "Skywell Magic TV Card", */ 3, 1, 0, No_Digital,
			0, 1,    0x000007, { 2, 1, 3, 0}, { 0, 0, 2, 0, 1} ,
			1,1,0, PLL_28, HwTuner::TC_Unknown },
		{/* "SDI Silk 200 (S-Video Jumper)", */ 4, 1, 3, CCIR656_Digital,
			NONE, 0, 0x1F800, { 2, 0, 1, 3, 2}, { 0xD, 0xE, 0xB, 0x7, 0} ,
			0,0,0, PLL_28, HwTuner::TC_Absent },
		{/* "SDI Silk 200 (C-Video Jumper)", */ 4, 1, 3, CCIR656_Digital,
			NONE, NONE, 0x1F800, { 0, 1, 2, 3, 2}, { 0xD, 0xE, 0xB, 0x7, 0} ,
			0,0,0, PLL_28, HwTuner::TC_Absent },
		{/* "Canopus WinDVR PCI (COMPAQ Presario 3524JP, 5112JP)", */ 3, 1, 0, No_Digital,
			0, 2, 0x7, { 2, 3, 1, 0}, { 0, 1, 2, 3, 4} ,
			0,0,0, PLL_28, HwTuner::TC_Unknown },

	/* 0x68 */
		{/* "Primavision Video Capture card", */ 4, 0, 0, No_Digital,
			NONE, NONE, 0x0, { 2, 2, 2, 2}, { 0, 0, 0, 0, 0} ,
			0,0,0, PLL_28, HwTuner::TC_Absent },

		{/* "GrandTec 16", */ 16, 1, 0, No_Digital,
			NONE, NONE, 0/*0x18e0*/, { 2, 1, 3, 0}, { 0, 0, 0, 0, 0},
			0,0,0, PLL_28, HwTuner::TC_Absent },

		{/* "Trust 814 PCI", */ 4, 1, 0, No_Digital,
			NONE, NONE, 0, { 2, 2, 2, 2}, { 0, 0, 0, 0, 0},
			0,0,0, PLL_28, HwTuner::TC_Absent },

		{/* "SweetSpot/PDI Deluxe", */ 2, 1, 4, CCIR656_Digital,
			NONE, 0, 0, { 1, 3, 0, 0, 0/*and up...*/}, { 0,0,0,0,0 },
			1,1,0, PLL_28, HwTuner::TC_Absent },

        {/* "Aimslab VHX Deluxe", */ 3, 1, 0, No_Digital,
			0, 2, 7, { 2, 1, 1}, { 0, 2, 1,/*1,2,*/ 3, 4}, // (Composite over SVideo)
			1,1,0, PLL_NONE,  HwTuner::TC_Unknown  },

	/* 0x6D */
 		{/* "Custom card...", */ 3, 1, 1, No_Digital,
 			0, 2, 0x18e0, { 2, 3, 0}, { 0x1080 , 0x18e0 , 0x1000 , 0x1000 , 0x1080 },
 			1,1,0, PLL_28, HwTuner::TC_Unknown }

};

// Table with relationship between Vendor and Subvendor code and card
/* ----------------------------------------------------------------------- */
/* list of card IDs for bt878+ cards                                       */

BtCard::CARD BtCard::Card[] = {
	{ 0x00011002, BtCard::ATI_TVWONDER			/*,"ATI TV Wonder"				*/},
	{ 0x00011461, BtCard::AVPHONE98				/*,"AVerMedia TVPhone98"		*/},
	{ 0x00021461, BtCard::AVERMEDIA98			/*,"Avermedia TVCapture 98"		*/},
	{ 0x00031002, BtCard::ATI_TVWONDERVE		/*,"ATI TV Wonder/VE"			*/},
	{ 0x00031461, BtCard::AVPHONE98				/*,"AVerMedia TVPhone98"		*/},
	{ 0x00041461, BtCard::AVERMEDIA98			/*,"AVerMedia TVCapture 98"		*/},
	{ 0x001211bd, BtCard::PINNACLE				/*,"Pinnacle PCTV"				*/},
	{ 0x001c11bd, BtCard::PINNACLE				/*,"Pinnacle PCTV Sat"			*/},
	{ 0x00790e11, BtCard::WINDVR				/*,"Canopus WinDVR PCI"			*/},
	{ 0x010114c7, BtCard::MODTEC_205			/*,"Modular Technology PCTV"	*/},
    { 0x010115cb, BtCard::GMV1					/*,"AG GMV1"					*/},
	{ 0x013214F1, BtCard::CONEXANTFOGHORNREVA	/*,"Conexant Foghorn NTSC/ATSC-A"*/ },
	{ 0x023214F1, BtCard::CONEXANTFOGHORNREVB	/*,"Conexant Foghorn NTSC/ATSC-B"*/ },
	{ 0x03116000, BtCard::SENSORAY311			/*,"Sensoray 311"				*/},
	{ 0x033214F1, BtCard::CONEXANTFOGHORNREVC	/*,"Conexant Foghorn NTSC/ATSC-C"*/ },
	{ 0x10b42636, BtCard::HAUPPAUGE878			/*,"STB ???"					*/},
	{ 0x1117153b, BtCard::TERRATVALUE			/*,"Terratec TValue"			*/},
	{ 0x1118153b, BtCard::TERRATVALUE			/*,"Terratec TValue"			*/},
	{ 0x1119153b, BtCard::TERRATVALUE			/*,"Terratec TValue"			*/},
	{ 0x111a153b, BtCard::TERRATVALUE			/*,"Terratec TValue"			*/},
	{ 0x1123153b, BtCard::TERRATVRADIO			/*,"Terratec TV Radio+"			*/},
	{ 0x1127153b, BtCard::TERRATV				/*,"Terratec TV+"				*/},
	{ 0x1134153b, BtCard::TERRATVALUE			/*,"Terratec TValue"			*/},
	{ 0x1135153b, BtCard::TERRATVALUER			/*,"Terratec TValue Radio"		*/},
	{ 0x1200bd11, BtCard::PINNACLE				/*,"Pinnacle PCTV"				*/},
	{ 0x1322127A, BtCard::ROCKWELLNTSCXEVK		/*,"Rockwell Bt878A NTSC XEVK"*/ },
	{ 0x13eb0070, BtCard::HAUPPAUGE878			/*,"Hauppauge WinTV"			*/},
	{ 0x14610002, BtCard::AVERMEDIA98			/*,"Avermedia TVCapture 98"		*/},
	{ 0x182214F1, BtCard::CONEXANTNTSCXEVK		/*,"Conexant Bt878A NTSC XEVK"*/ },
	{ 0x18501851, BtCard::CHRONOS_VS2			/*,"Flyvideo 98 (LR50)/ Chronos Video Shuttle II"	*/},
	{ 0x18511851, BtCard::FLYVIDEO98EZ			/*,"Flyvideo 98EZ (LR51)/ CyberMail AV"				*/},
	{ 0x18521852, BtCard::TYPHOON_TVIEW			/*,"Flyvideo 98FM (LR50)/ Typhoon TView TV/FM Tuner / CyberVideo Xtreme" */},
	{ 0x217d6606, BtCard::WINFAST2000			/*,"Leadtek WinFast TV 2000"	*/},
	{ 0x263610b4, BtCard::STB2					/*,"STB TV PCI FM, P/N 6000704" */},
	{ 0x3000121a, BtCard::VOODOOTV_FM			/*,"3Dfx VoodooTV FM/ VoodooTV 200"*/},
	{ 0x3000144f, BtCard::MAGICTVIEW063			/*,"(Askey Magic/others) TView99 CPH06x"			*/},
	{ 0x300014ff, BtCard::MAGICTVIEW061			/*,"TView 99 (CPH061)"			*/},
	{ 0x3002144f, BtCard::MAGICTVIEW061			/*,"(Askey Magic/others) TView99 CPH05x"			*/},
	{ 0x300214ff, BtCard::PHOEBE_TVMAS			/*,"Phoebe TV Master (CPH060)"			*/},
	{ 0x3005144f, BtCard::MAGICTVIEW061			/*,"(Askey Magic/others) TView99 CPH061/06L (T1/LC)"*/},
	{ 0x300214ff, BtCard::PHOEBE_TVMAS			/*,"Phoebe TV Master"			*/},
	{ 0x3100121a, BtCard::VOODOOTV_FM			/*,"3Dfx VoodooTV FM/ VoodooTV 200 (OEM)"*/},
	{ 0x31323334, BtCard::GRANDTEC				/*,"GrandTec XCapture"*/ },
	{ 0x39000070, BtCard::HAUPPAUGE878			/*,"Hauppauge WinTV-D"			*/},
	{ 0x400a15b0, BtCard::ZOLTRIX_GENIE			/*,"Zoltrix Genie TV"			*/},
	{ 0x400d15b0, BtCard::ZOLTRIX_GENIE			/*,"Zoltrix Genie TV / Radio"	*/},
	{ 0x401015b0, BtCard::ZOLTRIX_GENIE			/*,"Zoltrix Genie TV / Radio"	*/},
	{ 0x401615b0, BtCard::ZOLTRIX_GENIE			/*,"Zoltrix Genie TV / Radio"	*/},
 	{ 0x402010fc, BtCard::GVBCTV3PCI			/*,"I-O Data Co. GV-BCV3/PCI"	*/},
	{ 0x405010fc, BtCard::GVBCTV4PCI			/*,"I-O Data Co. GV-BCV4/PCI"	*/},
	{ 0x45000070, BtCard::HAUPPAUGE878			/*,"Hauppauge WinTV/PVR"		*/},
	{ 0x5018153b, BtCard::TERRATVALUE			/*,"Terratec TValue"			*/},
	{ 0x6606107d, BtCard::WINFAST2000			/*,"Leadtek WinFast TV 2000"	*/},
	{ 0xa0fca1a0, BtCard::ZOLTRIX				/*,"Face to Face Tvmax"			*/},    
	{ 0xff000070, BtCard::HAUPPAUGE878			/*,"Osprey-100"					*/},
	{ 0xff010070, BtCard::HAUPPAUGE878			/*,"Osprey-200"					*/},
	{ 0x81828384, BtCard::GRANDTEC16			/*,"GrandTec Grand X Guard 16"	*/},	
	{ 0x01020304, BtCard::TRUST814PCI			/*,"Trust 814 PCI"				*/},
#if 0 /* probably wrong */
	{ 0x6606217d, BtCard::WINFAST2000			/*,"Leadtek WinFast TV 2000"	*/},
#endif
	{ 0, BtCard::UNKNOWN						/*, NULL						*/}
};

// Miro Specific
/* Mapping from MIRO coding to the tuners supported */
HwTuner::Tuner BtCard::miro_tunermap[] = 
{	
	HwTuner::TC_TEMIC_PAL			,HwTuner::TC_TEMIC_NTSC		,HwTuner::TC_PHILIPS_NTSC	,HwTuner::TC_PHILIPS_SECAM,   
	HwTuner::TC_Absent				,HwTuner::TC_PHILIPS_PAL	,HwTuner::TC_TEMIC_NTSC		,HwTuner::TC_TEMIC_PAL,  
	HwTuner::TC_PHILIPS_SECAM		,HwTuner::TC_TEMIC_PAL		,HwTuner::TC_Absent			,HwTuner::TC_PHILIPS_PAL,  
	HwTuner::TC_PHILIPS_PAL			,HwTuner::TC_PHILIPS_NTSC	,HwTuner::TC_TEMIC_PAL_DK	,HwTuner::TC_PHILIPS_PAL_I,
				
	HwTuner::TC_TEMIC_4006FH5_PAL	,HwTuner::TC_PHILIPS_NTSC	,HwTuner::TC_TEMIC_NTSC_M	,HwTuner::TC_PHILIPS_PAL_I, 
	HwTuner::TC_Absent				,HwTuner::TC_PHILIPS_PAL_I	,HwTuner::TC_Absent			,HwTuner::TC_PHILIPS_SECAM,  
	HwTuner::TC_PHILIPS_PAL_I		,HwTuner::TC_PHILIPS_NTSC	,HwTuner::TC_TEMIC_PAL_DK	,HwTuner::TC_PHILIPS_PAL_I, 
	HwTuner::TC_Absent				,HwTuner::TC_Absent			,HwTuner::TC_Absent			,HwTuner::TC_Absent  
};

bool BtCard::miro_fmtuner[] = 
{ 
	false	,false	,false	,false	,   
	false	,false	,false	,false	,  
	false	,false	,false	,false	,  
	false	,false	,false	,true	,
	true	,true	,true	,true	,   
	true	,true	,true	,false	,  
	false	,false	,false	,false	,  
	false	,false	,false	,false 
};

// Hauppauge Specific
BtCard::hauppauge_tunerentry BtCard::hauppauge_tuner[] = 
{
	{ HwTuner::TC_Absent						/*,""						*/},
	{ HwTuner::TC_Absent						/*,"External"				*/},
	{ HwTuner::TC_Absent						/*,"Unspecified"			*/},
	{ HwTuner::TC_PHILIPS_PAL					/*,"Philips FI1216"			*/},
	/*4*/	
	{ HwTuner::TC_PHILIPS_SECAM					/*,"Philips FI1216MF"		*/},
	{ HwTuner::TC_PHILIPS_NTSC					/*,"Philips FI1236"			*/},
	{ HwTuner::TC_PHILIPS_PAL_I					/*,"Philips FI1246"			*/},
	{ HwTuner::TC_PHILIPS_PAL_DK				/*,"Philips FI1256"			*/},
	/*8*/
	{ HwTuner::TC_PHILIPS_PAL					/*,"Philips FI1216 MK2"		*/},
	{ HwTuner::TC_PHILIPS_SECAM					/*,"Philips FI1216MF MK2"	*/},
	{ HwTuner::TC_PHILIPS_NTSC					/*,"Philips FI1236 MK2"		*/},
	{ HwTuner::TC_PHILIPS_PAL_I					/*,"Philips FI1246 MK2"		*/},
	/*12*/
	{ HwTuner::TC_PHILIPS_PAL_DK				/*,"Philips FI1256 MK2"		*/},
	{ HwTuner::TC_TEMIC_NTSC					/*,"Temic 4032FY5"			*/},
	{ HwTuner::TC_TEMIC_PAL						/*,"Temic 4002FH5"			*/},
	{ HwTuner::TC_TEMIC_PAL_I					/*,"Temic 4062FY5"			*/},
	/*16*/
	{ HwTuner::TC_PHILIPS_PAL					/*,"Philips FR1216 MK2"		*/},
	{ HwTuner::TC_PHILIPS_SECAM					/*,"Philips FR1216MF MK2"	*/},
	{ HwTuner::TC_PHILIPS_NTSC					/*,"Philips FR1236 MK2"		*/},
	{ HwTuner::TC_PHILIPS_PAL_I					/*,"Philips FR1246 MK2"		*/},
	/*20*/
	{ HwTuner::TC_PHILIPS_PAL_DK				/*,"Philips FR1256 MK2"		*/},
	{ HwTuner::TC_PHILIPS_PAL					/*,"Philips FM1216"			*/},
	{ HwTuner::TC_PHILIPS_SECAM					/*,"Philips FM1216MF"		*/},
	{ HwTuner::TC_PHILIPS_NTSC					/*,"Philips FM1236"			*/},
	/*24*/
	{ HwTuner::TC_PHILIPS_PAL_I					/*,"Philips FM1246"			*/},
	{ HwTuner::TC_PHILIPS_PAL_DK				/*,"Philips FM1256"			*/},
	{ HwTuner::TC_TEMIC_4036FY5_NTSC			/*,"Temic 4036FY5"			*/},
	{ HwTuner::TC_Absent						/*,"Samsung TCPN9082D"		*/},
	/*28*/
	{ HwTuner::TC_Absent						/*,"Samsung TCPM9092P"		*/},
	{ HwTuner::TC_TEMIC_4006FH5_PAL				/*,"Temic 4006FH5"			*/},
	{ HwTuner::TC_Absent						/*,"Samsung TCPN9085D"		*/},
	{ HwTuner::TC_Absent						/*,"Samsung TCPB9085P"		*/},
	/*32*/
	{ HwTuner::TC_Absent						/*,"Samsung TCPL9091P"		*/},
	{ HwTuner::TC_TEMIC_4039FR5_NTSC			/*,"Temic 4039FR5"			*/},
	{ HwTuner::TC_PHILIPS_FQ1216ME				/*,"Philips FQ1216 ME"		*/},
	{ HwTuner::TC_TEMIC_4066FY5_PAL_I			/*,"Temic 4066FY5"			*/},
	/*36*/
	{ HwTuner::TC_Absent						/*,"Philips TD1536"			*/},
	{ HwTuner::TC_Absent						/*,"Philips TD1536D"		*/},
	{ HwTuner::TC_PHILIPS_NTSC					/*,"Philips FMR1236"		*/},
	{ HwTuner::TC_Absent						/*,"Philips FI1256MP"		*/},
	/*40*/
	{ HwTuner::TC_Absent						/*,"Samsung TCPQ9091P"		*/},
	{ HwTuner::TC_TEMIC_4006FN5_MULTI_PAL		/*,"Temic 4006FN5"			*/},
	{ HwTuner::TC_TEMIC_4009FR5_PAL				/*,"Temic 4009FR5"			*/},
	{ HwTuner::TC_TEMIC_4046FM5					/*,"Temic 4046FM5"			*/},
	/*44*/
	{ HwTuner::TC_TEMIC_4009FN5_MULTI_PAL_FM	/*,"Temic 4009FN5"			*/},
	{ HwTuner::TC_Absent						/*,"Philips TD1536D_FH_44"	*/},
	{ HwTuner::TC_LG_NTSC_FM					/*,"LG TP18NSR01F"			*/},
	{ HwTuner::TC_LG_PAL_FM						/*,"LG TP18PSB01D"			*/},
	{ HwTuner::TC_LG_PAL						/*,"LG TP18PSB11D"			*/},	
	{ HwTuner::TC_LG_PAL_I_FM					/*,"LG TAPC-I001D"			*/},
	{ HwTuner::TC_LG_PAL_I						/*,"LG TAPC-I701D"			*/}

};

/* ----------------------------------------------------------------------- */
/* AVermedia specific */

HwTuner::Tuner BtCard::avermedia_tuner_0_table[] = {
	HwTuner::TC_PHILIPS_NTSC,  HwTuner::TC_PHILIPS_PAL,
	HwTuner::TC_PHILIPS_PAL,   HwTuner::TC_PHILIPS_PAL,
	HwTuner::TC_PHILIPS_PAL,   HwTuner::TC_PHILIPS_PAL,
	HwTuner::TC_PHILIPS_SECAM, HwTuner::TC_PHILIPS_SECAM,
	HwTuner::TC_PHILIPS_SECAM, HwTuner::TC_PHILIPS_PAL};

#if 0
HwTuner::Tuner BtCard::avermedia_tuner_0_fm_table[] = {
	PHILIPS_FR1236_NTSC,  PHILIPS_FR1216_PAL,
	PHILIPS_FR1216_PAL,   PHILIPS_FR1216_PAL,
	PHILIPS_FR1216_PAL,   PHILIPS_FR1216_PAL,
	PHILIPS_FR1236_SECAM, PHILIPS_FR1236_SECAM,
	PHILIPS_FR1236_SECAM, PHILIPS_FR1216_PAL};
#endif

HwTuner::Tuner BtCard::avermedia_tuner_1_table[] = {
	HwTuner::TC_TEMIC_NTSC,  HwTuner::TC_TEMIC_PAL,
	HwTuner::TC_TEMIC_PAL,   HwTuner::TC_TEMIC_PAL,
	HwTuner::TC_TEMIC_PAL,   HwTuner::TC_TEMIC_PAL,
	HwTuner::TC_TEMIC_4012FY5, HwTuner::TC_TEMIC_4012FY5, //HwTuner::TC_TEMIC_SECAM
	HwTuner::TC_TEMIC_4012FY5, HwTuner::TC_TEMIC_PAL};

// I2C Addresses
#define I2C_STBEE       0xae
#define I2C_HAUPEE      0xa0

// PCX200
#define I2C_MAX517		0x5E
#define I2C_12C508		0x1E


// Audio functions: All these depend on the Hardware
bool BtCard::AudioSetInput( AudioConnector audioConnector )
{
	// If Specified Tuner OR Radio, use appropiate connector instead of specified
	if (audioConnector == AudioConTuner ||
		audioConnector == AudioConRadio) {
		audioConnector = TunerConnector;
	}

	// Only set the connector if unmuted and the connector has changed!
	if (!bAudioMuted && 
			(AudioMux != audioConnector) ) {

		// Get card information
		CInfo* Info = GetCardInfo(); 

		// Enable the corresponding GPIO pins to select audio sources...
		GetBtPisces().SetGPOE( GetBtPisces().GetGPOE() | Info->gpoe );

		// Set the correct bits to enable the Line Source
		ULONG Val;
		GetBtPisces().GetGPDATA(&Val,1,0);
		Val &= ~Info->gpoe;									// Keep all bits not involved into audio switching: all the others set to 0
		Val |=  Info->gpoe & Info->audiomux[audioConnector];	// And select the audio source without disrupting the other bits...
		GetBtPisces().SetGPDATA(&Val,1,0);
	
		// Now, For all the AudioChips, set the input!
		for (DWORD Idx = 0 ; Idx < AudioCount ; Idx ++) {
			AudioChip[Idx]->SetAudioConnector(audioConnector);
		}
	}

	// Remember last source !!
	AudioMux = audioConnector;

	return true;
}      

// Unmute card
void BtCard::AudioUnmute( void )
{
	// Only if we were muted ...
	if (bAudioMuted) {

		// Get card information
		CInfo* Info = GetCardInfo(); 

		// Enable the corresponding GPIO pins to select audio sources...
		GetBtPisces().SetGPOE( GetBtPisces().GetGPOE() | Info->gpoe );

		// Set the correct bits to enable the Line Source
		ULONG Val;
		GetBtPisces().GetGPDATA(&Val,1,0);
		Val &= ~Info->gpoe;							// Keep all bits not involved into audio switching: all the others set to 0
		Val |=  Info->gpoe & Info->audiomux[AudioMux];	// And select the audio source without disrupting the other bits...
		GetBtPisces().SetGPDATA(&Val,1,0);

		// Now, For all the AudioChips, set the input!
		for (DWORD Idx = 0 ; Idx < AudioCount ; Idx ++) {
			AudioChip[Idx]->SetAudioConnector(AudioMux);
		}

		// Unmute!
		bAudioMuted = FALSE; 
	}
}      

void BtCard::AudioMute( void )
{
	// Only if we were not unmuted
	if (!bAudioMuted) {

		// Get card information
		CInfo* Info = GetCardInfo(); 

		// Enable the corresponding GPIO pins to select audio sources...
		GetBtPisces().SetGPOE( GetBtPisces().GetGPOE() | Info->gpoe );

		// Set the correct bits to enable the Line Source
		ULONG Val;
		GetBtPisces().GetGPDATA(&Val,1,0);
		Val &= ~Info->gpoe;								// Keep all bits not involved into audio switching: all the others set to 0
		Val |=  Info->gpoe & Info->audiomux[AudioConMute];	// And select the audio source without disrupting the other bits...
		GetBtPisces().SetGPDATA(&Val,1,0);

		// Now, for all the AudioChips, Mute audio!
		for (DWORD Idx = 0 ; Idx < AudioCount ; Idx ++) {
			AudioChip[Idx]->SetAudioConnector(AudioConMute);
		}

		bAudioMuted = TRUE;
	}
}      

// Gets the Capabilities ...
ULONG BtCard::AudioGetCaps()
{
	// We start with mono Sound AND Lang A
	ULONG Caps = KS_TVAUDIO_MODE_MONO | KS_TVAUDIO_MODE_LANG_A;
	USHORT Idx;

	// Now, For all the AudioChips, Get Caps!
	for (Idx = 0 ; Idx < AudioCount ; Idx ++) {
		// Get Available modes from Audiochip
		AudioModes AvModes = AudioChip[Idx]->AudioModesAvailable();
		// If mono available, mark it!
		if (AvModes & Am_Mono)	 Caps |= KS_TVAUDIO_MODE_MONO   | KS_TVAUDIO_MODE_LANG_A;
		// If Stereo available, mark it!
		if (AvModes & Am_Stereo) Caps |= KS_TVAUDIO_MODE_STEREO | KS_TVAUDIO_MODE_LANG_A; // Yes, Stereo in the first language
		// If Lang A available, mark it!
		if (AvModes & Am_Lang1)	 Caps |= KS_TVAUDIO_MODE_LANG_B | KS_TVAUDIO_MODE_MONO;
		// If Lang B available, mark it!
		if (AvModes & Am_Lang2)	 Caps |= KS_TVAUDIO_MODE_LANG_C | KS_TVAUDIO_MODE_MONO;
	}

	// Now, for all tuners, Get Caps!
	for (Idx = 0 ; Idx < TunerCount ; Idx ++) {
		if (TunerChip[Idx]->AMStereoAvailable() ||
			TunerChip[Idx]->FMStereoAvailable()	||
			TunerChip[Idx]->TVStereoAvailable() ) Caps |= KS_TVAUDIO_MODE_STEREO | KS_TVAUDIO_MODE_LANG_A;
	}

	// Return capacities
	return Caps;
}

// Get all the Supported Modes Right now for the current mode
ULONG BtCard::AudioGetSupportedModes()
{
	// We start with mono Sound
	ULONG Modes = KS_TVAUDIO_MODE_MONO | KS_TVAUDIO_MODE_LANG_A;

	// Now, For all the AudioChips, Get Caps. This is valid for TV (and for FM and AM radio also)
	// Normally, audio Chips handles Stereo of TV, SAP of TV, AND Stereo of FM
	DWORD Idx;
	for (Idx = 0 ; Idx < AudioCount ; Idx ++) {
		// Get Detected modes..
		AudioModes DetModes = AudioChip[Idx]->AudioModesDetected();
		// If mono available, mark it!
		if (DetModes & Am_Mono)		Modes |= KS_TVAUDIO_MODE_MONO   | KS_TVAUDIO_MODE_LANG_A;
		// If Stereo available, mark it!
		if (DetModes & Am_Stereo)	Modes |= KS_TVAUDIO_MODE_STEREO | KS_TVAUDIO_MODE_LANG_A;
		// If Lang A available, mark it!
		if (DetModes & Am_Lang1)	Modes |= KS_TVAUDIO_MODE_LANG_B | KS_TVAUDIO_MODE_MONO;
		// If Lang B available, mark it!
		if (DetModes & Am_Lang2)	Modes |= KS_TVAUDIO_MODE_LANG_C | KS_TVAUDIO_MODE_MONO;
	}

	// Now, for all tuners, Get Caps. 
	// Here we must select the correct tuner only, because there can be more than 1 tuner in the system!
	for (Idx = 0 ; Idx < TunerCount ; Idx ++) {
		if ((m_CurrentTunerMode == HwTuner::Tm_TV && TunerChip[Idx]->CanTuneTV())) {
			if (TunerChip[Idx]->TVStereoDetected()) Modes |= KS_TVAUDIO_MODE_STEREO | KS_TVAUDIO_MODE_LANG_A;
		}
		if ((m_CurrentTunerMode == HwTuner::Tm_Radio_AM && TunerChip[Idx]->CanTuneAM())) {
			if (TunerChip[Idx]->AMStereoDetected()) Modes |= KS_TVAUDIO_MODE_STEREO | KS_TVAUDIO_MODE_LANG_A;
		}
		if ((m_CurrentTunerMode == HwTuner::Tm_Radio_FM && TunerChip[Idx]->CanTuneFM())) {
			if (TunerChip[Idx]->FMStereoDetected()) Modes |= KS_TVAUDIO_MODE_STEREO | KS_TVAUDIO_MODE_LANG_A;
		}
	}

	// Return capacities
	return Modes;
}

NTSTATUS BtCard::AudioSetMode(ULONG New)
{
	// Note! -- Argh!
	// Microsoft fucked this up! - 
	//		I was assuming that you should be able to specify either Mono or Stereo
	// for each language (A/B/C)... But default TV mode dialogs don't allow this ...
	// You can select one of: Mono,Stereo,LangA,LangB,LangC... That is wrong! - So i have to
	// cope with it. Also, LangA could be either Mono or Stereo... Mono if receiving a Dual
	// trnasmission, Stereo if receiving an Stereo transmission... So we will translate them!
	
	// Fill in all the missing pieces of info... If the calling app is not passing the right parameters... as Microsoft does.
	if (New == KS_TVAUDIO_MODE_LANG_A) {
		// I Assume a Mono transmission of a Dual Channel...
		New |= (KS_TVAUDIO_MODE_LANG_A | KS_TVAUDIO_MODE_MONO);
	} else 
	if (New == KS_TVAUDIO_MODE_LANG_B) {
		// I Assume a Mono transmission of a Dual Channel...
		New |= (KS_TVAUDIO_MODE_LANG_B | KS_TVAUDIO_MODE_MONO);
	} else 
	if (New == KS_TVAUDIO_MODE_LANG_C) {
		// I Assume a Mono transmission of a Dual Channel...
		New |= (KS_TVAUDIO_MODE_LANG_C | KS_TVAUDIO_MODE_MONO);
	} else
	if (New == KS_TVAUDIO_MODE_MONO) {
		// I Assume a Mono transmission of the default channel...
		New |= (KS_TVAUDIO_MODE_LANG_A | KS_TVAUDIO_MODE_MONO);
	} else
	if (New == KS_TVAUDIO_MODE_STEREO) {
		// I Assume an Stereo transmission of the default channel...
		New |= (KS_TVAUDIO_MODE_LANG_A | KS_TVAUDIO_MODE_STEREO);
	}

	// Get actually available modes...
	ULONG ValidModes = AudioGetSupportedModes();

	// Ensure at least one of STEREO / MONO is selected! (could be both)
	if ((New & (KS_TVAUDIO_MODE_MONO | KS_TVAUDIO_MODE_STEREO)) == 0) {

		// Try to get Stereo if supported if it is NOT specified
		if (ValidModes & KS_TVAUDIO_MODE_STEREO) {
			New |= KS_TVAUDIO_MODE_STEREO;
		} else {
			New |= KS_TVAUDIO_MODE_MONO;
		}
	}

	// Ensure at least LANG_A is selected!
	if ( (New & (KS_TVAUDIO_MODE_LANG_A | KS_TVAUDIO_MODE_LANG_B | KS_TVAUDIO_MODE_LANG_C) ) == 0) {
		New |= KS_TVAUDIO_MODE_LANG_A;
	}

	// If something that is NOT supported right now is asked, fail!
	if (
		( (New & (~ValidModes)) &
			(KS_TVAUDIO_MODE_MONO | KS_TVAUDIO_MODE_STEREO | 
			 KS_TVAUDIO_MODE_LANG_A | KS_TVAUDIO_MODE_LANG_B | KS_TVAUDIO_MODE_LANG_C )
		) != 0

		) {
		return STATUS_INVALID_PARAMETER;
	}

	// We usually can receive one of:
	//	-Mono
	//	-Stereo
	//	-Dual A/B
	//	-NICAM... 

	// Note that if we are receiving Dual channels, we won't receive in Stereo...

	// Only if mode has Changed ...
	if (New != m_AudioMode) {

		// Store New AudioMode
		m_AudioMode = New;

		// Translate WDM to our representation
		AudioMode am = Am_Mono;

		if (New & KS_TVAUDIO_MODE_LANG_C) {
			am = Am_Lang2;
		} else 
		if (New & KS_TVAUDIO_MODE_LANG_B) {
			am = Am_Lang1;
		} else 
		if (New & KS_TVAUDIO_MODE_STEREO) {
			am = Am_Stereo;
		}

		// Now, for all tuners, Set Mode!
		DWORD Idx;
		for (Idx = 0 ; Idx < TunerCount ; Idx ++) {
			TunerChip[Idx]->SetAudioMode(am);
		}

		// Now for all audioChips, Set mode and language
		for (Idx = 0 ; Idx < AudioCount ; Idx ++) {
			AudioChip[Idx]->SetAudioMode(am);
		}
	}

	return STATUS_SUCCESS;
}

bool BtCard::HasTunerInput( void ) const
{
	if (!VideoCount) 
		return GetCardInfo()->tuner0 != NONE ? true : false;

	bool c = false;
	for (DWORD Idx = 0 ; Idx < VideoCount ; Idx ++) {
		c |= VideoChip[Idx]->HasTunerInput();
	}
	return c;
}

int	BtCard::SVideoInputCount( void ) const
{
	if (!VideoCount) 
		return GetCardInfo()->svhs0 != NONE ? 1 : 0;

	int c = 0;
	for (DWORD Idx = 0 ; Idx < VideoCount ; Idx ++) {
		c += VideoChip[Idx]->SVideoCount();
	}
	return c;
}

int	BtCard::CompositeInputCount( void ) const
{
	if (!VideoCount) 
		return GetCardInfo()->analogvideoins - SVideoInputCount() - (HasTunerInput() ? 1 : 0);

	int c = 0;
	for (DWORD Idx = 0 ; Idx < VideoCount ; Idx ++) {
		c += VideoChip[Idx]->CompositeCount();
	}
	return c;
}

int	BtCard::SerialDigitalInputCount( void ) const
{
	if (!VideoCount) 
		return GetCardInfo()->digitalvideoins;

	int c = 0;
	for (DWORD Idx = 0 ; Idx < VideoCount ; Idx ++) {
		c += VideoChip[Idx]->SerialDigitalCount();
	}
	return c;
}

int	BtCard::ParallelDigitalInputCount( void ) const
{
	if (!VideoCount) 
		return 0;

	int c = 0;
	for (DWORD Idx = 0 ; Idx < VideoCount ; Idx ++) {
		c += VideoChip[Idx]->ParallelDigitalCount();
	}
	return c;
}

int	BtCard::RGBInputCount( void ) const
{
	if (!VideoCount) 
		return 0;

	int c = 0;
	for (DWORD Idx = 0 ; Idx < VideoCount ; Idx ++) {
		c += VideoChip[Idx]->RGBCount();
	}
	return c;
}

int	BtCard::ComponentVideoInputCount( void ) const
{
	if (!VideoCount) 
		return 0;

	int c = 0;
	for (DWORD Idx = 0 ; Idx < VideoCount ; Idx ++) {
		c += VideoChip[Idx]->ComponentVideoCount();
	}
	return c;
}

// Video Selection
void BtCard::VideoSetInput(VideoConnector conn)
{

	// Try to find a handler for this Video Connector selection
	condesc_t x = -1;
	BOOL done = false;
	for (DWORD Idx = 0 ; Idx < VideoCount ; Idx ++) {

		// Try to set it...
		x = VideoChip[Idx]->SetConnector(conn);

		// If we did it, break loop.
		if (x != -1) {

			// Set kind of required video...
			VideoSetVideoKind(CONDESC_DIGITAL(x) ? true : false);

			// And select the right mux to handle it.
			GetBtPisces().SetConnector(CONDESC_MUX(x) , CONDESC_SVIDEO(x) ? true : false );

			// Remember the last selected connector (required by following call)
			VideoMux = conn;

			// We are done!
			done = true;
		}
	}

	// If done, done!
	if (done) return;

	// Noone handled it, use the default way
	int idx = -1; // assume not present

	// Get Information about the card
	CInfo* Info = GetCardInfo();

	switch (conn) {
		case VideoConSVideo0:

			// set to composite or Y/C component video depends on video source
			idx = Info->svhs0;

			// If not present, don't set it
			if (idx < 0 || idx >= MAX_VIDEOINS) return;

			// This is analog video!
			VideoSetVideoKind(false);
			break;

		case VideoConTuner:

			// set to composite or Y/C component video depends on video source
			idx = Info->tuner0;

			// If not present, don't set it
			if (idx < 0 || idx >= MAX_VIDEOINS) return;

			// This is analog video!
			VideoSetVideoKind(false);
			break;

		case VideoConComposite0:
		case VideoConComposite1:
		case VideoConComposite2:
		case VideoConComposite3:
		case VideoConComposite4:
		case VideoConComposite5:
		case VideoConComposite6:
		case VideoConComposite7:
		case VideoConComposite8:
		case VideoConComposite9:
		case VideoConComposite10:
		case VideoConComposite11:
		case VideoConComposite12:
		case VideoConComposite13:
		case VideoConComposite14:
		case VideoConComposite15:
		case VideoConComposite16:
		case VideoConComposite17:
		case VideoConComposite18:
		case VideoConComposite19:
		case VideoConComposite20:
		case VideoConComposite21:
		case VideoConComposite22:
		case VideoConComposite23:
		case VideoConComposite24:
		case VideoConComposite25:
		case VideoConComposite26:
		case VideoConComposite27:
		case VideoConComposite28:
		case VideoConComposite29:
		case VideoConComposite30:
		case VideoConComposite31:
			{
				int cn = 3; // Else, default to the 3rd entry... Probably will be overriden by the videoChip
				switch (conn) {
					case VideoConComposite0: cn = 0; break;
					case VideoConComposite1: cn = 1; break;
					case VideoConComposite2: cn = 2; break;
					case VideoConComposite3: cn = 3; break;
				};

				// Get idx of composite #cn
				idx = 0;
				while (1) { // Mods by Kevin Liao (11/20/2001)
					// skip all the tuner and svideo i/dexes
					while (idx < MAX_VIDEOINS && (idx == Info->tuner0 || idx == Info->svhs0))
						idx++;
					if (cn--)
						idx++;
					else
						break;
				};

				// If not present, use the last valid...
				if (idx >= MAX_VIDEOINS) {
					idx = MAX_VIDEOINS-1; 
				}

				// Force switching to analog video
				VideoSetVideoKind(false);
			}
			break;

		case VideoConSDI0: // This is an special connector: We are forced to use the digital in
		case VideoConSDI1:
		case VideoConSDI2:
		case VideoConSDI3:
		case VideoConPDI0: // This is an special connector: We are forced to use the digital in
		case VideoConPDI1:
		case VideoConPDI2:
		case VideoConPDI3:

			{
				// Digital video does NOT need to set the video mux - dont do it!
				VideoMux = conn;

				// This is truly digital video!
				VideoSetVideoKind(true);

				return;
			}
			
		default:
			return;
	}

	// Get real value
	int val = Info->videomux[idx] & 3;

	// Set Video Connector
	GetBtPisces().SetConnector(val , conn == VideoConSVideo0 || conn == VideoConSVideo1 || conn == VideoConSVideo2 || conn == VideoConSVideo3 );

	// Remember the last selected connector
	VideoMux = conn;
}

/* Method: BtCard::GetSupportedStandards
 * Purpose: Returns all supported standards by Decoder
 * Input: None
 * Output: LONG: standard flags
 */
LONG BtCard::VideoGetSupportedStandards()
{
	return	( (HasPLL() || Has28Xtal()) ? ( KS_AnalogVideo_PAL_M | KS_AnalogVideo_NTSC_M | KS_AnalogVideo_NTSC_M_J ) : 0 ) |
			( (HasPAL_NC()) ? KS_AnalogVideo_PAL_NC : 0 ) |
			( (HasPLL() || Has35Xtal()) ? 
				(	KS_AnalogVideo_PAL_N | KS_AnalogVideo_PAL_B | KS_AnalogVideo_PAL_D |
					KS_AnalogVideo_PAL_G | KS_AnalogVideo_PAL_H	| KS_AnalogVideo_PAL_I |
					KS_AnalogVideo_SECAM_B | KS_AnalogVideo_SECAM_D	| KS_AnalogVideo_SECAM_G |
					KS_AnalogVideo_SECAM_H | KS_AnalogVideo_SECAM_K	| KS_AnalogVideo_SECAM_K1 |
					KS_AnalogVideo_SECAM_L ) : 0 ) |
			( (IsBt878Family()) ? (KS_AnalogVideo_NTSC_50 | KS_AnalogVideo_PAL_60) : 0 )
			;
}

NTSTATUS BtCard::VideoSetVideoKind( bool IsDigital )
{
	// Force switching!
	if (m_IsDigitalVideo != IsDigital) {
		m_IsDigitalVideo = IsDigital;

		// Force Switch to asked
		KS_AnalogVideoStandard old = m_ksCurrentVideoStandard;
		m_ksCurrentVideoStandard = KS_AnalogVideoStandard(0);
		return VideoSetCurrentStandard(old);
	}
	return STATUS_SUCCESS;
}

NTSTATUS BtCard::VideoSetCurrentStandard( KS_AnalogVideoStandard ksStandard )
{
	DUMP(( "Decoder::SetCurrentVideoDecoderStandard( 0x%08X )\n", (DWORD)ksStandard ));

	// Only if really changing Video Standard ...
	// (note that to switch to digital video m_ksCurrentVideoStandard
	//  is forced to 0 before calling this routine. Also to switch to analog
	//  video)
	if (ksStandard == m_ksCurrentVideoStandard) 
		return STATUS_SUCCESS;

	ULONG f_req = 0;

	VideoFormat m_vf;

	switch( ksStandard ) {

		case KS_AnalogVideo_NTSC_M:  
			m_vf = VFormat_NTSC;
			break;
		case KS_AnalogVideo_NTSC_M_J:
			m_vf = VFormat_NTSC_J;
			break;
		case KS_AnalogVideo_PAL_B:   
		case KS_AnalogVideo_PAL_D:   
		case KS_AnalogVideo_PAL_G:   
		case KS_AnalogVideo_PAL_H:   
		case KS_AnalogVideo_PAL_I:   
			m_vf = VFormat_PAL_BDGHI;
			break;
		case KS_AnalogVideo_PAL_M:   
			m_vf = VFormat_PAL_M;
			break;
		case KS_AnalogVideo_PAL_N:  
			m_vf = VFormat_PAL_N;
			break;
		case KS_AnalogVideo_PAL_NC:
			if (!HasPAL_NC()) 
				return false;
			m_vf = VFormat_PAL_N_COMB;
			break;
		case KS_AnalogVideo_SECAM_B: 
		case KS_AnalogVideo_SECAM_D: 
		case KS_AnalogVideo_SECAM_G: 
		case KS_AnalogVideo_SECAM_H: 
		case KS_AnalogVideo_SECAM_K: 
		case KS_AnalogVideo_SECAM_K1:
		case KS_AnalogVideo_SECAM_L: 
		case KS_AnalogVideo_SECAM_L1:
			m_vf = VFormat_SECAM;
			break;
		case KS_AnalogVideo_PAL_60:
			if (!IsBt878Family()) 
				return false;
			m_vf = VFormat_PAL_60;
			break;
		case KS_AnalogVideo_NTSC_50: // = NTSC-50
			if (!IsBt878Family()) 
				return false;
			m_vf = VFormat_NTSC_50;
			break;
		default:
			return STATUS_INVALID_PARAMETER;
	}

	// Translate to digital formats if we need to
	VideoFormat vf = m_vf;

	// If we are going to get digital video, Make sure to select a digital mode.
	if (m_IsDigitalVideo) {
		switch (GetCardInfo()->digxface) {
			default:
			case CCIR656_Digital:
				if (vf == VFormat_NTSC_J || vf == VFormat_NTSC) {
					// Use NTSC as the video norm for digital video
					vf = VFormat_NTSC_Digital;
				} else {
					// All the other formats are assumed to be PAL
					vf = VFormat_PAL_Digital;
				}
				break;
			case RQS_Digital:
				vf = VFormat_RQS_Digital;
				break;
			case SVI_Digital:
				vf = VFormat_SVI_Digital;
				break;
			case LVM_Digital:
				vf = VFormat_LVM_Digital;
				break;
		}
	}

	// Now set the videoMode
	if( GetBtPisces().SetVideoFormat( m_xtal0freq, m_xtal1freq, vf ) == Fail )
	{
		ERROR(( " Failed SetVideoFormat( %d )\n", (DWORD)vf ));
		return STATUS_INVALID_PARAMETER;
	}

	// Inform all videoChips of change!
	for (DWORD Idx = 0 ; Idx < VideoCount ; Idx ++) {
		VideoChip[Idx]->SetFormat(m_vf); // (use the untranslated std)
	}

	// Remember Standard
	m_ksCurrentVideoStandard = ksStandard;

	return STATUS_SUCCESS;
}

// Tuner Functions
LONG BtCard::TunerGetSupportedModes()
{
	// Verify all tuners , checking for available Modes
	ULONG Modes = 0;	// By default, nothing!

	// Go tuner by tuner checking its type
	for (DWORD Idx = 0 ; Idx < TunerCount ; Idx ++) {
		if (TunerChip[Idx]->CanTuneTV()) {
			Modes |= KSPROPERTY_TUNER_MODE_TV;
		}
		if (TunerChip[Idx]->CanTuneAM()) {
			Modes |= KSPROPERTY_TUNER_MODE_AM_RADIO;
		}
		if (TunerChip[Idx]->CanTuneFM()) {
			Modes |= KSPROPERTY_TUNER_MODE_FM_RADIO;
		}
	}

	// Return the available modes
	return Modes;
}

LONG BtCard::TunerGetSupportedStandards()
{
	// Verify all tuners , checking for available Stds
	ULONG Stds = 0;	// By default, no standards

	// Consider only the TV Tuners (obviously!)
	for (DWORD Idx = 0 ; Idx < TunerCount ; Idx ++) {
		if (TunerChip[Idx]->CanTuneTV()) {
			Stds |= TunerChip[Idx]->TVTunerVideoStandardsSupported();
		}
	}

	// Return the detected standards
	return Stds;
}

NTSTATUS BtCard::TunerSetCurrentStandard( KS_AnalogVideoStandard ksStandard )
{
	TRACE(("BtCard::TunerSetCurrentStandard"));

	// Assume failure
	NTSTATUS st = STATUS_INVALID_PARAMETER;
	USHORT Idx;

	// Only if really changing VideoStandard ...
	if (ksStandard != m_ksCurrentTunerStandard) {

		// Look for a tuner that supports this Standard (There's a potential IF change, so this is required...)
		for (Idx = 0 ; Idx < TunerCount ; Idx ++) {

			// Only try this if Tuner supports TV mode...
			if (TunerChip[Idx]->CanTuneTV()) {

				// If tuner supports the requested standard...
				if (TunerChip[Idx]->TVTunerVideoStandardsSupported() & ksStandard) {

					// Set the tuner Standard!
					if (m_CurrentTunerMode == HwTuner::Tm_TV) {

						ULONG RealFreq = TunerChip[Idx]->SetFrequency(ksStandard,m_CurrentTunerMode,m_CurrentTunerFrequency ,HwTuner::Tm_Fine);
						if (RealFreq) {
							m_CurrentTunerFrequency = RealFreq;
						}
						m_TunerSignalDetected = (RealFreq != 0);

					} else {
						// We are not on TV mode...
						DUMP(("m_CurrentTumerMode != HWTuner::Tm_TV, but %d\n",m_CurrentTunerMode));
					}

					// Remember the current standard...
					m_ksCurrentTunerStandard = ksStandard;

					// And that we changed things successfully.
					st = STATUS_SUCCESS;

				} else {

					DUMP(("TunerChip[%d] does NOT support standard %08X, supported are %08X\n",Idx,ksStandard,TunerChip[Idx]->TVTunerVideoStandardsSupported()));
				}
			} else {
				DUMP(("TunerChip[%d] can NOT tune TV!\n", Idx));
			}
		}

		// If we were able to change...
		if ( st == STATUS_SUCCESS ) {

			// Inform of the video norm change...
			for (Idx = 0 ; Idx < AudioCount ; Idx ++) {
				AudioChip[Idx]->VideoStandardChanged(ksStandard);
			}
			
			// Now, Inform the audioChips that Channel has Changed ...
			for (Idx = 0 ; Idx < AudioCount ; Idx ++) {
				AudioChip[Idx]->ChannelChanged();
			}
		}

	} else {
		st = STATUS_SUCCESS;
	}

	DUMP(("Returning status %d, current standard %d, requested standard %d\n", st, m_ksCurrentTunerStandard, ksStandard));
	return st;
}

NTSTATUS BtCard::TunerSetMode( ULONG Mode )
{
	HwTuner::TunerMode tm = HwTuner::Tm_TV;

	// Translate from WDM to our internal form
	switch (Mode) {
		case 0: // This is the DEFAULT mode based on DirectShow Docs (feature not documented on ntddk)
		case KSPROPERTY_TUNER_MODE_TV:
			tm = HwTuner::Tm_TV;
			break;
		case KSPROPERTY_TUNER_MODE_AM_RADIO:
			tm = HwTuner::Tm_Radio_AM;
			break;
		case KSPROPERTY_TUNER_MODE_FM_RADIO:
			tm = HwTuner::Tm_Radio_FM;
			break;
		default:
			return STATUS_INVALID_PARAMETER;
	}

	// Only do if really changing mode !
	if (m_CurrentTunerMode != tm) {

		// Remember new Tuner Mode
		m_CurrentTunerMode = tm;

		// Make things easier : Despite WDM does not differenciate between Radio and TV tuner, we do, so remember mode 
		if (tm == HwTuner::Tm_TV || m_bTVAudioUsedForFM) {
			// TV Tuner mode
			TunerConnector = AudioConTuner;
		} else {
			// Radio Tuner mode
			TunerConnector = AudioConRadio;
		}

		// Switch to appropiate connector if Tuner is selected
		if ((AudioMux == AudioConTuner ||
			AudioMux == AudioConRadio) && 
			!bAudioMuted ) {

			// Set the correct connector for Tuner!
			AudioMux = TunerConnector;

			// Get card information
			CInfo* Info = GetCardInfo(); 

			// Enable the corresponding GPIO pins to select audio sources...
			GetBtPisces().SetGPOE( GetBtPisces().GetGPOE() | Info->gpoe );

			// Set the correct bits to enable the Line Source
			ULONG Val;
			GetBtPisces().GetGPDATA(&Val,1,0);
			Val &= ~Info->gpoe;							// Keep all bits not involved into audio switching: all the others set to 0
			Val |=  Info->gpoe & Info->audiomux[AudioMux];	// And select the audio source without disrupting the other bits...
			GetBtPisces().SetGPDATA(&Val,1,0);

			// Now, For all the AudioChips, set the input!
			for (DWORD Idx = 0 ; Idx < AudioCount ; Idx ++) {
				AudioChip[Idx]->SetAudioConnector(AudioMux);
			}

		}

		// Now, Set the mode!
		// Based on mode requested, select the adequate handler )a possible IF change makes this a must...)
		switch (m_CurrentTunerMode) {
			case HwTuner::Tm_TV: 
				{
					// Find the right chip
					for (DWORD Idx = 0 ; Idx < TunerCount ; Idx ++) {
						if (TunerChip[Idx]->CanTuneTV()) {
							// This one should work ...
							ULONG RealFreq = TunerChip[Idx]->SetFrequency(m_ksCurrentTunerStandard,m_CurrentTunerMode,m_CurrentTunerFrequency,HwTuner::Tm_Fine);
							if (RealFreq) {
								m_CurrentTunerFrequency = RealFreq;
							}
							m_TunerSignalDetected = (RealFreq != 0);
						}
					}
				}
				break;
			case HwTuner::Tm_Radio_AM: 
				{
					// Find the right chip
					for (DWORD Idx = 0 ; Idx < TunerCount ; Idx ++) {
						if (TunerChip[Idx]->CanTuneAM()) {
							// This one should work ...
							ULONG RealFreq = TunerChip[Idx]->SetFrequency(m_ksCurrentTunerStandard,m_CurrentTunerMode,m_CurrentTunerFrequency,HwTuner::Tm_Fine);
							if (RealFreq) {
								m_CurrentTunerFrequency = RealFreq;
							}
							m_TunerSignalDetected = (RealFreq != 0);
						}
					}
				}
				break;
			case HwTuner::Tm_Radio_FM: 
				{
					// Find the right chip
					for (DWORD Idx = 0 ; Idx < TunerCount ; Idx ++) {
						if (TunerChip[Idx]->CanTuneFM()) {
							// This one should work ...
							ULONG RealFreq = TunerChip[Idx]->SetFrequency(m_ksCurrentTunerStandard,m_CurrentTunerMode,m_CurrentTunerFrequency,HwTuner::Tm_Fine);
							if (RealFreq) {
								m_CurrentTunerFrequency = RealFreq;
							}
							m_TunerSignalDetected = (RealFreq != 0);
						}
					}
				}
				break;
		}
		// Now, Inform the audioChips that Channel has Changed ...
		for (DWORD Idx = 0 ; Idx < AudioCount ; Idx ++) {
			AudioChip[Idx]->ChannelChanged();
		}
		return STATUS_SUCCESS;
	} 
	return STATUS_SUCCESS;
}

ULONG BtCard::TunerGetMode()
{
	// Translate from our internal form to WDM
	switch (m_CurrentTunerMode) {
		default:
		case HwTuner::Tm_TV:
			return KSPROPERTY_TUNER_MODE_TV;
		case HwTuner::Tm_Radio_AM:
			return KSPROPERTY_TUNER_MODE_AM_RADIO;
		case HwTuner::Tm_Radio_FM:
			return KSPROPERTY_TUNER_MODE_FM_RADIO;
	}
	return KSPROPERTY_TUNER_MODE_TV;
}

// Get capabilities for a given mode
NTSTATUS BtCard::TunerGetModeCaps( PKSPROPERTY_TUNER_MODE_CAPS_S pCaps)
{
	switch (pCaps->Mode) {
		case KSPROPERTY_TUNER_MODE_TV:
			{
				// Look for the first tuner that handles this ... And Get Caps!
				for (DWORD Idx = 0 ; Idx < TunerCount ; Idx ++) {
					if (TunerChip[Idx]->CanTuneTV()) {
						// This one should work ...
						TunerChip[Idx]->TVTunerCaps(
							pCaps->MinFrequency,pCaps->MaxFrequency,
							pCaps->TuningGranularity,pCaps->SettlingTime);
						pCaps->StandardsSupported = TunerChip[Idx]->TVTunerVideoStandardsSupported();
						return STATUS_SUCCESS;
					}
				}
				break;
			}
		case KSPROPERTY_TUNER_MODE_AM_RADIO:
			{
				// Look for the first tuner that handles this ... And Get Caps!
				for (DWORD Idx = 0 ; Idx < TunerCount ; Idx ++) {
					if (TunerChip[Idx]->CanTuneAM()) {
						// This one should work ...
						TunerChip[Idx]->AMTunerCaps(
							pCaps->MinFrequency,pCaps->MaxFrequency,
							pCaps->TuningGranularity,pCaps->SettlingTime);
						pCaps->StandardsSupported = 0;
						return STATUS_SUCCESS;
					}
				}
				break;
			}
		case KSPROPERTY_TUNER_MODE_FM_RADIO:
			{
				// Look for the first tuner that handles this ... And Get Caps!
				for (DWORD Idx = 0 ; Idx < TunerCount ; Idx ++) {
					if (TunerChip[Idx]->CanTuneFM()) {
						// This one should work ...
						TunerChip[Idx]->FMTunerCaps(
							pCaps->MinFrequency,pCaps->MaxFrequency,
							pCaps->TuningGranularity,pCaps->SettlingTime);
						pCaps->StandardsSupported = 0;
						return STATUS_SUCCESS;
					}
				}
				break;
			}
	}
	return STATUS_INVALID_PARAMETER;
}



NTSTATUS BtCard::TunerSetFrequency(	DWORD Frequency ,KS_TUNER_TUNING_FLAGS tf)			// Frequency of channel
{
	TRACE(("BtCard::TunerSetFrequency"));

	// Translate the Tuning Method
	HwTuner::TuningMethod method = HwTuner::Tm_Fine;

	DUMP(("  Frequency requested %d, using tuning flags %d\n",
		Frequency, tf ));

	// This should be in this way, but the Default TV tuner dialog box does not 
	// pass KS_TUNER_TUNING_FINE, so we will have to add some workarounds...
	if (tf == KS_TUNER_TUNING_COARSE) {
		method = HwTuner::Tm_Coarse;
	} else if (tf == KS_TUNER_TUNING_EXACT) {
		method = HwTuner::Tm_Exact;
	}


	// Remember the current tuning method...
	m_TunerCurrMethod = tf;

	// Only set Frequency if frequency and/or tuning mode has really changed!
	if (Frequency != m_CurrentTunerFrequency || method != m_TunerLastMethod) {

		// Remember last tuning method
		m_TunerLastMethod = method;

		// Remember muted state and mute
		BOOL PrevMuted = bAudioMuted;
		AudioMute();

		// Based on mode requested, select the adequate handler
		switch (m_CurrentTunerMode) {
			case HwTuner::Tm_TV: 
				{
					// Find the right chip
					for (DWORD Idx = 0 ; Idx < TunerCount ; Idx ++) {
						if (TunerChip[Idx]->CanTuneTV()) {
							// This one should work ...
							ULONG RealFreq = TunerChip[Idx]->SetFrequency(m_ksCurrentTunerStandard,m_CurrentTunerMode,Frequency,method);

							// Delay to allow the video signal to stabilise
							mdelay( 35 );

							// If the requested method isn't fine tuning, but we locked on a signal
							// then tune again with fine tuning but only when the new tuning
							// has been selected
							if (method != HwTuner::Tm_Fine && IsHLocked() ) {
								DUMP(( "SetFrequency found signal lock, fine tuning\n" ));
								RealFreq = TunerChip[Idx]->SetFrequency(m_ksCurrentTunerStandard,m_CurrentTunerMode,Frequency,HwTuner::Tm_Fine );
							}

							if (RealFreq) {
								m_CurrentTunerFrequency = RealFreq;
							} else {
								m_CurrentTunerFrequency = Frequency;
							}
							m_TunerSignalDetected = (RealFreq != 0);
						}
					}
				}
				break;
			case HwTuner::Tm_Radio_AM: 
				{
					// Find the right chip
					for (DWORD Idx = 0 ; Idx < TunerCount ; Idx ++) {
						if (TunerChip[Idx]->CanTuneAM()) {
							// This one should work ...
							ULONG RealFreq = TunerChip[Idx]->SetFrequency(m_ksCurrentTunerStandard,m_CurrentTunerMode,Frequency,method);
							if (RealFreq) {
								m_CurrentTunerFrequency = RealFreq;
							} else {
								m_CurrentTunerFrequency = Frequency;
							}
							m_TunerSignalDetected = (RealFreq != 0);
						}
					}
				}
				break;
			case HwTuner::Tm_Radio_FM: 
				{
					// Find the right chip
					for (DWORD Idx = 0 ; Idx < TunerCount ; Idx ++) {
						if (TunerChip[Idx]->CanTuneFM()) {
							// This one should work ...
							ULONG RealFreq= TunerChip[Idx]->SetFrequency(m_ksCurrentTunerStandard,m_CurrentTunerMode,Frequency,method);
							if (RealFreq) {
								m_CurrentTunerFrequency = RealFreq;
							} else {
								m_CurrentTunerFrequency = Frequency;
							}
							m_TunerSignalDetected = (RealFreq != 0);
						}
					}
				}
				break;
		}

		// If previously unmuted, unmute now!
		if (!PrevMuted) {
			AudioUnmute();
		}

		// Now, Inform the audioChips that Channel has Changed ...
		for (DWORD Idx = 0 ; Idx < AudioCount ; Idx ++) {
			AudioChip[Idx]->ChannelChanged();
		}

	}
	return STATUS_SUCCESS;
}

// Power management
NTSTATUS BtCard::PowerDown()
{
	// Wait until all chips are idle
	DWORD Idx;
	for (Idx = 0 ; Idx < IRCount ; Idx ++) {
		IRChip[Idx]->Suspend();
	}

	for (Idx = 0 ; Idx < TunerCount ; Idx ++) {
		TunerChip[Idx]->Suspend();
	}

	for (Idx = 0 ; Idx < VideoCount ; Idx ++) {
		VideoChip[Idx]->Suspend();
	}

	for (Idx = 0; Idx < AudioCount ; Idx++) {
		AudioChip[Idx]->Suspend();
	}

	// If tuner is available, power down it ?

	// Power down the Bt chip
	GetBtPisces().PowerDown();

	// And return success!
	return STATUS_SUCCESS;
}

void BtCard::InitCard()
{
	// If a Hauppauge card is present, reset some chips
	if (dwCard == BtCard::HAUPPAUGE || 
		dwCard == BtCard::HAUPPAUGE878) {

		// reset/enable the MSP on some Hauppauge cards 
		// Thanks to Kysti Mlkki (kmalkki@cc.hut.fi)! 
	
		GetBtPisces().SetGPOE( 
			( (GetBtPisces().GetGPOE() & (~(1<<5)) ) | (1<<5) )	
					);
		ULONG Val;
		GetBtPisces().GetGPDATA(&Val,1,0);
		Val &= ~ (1<<5);
		GetBtPisces().SetGPDATA(&Val,1,0);
		mdelay(10);
		GetBtPisces().GetGPDATA(&Val,1,0);
		Val &= ~ (1<<5);
		Val |= (1<<5);
		GetBtPisces().SetGPDATA(&Val,1,0);
	}

	// If VOODOO TM, init MSP
	if (dwCard == BtCard::VOODOOTV_FM) {

		GetBtPisces().SetGPOE( 
			( (GetBtPisces().GetGPOE() & (~(1<<20)) ) | (1<<20) )	
				);

		ULONG Val;
		GetBtPisces().GetGPDATA(&Val,1,0);
		Val &= ~ (1<<20);
		GetBtPisces().SetGPDATA(&Val,1,0);
		mdelay(10);
		GetBtPisces().GetGPDATA(&Val,1,0);
		Val &= ~ (1<<20);
		Val |= (1<<20); // Enable MSP
		GetBtPisces().SetGPDATA(&Val,1,0);
	}

	// If PXC200, init card 
	if (dwCard == BtCard::PXC200) {
		static const BYTE vals[] = 
			{ 0x08, 0x09, 0x0a, 0x0b, 0x0d, 0x0d,
			0x01, 0x02, 0x03, 0x04, 0x05, 0x06,
			0x00 };

		int i;

		// Initialise GPIO-connected stuff 
		GetBtPisces().SetGPOE( 1<<13 );		// Reset pin only 
		ULONG Val = 0;
		GetBtPisces().SetGPDATA(&Val,1,0);
		udelay(3);
		Val = 1<<13;
		GetBtPisces().SetGPDATA(&Val,1,0);

		// GPIO inputs are pulled up, so no need to drive 
		// reset pin any longer 
		GetBtPisces().SetGPOE( 0 );

		// we could/should try and reset/control the AD pots? but
		// right now  we simply  turned off the crushing.  Without
		// this the AGC drifts drifts
		// remember the EN is reverse logic -->
		// setting BT848_ADC_AGC_EN disable the AGC
		// tboult@eecs.lehigh.edu
		WRITE_REGISTER_UCHAR( LPBYTE(BT848_REG_ADC + dwBaseAddress) , (BYTE)0x90 );
	
		//	Initialise MAX517 DAC */
		// Setting DAC reference voltage level ...
		GetBtPisces().I2CWrite(I2C_MAX517,0,0x80);
	
		// Initialise 12C508 PIC 
		// The I2CWrite and I2CRead commmands are actually to the 
		// same chips - but the R/W bit is included in the address
		// argument so the numbers are different 
		
		// Initialising 12C508 PIC chip ...
		GetBtPisces().I2CSetFreq(6000);		// Lower Speed to allow PIC to understand
		for (i = 0; i < sizeof(vals)/sizeof(int); i++) {
			GetBtPisces().I2CWrite(I2C_12C508,vals[i]);
			BYTE ret;
			GetBtPisces().I2CRead(I2C_12C508,&ret);
			DUMP(("I2C Write(0x08) = %i\nI2C Read () = %x\n", vals[i] ,ret));
		}
	}

	// MIRO needs to enable TV tuner access...
	if (dwCard == BtCard::MIRO ||
		dwCard == BtCard::MIROPRO ||
		dwCard == BtCard::PINNACLE ||
		dwCard == BtCard::PINNACLEPRO) {

		static BYTE init_0[] = { 0x00, 0x54, 0x70, 0x49 };
		static BYTE init_1[] = { 0x00, 0xD4, 0x70, 0x49 };

		// Newer PINNACLE cards use an MT2032/MT2050 tuner... We must enable it using the TDA9885/6/7... or tuning won't work!
		GetBtPisces().I2CWR(I2C_TDA9887_0 /* =0x86*/, init_1,4,NULL,0,true); // It does not matter if it acks...
		GetBtPisces().I2CWR(I2C_TDA9887_0 /* =0x86*/, init_0,4,NULL,0,true); // It does not matter if it acks...
		GetBtPisces().I2CWR(I2C_TDA9887_0 /* =0x86*/, init_1,4,NULL,0,true); // It does not matter if it acks...
		GetBtPisces().I2CWR(I2C_TDA9887_0 /* =0x86*/, init_1,4,NULL,0,true); // It does not matter if it acks...
	}
}

NTSTATUS BtCard::PowerUp()
{
	DWORD Idx;
	// Power up the chipset!
	GetBtPisces().PowerUp();

	// BtPisces was successfully powered up; Now we must restore all the associated settings of the I2C chips
		
	// Ensure the compatibility
	SetChipsetCompatibility(true);

	// Reset the i2c bus... (not needed , but...)
	I2CSetFreq(I2CGetFreq());

	// Reinit special chips in cards
	InitCard();

	// Reinit tuner chips
	for (Idx = 0 ; Idx < TunerCount ; Idx ++) {
		TunerChip[Idx]->Restart();
	}

	// Reinit the video chips
	for (Idx = 0 ; Idx < VideoCount ; Idx ++) {
		VideoChip[Idx]->Restart();
	}

	// Reinit the IR chips
	for (Idx = 0 ; Idx < IRCount ; Idx ++) {
		IRChip[Idx]->Restart();
	}

	// Reinit audio chips and set volume of the audioChips to the one that was before!
	for (Idx = 0; Idx < AudioCount ; Idx++) {
		AudioChip[Idx]->Restart();
		AudioChip[Idx]->SetBassTreble(m_Bass,m_Treble);
		AudioChip[Idx]->SetLoudness(m_Loudness);
		AudioChip[Idx]->SetEqualizer(m_EqBands);
		AudioChip[Idx]->SetDolby(m_DolbyParams);
		AudioChip[Idx]->SetVolume(m_VolumeLeft,m_VolumeRight);
	}

	// Now we must (Re)set all the choices...

	// ---- Set the audio input ----

	// Get card information
	CInfo* Info = GetCardInfo(); 

	// Enable the corresponding GPIO pins to select audio sources...
	GetBtPisces().SetGPOE( GetBtPisces().GetGPOE() | Info->gpoe );

	// Set the correct bits to enable the Line Source
	ULONG Val;
	GetBtPisces().GetGPDATA(&Val,1,0);

	if (!bAudioMuted) {	// we were not muted...

		Val &= ~Info->gpoe;							// Keep all bits not involved into audio switching: all the others set to 0
		Val |=  Info->gpoe & Info->audiomux[AudioMux];	// And select the audio source without disrupting the other bits...
		GetBtPisces().SetGPDATA(&Val,1,0);
	
		// Now, For all the AudioChips, set the input!
		for (DWORD Idx = 0 ; Idx < AudioCount ; Idx ++) {
			AudioChip[Idx]->SetAudioConnector(AudioMux);
		}


	} else { // We were muted...

		Val &= ~Info->gpoe;								// Keep all bits not involved into audio switching: all the others set to 0
		Val |=  Info->gpoe & Info->audiomux[AudioConMute];	// And select the audio source without disrupting the other bits...
		GetBtPisces().SetGPDATA(&Val,1,0);

		// Now, for all the AudioChips, Mute audio!
		for (DWORD Idx = 0 ; Idx < AudioCount ; Idx ++) {
			AudioChip[Idx]->SetAudioConnector(AudioConMute);
		}

	}

	// Now (re)select the video input... (as a side effect, will reinit VideoChips)
	VideoSetInput(VideoMux);

	// Now, Inform the video/audioChips that we are about to change channels...
	for (Idx = 0 ; Idx < AudioCount ; Idx ++) {
		AudioChip[Idx]->VideoStandardChanged(m_ksCurrentTunerStandard);
	}

	// Now set tuner mode/frequency and standard
	// Based on current tuner mode, select the adequate handler
	switch (m_CurrentTunerMode) {
		case HwTuner::Tm_TV: 
			{
				// Find the right chip
				for (Idx = 0 ; Idx < TunerCount ; Idx ++) {
					if (TunerChip[Idx]->CanTuneTV()) {
						// This one should work ...
						ULONG RealFreq = TunerChip[Idx]->SetFrequency(m_ksCurrentTunerStandard,m_CurrentTunerMode,m_CurrentTunerFrequency,HwTuner::Tm_Fine);
						if (RealFreq) {
							m_CurrentTunerFrequency = RealFreq;
						}
						m_TunerSignalDetected = (RealFreq != 0);
					}
				}
			}
			break;
		case HwTuner::Tm_Radio_AM: 
			{
				// Find the right chip
				for (Idx = 0 ; Idx < TunerCount ; Idx ++) {
					if (TunerChip[Idx]->CanTuneAM()) {
						// This one should work ...
						ULONG RealFreq = TunerChip[Idx]->SetFrequency(m_ksCurrentTunerStandard,m_CurrentTunerMode,m_CurrentTunerFrequency,HwTuner::Tm_Fine);
						if (RealFreq) {
							m_CurrentTunerFrequency = RealFreq;
						}
						m_TunerSignalDetected = (RealFreq != 0);
					}
				}
			}
			break;
		case HwTuner::Tm_Radio_FM: 
			{
				// Find the right chip
				for (Idx = 0 ; Idx < TunerCount ; Idx ++) {
					if (TunerChip[Idx]->CanTuneFM()) {
						// This one should work ...
						ULONG RealFreq = TunerChip[Idx]->SetFrequency(m_ksCurrentTunerStandard,m_CurrentTunerMode,m_CurrentTunerFrequency,HwTuner::Tm_Fine);
						if (RealFreq) {
							m_CurrentTunerFrequency = RealFreq;
						}
						m_TunerSignalDetected = (RealFreq != 0);
					}
				}
			}
			break;
	}
	// Now, Inform the audioChips that norm (maybe) has changed and Channel has Changed ...
	for (Idx = 0 ; Idx < AudioCount ; Idx ++) {
		AudioChip[Idx]->ChannelChanged();
	}

	// Translate WDM to our representation
	AudioMode am = Am_Mono;

	if (m_AudioMode & KS_TVAUDIO_MODE_LANG_C) {
		am = Am_Lang2;
	} else 
	if (m_AudioMode & KS_TVAUDIO_MODE_LANG_B) {
		am = Am_Lang1;
	} else 
	if (m_AudioMode & KS_TVAUDIO_MODE_STEREO) {
		am = Am_Stereo;
	}

	// Now, for all tuners, Set Mode!
	for (Idx = 0 ; Idx < TunerCount ; Idx ++) {
		TunerChip[Idx]->SetAudioMode(am);
	}

	// Now for all audioChips, Set mode and language
	for (Idx = 0 ; Idx < AudioCount ; Idx ++) {
		AudioChip[Idx]->SetAudioMode(am);
	}

	// We are done!
	return STATUS_SUCCESS;

}

/* Function: BtCard::SetChipsetCompatibility
 * Purpose: Ensures compatibility between the Bt8xx and the
 *			Motherboard PCI chipset.
 * Methods:	
 *			-Flips the magic bit in certain SIS PCI Bridges.
 *			-Turns on the VIA Bit in Fusion PCI config space.
 *			-Turns on the Triton Bit in Fusion PCI config space.
 */
void BtCard::SetChipsetCompatibility(bool s)
{
	const WORD SIS_ID		= 0x1039;				// SIS manufacturer ID
	const WORD SISdevID1	= 0x5511;				// SIS Part Numbers
	const WORD SISdevID2	= 0x5596;
	const WORD SISdevID3	= 0x5597;
	const WORD SISdevID4	= 0x5598;


	// Registry settings
	Registry reg(PDO);
	LONG lVal = 0;

	if (reg.ReadKey("NeedSIS5511or5596Fix",lVal ,LONG(0)) && lVal != 0 ) {
		PCI_IO pci;		// PCI Bus
		if (pci.FindDevice(SIS_ID, SISdevID1 ) ||
			pci.FindDevice(SIS_ID, SISdevID2 ) ) {
			DWORD val ;

			DUMP(("SIS 5511 or 5596\n"));

			pci.ReadConfigDWORD( 0x080 , val ) ;
			if (s)
				val |= 0x0100 ;           // set the magic bit
			else
				val &= ~0x0100 ;          // clear the magic bit
			pci.WriteConfigDWORD( 0x080 , val ) ;
		}
	}

	if (reg.ReadKey("NeedSIS5597or5598Fix",lVal ,LONG(0)) && lVal != 0 ) {
		PCI_IO pci;		// PCI Bus
		if (pci.FindDevice(SIS_ID, SISdevID3 ) ||
			pci.FindDevice(SIS_ID, SISdevID4 ) ) {
			DWORD val ;
			const DWORD mask = 0x06 ; // bit 1 and 2 set

			DUMP(("SIS 5597 or 5598\n"));

			pci.ReadConfigDWORD( 0x081 , val ) ;
			if (s)
				val &= ~mask ;          // clear bits 1 and 2
			else
				val |= mask ;           // reset bits 1 and 2 
			pci.WriteConfigDWORD( 0x081 , val ) ;
		}
	}

	if (IsBt878Family()) {
		reg.ReadKey("NeedVFSX",lVal ,LONG(0));
		if (lVal != 0 ) {

			PCI pciBt(PDO); // PCI Space of Driver
			DWORD val ;
            DUMP(("Flipping Vfsx bit in Fusion PCI Config space [%x]\n",s)) ;

            pciBt.ReadConfigDWORD( 0x040 , val ) ;
            if (s)
               val |= 0x04 ;           // set the magic bit
            else
               val &= ~0x04 ;          // clear the magic bit

            pciBt.WriteConfigDWORD( 0x040 , val ) ;
		}
	}

	Triton848 = FALSE;
	reg.ReadKey("NeedTriton",lVal ,LONG(0)); // By default, triton fix is off!
	if (lVal != 0 ) {
		if (IsBt878Family()) {

			PCI pciBt(PDO); // PCI Space of Driver
			DWORD val ;
			DUMP(("Flipping Triton bit in Fusion PCI Config space [%x]\n",s)) ;

			pciBt.ReadConfigDWORD( 0x040 , val ) ;
			if (s)
	            val |= 0x02 ;           // set the magic bit
			else
	            val &= ~0x02 ;          // clear the magic bit
			pciBt.WriteConfigDWORD( 0x040 , val ) ;
		} else {
			Triton848 = TRUE;
		}
	}
}

BtCard::BtCard(
	const PDEVICE_OBJECT pPDO,
	const PVOID HwDeviceExtension_,
	const DWORD_PTR dwBase ) : 

		Triton848(FALSE),

		// Store PDO
		PDO(pPDO),

		// Store base address!
		dwBaseAddress(dwBase),

		// Store DeviceExtension
		HwDeviceExtension(HwDeviceExtension_),

		// Audio Stuff
		TunerConnector(AudioConnector(-1)),	// Set to an invalid value, to force routines to make the change!
		AudioMux(AudioConTuner),
		bAudioMuted(TRUE),
		m_AudioMode(0),						// Set to an invalid value, to force routines to make the change!

		// Video Stuff
		VideoMux(VideoConSVideo0),

		// HwCapture Stuff
		Pisces(NULL),
		// HwTuner Stuff
		TunerCount(0), 
		// HwAudio Stuff
		AudioCount(0) ,
		// HwVideo Stuff
		VideoCount(0) ,
		// HwIR stuff
		IRCount(0),
		// HwIFDem stuff
		IFDemCount(0),
		
		// Videonorms
		m_ksCurrentVideoStandard(KS_AnalogVideoStandard(0)), // Set to an invalid value, to force routines to make the change!

		// We default to analog video standard
		m_IsDigitalVideo(false),

		// Tuner Info
		m_CurrentTunerFrequency(1), 
		m_ksCurrentTunerStandard(KS_AnalogVideoStandard(0)), // Set to an invalid value, to force routines to make the change!

		// Current Tuner mode
		m_CurrentTunerMode(HwTuner::TunerMode(-1)), // Set to an invalid value, to force routines to make the change!

		// Last tuning method
		m_TunerLastMethod(HwTuner::Tm_Coarse),
		m_TunerCurrMethod(KS_TUNER_TUNING_FINE),

		// No signal was detected in last tuning Op.
		m_TunerSignalDetected(FALSE),

		// Start with normal audio settings!
		m_VolumeLeft(0),
		m_VolumeRight(0),
		m_Loudness(0),
		m_Bass(0),
		m_Treble(0),

		// Count of current open video channels
		m_VCOpened(0),

		// Start by NOT using the TV tuner to get FM radio.
		m_bTVAudioUsedForFM(false),

		// Init the XTal freqs
		m_xtal0freq(28636363), // By default
		m_xtal1freq(35468950), // By default

		// Not inited already
		m_InitedOK(false),

		// We are not on the ISR
		ISRLock(0)
{
	TRACE(("BtCard::BtCard"));
}

bool BtCard::Init()
{
	TRACE(("BtCard::Init"));

	// Configuration variables:
	HwTuner::Tuner	TVTuner = HwTuner::TC_Unknown;	// Type of tuner
	BYTE			TVTunerI2CAddr = I2C_TUNER_0;	// Address of Tuner
	bool			TVTunerDoesFM	= false;		// If TV tuner does tune FM
	bool			HasTEA5757		= false;		// If card has a TEA5757 as a Radio Tuner
	bool			HasRemoteControl= true;		// If card has a remote control receiver

	// Start with neutral equalizer
	memset(m_EqBands,0,sizeof(m_EqBands));

	// And with neutral Dolby
	memset(&m_DolbyParams,0,sizeof(m_DolbyParams));
	m_DolbyParams.mode = DPL_Through;

	// Instanciate a PCI object
	PCI pci(PDO);

	//  --------------- Get Device and vendor
	dwDeviceId = pci.GetDeviceID();
	dwVendorId = pci.GetVendorID();

	// ---------------- Get Chip Revision
	DWORD dwVal;
	if (!pci.ReadConfigDWORD(0x8 ,dwVal)) {
		dwVal = 0;
	}
	dwRevision = (dwVal) & 0xff;
   
	DUMP(("DeviceID: %04x\n", dwDeviceId));
	DUMP(("VendorID: %04x\n", dwVendorId));
	DUMP(("Revision: %02x\n", dwRevision));

	// ---------------- Get Subsystem ID and Subsystem Vendor ID:
	//						Only available on Bt878 Family
	if (IsBt878Family()) {

		pci.ReadConfigDWORD(0x2c ,dwVal);

		dwSubsystemId = (dwVal >> 16);
		dwSubsystemVendorId = (dwVal & 0xffff);
   
		DUMP(("SubsystemId      : %04x\n", dwSubsystemId));
		DUMP(("SubsystemVendorId: %04x\n", dwSubsystemVendorId));

	} else {
		dwSubsystemId = 0;
		dwSubsystemVendorId = 0;
	}

	// Find out if any of the problematic chipsets were found and enable compatibilities
	SetChipsetCompatibility(true);

	// With this info available, instanciate a BtPisces object!!
	// --- Needed to use the I2C Bus!. As soon as we assign something to Pisces, IRQs begin to be routed there
	Pisces = new BtPisces(this,PDO,HwDeviceExtension,dwBaseAddress);

	// If failure, end Here!
	if (!Pisces) return false;

	// try to autodetect the card:
	Registry reg(PDO);

	// ---------- Read some Misc settings
	
	LONG lVal;
	// Read user specified card type
	reg.ReadKey("Card",lVal ,LONG(BtCard::UNKNOWN));
	dwCard = BtCard::Model( lVal );

	// If user asked for Custom card, load those settings, and set it!
	if (dwCard == 0xFF || dwCard == BtCard::CUSTOM ) { // Means custom card
		reg.ReadKey("Custom_VideoIns",lVal ,1L);
		BtCard::Info[BtCard::CUSTOM].analogvideoins = (BYTE) lVal;
		// We assume a digital in
		BtCard::Info[BtCard::CUSTOM].digitalvideoins = (BYTE) 1;
		reg.ReadKey("Custom_AudioIns",lVal ,1L);
		BtCard::Info[BtCard::CUSTOM].audioins = (BYTE) lVal;
		reg.ReadKey("Custom_Tuner",lVal ,0L);
		BtCard::Info[BtCard::CUSTOM].tuner0 = (BYTE) lVal;
		reg.ReadKey("Custom_Svhs",lVal ,2L);
		BtCard::Info[BtCard::CUSTOM].svhs0 = (BYTE) lVal;
		reg.ReadKey("Custom_GPIOMask",lVal ,2L);
		BtCard::Info[BtCard::CUSTOM].gpoe = lVal;
		reg.ReadKey("Custom_VideoMux0",lVal ,2L);
		BtCard::Info[BtCard::CUSTOM].videomux[0] = (BYTE) lVal;
		reg.ReadKey("Custom_VideoMux1",lVal ,3L);
		BtCard::Info[BtCard::CUSTOM].videomux[1] = (BYTE) lVal;
		reg.ReadKey("Custom_VideoMux2",lVal ,1L);
		BtCard::Info[BtCard::CUSTOM].videomux[2] = (BYTE) lVal;
		reg.ReadKey("Custom_VideoMux3",lVal ,0L);
		BtCard::Info[BtCard::CUSTOM].videomux[3] = (BYTE) lVal;
		reg.ReadKey("Custom_AudioMux0",lVal ,0L);
		BtCard::Info[BtCard::CUSTOM].audiomux[0] = (BYTE) lVal;
		reg.ReadKey("Custom_AudioMux1",lVal ,0L);
		BtCard::Info[BtCard::CUSTOM].audiomux[1] = (BYTE) lVal;
		reg.ReadKey("Custom_AudioMux2",lVal ,0L);
		BtCard::Info[BtCard::CUSTOM].audiomux[2] = (BYTE) lVal;
		reg.ReadKey("Custom_AudioMux3",lVal ,0L);
		BtCard::Info[BtCard::CUSTOM].audiomux[3] = (BYTE) lVal;
		reg.ReadKey("Custom_AudioMux4",lVal ,0L);
		BtCard::Info[BtCard::CUSTOM].audiomux[4] = (BYTE) lVal;
		// Let the driver detect this
		BtCard::Info[BtCard::CUSTOM].msp34xx = 1;
		BtCard::Info[BtCard::CUSTOM].tvaudio = 1;
		reg.ReadKey("Custom_HasRadio",lVal ,0L);
		if (lVal == 1) {
			BtCard::Info[BtCard::CUSTOM].has_radio = 1;
		} else {
			BtCard::Info[BtCard::CUSTOM].has_radio = 0;
		}
		
		// other settings..
		BtCard::Info[BtCard::CUSTOM].pll = PLL_NONE; // Will be autodetected
		reg.ReadKey("Custom_HasTV",lVal ,0L);
		if (lVal == 0) {
			BtCard::Info[BtCard::CUSTOM].tuner_type = HwTuner::TC_Absent;
		} else {
			BtCard::Info[BtCard::CUSTOM].tuner_type = HwTuner::TC_Unknown;
		}

		// Make sure we are using the Custom card!
		dwCard = BtCard::CUSTOM;
	}

	// If user asked for autodetection, perform it!
	if (dwCard == BtCard::UNKNOWN) {

		// many bt878 cards have a eeprom @ 0xa0 => read ID
		// and try to identify it 
		BYTE Val = 0;
		if (GetBtPisces().I2CRead(0xA0,&Val)) {
			// Get Contents!
			EEProm eeprom(GetBtPisces().GetI2C(),0xA0);

			// Try to Id card
			int i;
			UINT id = (UINT(eeprom[252]) << 24) |
				(UINT(eeprom[253]) << 16) |
				(UINT(eeprom[254]) << 8)  |
				(UINT(eeprom[255]));

			// If ID is valid ...
			if (id != 0 && id != 0xffffffff) {

				// look for the card 
				for (i = 0; BtCard::Card[i].id != BtCard::UNKNOWN; i++)
					if (BtCard::Card[i].id  == id) {
						dwCard = BtCard::Card[i].cardnr;
					}
			}
		}

		// STB cards have a eeprom @ 0xae (old bt848) 
		if (dwCard == BtCard::UNKNOWN && GetBtPisces().I2CRead(I2C_STBEE,&Val)) {
			dwCard = BtCard::STB;
		}
	}

	// We must assume the card model is known, proceed to the tuner detection.
	if (reg.ReadKey("Tuner",lVal ,LONG(HwTuner::TC_Unknown))) {
		TVTuner = HwTuner::Tuner( lVal );
		// If unspecified, Get tuner from card model
		if (TVTuner == HwTuner::TC_Unknown) {
			TVTuner = BtCard::Info[dwCard].tuner_type;
		}
	} else {
		TVTuner = BtCard::Info[dwCard].tuner_type;
	}

	
	// We know if card receives FM using info of card
	TVTunerDoesFM = (BtCard::Info[dwCard].has_radio == 1);	

	// Now, we try to autodetect features
	if (dwCard == BtCard::LIFETEC_9415) {
		TVTunerDoesFM = false;
		ULONG id;
		GetBtPisces().SetGPOE(0x18e0);
		GetBtPisces().GetGPDATA(&id,1,0);
		if (id & 0x4000) {
			TVTunerDoesFM = true;
		}
	}

	// Miro or Pinnacle can autodetect Tuner and Radio ... - Use it!
	ULONG PinnacleID = 0;
	if (dwCard == BtCard::MIRO      ||
		dwCard == BtCard::MIROPRO   ||
		dwCard == BtCard::PINNACLE  ||
	    dwCard == BtCard::PINNACLEPRO) {

		TVTunerDoesFM = false;

		// Set to inputs the appropiate lines 
		GetBtPisces().SetGPOE(0x00000); // All ins

		// Get Tuner ID
		ULONG id;
		GetBtPisces().GetGPDATA(&id,1,0);
		id = ((id >> 10) & 63) - 1;

		// If dealing with an older Pinnacle card...
		if (id<32) {

			// Map MIRO tuner codes to our Tuners.
			if (TVTuner == HwTuner::TC_Unknown) {
				TVTuner = miro_tunermap[id];
			}
		
			// Find out if radio tuner present
			ULONG val;
			GetBtPisces().GetGPDATA(&val,1,0);
			if (val & 0x20) {
				TVTunerDoesFM = true;
				if (miro_fmtuner[id]) {
					HasTEA5757 = true;
					TVTunerDoesFM = false;
				}
			}
		} else {

			// New Pinnacle Cards
			id = 63-id;	
			PinnacleID = id;
			TVTunerDoesFM = true;

			// Only if tuner model not known...
			if (TVTuner == HwTuner::TC_Unknown) {

				// Enable I2C access to tuner ...
				InitCard();

				// Differenciate between MT2032 and MT2050
				BYTE buf[21];
				buf[0]=0; buf[0x14]=0;buf[0]=0x0;

				// Get information from tuner
				if (!GetBtPisces().I2CWR(I2C_TUNER_0,buf,1,buf,21)) {
					buf[0]=0; buf[0x14]=0;buf[0]=0x0;
					if (!GetBtPisces().I2CWR(I2C_TUNER_1,buf,1,buf,21)) {
						buf[0]=0; buf[0x14]=0;buf[0]=0x0;
						if (!GetBtPisces().I2CWR(I2C_TUNER_2,buf,1,buf,21)) {
							buf[0]=0; buf[0x14]=0;buf[0]=0x0;
							if (!GetBtPisces().I2CWR(I2C_TUNER_3,buf,1,buf,21)) {
								buf[0]=0; buf[0x14]=0;buf[0]=0x0;
								if (!GetBtPisces().I2CWR(I2C_TUNER_4,buf,1,buf,21)) {
									buf[0]=0; buf[0x14]=0;buf[0]=0x0;
									if (!GetBtPisces().I2CWR(I2C_TUNER_5,buf,1,buf,21)) {
										buf[0]=0; buf[0x14]=0;buf[0]=0x0;
										if (!GetBtPisces().I2CWR(I2C_TUNER_6,buf,1,buf,21)) {
											buf[0]=0; buf[0x14]=0;buf[0]=0x0;
											GetBtPisces().I2CWR(I2C_TUNER_7,buf,1,buf,21);
										}
									}
								}
							}
						}
					}
				}

				DUMP(("MT20xx: Companycode=%02x%02x Part=%02x Revision=%02x\n",
					buf[0x11],buf[0x12],buf[0x13],buf[0x14]));

				// Look for MT2032 id:
				// part= 0x04(MT2032), 0x06(MT2030), 0x07(MT2040)
				if((buf[0x11] == 0x4d) && (buf[0x12] == 0x54) && (buf[0x13] == 0x04)) {
					DUMP(("tuner: MT2032 detected.\n"));
					TVTuner = HwTuner::TC_MT2032;
				} else if(buf[0]==0x42) {
					DUMP(("tuner: MT2050 detected.\n"));
					TVTuner = HwTuner::TC_MT2050;
				} else {
					DUMP(("tuner: no MT2032 nor MT2050 detected. Guessing MT2032...\n"));
					switch (id) {

						case 1: //PAL / mono
							TVTuner = HwTuner::TC_MT2032/*_PAL*/;
							break;
						case 2: //PAL+SECAM / stereo
							TVTuner = HwTuner::TC_MT2032/*_PAL*/;
							break;
						case 3: //NTSC / stereo
							TVTuner = HwTuner::TC_MT2032;
							break;
						case 4: //PAL+SECAM / mono
							TVTuner = HwTuner::TC_MT2032/*_PAL*/;
							break;
						case 5: //NTSC / mono
							TVTuner = HwTuner::TC_MT2032;
							break;
						case 6: //NTSC / stereo
							TVTuner = HwTuner::TC_MT2032;
							break;
						default:
							//unknown...
							TVTuner = HwTuner::TC_Absent;
							break;
					}

				}
			}

		}

		// If user says yes, force TEA5757 settings
		if (!HasTEA5757) {
			if (reg.ReadKey("ForceTEA5757",lVal ,LONG(false))) {
				if (lVal && lVal != 0xFF) HasTEA5757 = true;
			}
		}

	}

	// All the following ones can autodetect tuners and other goodies
	if (dwCard == BtCard::FLYVIDEO_98		||
		dwCard == BtCard::TYPHOON_TVIEW		||
		dwCard == BtCard::CHRONOS_VS2		||
	    dwCard == BtCard::FLYVIDEO_98FM		||
		dwCard == BtCard::FLYVIDEO2000		||
		dwCard == BtCard::FLYVIDEO2000S		||
	    dwCard == BtCard::FLYVIDEO98EZ)		{

		ULONG out = GetBtPisces().GetGPOE();
		// Set to inputs the appropiate lines 
		GetBtPisces().SetGPOE(0x00000); // All ins
		udelay(8);	// without this we would see the 0x1800 mask
		ULONG gpio;
		GetBtPisces().GetGPDATA(&gpio,1,0);
		GetBtPisces().SetGPOE(out);
		// all cards provide GPIO info, some have an additional eeprom

		// lowest 3 bytes are remote control codes (no handshake needed)
		ULONG ttype = (gpio&0x0f0000)>>16;
		if (TVTuner == HwTuner::TC_Unknown) {
			switch(ttype) {
				case 0: 
					TVTuner = HwTuner::TC_Absent;			// None
					break;
				case 4: 
					TVTuner = HwTuner::TC_PHILIPS_PAL;		// Philips PAL
					break;
				case 6: 
					TVTuner = HwTuner::TC_LG_PAL_NEW_TAPC;	// LG PAL (newer TAPC series)
					break;
				case 0xC: 
					TVTuner = HwTuner::TC_PHILIPS_SECAM;	// Philips SECAM(+PAL)
					break;
				default:
					DUMP(("flyvideo: unknown tuner type.\n"));
			}
		}

		DUMP(( "Flyvideo Radio=%s RemoteControl=%s Tuner=%d gpio=0x%06x\n", 
	       gpio&0x400000? "yes":"no",
	       gpio&0x800000? "yes":"no", TVTuner, gpio));

		TVTunerDoesFM = (gpio&0x400000) != 0;
		HasRemoteControl = (gpio&0x800000) != 0;
	}

	// If a Hauppauge card is present, pick info from the EEprom!
	if (dwCard == BtCard::HAUPPAUGE || 
		dwCard == BtCard::HAUPPAUGE878) {

		// Get Contents!
		EEProm eeprom(GetBtPisces().GetI2C(),0xA0);

		if (eeprom[0] != 0x84 || eeprom[2] != 0) {
			ERROR(("Hauppauge E2PROM invalid!\n"));
		}

		// Extract tuner type if Possible
		if (eeprom[9] < sizeof(hauppauge_tuner)/sizeof(hauppauge_tuner[0])) {
			if (TVTuner == HwTuner::TC_Unknown) {
				TVTuner = hauppauge_tuner[eeprom[9]].id;
			}
		}

		/* Block 2 starts after len+3 bytes header */
		int blk2 = eeprom[1] + 3;
	
		/* decode + use some config infos */
		//model = eeprom_data[12] << 8 | eeprom_data[11];
		//tuner = eeprom_data[9];
		int radio = eeprom[blk2-1] & 0x01;
		int infrared = eeprom[blk2-1] & 0x04;
		TVTunerDoesFM = false;
		if (radio) {
			TVTunerDoesFM = true;
		}
		HasRemoteControl = false;
		if (infrared) {
			HasRemoteControl = true;
		}
	}

	// If Avermedia, get contents from eeprom
	if (dwCard == BtCard::AVERMEDIA98 ||
		dwCard == BtCard::AVPHONE98 ||
		dwCard == BtCard::AVERTVOLD ) {

		// Get Contents!
		EEProm eeprom(GetBtPisces().GetI2C(),0xA0);

		BYTE tuner_make,tuner_tv_fm,tuner_format,tuner=0;

		tuner_make   = (eeprom[0x41] & 0x7);
	    tuner_tv_fm  = (eeprom[0x41] & 0x18) >> 3;
		tuner_format = (eeprom[0x42] & 0xf0) >> 4;

		if (tuner_make == 0 || tuner_make == 2)
			if(tuner_format <=9)
				if (TVTuner == HwTuner::TC_Unknown) {
					TVTuner = avermedia_tuner_0_table[tuner_format];
				}
		if (tuner_make == 1)
			if(tuner_format <=9)
				if (TVTuner == HwTuner::TC_Unknown) {
					TVTuner = avermedia_tuner_1_table[tuner_format];
				}
	
		DUMP(( "Avermedia eeprom[0x%02x%02x]: tuner=",
			eeprom[0x41],eeprom[0x42]));

		// We assume we have a remote control
		HasRemoteControl = true;
	}

	// VHX has a TEA5757 as its tuner, flag it!
	if (dwCard == BtCard::VHX ||
		dwCard == BtCard::VHXDELUXE	) {
		TVTunerDoesFM = false;
		HasTEA5757 = true;
	}

	if (dwCard == BtCard::LIFETEC_9415) {
		ULONG Val;
		GetBtPisces().GetGPDATA(&Val,1,0);
		if (Val & 0x4000) {
			DUMP(("lifetec: tv mono/fm stereo card"));
		} else {
			DUMP(("lifetec: stereo(TDA9821) card"));
		}
	}

	if (dwCard == BtCard::MAGICTVIEW061) {
		// Get Contents!
		EEProm eeprom(GetBtPisces().GetI2C(),0xA0);

		// Try to Id card
		UINT id = (UINT(eeprom[252]) << 24) |
			(UINT(eeprom[253]) << 16) |
			(UINT(eeprom[254]) << 8)  |
			(UINT(eeprom[255]));

		if(id == 0x4002144f) {
			TVTunerDoesFM = true;
			DUMP(("radio detected by subsystem id (CPH05x)"));
		}
	}

	// If user says yes, force TVTuner settings
	if (reg.ReadKey("TVTunerDoesFM",lVal ,LONG(false))) {
		if (lVal && lVal != 0xFF) TVTunerDoesFM = true;
	}

	// If user says yes, force Remote settings
	if (reg.ReadKey("HasRemote",lVal ,LONG(true))) {
		if (lVal != 0xFF) HasRemoteControl = (lVal != 0);
	}

	// If the TV tuner does not natively support FM, use TVAudio to try to get it
	// The user can override this so he/she wont get FM at all.
	if (!TVTunerDoesFM) {
		reg.ReadKey("TVAudioUsedForFM",lVal ,LONG(!HasTEA5757) );
		m_bTVAudioUsedForFM = (lVal != 0);
		if (m_bTVAudioUsedForFM) {
			HasTEA5757 = false;
		}
	}

	// We must assume that we know the Card Type and the Tuner .... 

	// Autodetect the xtal frequencies of each port of the card

	// Start using some default values based on given info
	if (Info[dwCard].pll == BtCard::PLL_35) { // Not recommended by brooktree

		m_xtal0freq = 35468950;
		m_xtal1freq = 28636363;

	} else 
	if (Info[dwCard].pll == BtCard::PLL_14) {	/* Other special case - Some crazy cards have this! */

		m_xtal0freq = 14318181;
		m_xtal1freq = 35468950;

	} else /* all the other cases assume normal xtals */
	  /*if (Info[dwCard].pll == BtCard::PLL_28 ||
	  	    Info[dwCard].pll == BtCard::PLL_NONE) )*/ 
	{
		m_xtal0freq = 28636363;
		m_xtal1freq = 35468950;
	}

	// And YES, if we have a PLL available, make sure to use it (better than to guess
	// 2 crystal frequencies is to guess just one of them!)
	if (HasPLL()) {
		m_xtal1freq = 0;
	}

	// If user says NO, don't autodetect crystal frequencies... 
	// There are problems with this piece of code under Win2000/WinXP, so
	// it will be disabled there
	lVal = 0x00;
	reg.ReadKey("TryXtalAutodetection",lVal ,LONG(0x00));

	// If user allowed it, try autodetection...
	if (lVal) {

		// Try to autodetect frequency of xtal0!
		ULONG FieldTimeInMsx10 = GetBtPisces().TimeDefaultField(Crystal_XT0); // Measure time between fields

		// If we were able to do it...
		if (FieldTimeInMsx10) { 

			// In NTSC, field timing should be 63556 ns per line, and 525/2 lines per field. 
			// Use it to calculate the Crystal frequency.

			ULONG xtal0freq = ULONG( (28636363i64 * 167i64) / FieldTimeInMsx10);
			DUMP(("Detected xtal0 freq (aprox): %d\n",xtal0freq));

			// Here we try to deduce the possible Crystal freq. We will try some very used ones!
			ULONG Try = 28636363;
			while (Try > 1000000) {

				// Round if we are close enough
				if ( abs( int(xtal0freq - Try) ) < int(Try/100) ) {
					xtal0freq = Try;
					break;
				}

				// Try the lower multiple
				Try >>= 1;
			};

			Try = 35468950;
			while (Try > 1000000) {

				// Round if we are close enough
				if ( abs( int(xtal0freq - Try) ) < int(Try/100) ) {
					xtal0freq = Try;
					break;
				}

				// Try the lower multiple
				Try >>= 1;
			};
		

			// And set the XT0 frequency!
			m_xtal0freq = xtal0freq;
		}

		FieldTimeInMsx10 = GetBtPisces().TimeDefaultField(Crystal_XT1);		// Measure time between fields

		// If we were able to do it...
		if (FieldTimeInMsx10) { 

			// In PAL, field timing should be 64000 ns per line, and 625/2 lines per field. 
			// Use it to calculate the Crystal frequency.

			ULONG xtal1freq = ULONG( (35468950i64 * 200i64) / FieldTimeInMsx10);
			DUMP(("Detected xtal1 freq (aprox): %d\n",xtal1freq));

			// Here we try to deduce the possible Crystal freq. We will try some very used ones!
			ULONG Try = 28636363;
			while (Try > 1000000) {

				// Round if we are close enough (enough seems to be 1% of Try for this algorithm)
				if ( abs( int(xtal1freq - Try) ) < int(Try/100) ) {
					xtal1freq = Try;
					break;
				}

				// Try the lower multiple
				Try >>= 1;
			};

			Try = 35468950;
			while (Try > 1000000) {

				// Round if we are close enough
				if ( abs( int(xtal1freq - Try) ) < int(Try/100) ) {
					xtal1freq = Try;
					break;
				}

				// Try the lower multiple
				Try >>= 1;
			};

			// And set the XT1 frequency!
			m_xtal1freq = xtal1freq;
		}
	}

	// Let user override Xtal settings
	if (reg.ReadKey("Xtal0",lVal ,LONG(0xFF))) {
		if (lVal != 0xFF) m_xtal0freq = lVal;
	}
	if (reg.ReadKey("Xtal1",lVal ,LONG(0xFF))) {
		if (lVal != 0xFF) m_xtal1freq = lVal;
	}

	// If both frequencies are the same, means that no xtal is connected to XT1.
	// (because Bt8xx used XT0 because XT1 was unconnected)
	if (m_xtal0freq == m_xtal1freq) {
		// Means there is only one crystal present!
		m_xtal1freq = 0; // Remember this!
	}

	DUMP(("Xtal0 frequency = %u\n",m_xtal0freq));
	DUMP(("Xtal1 frequency = %u\n",m_xtal1freq));

	// Reinit special chips in cards.... Specially needed to enable i2c chips!!!
	InitCard();

	// Begin autodetection of all the other Chips!

	// The address of each chip
	// and Detected flag
	bool HasMSP34xx		= false;
	BYTE MSP34xxI2CAddr = I2C_MSP3400C_0;
	bool HasTDA7432		= false;
	BYTE TDA7432I2CAddr = I2C_TDA7432;
	bool HasTDA8425		= false;
	BYTE TDA8425I2CAddr = I2C_TDA8425;
	bool HasTDA9840		= false;
	BYTE TDA9840I2CAddr = I2C_TDA9840;
	bool HasTDA9850		= false;
	BYTE TDA9850I2CAddr = I2C_TDA9850_0;
	bool HasTDA9855		= false;
	BYTE TDA9855I2CAddr = I2C_TDA9855_0;
	bool HasTDA9873		= false;
	BYTE TDA9873I2CAddr = I2C_TDA9873_0;
	bool HasTDA9874		= false;
	BYTE TDA9874I2CAddr = I2C_TDA9874_0;
	bool HasTDA9875		= false;
	BYTE TDA9875I2CAddr = I2C_TDA9875_0;
	bool HasTEA6300		= false;
	BYTE TEA6300I2CAddr = I2C_TEA6300;
	bool HasTEA6320		= false;
	BYTE TEA6320I2CAddr = I2C_TEA6320;
	bool HasTEA6420		= false;
	BYTE TEA6420I2CAddr = I2C_TEA6420_0;
	bool HasTDA8540		= false;
	BYTE TDA8540I2CAddr = I2C_TDA8540_0;
	bool HasPV951		= false;
	BYTE PV951I2CAddr   = I2C_PV951;
	bool HasHauppIR		= false;
	BYTE HauppIRAddr    = I2C_HAUPPAUGEIR_0;
	bool HasKNC1IR		= false;
	BYTE KNC1IRAddr     = I2C_KNC1IR_0;
	bool HasPVIEWIR		= false;
	BYTE PVIEWIRAddr    = I2C_PVIEWIR_0;
	bool HasDPL3518		= false;
	BYTE DPL3518Addr	= I2C_DPL3518;
	bool HasTDA9887		= false;
	BYTE TDA9887Addr	= I2C_TDA9887_0;

	// To Scan [0xc0] - [0xce]
	// if we have the tuner type, try to autodetect its address... 
	// Seems to be reasonable to wait until tuner is locked
	if (TVTuner != HwTuner::TC_Absent) {
		BYTE v[1];
		if (GetBtPisces().I2CRead(I2C_TUNER_0,v) && (v[0] & 0x40) ) { 
			TVTunerI2CAddr = I2C_TUNER_0;
		} else 
		if (GetBtPisces().I2CRead(I2C_TUNER_1,v) && (v[0] & 0x40) ) { 
			TVTunerI2CAddr = I2C_TUNER_1;
		} else 
		if (GetBtPisces().I2CRead(I2C_TUNER_2,v) && (v[0] & 0x40) ) { 
			TVTunerI2CAddr = I2C_TUNER_2;
		} else 
		if (GetBtPisces().I2CRead(I2C_TUNER_3,v) && (v[0] & 0x40) ) { 
			TVTunerI2CAddr = I2C_TUNER_3;
		} else 
		if (GetBtPisces().I2CRead(I2C_TUNER_4,v) && (v[0] & 0x40) ) { 
			TVTunerI2CAddr = I2C_TUNER_4;
		} else 
		if (GetBtPisces().I2CRead(I2C_TUNER_5,v) && (v[0] & 0x40) ) { 
			TVTunerI2CAddr = I2C_TUNER_5;
		} else 
		if (GetBtPisces().I2CRead(I2C_TUNER_6,v) && (v[0] & 0x40) ) { 
			TVTunerI2CAddr = I2C_TUNER_6;
		} else 
		if (GetBtPisces().I2CRead(I2C_TUNER_7,v) && (v[0] & 0x40) ) { 
			TVTunerI2CAddr = I2C_TUNER_7;
		} else 
		// Assume MT2032 ... Try again!
		if (GetBtPisces().I2CRead(I2C_TUNER_0,v)) { 
			TVTunerI2CAddr = I2C_TUNER_0;
		} else 
		if (GetBtPisces().I2CRead(I2C_TUNER_1,v)) { 
			TVTunerI2CAddr = I2C_TUNER_1;
		} else 
		if (GetBtPisces().I2CRead(I2C_TUNER_2,v)) { 
			TVTunerI2CAddr = I2C_TUNER_2;
		} else 
		if (GetBtPisces().I2CRead(I2C_TUNER_3,v)) { 
			TVTunerI2CAddr = I2C_TUNER_3;
		} else 
		if (GetBtPisces().I2CRead(I2C_TUNER_4,v)) { 
			TVTunerI2CAddr = I2C_TUNER_4;
		} else 
		if (GetBtPisces().I2CRead(I2C_TUNER_5,v)) { 
			TVTunerI2CAddr = I2C_TUNER_5;
		} else 
		if (GetBtPisces().I2CRead(I2C_TUNER_6,v)) { 
			TVTunerI2CAddr = I2C_TUNER_6;
		} else 
		if (GetBtPisces().I2CRead(I2C_TUNER_7,v)) { 
			TVTunerI2CAddr = I2C_TUNER_7;
		} else {
			TVTunerI2CAddr = I2C_TUNER_0;
		}
	}

	// Detection methods:
	// ! = requires ACK AND test to be detected : -> Much more reliable.
	// ? = Only if ACK	
	// : = Only if this card is detected...

	// 0x1e 12C508pic
	// 0x30 IR receiver Hauppauge
	// 0x34 IR receiver Hauppauge
	// 0x42 SAA7118(in SweetSpotPDI)
	// 0x5e MAX517
	// 0x60 KNC IR
	// 0x72 PCF8575A (Primavision)
	// 0x80 MSP4300! TEA6300? TEA6320?
	// 0x82 TDA8425?
	// 0x84 TDA9840! DPL3518? TDA9885/6/7?
	// 0x86 TDA9885/6/7? (only used by pinnacle)
	// 0x8a TDA7432?
	// 0x90 TDA8540?
	// 0x94 TDA9885/6/7?
	// 0x96 PIC16c54? TDA9885/6/7?
	// 0x98 TEA6420?
	// 0xa0 HAUPEE
	// 0xaa Tekram I2C audio chip...
	// 0xae STBEE
	// 0xb0 TDA9874! TDA9875! SILK200Sk?
	// 0xb4 TDA9873! TDA9850? TDA9855?
	// 0xb6 TDA9873! TDA9850? TDA9855?
	// 0xc0 VHX
	// 0xc2 TSA5522
	// 0xc8 PixelView IR
	
	// Now, autodetect the Hauppauge IR receiver
	if (dwCard == BtCard::HAUPPAUGE || 
		dwCard == BtCard::HAUPPAUGE878 ) {

		BYTE v[1];
		if (GetBtPisces().I2CRead(I2C_HAUPPAUGEIR_0,v)) { 
			HasHauppIR = true;
			HauppIRAddr = I2C_HAUPPAUGEIR_0;
		} else {
			if (GetBtPisces().I2CRead(I2C_HAUPPAUGEIR_1,v)) { 
				HasHauppIR = true;
				HauppIRAddr = I2C_HAUPPAUGEIR_1;
			} else {
				if (HasRemoteControl) { // If failure to detect , assume internal control
					HasHauppIR = true;
					HauppIRAddr = I2C_HAUPPAUGEIR_0;
				}
			}
		}
	}

	// Autodetect the Typhoon/KNC1 IR receiver.
	{
		BYTE v[1];
		if (GetBtPisces().I2CRead(I2C_KNC1IR_0,v)) { 

			HasKNC1IR = true;
			HasRemoteControl = true;
			KNC1IRAddr = I2C_KNC1IR_0;
			DUMP(("Thypoon/KNC1 IR is present at 0x%02x\n",KNC1IRAddr));
		}
	}

	// Autodetect the PixelView IR receiver.
	{
		BYTE v[1];
		if (TVTunerI2CAddr != I2C_PVIEWIR_0 &&
			GetBtPisces().I2CRead(I2C_PVIEWIR_0,v)) { 

			HasPVIEWIR = true;
			HasRemoteControl = true;
			PVIEWIRAddr = I2C_PVIEWIR_0;
			DUMP(("PixelView IR is present at 0x%02x\n",PVIEWIRAddr));
		}
	}

	// Now, autodetect the MSP chip. Complex test to ensure we have a MSP installed
	// To Scan [0x80]
	if (Info[dwCard].msp34xx) {
		BYTE v[1];

		static BYTE reset_off[3] = { 0x00, 0x80, 0x00 };
		static BYTE reset_on[3]  = { 0x00, 0x00, 0x00 };
		BYTE writebf[3];
		BYTE readbf[2];
		WORD rev1,rev2;
		
		if (GetBtPisces().I2CRead(I2C_MSP3400C_0,v)) { 
			// Its responding ... Try a better check!
			// Reset MSP!
			GetBtPisces().I2CWR(I2C_MSP3400C_0,reset_off,3,NULL,0,true);  /* XXX ignore errors here */
			if (GetBtPisces().I2CWR(I2C_MSP3400C_0, reset_on, 3,NULL,0)) {
				// If Reset went ok, Get revision
				writebf[0] = I2C_MSP3400C_DFP + 1;
				writebf[1] = 0x1e >> 8;
				writebf[2] = 0x1e & 0xff;
				if (GetBtPisces().I2CWR(I2C_MSP3400C_0,writebf,3,readbf,2)) {				
					rev1 = ( WORD(readbf[0]) << 8) | readbf[1] ;
					writebf[0] = I2C_MSP3400C_DFP + 1;
					writebf[1] = 0x1f >> 8;
					writebf[2] = 0x1f & 0xff;
					if (GetBtPisces().I2CWR(I2C_MSP3400C_0,writebf,3,readbf,2)) {				
						rev2 = ( WORD(readbf[0]) << 8) | readbf[1] ;
						if ( (rev1 != rev2) ||
							(rev1 != 0 && rev1 != 0xFFFF) ) {
							// Revision seems to be OK! ... Accept!
							HasMSP34xx = true;
							MSP34xxI2CAddr = I2C_MSP3400C_0;
						}
					}
				}
			}
		}
	}

	if (Info[dwCard].tvaudio) {
		// Now, autodetect TDA9840
		// To Scan [0x84]
		// Data read & 0x1f Must be 0, to be in presence of a TDA9840. 
		// Also, writes to registers above 3, wont ack! (else, MSP here!)
		{
			BYTE v[1];
			if (GetBtPisces().I2CRead(I2C_TDA9840,v)) { 
				// Ackd ... Chk valid Status
				if ((v[0] & 0x1F) == 0) { 
					// Check that we are ackd a write to the 4th reg, but not, a write to the 5th reg
					if (GetBtPisces().I2CWrite(I2C_TDA9840,3,0)) {
						// Write to 4th register succedeed, try to the next (Must NOT succeed)
						if (!GetBtPisces().I2CWrite(I2C_TDA9840,4,0)) {
							// Detected Chip!
							HasTDA9840 = true;
							TDA9840I2CAddr = I2C_TDA9840;
						}
					}
				}
			} 
		}

		// Autodetect DPL3518
		if (!HasTDA9840) {
			BYTE v[1];

			static BYTE reset_off[3] = { 0x00, 0x80, 0x00 };
			static BYTE reset_on[3]  = { 0x00, 0x00, 0x00 };
			BYTE writebf[3];
			BYTE readbf[2];
		
			if (GetBtPisces().I2CRead(I2C_DPL3518,v)) { 
				// Its responding ... Try a better check!
				// Reset DPL!
				GetBtPisces().I2CWR(I2C_DPL3518,reset_off,3,NULL,0,true);  /* XXX ignore errors here */
				if (GetBtPisces().I2CWR(I2C_DPL3518, reset_on, 3,NULL,0)) {
					HasDPL3518 = true;
					DPL3518Addr = I2C_DPL3518;
				}
			}
		}

		// Now, autodetect TDA9873
		// To Scan [0xb4] ##### WARNING: CLASH with TDA9875
		// To Scan [0xb6] ##### WARNING: CLASH with TDA9855!
		{
			BYTE readbf[1];
			BYTE writebf[1];
			BYTE v[1];
			if (GetBtPisces().I2CRead(I2C_TDA9873_0,v)) { 
				// Try a little Harder .... 
				// [254] (Register [254] & ~0x1f) == 0x80
				writebf[0] = 254;
				if (GetBtPisces().I2CWR(I2C_TDA9873_0,writebf,1,readbf,1)) {
					if ( ( readbf[0] & ~0x1f) == 0x80 ) {
						HasTDA9873 = true;
						TDA9873I2CAddr = I2C_TDA9873_0;
					}
				}
			} 

			if (!HasTDA9873 && GetBtPisces().I2CRead(I2C_TDA9873_1,v)) { 
				// Try a little Harder .... 
				// [254] (Register [254] & ~0x1f) == 0x80
				writebf[0] = 254;
				if (GetBtPisces().I2CWR(I2C_TDA9873_1,writebf,1,readbf,1)) {
					if ( ( readbf[0] & ~0x1f) == 0x80 ) {
						HasTDA9873 = true;
						TDA9873I2CAddr = I2C_TDA9873_1;
					}
				}
			} 
		}

		// Now autodetect TDA9874
		// To Scan [0xb0]
		{
			BYTE readbf[1];
			BYTE writebf[1];
			BYTE v[1];
			if (GetBtPisces().I2CRead(I2C_TDA9874_0,v)) { 
				// Try a little Harder .... 
				// [254] (Register [254] & ~0x1f) == 0x80
				writebf[0] = 254;
				if (GetBtPisces().I2CWR(I2C_TDA9874_0,writebf,1,readbf,1)) {
					if ( (readbf[0] & 0xff) == 0x11 ||	// TDA9874A
						 (readbf[0] & 0xff) == 0x07		// TDA9874H
						) {
						HasTDA9874 = true;
						TDA9874I2CAddr = I2C_TDA9874_0;
					}
				}
			}
		}

		// Now, autodetect TDA9875
		// To Scan [0xb0] (clashes with TDA9874)
		if (!HasTDA9874) {
			BYTE readbf[1];
			BYTE writebf[1];
			BYTE v[1];
			if (GetBtPisces().I2CRead(I2C_TDA9875_0,v)) { 
				// Try a little Harder .... 
				// Register [253] >= 7    && < 255
				// Register [252] >= 0x2D && < 255
				// Register [Register [253] + 1] == 0
				writebf[0] = 252;
				if (GetBtPisces().I2CWR(I2C_TDA9875_0,writebf,1,readbf,1)) {
					if (readbf[0] >= 0x2D && readbf[0] < 255) {
						writebf[0] = 253;
						if (GetBtPisces().I2CWR(I2C_TDA9875_0,writebf,1,readbf,1)) {
							if (readbf[0] >= 7 && readbf[0] < 255) {
								writebf[0] = readbf[0] + 1;
								if (GetBtPisces().I2CWR(I2C_TDA9875_0,writebf,1,readbf,1)) {
									if (readbf[0] == 0xFF) {
										// All the tests passed! ... Hope we are in the right way
										HasTDA9875 = true;
										TDA9875I2CAddr = I2C_TDA9875_0;
									}
								}
							}
						}	
					}
				}
			}
		}

		// Check if Card has a TDA9855
		reg.ReadKey("HasTDA9855",lVal ,LONG(false));
		HasTDA9855 = (lVal != 0);

		// Now, autodetect TDA9850
		// To Scan [0xb6], [0xb4] 
		if (!HasTDA9873 && !HasTDA9855) {
			BYTE v[1];
			if (!HasTDA9873 && GetBtPisces().I2CRead(I2C_TDA9850_0,v)) { 
				TDA9850I2CAddr = I2C_TDA9850_0;
				HasTDA9850 = true;
			} else if (GetBtPisces().I2CRead(I2C_TDA9850_1,v)) { 
				TDA9850I2CAddr = I2C_TDA9850_1;
				HasTDA9850 = true;
			}
		} 

		// Now, autodetect TDA9855
		// To Scan [0xb6], [0xb4]
		if (!HasTDA9873 && !HasTDA9850) {
			BYTE v[1];
			if (GetBtPisces().I2CRead(I2C_TDA9855_0,v)) { 
				TDA9855I2CAddr = I2C_TDA9855_0;
				HasTDA9855 = true;
			} else if (GetBtPisces().I2CRead(I2C_TDA9855_1,v)) { 
				TDA9855I2CAddr = I2C_TDA9855_1;
				HasTDA9855 = true;
			}
		}
	
		// Now, autodetect TEA6300 ... This Chip does NOT ack so force it if and STB card is found
		// To Scan [0x80] (clashes with MSP34xx) 
		if (!HasMSP34xx) {
			BYTE v[1];
			if (dwCard == BtCard::STB ||
				dwCard == BtCard::STB2 ||
				GetBtPisces().I2CRead(I2C_TEA6300,v)) { 
				TEA6300I2CAddr = I2C_TEA6300;
				HasTEA6300 = true;
			}
		} 

		// Now, autodetect TEA6320 ... This Chip does NOT ack so force it if an CEI Raffles card is found
		// To Scan [0x80] (clashes with MSP34xx) 
		if (!HasMSP34xx) {
			BYTE v[1];
			if (dwCard == BtCard::CEI_RAFFLES ||
				GetBtPisces().I2CRead(I2C_TEA6320,v)) { 
				TEA6320I2CAddr = I2C_TEA6320;
				HasTEA6320 = true;
			}
		} 

		// Now, autodetect TEA6420 ... Seems to be present on STB2 boards...
		// To Scan [0x98]
		{
			BYTE v[1];
			if (GetBtPisces().I2CRead(I2C_TEA6420_0,v)) { 
				TEA6420I2CAddr = I2C_TEA6420_0;
				HasTEA6420 = true;
			} else 
			if (dwCard == BtCard::STB ||
				dwCard == BtCard::STB2) {
				// Some STB cards have it... Won't hurt if we force it there!
				TEA6420I2CAddr = I2C_TEA6420_0;
				HasTEA6420 = true;
			}

		}

		// Now, autodetect TDA8425 
		// To Scan [0x82]
		{
			BYTE v[1];
			if (GetBtPisces().I2CRead(I2C_TDA8425,v)) { 
				TDA8425I2CAddr = I2C_TDA8425;
				HasTDA8425 = true;
			}
		}

		// Now, autodetect PIC16c54 on PV951
		// To Scan [0x96]
		{
			BYTE v[1];
			DWORD cf = GetBtPisces().I2CGetFreq();
			GetBtPisces().I2CSetFreq(6000); // This device is slow , so I2C must go slowly!
			if (GetBtPisces().I2CRead(I2C_PV951,v)) { 
				PV951I2CAddr = I2C_PV951;
				HasPV951 = true;
			} else {
				// If PV951 type card, select the right Audio chip for it!
				if (dwCard == BtCard::PV951) {
					HasPV951 = true;
					PV951I2CAddr = I2C_PV951;
				} else {
					// Restore default speed
					GetBtPisces().I2CSetFreq(cf);
				}
			}
		} 

		// Now, autodetect TDA7432 (Seems there is a way to receive something ... Not Documented ! 
		// To Scan [0x8a]
		{
			BYTE v[1];
			if (GetBtPisces().I2CRead(I2C_TDA7432,v)) { 
				TDA7432I2CAddr = I2C_TDA7432;
				HasTDA7432 = true;
			} else 
			// Some STB cards have it.. Won't hurt if forced...
			if (dwCard == BtCard::STB ||
				dwCard == BtCard::STB2) {
				// Better an slight delay than not to drive them if present!
				TDA7432I2CAddr = I2C_TDA7432;
				HasTDA7432 = true;
			}
		}
	}

	// Now, autodetect TDA8540
	// To Scan 0x90
	{
		BYTE v[1];
		if (GetBtPisces().I2CRead(I2C_TDA8540_0,v)) { 
			TDA8540I2CAddr = I2C_TDA8540_0;
			HasTDA8540 = true;
		} else {
			if (dwCard == BtCard::SDISILK100_S ||
				dwCard == BtCard::SDISILK100_C ||
				dwCard == BtCard::SASEM_S ||
				dwCard == BtCard::SDISILK200_S ||
				dwCard == BtCard::SDISILK200_C) {

				// We know that those cards have it!
				TDA8540I2CAddr = I2C_TDA8540_0;
				HasTDA8540 = true;
			}
		}
	} 

	// Now, autodetect TDA9887
	// To Scan 0x86
	{
		BYTE v[1];
		if (GetBtPisces().I2CRead(I2C_TDA9887_0,v)) { 
			TDA9887Addr = I2C_TDA9887_0;
			HasTDA9887 = true;
		} 
	} 

	// Let registry settings force all of this!
	if (reg.ReadKey("TunerI2CAddr"	,lVal ,LONG(0xce))	&& lVal != 0xff ) TVTunerI2CAddr = BYTE(lVal);
	if (reg.ReadKey("HasMSP34xx"	,lVal ,LONG(false)) && lVal != 0xff ) HasMSP34xx = (lVal != 0);
	if (reg.ReadKey("HasTDA7432"	,lVal ,LONG(false)) && lVal != 0xff ) HasTDA7432 = (lVal != 0);
	if (reg.ReadKey("HasTDA8425"	,lVal ,LONG(false)) && lVal != 0xff ) HasTDA8425 = (lVal != 0);
	if (reg.ReadKey("HasTDA9840"	,lVal ,LONG(false)) && lVal != 0xff ) HasTDA9840 = (lVal != 0);
	if (reg.ReadKey("HasTDA9850"	,lVal ,LONG(false)) && lVal != 0xff ) HasTDA9850 = (lVal != 0);
	if (reg.ReadKey("HasTDA9855"	,lVal ,LONG(false)) && lVal != 0xff ) HasTDA9855 = (lVal != 0);
	if (reg.ReadKey("HasTDA9873"	,lVal ,LONG(false)) && lVal != 0xff ) HasTDA9873 = (lVal != 0);
	if (reg.ReadKey("HasTDA9874"	,lVal ,LONG(false)) && lVal != 0xff ) HasTDA9874 = (lVal != 0);
	if (reg.ReadKey("HasTDA9875"	,lVal ,LONG(false)) && lVal != 0xff ) HasTDA9875 = (lVal != 0);
	if (reg.ReadKey("HasTEA6300"	,lVal ,LONG(false)) && lVal != 0xff ) HasTEA6300 = (lVal != 0);
	if (reg.ReadKey("HasTEA6320"	,lVal ,LONG(false)) && lVal != 0xff ) HasTEA6320 = (lVal != 0);
	if (reg.ReadKey("HasTEA6420"	,lVal ,LONG(false)) && lVal != 0xff ) HasTEA6420 = (lVal != 0);
	if (reg.ReadKey("HasTDA8540"	,lVal ,LONG(false)) && lVal != 0xff ) HasTDA8540 = (lVal != 0);
	if (reg.ReadKey("HasPV951"		,lVal ,LONG(false)) && lVal != 0xff ) HasPV951	 = (lVal != 0);
	if (reg.ReadKey("HasHauppIR"	,lVal ,LONG(false)) && lVal != 0xff ) HasHauppIR = (lVal != 0);
	if (reg.ReadKey("HasKNC1IR"		,lVal ,LONG(false)) && lVal != 0xff ) HasKNC1IR  = (lVal != 0);
	if (reg.ReadKey("HasPVIEWIR"	,lVal ,LONG(false)) && lVal != 0xff ) HasPVIEWIR = (lVal != 0);
	if (reg.ReadKey("HasDPL3518"	,lVal ,LONG(false)) && lVal != 0xff ) HasDPL3518 = (lVal != 0);
	if (reg.ReadKey("HasTDA9887"	,lVal ,LONG(false)) && lVal != 0xff ) HasTDA9887 = (lVal != 0);
	if (reg.ReadKey("MSP34xxI2CAddr",lVal ,LONG(0xff)) && lVal != 0xff ) MSP34xxI2CAddr = BYTE(lVal);
	if (reg.ReadKey("TDA7432I2CAddr",lVal ,LONG(0xff)) && lVal != 0xff ) TDA7432I2CAddr = BYTE(lVal);
	if (reg.ReadKey("TDA8425I2CAddr",lVal ,LONG(0xff)) && lVal != 0xff ) TDA8425I2CAddr = BYTE(lVal);
	if (reg.ReadKey("TDA9840I2CAddr",lVal ,LONG(0xff)) && lVal != 0xff ) TDA9840I2CAddr = BYTE(lVal);
	if (reg.ReadKey("TDA9850I2CAddr",lVal ,LONG(0xff)) && lVal != 0xff ) TDA9850I2CAddr = BYTE(lVal);
	if (reg.ReadKey("TDA9855I2CAddr",lVal ,LONG(0xff)) && lVal != 0xff ) TDA9855I2CAddr = BYTE(lVal);
	if (reg.ReadKey("TDA9873I2CAddr",lVal ,LONG(0xff)) && lVal != 0xff ) TDA9873I2CAddr = BYTE(lVal);
	if (reg.ReadKey("TDA9874I2CAddr",lVal ,LONG(0xff)) && lVal != 0xff ) TDA9874I2CAddr = BYTE(lVal);
	if (reg.ReadKey("TDA9875I2CAddr",lVal ,LONG(0xff)) && lVal != 0xff ) TDA9875I2CAddr = BYTE(lVal);
	if (reg.ReadKey("TEA6300I2CAddr",lVal ,LONG(0xff)) && lVal != 0xff ) TEA6300I2CAddr = BYTE(lVal);
	if (reg.ReadKey("TEA6320I2CAddr",lVal ,LONG(0xff)) && lVal != 0xff ) TEA6320I2CAddr = BYTE(lVal);
	if (reg.ReadKey("TEA6420I2CAddr",lVal ,LONG(0xff)) && lVal != 0xff ) TEA6420I2CAddr = BYTE(lVal);
	if (reg.ReadKey("TDA8540I2CAddr",lVal ,LONG(0xff)) && lVal != 0xff ) TDA8540I2CAddr = BYTE(lVal);
	if (reg.ReadKey("PV951I2CAddr"	,lVal ,LONG(0xff)) && lVal != 0xff ) PV951I2CAddr	= BYTE(lVal);
	if (reg.ReadKey("HauppIRAddr"	,lVal ,LONG(0xff)) && lVal != 0xff ) HauppIRAddr	= BYTE(lVal);
	if (reg.ReadKey("KNC1IRAddr"	,lVal ,LONG(0xff)) && lVal != 0xff ) KNC1IRAddr		= BYTE(lVal);
	if (reg.ReadKey("PVIEWIRAddr"	,lVal ,LONG(0xff)) && lVal != 0xff ) PVIEWIRAddr	= BYTE(lVal);
	if (reg.ReadKey("DPL3518Addr"	,lVal ,LONG(0xff)) && lVal != 0xff ) DPL3518Addr	= BYTE(lVal);
	if (reg.ReadKey("TDA9887Addr"	,lVal ,LONG(0xff)) && lVal != 0xff ) TDA9887Addr	= BYTE(lVal);

	// Get from the registry the capabilities of tuner
	bool FastTune = false;
	bool HalfStep = false;
	if (reg.ReadKey("TunerFastTune"	,lVal ,LONG(false)) && lVal != 0xff ) FastTune = (lVal != 0);
	if (reg.ReadKey("TunerHalfStep"	,lVal ,LONG(false)) && lVal != 0xff ) HalfStep = (lVal != 0);

	// Correct Guess if MIRO card
	if (HasMSP34xx) {
		if (dwCard == BtCard::MIRO)
			dwCard = BtCard::MIROPRO;
		if (dwCard == BtCard::PINNACLE)
			dwCard = BtCard::PINNACLEPRO;
	}

	// Correct guess if flyvideo card
	if (HasTDA9874) {
		if (dwCard == BtCard::FLYVIDEO2000)
			dwCard = BtCard::FLYVIDEO2000S;
	}
	
	// Now, the last AudioChips are forced, depending upon the model of card. Begin Creation of objects!
	DUMP(("Detected/Read Card Type %d\n",dwCard));

	// Now create the audio Mixers
	if (HasDPL3518) {
		AudioChip[AudioCount] = new DPL3518(GetBtPisces().GetI2C(),DPL3518Addr);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found DPL3518 at 0x%02x\n",DPL3518Addr));
	}

	if (HasMSP34xx) {
		AudioChip[AudioCount] = new MSP34xx(GetBtPisces().GetI2C(),MSP34xxI2CAddr,HasDPL3518, (TVTuner != HwTuner::TC_MT2032) ); // Mt2032 uses MSP to decode FM radio
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found MSP34xx at 0x%02x\n",MSP34xxI2CAddr));
	}

	if (HasTDA7432) {
		AudioChip[AudioCount] = new TDA7432(GetBtPisces().GetI2C(),TDA7432I2CAddr);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found TDA7432 at 0x%02x\n",TDA7432I2CAddr));
	}

	if (HasTDA8425) {
		AudioChip[AudioCount] = new TDA8425(GetBtPisces().GetI2C(),TDA8425I2CAddr);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found TDA8425 at 0x%02x\n",TDA8425I2CAddr));
	}

	if (HasTDA9840) {
		AudioChip[AudioCount] = new TDA9840(GetBtPisces().GetI2C(),TDA9840I2CAddr);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found TDA9840 at 0x%02x\n",TDA9840I2CAddr));
	}

	if (HasTDA9850) {
		AudioChip[AudioCount] = new TDA9850(GetBtPisces().GetI2C(),TDA9850I2CAddr);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found TDA9850 at 0x%02x\n",TDA9850I2CAddr));
	}

	if (HasTDA9855) {
		AudioChip[AudioCount] = new TDA9855(GetBtPisces().GetI2C(),TDA9855I2CAddr);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found TDA9855 at 0x%02x\n",TDA9855I2CAddr));
	}

	if (HasTDA9873) {
		AudioChip[AudioCount] = new TDA9873(GetBtPisces().GetI2C(),TDA9873I2CAddr);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found TDA9873 at 0x%02x\n",TDA9873I2CAddr));
	}

	if (HasTDA9874) {
		AudioChip[AudioCount] = new TDA9874(GetBtPisces().GetI2C(),TDA9874I2CAddr);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found TDA9874 at 0x%02x\n",TDA9874I2CAddr));
	}

	if (HasTDA9875) {
		AudioChip[AudioCount] = new TDA9875(GetBtPisces().GetI2C(),TDA9875I2CAddr);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found TDA9875 at 0x%02x\n",TDA9875I2CAddr));
	}

	if (HasTEA6300) {
		AudioChip[AudioCount] = new TEA6300(GetBtPisces().GetI2C(),TEA6300I2CAddr);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found TEA6300 at 0x%02x\n",TEA6300I2CAddr));
	}

	if (HasTEA6320) {
		AudioChip[AudioCount] = new TEA6320(GetBtPisces().GetI2C(),TEA6320I2CAddr);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found TEA6320 at 0x%02x\n",TEA6320I2CAddr));
	}

	if (HasTEA6420) {
		AudioChip[AudioCount] = new TEA6420(GetBtPisces().GetI2C(),TEA6420I2CAddr);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found TEA6420 at 0x%02x\n",TEA6420I2CAddr));
	}

	if (HasPV951) {
		AudioChip[AudioCount] = new PV951AC(GetBtPisces().GetI2C(),PV951I2CAddr);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found PV951 at 0x%02x\n",PV951I2CAddr));
	}

   	if (dwCard == BtCard::HOONTECH) {
		AudioChip[AudioCount] = new Hoontech(GetBtPisces().GetI2C(),I2C_Hoontech);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found Hoontech at 0x%02x\n",I2C_Hoontech));
	}

	// Based on Card model, create the missing Handlers
	if (dwCard == BtCard::AVERMEDIA) {
		AudioChip[AudioCount] = new AvermediaTvPhone(GetBtPisces().GetGPIO());
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found Avermedia TVPhone audiochip\n"));
	}

	if (dwCard == BtCard::AVERTVOLD) {
		AudioChip[AudioCount] = new AvermediaTvPhoneOld(GetBtPisces().GetGPIO());
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found Avermedia TVPhone (old) audiochip\n"));
	}

	// This one uses the PT2254
	if (dwCard == BtCard::WINVIEW_601) {
		AudioChip[AudioCount] = new PT2254(GetBtPisces().GetGPIO(),
			6, // CLK
			5, // DATA
			7  // STROBE
			);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found PT2254 audiochip (clk=6,Data=5,Strobe=7\n"));
	}

	// This also uses the PT2254
	if (dwCard == BtCard::TEKRAMM205) {
		AudioChip[AudioCount] = new PT2254(GetBtPisces().GetGPIO(),
			0, // CLK
			1, // DATA
			2  // STROBE
			);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found PT2254 audiochip (clk=0,Data=1,Strobe=2\n"));
	}

	if (dwCard == BtCard::TERRATV) {
		AudioChip[AudioCount] = new TerraTV(GetBtPisces().GetGPIO());
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found TerraTV audiochip\n"));
	}

	
	if (dwCard == BtCard::GVBCTV3PCI) {
		AudioChip[AudioCount] = new GVBCTV3(GetBtPisces().GetGPIO());
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif

		DUMP(("Found GVBCTV3 audiochip\n"));
	}

	if (dwCard == BtCard::LIFETEC_9415) {
		AudioChip[AudioCount] = new LT9415(GetBtPisces().GetGPIO());
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found LT9415 audiochip\n"));
	}

	if (dwCard == BtCard::WINFAST2000) {
		AudioChip[AudioCount] = new WinFast2000(GetBtPisces().GetGPIO());
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found WinFast2000 audiochip\n"));
	}

	if (dwCard == BtCard::PV_BT878P_9B) {
		AudioChip[AudioCount] = new pvbt878p9b(GetBtPisces().GetGPIO());
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found PV_BT878P_9B audiochip\n"));
	}

	// Tekram also uses a custom I2C chip (a PIC16c54)
	if (dwCard == BtCard::TEKRAMM205) {
		AudioChip[AudioCount] = new Tekram205(GetBtPisces().GetI2C(),I2C_TEKRAM205);
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found TEKRAMM205 audiochip\n"));
	}

	if (dwCard == BtCard::WINDVR) {
		AudioChip[AudioCount] = new WinDVR(GetBtPisces().GetGPIO());
		if (AudioChip[AudioCount]) AudioCount++;
#if DBG
		else ERROR(("Out of Mem! - No audioChip Will be registered!\n"));
#endif
		DUMP(("Found WINDVR audiochip\n"));
	}

	// VideoChips
	if (HasTDA8540) {
		VideoChip[VideoCount] = new TDA8540(GetBtPisces().GetI2C(),TDA8540I2CAddr,((dwCard == BtCard::SDISILK100_C || (dwCard == BtCard::SDISILK200_C)) ? false : true) );
		if (VideoChip[VideoCount]) VideoCount++;
#if DBG
		else ERROR(("Out of Mem! - No videoChip Will be registered!\n"));
#endif
		DUMP(("Found TDA8540 at 0x%02x\n",TDA8540I2CAddr));
	}

	if (dwCard == BtCard::RS_BT) {
		VideoChip[VideoCount] = new RSBT(GetBtPisces().GetGPIO());
		if (VideoChip[VideoCount]) VideoCount++;
#if DBG
		else ERROR(("Out of Mem! - No videoChip Will be registered!\n"));
#endif
		DUMP(("Found RS_BT\n"));
	}

	if (dwCard == BtCard::SDISILK200_S || 
		dwCard == BtCard::SDISILK200_C) {
		VideoChip[VideoCount] = new SILK200(GetBtPisces().GetI2C(), I2C_SILK200_0 );
		if (VideoChip[VideoCount]) VideoCount++;
#if DBG
		else ERROR(("Out of Mem! - No videoChip Will be registered!\n"));
#endif
		DUMP(("Found SDI Silk200 video switch\n"));
	}

	if (dwCard == BtCard::PRIMAVISION) {
		VideoChip[VideoCount] = new PCF8575A(GetBtPisces().GetI2C(), I2C_PCF8575A_0 );
		if (VideoChip[VideoCount]) VideoCount++;
#if DBG
		else ERROR(("Out of Mem! - No videoChip Will be registered!\n"));
#endif
		DUMP(("Found Primavision I2C expander\n"));
	}

	if (dwCard == BtCard::GRANDTEC16) {
		VideoChip[VideoCount] = new GRANDTECx16(GetBtPisces().GetGPIO());
		if (VideoChip[VideoCount]) VideoCount++;
#if DBG
		else ERROR(("Out of Mem! - No videoChip Will be registered!\n"));
#endif
		DUMP(("Found GrandTec X Guard 16 port\n"));
	}

	if (dwCard == BtCard::TRUST814PCI) {
		VideoChip[VideoCount] = new CTRUST814PCI(GetGPIO());
		if (VideoChip[VideoCount]) VideoCount++;
#if DBG
		else ERROR(("Out of Mem! - No videoChip Will be registered!\n"));
#endif
		DUMP(("Found Trust 814 PCI port\n"));
	}

	if (dwCard == BtCard::SWEETSPOTPDI) {
		VideoChip[VideoCount] = new SAA7118(GetI2C(),I2C_SAA7118_0);
		if (VideoChip[VideoCount]) VideoCount++;
#if DBG
		else ERROR(("Out of Mem! - No videoChip Will be registered!\n"));
#endif
		DUMP(("Found SAA7118 video port driver\n"));
	}
	
	// IR chips
	if (HasHauppIR) {
		IRChip[IRCount] = new HauppaugeIR(GetBtPisces().GetI2C(),HauppIRAddr);
		if (IRChip[IRCount]) IRCount++;
#if DBG
		else ERROR(("Out of Mem! - No IRChip Will be registered!\n"));
#endif
		DUMP(("Found HauppIR at 0x%02x\n",HauppIRAddr));

	}

	if (HasKNC1IR) {
		IRChip[IRCount] = new KNC1IR(GetBtPisces().GetI2C(),KNC1IRAddr);
		if (IRChip[IRCount]) IRCount++;
#if DBG
		else ERROR(("Out of Mem! - No IRChip Will be registered!\n"));
#endif
		DUMP(("Found KNC1 IR at 0x%02x\n",KNC1IRAddr));
	}

	if (HasPVIEWIR) {
		IRChip[IRCount] = new PixelViewIR(GetBtPisces().GetI2C(),PVIEWIRAddr);
		if (IRChip[IRCount]) IRCount++;
#if DBG
		else ERROR(("Out of Mem! - No IRChip Will be registered!\n"));
#endif
		DUMP(("Found PixelView IR at 0x%02x\n",PVIEWIRAddr));

	}

	if (HasPV951) {
		IRChip[IRCount] = new PV951IR(GetBtPisces().GetI2C(),PV951I2CAddr);
		if (IRChip[IRCount]) IRCount++;
#if DBG
		else ERROR(("Out of Mem! - No IRChip Will be registered!\n"));
#endif
		DUMP(("Found PV951IR at 0x%02x\n",PV951I2CAddr));
	}

	// Tekram also uses a custom I2C chip (a PIC16c54)
	if (dwCard == BtCard::TEKRAMM205) {
		IRChip[IRCount] = new TEKRAMIR(GetBtPisces().GetI2C(),I2C_TEKRAM205);
		if (IRChip[IRCount]) IRCount++;
#if DBG
		else ERROR(("Out of Mem! - No IRChip Will be registered!\n"));
#endif
		DUMP(("Found TEKRAMM205 IR\n"));
	}

	
	/*
		- And the GPIO connected decoders (polled ones):
		Pixel View TV pro
			Mask:0x1f00		, Lock:0x8000	,Xor:0
		AverMedia
			Mask:0x0f88000	, Lock:0x010000	,Xor:0x010000 
		

		"AverMedia 98",					0x003b8000, 0x00004000, 0x00800000,          0, KeyTableAvermedia98,
		"Apac ViewComp 878",			0x00001f00,          0, 0x00008000, 0x00008000, KeyTableApacViewComp,
		"Art TV Tuner",					0x00001f00,          0, 0x00004000, 0x00004000, KeyTableArtTVTuner,
		"Askey Magic TViewer CPH031",	0x00001f00,          0, 0x00004000, 0x00004000, KeyTableAskeyCPH031,
		"BT848 Capture",				0x00001f00,          0, 0x00004000, 0x00004000, KeyTableBt848Capture,

		"FlyVideo II",					0x000000f8,          0, 0x000000f8,          0, KeyTableFlyVideo,


		"Lenco MXR 9577",				0x00001f00,          0, 0x00008000, 0x00008000, KeyTablePixelViewPro,
		"Mach TV",						0x00001f00,          0, 0x00004000, 0x00004000, KeyTableMachTV,
	//	"Miro PCTV",					0x00001f00,          0, 0x00004000, 0x00004000, NULL,
		"TTT Tritan Technology",		0x00aaf000,			 0, 0x00aaf000,          0, KeyTableTritan1,
		"TTT Tritan Technology (*)",	0x00001f00,			 0, 0x00000000, 0x00000000, KeyTableTritan2,

		"TV Capturer",					0x00001f00,          0, 0x00008000, 0x00008000, KeyTableTVCapturer,
		"Leadtek WinView 601 TV/FM (*)",0x00001f00,			 0, 0x00000000, 0x00000000, KeyTableWinview601,
  		"KWorld TV-FM",					0x00001f00,          0, 0x00004000, 0x00004000, KeyTableArtTVTuner,
  		"Dynalink TV",					0x007cf000,          0, 0x00800000, 0x00800000, KeyTableDynalink,
  		"Askey Magic TView 99 CPH053",	0x0028e000,          0, 0x00020000, 0x00020000, KeyTableAskeyTView,
  		"Avermedia TVCapture 95",		0x00f88000,          0, 0x00010000,          0, KeyTableAvermedia95,
	//	"Avermedia TVPhone 98",			0x00f88000,          0, 0x00010000,          0, NULL,
		"Avermedia TVPhone 98 w/VCR",	0x00f88000,          0, 0x00010000,          0, KeyTableAverPhone98VCR,
		"Avermedia TVStudio",			0x00f88000,          0, 0x00010000,          0, KeyTableAverPhone98VCR,
		"PixelView PlayTV Pro",			0x00001f00,          0, 0x00008000, 0x00008000, KeyTablePixelViewPro,
		"PixelView PlayTV Pak",			0x00003e00,			 0,	0x00010000,	0x00010000, KeyTablePixelViewPak,

	*/
	if (HasRemoteControl) {

		if (dwCard == BtCard::TVCAPTURER) {

			IRChip[IRCount] = new GPIOIR(GetBtPisces().GetGPIO(),0x00001f00,0x00008000,0);
			if (IRChip[IRCount]) IRCount++;
#if DBG
			else ERROR(("Out of Mem! - No IRChip Will be registered!\n"));
#endif
			DUMP(("Found TVCAPTURER IR\n"));
		}

		if (dwCard == BtCard::WINVIEW_601) {

			IRChip[IRCount] = new GPIOIR(GetBtPisces().GetGPIO(),0x00001f00,0,0);
			if (IRChip[IRCount]) IRCount++;
#if DBG
			else ERROR(("Out of Mem! - No IRChip Will be registered!\n"));
#endif
			DUMP(("Found WINVIEW_601 IR\n"));
		}
		
		if (dwCard == BtCard::KWORLD878) {

			IRChip[IRCount] = new GPIOIR(GetBtPisces().GetGPIO(),0x00001f00,0x00004000,0);
			if (IRChip[IRCount]) IRCount++;
#if DBG
			else ERROR(("Out of Mem! - No IRChip Will be registered!\n"));
#endif
			DUMP(("Found KWORLD878 IR\n"));
		}

		if (dwCard == BtCard::DYNALINK) {

			IRChip[IRCount] = new GPIOIR(GetBtPisces().GetGPIO(),0x007cf000,0x00800000,0);
			if (IRChip[IRCount]) IRCount++;
#if DBG
			else ERROR(("Out of Mem! - No IRChip Will be registered!\n"));
#endif
			DUMP(("Found DYNALINK IR\n"));
		}

		if (dwCard == BtCard::MAGICTVIEW061 /*CPH053*/) {

			IRChip[IRCount] = new GPIOIR(GetBtPisces().GetGPIO(),0x0028e000,0x00020000,0);
			if (IRChip[IRCount]) IRCount++;
#if DBG
			else ERROR(("Out of Mem! - No IRChip Will be registered!\n"));
#endif
			DUMP(("Found MAGICTVIEW061 IR\n"));
		}

		if (dwCard == BtCard::PXELVWPLTVPRO) {

			IRChip[IRCount] = new GPIOIR(GetBtPisces().GetGPIO(),0x1f00,0x8000,0);
			if (IRChip[IRCount]) IRCount++;
#if DBG
			else ERROR(("Out of Mem! - No IRChip Will be registered!\n"));
#endif
			DUMP(("Found PXELVWPLTVPRO IR\n"));
		}

		if (dwCard == BtCard::PXELVWPLTVPAK) {
			IRChip[IRCount] = new GPIOIR(GetBtPisces().GetGPIO(),0x00003e00,0x0010000,0);
			if (IRChip[IRCount]) IRCount++;
#if DBG
			else ERROR(("Out of Mem! - No IRChip Will be registered!\n"));
#endif
			DUMP(("Found PXELVWPLTVPAK IR\n"));
		}


		if (dwCard == BtCard::AVERMEDIA ||
			dwCard == BtCard::AVERMEDIA98 ||
			dwCard == BtCard::AVERTVOLD ) {

			IRChip[IRCount] = new GPIOIR(GetBtPisces().GetGPIO(),0x0f88000,0x010000,0x010000);
			if (IRChip[IRCount]) IRCount++;
#if DBG
			else ERROR(("Out of Mem! - No IRChip Will be registered!\n"));
#endif
			DUMP(("Found AVERMEDIA IR\n"));
		}

		/*
		-The interrupt - driven ones:

		Leadtek Winfast tv2000
 			Mask:0x1F8	, and an GPIO int when a new code arrives
			Uses GPIO interrupts to get codes!
			the 9:th bit in gpio output enable must be 0 or there will be no gpio interupt

		FlyVideo 98
			Mask:0x1F8	, and an GPIO int when a new code arrives
			Uses GPIO interrupts to get codes!

  		"Series TV Capture BCM",		0x000000f8,
		"Dart TV Live",					0x000000f8,
		"Genius VideoWonder Pro2",		0x000000f8,

		"FlyVideo '98",					0x000000f8,
		"Leadtek WinFast TV 2000",		0x000000f8,
		"FlyVideo 2000S",				0x0000001f,
		"Chronos Video Shuttle II",		0x000000f8,
		"Typhoon TView",				0x000000f8,

		*/


		if (dwCard == BtCard::WINFAST2000		||
			dwCard == BtCard::FLYVIDEO_98		||
			dwCard == BtCard::TYPHOON_TVIEW		||
			dwCard == BtCard::CHRONOS_VS2		||
			dwCard == BtCard::FLYVIDEO_98FM		||
			/*dwCard == BtCard::FLYVIDEO2000		||(Can't have remotes) */
			dwCard == BtCard::FLYVIDEO98EZ)		{

			//the 9:th bit in gpio output enable must be 0 or there will be no gpio interrupt
			IRChip[IRCount] = new GPIO_INTIR(GetBtPisces().GetGPIO(),0x1F8);
			if (IRChip[IRCount]) IRCount++;
#if DBG
			else ERROR(("Out of Mem! - No IRChip Will be registered!\n"));
#endif
			DUMP(("Found FLYVIDEO/WINFAST2000 IR\n"));
		}

		if (dwCard == BtCard::FLYVIDEO2000S)		{

			//the 9:th bit in gpio output enable must be 0 or there will be no gpio interrupt
			IRChip[IRCount] = new GPIO_INTIR(GetBtPisces().GetGPIO(),0x11f);
			if (IRChip[IRCount]) IRCount++;
#if DBG
			else ERROR(("Out of Mem! - No IRChip Will be registered!\n"));
#endif
			DUMP(("Found FLYVIDEO2000S IR\n"));
		}

	}

	// The IF Demodulators
	if (HasTDA9887) {
		if (dwCard == BtCard::MIRO      ||
			dwCard == BtCard::MIROPRO   ||
			dwCard == BtCard::PINNACLE  ||
			dwCard == BtCard::PINNACLEPRO) {
			// Miro Cards...
			IFDemChip[IFDemCount] = new TDA988xPinnacle(GetBtPisces().GetI2C(),TDA9887Addr,PinnacleID);
			if (IFDemChip[IFDemCount]) IFDemCount++;
#if DBG
			else ERROR(("Out of Mem! - No IFDemChip Will be registered!\n"));
#endif
			DUMP(("Found TDA988xPinnacle at 0x%02x\n",TDA9887Addr));

		} else {
			// Non Miro Cards...
			IFDemChip[IFDemCount] = new TDA9887(GetBtPisces().GetI2C(),TDA9887Addr);
			if (IFDemChip[IFDemCount]) IFDemCount++;
#if DBG
			else ERROR(("Out of Mem! - No IFDemChip Will be registered!\n"));
#endif
			DUMP(("Found TDA9887 at 0x%02x\n",TDA9887Addr));
		}
	}

	if (dwCard == BtCard::VOODOOTV_FM) {
		IFDemChip[IFDemCount] = new VoodooTV(GetBtPisces().GetGPIO());
		if (IFDemChip[IFDemCount]) IFDemCount++;
#if DBG
		else ERROR(("Out of Mem! - No IFDemChip Will be registered!\n"));
#endif
		DUMP(("Found VoodooTV\n"));
	}

	// Create an other tuner if we have a TEA5757 tuner.
	if (HasTEA5757) {

		if (dwCard == BtCard::MIROPRO ||
			dwCard == BtCard::MIRO ||
			dwCard == BtCard::PINNACLE ||
			dwCard == BtCard::PINNACLEPRO) {

			TunerChip[TunerCount] = new TEA5757(
				GetBtPisces().GetGPIO(),
				6, // WE
				8, // CLK
				9, // DATA
				7  // MOST
				);

			if (TunerChip[TunerCount]) TunerCount++;
#if DBG
			else ERROR(("Out of Mem! - No tunerChip Will be registered!\n"));
#endif
	
			DUMP(("TEA5757 Present (WE:%d,CLK:%d,DATA:%d.MOST:%d\n",
				6, // WE
				8, // CLK
				9, // DATA
				7  // MOST
				));
		} else if (
			dwCard == BtCard::VHX ||
			dwCard == BtCard::VHXDELUXE ) {

			TunerChip[TunerCount] = new TEA5757(
				GetBtPisces().GetGPIO(),
				5, // WE
				3, // CLK
				4, // DATA
				6  // MOST
				);

			if (TunerChip[TunerCount]) TunerCount++;
#if DBG
			else ERROR(("Out of Mem! - No tunerChip Will be registered!\n"));
#endif
	
			DUMP(("TEA5757 Present (WE:%d,CLK:%d,DATA:%d.MOST:%d\n",
				5, // WE
				3, // CLK
				4, // DATA
				6  // MOST
				));

		} 
#if DBG
		else {
			ERROR(("Don't know how to drive TDA5757 for this card\n"));
		}
#endif
	}

	// Create an MT2032 tuner if needed and pass it a pointer to the IFDem chip 
	if (TVTuner == HwTuner::TC_MT2032 ) {
		TunerChip[TunerCount] = new MT2032(GetBtPisces().GetI2C(),TVTunerI2CAddr, (IFDemCount) ? IFDemChip[0] : NULL );

		if (TunerChip[TunerCount]) TunerCount++;
#if DBG
		else ERROR(("Out of Mem! - No tuerChip (mt2032) Will be registered!\n"));
#endif

	} else 
	if (TVTuner == HwTuner::TC_MT2050 ) {
		TunerChip[TunerCount] = new MT2050(GetBtPisces().GetI2C(),TVTunerI2CAddr, (IFDemCount) ? IFDemChip[0] : NULL );

		if (TunerChip[TunerCount]) TunerCount++;
#if DBG
		else ERROR(("Out of Mem! - No tunerChip (mt2050) Will be registered!\n"));
#endif

	} else 
	// Create object Representing the TV Tuner!
	if (TVTuner != HwTuner::TC_Absent &&
		TVTuner != HwTuner::TC_Unknown &&
		TVTuner != HwTuner::TC_TEA5757 &&
		TVTuner != HwTuner::TC_MT2050 &&
		TVTuner != HwTuner::TC_MT2032) { 
		TunerChip[TunerCount] =
			new I2CTuner(GetBtPisces().GetI2C(),TVTunerI2CAddr,TVTuner,TVTunerDoesFM,m_bTVAudioUsedForFM,FastTune,HalfStep);

		if (TunerChip[TunerCount]) TunerCount++;
#if DBG
		else ERROR(("Out of Mem! - No tunerChip Will be registered!\n"));
#endif

		DUMP(("Found tuner %d at 0x%02x with%s FM support\n",TVTuner,TVTunerI2CAddr, (TVTunerDoesFM) ? "" : "out" ));
	}

	// All Ok! - Go On!
	m_InitedOK = true;

	// Start by setting volume of the audioChips to Normal!
	for (DWORD Idx = 0; Idx < AudioCount ; Idx++) {
		AudioChip[Idx]->SetBassTreble(m_Bass,m_Treble);
		AudioChip[Idx]->SetLoudness(m_Loudness);
		AudioChip[Idx]->SetEqualizer(m_EqBands);
		AudioChip[Idx]->SetDolby(m_DolbyParams);
		AudioChip[Idx]->SetVolume(m_VolumeLeft,m_VolumeRight);
	}

	// Set the initial values (Assumes TV tuner present!)
	TunerSetCurrentStandard(KS_AnalogVideo_NTSC_M);
	TunerSetMode(KSPROPERTY_TUNER_MODE_TV);
	TunerSetFrequency(100000000,KS_TUNER_TUNING_FINE);
	VideoSetInput(VideoConTuner);
	VideoSetCurrentStandard(KS_AnalogVideo_NTSC_M);
	AudioSetInput(AudioConTuner);
	// Try to set stereo if asked to
	AudioSetMode(
		(AudioGetCaps() & KS_TVAUDIO_MODE_STEREO) ?
			KS_TVAUDIO_MODE_STEREO :
			KS_TVAUDIO_MODE_MONO
		);
	AudioMute();

	// All done!
	m_InitedOK = true;
	return true;
}

void BtCard::AudioSetVolume(int Left,int Right)
{
	m_VolumeLeft = Left;
	m_VolumeRight = Right;
	for (DWORD Idx = 0; Idx < AudioCount ; Idx++) {
		AudioChip[Idx]->SetVolume(m_VolumeLeft,m_VolumeRight);
	}
}

void BtCard::AudioSetLoudness(int Loudness)
{
	m_Loudness = Loudness;
	for (DWORD Idx = 0; Idx < AudioCount ; Idx++) {
		AudioChip[Idx]->SetLoudness(m_Loudness);
	}
}

void BtCard::AudioSetBassTreble(int Bass,int Treble)
{
	m_Bass = Bass;
	m_Treble = Treble;
	for (DWORD Idx = 0; Idx < AudioCount ; Idx++) {
		AudioChip[Idx]->SetBassTreble(m_Bass,m_Treble);
	}
}

void BtCard::AudioControlCaps(DWORD& Caps,DWORD& Bands, DWORD* BandFreq)
{
	Caps = 0;
	Bands = 0;
	for (DWORD Idx = 0; Idx < AudioCount ; Idx++) {
		if (AudioChip[Idx]->IsVolumeAvailable()) Caps |= KSPROPERTY_AUDIO_CAPS_HASVOLUME;
		if (AudioChip[Idx]->IsBassTrebleAvailable()) Caps |= KSPROPERTY_AUDIO_CAPS_HASBASSTREBLE;
		if (AudioChip[Idx]->IsLoudnessAvailable()) Caps |= KSPROPERTY_AUDIO_CAPS_HASLOUDNESS;
		if (Bands == 0 && AudioChip[Idx]->EqualizerBandsAvailable(BandFreq)) {
			Caps |= KSPROPERTY_AUDIO_CAPS_HASEQUALIZER;
			Bands = AudioChip[Idx]->EqualizerBandsAvailable(BandFreq);
		}
		if (AudioChip[Idx]->IsDolbyAvailable()) Caps |= KSPROPERTY_AUDIO_CAPS_HASDOLBY;
	}
}

void BtCard::AudioSetEqualizer(LONG* bands)
{
	// Remember the band settings
	RtlCopyMemory(m_EqBands,bands,sizeof(m_EqBands));

	// And set them
	for (DWORD Idx = 0; Idx < AudioCount ; Idx++) {
		AudioChip[Idx]->SetEqualizer(bands);
	}
}

void BtCard::AudioGetEqualizer(LONG* bands)
{
	// Return the band settings
	RtlCopyMemory(bands,m_EqBands,sizeof(m_EqBands));
}

void BtCard::AudioSetDolby(DOLBY_PARAMS& params)
{
	// Remember the Dolby settings
	RtlCopyMemory(&m_DolbyParams,&params,sizeof(m_DolbyParams));

	// And set them
	for (DWORD Idx = 0; Idx < AudioCount ; Idx++) {
		AudioChip[Idx]->SetDolby(params);
	}
}

void BtCard::AudioGetDolby(DOLBY_PARAMS& params)
{
	// Return the Dolby settings
	RtlCopyMemory(&params,&m_DolbyParams,sizeof(m_DolbyParams));
}

DWORD BtCard::IRGetCode()
{
	if (IRCount) {
		return IRChip[0]->GetCode();
	}
	return 0;
}

void BtCard::Uninit()
{
	// We wont be inited anymore
	m_InitedOK = false;

	// Destroy all the IRChips
	while (IRCount) {
		IRCount--;
		delete IRChip[IRCount];
	};

	// Destroy all the VideoChips
	while (VideoCount) {
		VideoCount--;
		delete VideoChip[VideoCount];
	};

	// Destroy all the AudioChips
	while (AudioCount) {
		AudioCount--;
		delete AudioChip[AudioCount];
	};

	// Destroy all Tuners
	while (TunerCount) {
		TunerCount--;
		delete TunerChip[TunerCount];
	};

	// Destroy all IF Demods
	while (IFDemCount) {
		IFDemCount--;
		delete IFDemChip[IFDemCount];
	};

	// Finally, Destroy the BtPisces (not used by anything!)
	if (Pisces) {

		// Make sure no more ints are routed there...
		AcquireISR();
		BtPisces* p = Pisces; Pisces = NULL;
		ReleaseISR();

		// And destroy the adapter
		delete p;
	}
}

BtCard::~BtCard()
{
}

void BtCard::ReinitIfIdle()
{
}

// VideoChannel Creation / Destruction
VideoChannel* BtCard::OpenVideoChannel(
			const VCMode mode,			// Mode of opening
			VideoChanIface * xface,		// Data buffer provider - consumer
			const MRect& analogwin,		// AnalogWindow
			const MRect& digitalwin,	// DigitalWindow
			const ColFmt ColorFormat	// Color format
			) 
{ 
	// Create the Video channel
	VideoChannel *ret = GetBtPisces().OpenVideoChannel(mode,xface,analogwin,digitalwin,ColorFormat); 

	// Keep track of number of open video channels
	if (ret) m_VCOpened++;

	// Return video channels
	return ret;
}

void BtCard::CloseVideoChannel( VideoChannel * chan ) 
{ 
	// Keep track of number of open video channels
	if (chan) m_VCOpened--;

	// Delete the video channel
	GetBtPisces().CloseVideoChannel(chan); 
}


// Called when an Int happens...
BOOLEAN BtCard::HwInterrupt() 
{ 
	// Adapter exists... Make sure we can actually call this...
	if (InterlockedIncrement(&ISRLock) > 1) {
		// Someone else is owning the ISR, don't call our ISR
		InterlockedDecrement(&ISRLock);
		// We were not causing the IRQ.
		return FALSE;
	}

	// If BtPisces not inited yet, do not pass the ints to it (shared ints are dangerous)
	if (Pisces == NULL) {

		// Release ISR.
		InterlockedDecrement(&ISRLock);

		return FALSE;
	}

	BOOLEAN res = GetBtPisces().HwInterrupt(); 

	// Release ISR.
	InterlockedDecrement(&ISRLock);

	// Return the result.
	return res;

}

void BtCard::AcquireISR( void )
{
	TRACE(("BtCard::AcquireISR"));

	// Get the ISR lock...
	while (InterlockedIncrement(&ISRLock) > 1) {
		InterlockedDecrement(&ISRLock);
		mdelay(1);
		DUMP(("waiting for ISR to be idle\n"));
	}
}

void BtCard::ReleaseISR( void )
{
	TRACE(("BtCard::ReleaseISR"));
	InterlockedDecrement(&ISRLock);
}


#ifdef __FULLDECODERACCESS__ // NOT RECOMMENDED AT ALL

// The following functions were meant not to be called from userland...
DWORD BtCard::ReadDecoder(const DWORD dwAddress)
{
	// Read the required register...
	return READ_REGISTER_ULONG( PULONG(dwAddress + dwBaseAddress) );
}

void BtCard::WriteDecoder(const DWORD dwAddress,const DWORD dwValue)
{
	// Write to the required register...
	WRITE_REGISTER_ULONG( PULONG(dwAddress + dwBaseAddress) , dwValue );
}

#endif